package org.ow2.choreos.abstractionBase;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.InterfacesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups.JointGroup;
import org.ow2.choreos.gui.UserSettings;
import org.ow2.choreos.serviceRegistration.ServiceRegistration;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;
import org.ow2.choreos.unknownClasses.UnknownClassesController;


public class AbstractionBaseController {


	//Variables.
	private Map<Integer, Hierarchy> hierarchies = new Hashtable<Integer, Hierarchy>();//The id of a hierarchy is the same with the id of a collection!

	private ServiceRegistration serviceRegistration;


	//Constructor.
	public AbstractionBaseController( ServiceRegistration serviceRegistration ){

		this.serviceRegistration = serviceRegistration;
	}


	//Verifiers.
	public static boolean isHierarchyKind( Object object ){

		return object != null ? object instanceof Hierarchy : false;
	}

	public static boolean isFunctionalAbstractionKind( Object object ){

		return object != null ? object instanceof FunctionalAbstraction : false;
	}

	public static boolean isAbstractionBaseKind( Object object ){

		if( isHierarchyKind( object ) ) return true;

		if( isFunctionalAbstractionKind( object ) ) return true;


		return false;
	}


	//Create an object for the new hierarchy in the abstraction base.
	public Hierarchy initHierarchy( UserSettings options, List<ServiceInterface> allInterfaces ){

		Hierarchy hierarchy = lookUp( options.getCollectionID() );


		if( hierarchy == null ) hierarchy = createHierarchy( options, allInterfaces );

		else if( hierarchy.getMode() != options.getMode() ){

			hierarchies.remove( hierarchy.getCollectionID() );//TODO: free objects!

			hierarchy = createHierarchy( options, allInterfaces );
		}

		else{

			if( options.getMode() == 1 ) ( (FunctionalHierarchyWithNestedNFAbstractions) hierarchy).update( options, allInterfaces);

			else if( options.getMode() == 2 ){//TODO: same as the other options!
				//hierarchy = new NonFunctionalHierarchyWithNestedFAbstractions( namesDistanceCalculator, options, allInterfaces );
			}

			else if( options.getMode() == 3 ) ( (IndependentHierarchies) hierarchy).update( options, allInterfaces );
		}


		return hierarchy;
	}

	private Hierarchy createHierarchy( UserSettings options, List<ServiceInterface> allInterfaces ){

		Hierarchy hierarchy = null;


		if( options.getMode() == 1 ) hierarchy = new FunctionalHierarchyWithNestedNFAbstractions( options, allInterfaces );

		else if( options.getMode() == 2 ) hierarchy = new NonFunctionalHierarchyWithNestedFAbstractions( options, allInterfaces );

		else if( options.getMode() == 3 ) hierarchy = new IndependentHierarchies( options, allInterfaces );


		hierarchies.put( options.getCollectionID(), hierarchy );


		return hierarchy;
	}


	//Getters.
	public Map<Integer, FunctionalAbstraction> get( Integer collectionID ){

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy == null )

			return null;

		if( hierarchy.getMode() == 1 )

			return null;

		if( hierarchy.getMode() == 2 )

			return null;

		if( hierarchy.getMode() == 3 )

			return ((IndependentHierarchies) hierarchy).getFunctionaLAbstractions();

		return null;
	}
	
	public Hashtable<Integer,NonFunctionalAbstraction> getNonFunctional( Integer collectionID ){

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy == null )

			return null;

		if( hierarchy.getMode() == 1 )

			return null;

		if( hierarchy.getMode() == 2 )

			return null;

		if( hierarchy.getMode() == 3 )

			return ((IndependentHierarchies) hierarchy).getNonFunctionalAbstractions();

		return null;
	}
	
	public Map<Integer, Hierarchy> getHierarchies(){

		return hierarchies;
	}

	public boolean exists( Integer collectionID ){

		if( lookUp( collectionID ) != null ) return true;

		return false;
	}

	public Hierarchy lookUp( Integer collectionID ){

		return hierarchies.get( collectionID );
	}

	public Integer getThreshold( Integer collectionID ){

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy.getMode() == 1 ) return ( (FunctionalHierarchyWithNestedNFAbstractions) hierarchy).getThreshold();

		else if( hierarchy.getMode() == 2 ) return ( (NonFunctionalHierarchyWithNestedFAbstractions) hierarchy).getThreshold();

		else if( hierarchy.getMode() == 3 ) return ( (IndependentHierarchies) hierarchy).getNonFunctionaLHierarchy().getThreshold();

		return null;
	}

	//Adding a functional abstraction (mode = 1 or 3).
	public ServiceInterface addAbstraction( Integer collectionID, Integer functionalAbstractionID, Integer descendant1, Integer descendant2, InterfacesMapping interfacesMapping, List<ServiceInterface> unRepresentedInterfaces ){

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy.getMode() == 1 ) return ((FunctionalHierarchyWithNestedNFAbstractions) hierarchy).addAbstraction( functionalAbstractionID, descendant1, descendant2, interfacesMapping, unRepresentedInterfaces);

		else if( hierarchy.getMode() == 3 ) return ( (IndependentHierarchies) hierarchy ).getFunctionalHierarchy().addAbstraction( functionalAbstractionID, descendant1, descendant2, interfacesMapping, unRepresentedInterfaces);


		return null;
	}

	public List<ServiceInterface> getRepresentedInterfaces ( Integer collectionID, Integer functionalAbstractionID ){
		Hierarchy hierarchy = lookUp( collectionID );

		return ((FunctionalHierarchy) hierarchy).getRepresentedInterfaces( functionalAbstractionID );
	}

	//Adding a non-functional abstraction (mode = 1).
	public NonFunctionalAbstraction addAbstraction( Integer collectionID, Integer functionalAbstractionID, Integer nonFunctionalAbstractionID, Integer ancestorID, JointGroup jointGroup ){

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy == null ) return null;

		return ((FunctionalHierarchyWithNestedNFAbstractions) hierarchy).addAbstraction( functionalAbstractionID, nonFunctionalAbstractionID, ancestorID, jointGroup );
	}


	//Adding a non-functional abstraction ( mode = 2 or 3 ).
	public NonFunctionalAbstraction addAbstraction( Integer collectionID, Integer nonFunctionalAbstractionID, Integer ancestorID, JointGroup jointGroup ){

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy == null ) return null;

		if( hierarchy.getMode() == 2 ) return ( (NonFunctionalHierarchyWithNestedFAbstractions) hierarchy).addAbstraction( serviceRegistration, nonFunctionalAbstractionID, ancestorID, jointGroup );

		else if( hierarchy.getMode() == 3 ) return ( (IndependentHierarchies) hierarchy).getNonFunctionaLHierarchy().addAbstraction( serviceRegistration, nonFunctionalAbstractionID, ancestorID, jointGroup );


		return null;
	}

	//Adding a functional abstraction (mode = 2).
	public ServiceInterface addAbstraction( Integer collectionID, Integer nonFunctionalAbstractionID, Integer functionalAbstractionID, Integer descendant1, Integer descendant2, InterfacesMapping interfacesMapping, ArrayList<ServiceInterface> unRepresentedInterfaces ){

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy == null ) return null;


		return ( (NonFunctionalHierarchyWithNestedFAbstractions) hierarchy).addAbstraction( nonFunctionalAbstractionID, functionalAbstractionID, descendant1, descendant2, interfacesMapping, unRepresentedInterfaces );
	}

	//Methods for updating a hierarchy.
	public void updateHierarchy( UserSettings options, ArrayList<ServiceInterface> interfaces ){//TODO: updated interfaces, so ...
		
	}

	//Remove hierarchy.
	public void removeHierarchy( Integer collectionID ){
		Hierarchy hierarchy = lookUp(collectionID);

		if( hierarchy != null ) hierarchies.remove( collectionID );//TODO: free objects!
	}

	//Visualization.
	public DefaultMutableTreeNode visualize( Integer collectionID ){

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy == null ) return null;

		if( hierarchy.getMode() == 1 ) return ( (FunctionalHierarchyWithNestedNFAbstractions) hierarchy).visualize( serviceRegistration );

		else if( hierarchy.getMode() == 2 ) return ((NonFunctionalHierarchyWithNestedFAbstractions) hierarchies.get(collectionID)).visualize();

		return null;
	}

	public DefaultMutableTreeNode visualizeFH( Integer collectionID ){

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy == null ) return null;


		return ((IndependentHierarchies) hierarchy).visualizeFH();
	}

	public String getFinalConstraintViolationFH( Integer collectionID ){

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy == null ) return null;


		return ( (IndependentHierarchies) hierarchy).getFinalConstraintViolation();
	}

	public DefaultMutableTreeNode visualizeNFH( Integer collectionID ){//This function is only for independent non-functional hierarchy.

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy == null ) return null;

		return ( (IndependentHierarchies) hierarchy ).visualizeNFH();
	}

	public DefaultMutableTreeNode visualizeFA( Integer collectionID, Integer functionalAbstractionID ){//This function is only for a functional abstraction.

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy == null ) return null;

		return ((IndependentHierarchies) hierarchy).visualizeFA( functionalAbstractionID );
	}

	public DefaultMutableTreeNode visualizeFAWithNestedNF( Integer collectionID, Integer functionalAbstractionID ){//This function is only for a functional abstraction with nested non-functional abstractions.

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy == null ) return null;

		return ( (FunctionalHierarchyWithNestedNFAbstractions) hierarchy).visualize( functionalAbstractionID );
	}

	public DefaultMutableTreeNode visualizeNFA( Integer collectionID, Integer nonFunctionalAbstractionID ){//This function is only for a non-functional abstraction.

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy == null ) return null;

		return ((IndependentHierarchies) hierarchy).visualizeNFA( serviceRegistration, nonFunctionalAbstractionID );
	}

	public DefaultMutableTreeNode visualizeServiceInterface( Integer collectionID, Integer interfaceID, Integer ancestorID ){//This function is only for independent functional hierarchy.

		Hierarchy hierarchy = lookUp( collectionID );

		if( hierarchy == null ) return null;

		return ( (IndependentHierarchies) hierarchy).visualizeServiceInterface( interfaceID, ancestorID );
	}


	//Getters.
	public static String getTag( Object object ){

		if( isHierarchyKind( object ) ) return ( (Hierarchy) object).getTag();

		if( isFunctionalAbstractionKind( object ) ) return ( (FunctionalAbstraction) object).getTag();


		return UnknownClassesController.getUnknownTagName();
	}

	public static String getConstraintViolation( Object object ){

		if( isHierarchyKind( object ) ) return ( (Hierarchy) object).getConstraintViolation();

		if( isFunctionalAbstractionKind( object ) ) return ( (FunctionalAbstraction) object).getConstraintViolation();


		return UnknownClassesController.getUnknownTagName();
	}


	//Setters.
	public static String setConstraintViolation( Object object, String constraintViolation ){

		if( isHierarchyKind( object ) ) return ( (Hierarchy) object).setConstraintViolation( constraintViolation );

		if( isFunctionalAbstractionKind( object ) ) return ( (FunctionalAbstraction) object).setConstraintViolation( constraintViolation );


		return UnknownClassesController.getUnknownTagName();
	}
}
