package org.ow2.choreos.abstractionBase;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import javax.swing.tree.DefaultMutableTreeNode;

import org.omg.CORBA.IntHolder;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.InterfacesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups.JointGroup;
import org.ow2.choreos.gui.UserSettings;
import org.ow2.choreos.serviceRegistration.ServiceRegistration;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


public class FunctionalHierarchyWithNestedNFAbstractions extends FunctionalHierarchy {//This class is exclusively for the mode = 1!


	//Variables.
	private Integer threshold;//Non-functional options.


	//Constructor.
	FunctionalHierarchyWithNestedNFAbstractions( UserSettings options, List<ServiceInterface> unRepresentedInterfaces ) {

		super( options, unRepresentedInterfaces );

		threshold = options.getThreshold();
	}


	//Getters.
	Integer getThreshold(){
		return threshold;
	}


	List<ServiceInterface> getRepresentedInterfaces ( Integer functionalAbstractionID ){

		FunctionalAbstraction abstraction = lookUp( functionalAbstractionID );

		if( abstraction == null ) return null;


		return ( (FunctionalAbstractionWithNestedNFAbstractions) abstraction).getRepresentedInterfaces();
	}

	//Setters.
	//Remove all the elements of the existing hierarchy because the options have been changed.
	void update( UserSettings options, List<ServiceInterface> cUnRepresentedInterfaces ){//This function is only to re-produce the hierarchy of the same versions of interfaces.

		mode = options.getMode();
		namesDistanceCalculatorAlgorithm = options.getNamesDistanceCalculatorAlgorithm();
		schemaRepresentationModel = options.getSchemaRepresentationModel();

		unRepresentedInterfaces = cUnRepresentedInterfaces;

		threshold = options.getThreshold();//Non-functional options.
	}


	//Methods for adding a functional abstraction (mode = 1).
	ServiceInterface addAbstraction( Integer functionalAbstractionID, Integer descendant1, Integer descendant2, InterfacesMapping interfacesMapping, List<ServiceInterface> unRepresentedInterfaces ){

		this.unRepresentedInterfaces = unRepresentedInterfaces;//Update the 'unRepresentedInterfaces' list at each step!

		ArrayList<FunctionalAbstraction> descenadants = null;
		FunctionalAbstraction functionalAbstractionDescendant1 = null;
		FunctionalAbstraction functionalAbstractionDescendant2 = null;

		if( functionalAbstractions == null )
			functionalAbstractions = new Hashtable<Integer, FunctionalAbstraction>();

		else{
			functionalAbstractionDescendant1 = lookUp( descendant1 );//The descendants may refer to concrete interface, not to interface of abstraction.
			functionalAbstractionDescendant2 = lookUp( descendant2 );

			if( functionalAbstractionDescendant1 != null ){
				descenadants = new ArrayList<FunctionalAbstraction>();
				descenadants.add( functionalAbstractionDescendant1 );
			}

			if( functionalAbstractionDescendant2 != null ){
				if( descenadants == null ) descenadants = new ArrayList<FunctionalAbstraction>();
				descenadants.add( functionalAbstractionDescendant2 );
			}
		}

		FunctionalAbstraction addedFunctionalAbstraction = new FunctionalAbstractionWithNestedNFAbstractions( namesDistanceCalculatorAlgorithm, functionalAbstractionID, descenadants, interfacesMapping );

		functionalAbstractions.put( functionalAbstractionID, addedFunctionalAbstraction );

		if( functionalAbstractionDescendant1 != null ) functionalAbstractionDescendant1.setAncestor( addedFunctionalAbstraction );
		if( functionalAbstractionDescendant2 != null ) functionalAbstractionDescendant2.setAncestor( addedFunctionalAbstraction );

		return functionalAbstractions.get( functionalAbstractionID ).getInterface();
	}

	//Adding a non-functional abstraction (mode = 1).
	NonFunctionalAbstraction addAbstraction( Integer functionalAbstractionID, Integer nonFunctionalAbstractionID, Integer ancestorID, JointGroup jointGroup ){

		FunctionalAbstraction functionalAbstraction = lookUp( functionalAbstractionID );

		if( functionalAbstraction == null ) return null;

		return ((FunctionalAbstractionWithNestedNFAbstractions) functionalAbstraction).addAbstraction( nonFunctionalAbstractionID, ancestorID, jointGroup );
	}


	//Visualization.
	DefaultMutableTreeNode visualize( Integer functionalAbstractionID ){

		functionalAbstractionID = functionalAbstractionID + totalNumOfInterfaces - 1;

		FunctionalAbstraction functionalAbstraction = lookUp( functionalAbstractionID );

		if( functionalAbstraction == null ) return null;
		
		return ( (FunctionalAbstractionWithNestedNFAbstractions) functionalAbstraction).visualize( totalNumOfInterfaces );
	}

	DefaultMutableTreeNode visualize( ServiceRegistration serviceRegistration ){

		DefaultMutableTreeNode root = new DefaultMutableTreeNode("Hierarchy");

		if( unRepresentedInterfaces != null && ! unRepresentedInterfaces.isEmpty() ){

			DefaultMutableTreeNode child1 = new DefaultMutableTreeNode("Unrepresented service interfaces");

			IntHolder labelOfInstances = new IntHolder(0);

			for( int i = 0; i < unRepresentedInterfaces.size(); ++i )

				child1 = serviceRegistration.visualize( labelOfInstances, collectionID, unRepresentedInterfaces.get(i), child1 );

			root.add(child1);
		}

		if( functionalAbstractions != null ){

			DefaultMutableTreeNode child2 = new DefaultMutableTreeNode("Functional abstractions");
			root.add(child2);

			for (Enumeration<Integer> e = functionalAbstractions.keys(); e.hasMoreElements();){

				Integer k = e.nextElement(); 

				if( functionalAbstractions.get( k ).ancestor == null )

					child2 = ((FunctionalAbstractionWithNestedNFAbstractions) functionalAbstractions.get( k )).visualize( serviceRegistration, collectionID, totalNumOfInterfaces, child2 );
			}
		}

    	return root;
	}
}
