package org.ow2.choreos.abstractionBase;

import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups.JointGroup;
import org.ow2.choreos.gui.UserSettings;
import org.ow2.choreos.serviceRegistration.ServiceRegistration;


public class NonFunctionalHierarchy extends Hierarchy {


	// Variables.
	protected Integer threshold;//Non-functional options.

	protected Hashtable<Integer, NonFunctionalAbstraction> nonFunctionalAbstractions;


	//Constructor.
	public NonFunctionalHierarchy( Integer collectionID, Integer mode, Integer cThreshold ){//This class is exclusively for the mode = 3!

		super( collectionID, mode );

		threshold = cThreshold;
	}


	//Setters.
	//Remove all the elements of the existing hierarchy because the options have been changed.
	void update( UserSettings options ){//This function is only to re-produce the hierarchy of the same versions of interfaces.

		this.mode = options.getMode();
		this.threshold = options.getThreshold();//Non-functional options!

		freeObjects();
	}

	void freeObjects(){

		for( int i = 0; nonFunctionalAbstractions != null && i < nonFunctionalAbstractions.size(); ++i ) nonFunctionalAbstractions.get(i).freeObjects();

		nonFunctionalAbstractions = null;
	}


	//Getters.
	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		//It did not find any checkConstraint method.


		if( hasConstraintViolation() ) return getConstraintViolation();


		//TODO: ...


		return getConstraintViolation();
	}

	public String getTag(){

		return "Non-functional hierarchy of the collection " + collectionID; 
	}

	Integer getThreshold(){

		return threshold;
	}

	NonFunctionalAbstraction lookUp( Integer nonFunctionalAbstractionID ){

		if( nonFunctionalAbstractions == null ) return null;

		return nonFunctionalAbstractions.get( nonFunctionalAbstractionID );
	}
	
	Hashtable<Integer, NonFunctionalAbstraction> get(){

		return nonFunctionalAbstractions;
	}

	int getNumOfUnrepresentedInstances(){

		int numOfUnrepresentedInterfaces = 0;


		return numOfUnrepresentedInterfaces;
	}

	int getTotalNumOfRepresentedInstances(){

		int numOfRepresentedInstances = 0;

		if( nonFunctionalAbstractions != null ){

			for( Enumeration<Integer> e = nonFunctionalAbstractions.keys(); e.hasMoreElements(); ){

				Integer k = e.nextElement(); 

				if( nonFunctionalAbstractions.get( k ).ancestor == null && nonFunctionalAbstractions.get( k ).getRepresentedServiceInstances() != null ) numOfRepresentedInstances += nonFunctionalAbstractions.get( k ).getRepresentedServiceInstances().size(); 
			}
		}


    	return numOfRepresentedInstances;
	}


	//Methods for updating the hierarchy.
	void updateHierarchy( UserSettings options ){

		mode = options.getMode();

		for( int i = 0; nonFunctionalAbstractions != null && i < nonFunctionalAbstractions.size(); ++i ) nonFunctionalAbstractions.get(i).freeObjects();
	}

	//Adding a non-functional abstraction (mode = 3).
	NonFunctionalAbstraction addAbstraction( ServiceRegistration serviceRegistration, Integer nonFunctionalAbstractionID, Integer ancestorID, JointGroup jointGroup ){

		NonFunctionalAbstraction nonFunctionalAbstractionAncestor = null;


		if( nonFunctionalAbstractions == null ) nonFunctionalAbstractions = new Hashtable<Integer, NonFunctionalAbstraction>();

		else nonFunctionalAbstractionAncestor = lookUp( ancestorID );


		NonFunctionalAbstraction addedNonFunctionalAbstraction = new NonFunctionalAbstraction( nonFunctionalAbstractionID, jointGroup, nonFunctionalAbstractionAncestor );

		if( nonFunctionalAbstractionAncestor != null ) nonFunctionalAbstractionAncestor.addDescendant( addedNonFunctionalAbstraction );

		nonFunctionalAbstractions.put( nonFunctionalAbstractionID, addedNonFunctionalAbstraction );


		return addedNonFunctionalAbstraction;
	}

	//Visualization.
	public DefaultMutableTreeNode visualize(){//This is the case of visualizing an independent non-functional hierarchy (mode = 3).

		DefaultMutableTreeNode root = new DefaultMutableTreeNode( "Hierarchy" );

		if( nonFunctionalAbstractions != null ){

			DefaultMutableTreeNode child2 = new DefaultMutableTreeNode( "Non-functional abstractions" );

			root.add( child2 );

			for (Enumeration<Integer> e = nonFunctionalAbstractions.keys(); e.hasMoreElements();){

				Integer k = e.nextElement(); 

				if( nonFunctionalAbstractions.get( k ).ancestor == null ) child2 = nonFunctionalAbstractions.get( k ).visualize( child2 );
			}
		}

    	return root;
	}

	DefaultMutableTreeNode visualize( ServiceRegistration serviceRegistration, Integer nonFunctionalAbstractionID ){

		nonFunctionalAbstractionID = nonFunctionalAbstractionID - 1;

		NonFunctionalAbstraction nonFunctionalAbstraction = lookUp( nonFunctionalAbstractionID );

		if( nonFunctionalAbstraction == null ) return null;
		
		return nonFunctionalAbstraction.visualize( serviceRegistration, collectionID, null );
	}
}