package org.ow2.choreos.abstractionDrivenServiceOrganization.functionalOrganization.hierarchicalClustering;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import org.omg.CORBA.IntHolder;
import org.omg.CORBA.LongHolder;
import org.ow2.choreos.abstractionBase.AbstractionBaseController;
import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.service.InterfacesDistanceCalculator;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.InterfacesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.NonFunctionalOrganization;
import org.ow2.choreos.gui.UserSettings;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.gui.ServiceOrganizationVisualizer;
import org.ow2.choreos.gui.interfaceMappingsVisualizer.ParameterizedServiceOrganizationVisualizer;
import org.ow2.choreos.measurements.number.DecimalPrecision;
import org.ow2.choreos.measurements.time.Delay;
import org.ow2.choreos.measurements.time.ElapsedTimeCalculator;
import org.ow2.choreos.serviceRegistration.ServiceRegistration;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


public class FunctionalOrganizationHierarchicalClustering {


	public static void produceFH3( ServiceRegistration serviceRegistration, AbstractionBaseController abstractionBase, UserSettings options, List<ServiceInterface> allInterfaces, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){//Mode = 3.

		if( allInterfaces == null ) return;//I check the non existence of interfaces without knowing if it has been already checked by the caller to be sure for the future that this function will be functional!
		
		reset( options.getDelay(), serviceOrganizationVisualizer );//GUI!

		List<ServiceInterface> unRepresentedInterfaces = new ArrayList<ServiceInterface>();

		for( int i = 0; allInterfaces != null && i < allInterfaces.size(); ++i ) unRepresentedInterfaces.add( allInterfaces.get( i ) );

		Map<Integer, ServiceInterface> interfaces = initializeInterfacesHashTable( allInterfaces );//Construct the hash table of the interfaces!

		loadExecutionLabel( options.getDelay(), serviceOrganizationVisualizer );//GUI!


		//Initialization phase.
		LongHolder millis = new LongHolder( 0 );


		int initialNumOfInterfaces = allInterfaces.size();

		Map<String, InterfacesMapping> allInterfacesMappings = calculateDistancesAndMappings( millis, options, interfaces, serviceOrganizationVisualizer );//Construct the hash table of the mapped interfaces!


		int initialPairs = ( allInterfaces.size() * ( allInterfaces.size() - 1 ) ) / 2;

		if( allInterfacesMappings != null ) loadInitializationCompleted( options.getDelay(), initialPairs, allInterfacesMappings.size(), serviceOrganizationVisualizer );//GUI!

		else loadInitializationCompleted( options.getDelay(), initialPairs, 0, serviceOrganizationVisualizer );//GUI!


		Long startTime = System.currentTimeMillis();

		Integer functionalAbstractionID = initialNumOfInterfaces - 1;//If there is no initial mapping, then it used only for visualization purposes.


		if( allInterfacesMappings != null ){

			printAllInterfaceMapping( allInterfacesMappings );

			IntHolder member1OfPair = new IntHolder( -1 ), member2OfPair = new IntHolder( -1 );

			do {//Clustering interfaces and extracting abstractions phase.
				
				member1OfPair.value = -1;
				member2OfPair.value = -1;

				InterfacesMapping minInterfacesMapping = findMin( allInterfacesMappings, member1OfPair, member2OfPair );

				PrintMessages.print( PrintMessages.organizationPrint, "Min distance has the pair of interfaces: (" + minInterfacesMapping.getServiceInterfaces().get( 0 ).getName() + ", " + minInterfacesMapping.getServiceInterfaces().get( 1 ).getName() + ") - id1 = " + member1OfPair.value + " - id2 = " + member2OfPair.value );


				//Extract functional abstraction!
				++functionalAbstractionID;

				unRepresentedInterfaces.remove( minInterfacesMapping.getServiceInterfaces().get( 0 ) );
				unRepresentedInterfaces.remove( minInterfacesMapping.getServiceInterfaces().get( 1 ) );

				ServiceInterface interfaceOfAbstraction = abstractionBase.addAbstraction( options.getCollectionID(), functionalAbstractionID, member1OfPair.value, member2OfPair.value, minInterfacesMapping, unRepresentedInterfaces );


				millis.value = millis.value + System.currentTimeMillis() - startTime;//Pause!

				loadFunctionalHierarchy( options.getDelay(), abstractionBase, options.getCollectionID(), serviceOrganizationVisualizer );//GUI!


				if( interfaceOfAbstraction == null ) break;

				loadProducedFALabels( options.getDelay(), functionalAbstractionID - initialNumOfInterfaces + 1, interfaceOfAbstraction.getName(), serviceOrganizationVisualizer );//GUI!


				startTime = System.currentTimeMillis();


				interfaces.remove( member1OfPair.value );//Update the contents of the 'allInterfacesMapping' hashtable and 'interfaces' hashtable.
				interfaces.remove( member2OfPair.value );

				allInterfacesMappings = removeAllInterfaceMappings( allInterfacesMappings, member1OfPair.value, member2OfPair.value );

				millis.value = millis.value + System.currentTimeMillis() - startTime;//Pause!


				allInterfacesMappings = addToAllInterfaceMappings( millis, options, functionalAbstractionID, interfaceOfAbstraction, interfaces, allInterfacesMappings, serviceOrganizationVisualizer );

				interfaces.put( functionalAbstractionID, interfaceOfAbstraction );


				resetLabelsOfFA( options.getDelay(), serviceOrganizationVisualizer );//GUI!


				startTime = System.currentTimeMillis();


				printAllInterfaceMapping( allInterfacesMappings );


			} while( ! allInterfacesMappings.isEmpty() && interfaces.size() > 1 );
		}


		resetLabelsOfFA( options.getDelay(), serviceOrganizationVisualizer );//GUI!

		loadProducedLabel( options.getDelay(), functionalAbstractionID - initialNumOfInterfaces + 1, serviceOrganizationVisualizer );//GUI!

		loadTerminationLabel( options.getDelay(), ElapsedTimeCalculator.getTime( millis.value ), serviceOrganizationVisualizer );//GUI!


		loadFunctionalHierarchy( options.getDelay(), abstractionBase, options.getCollectionID(), serviceOrganizationVisualizer );//GUI!

		enableStoreButton( options.getDelay(), serviceOrganizationVisualizer );//GUI!
	}
	
//Parameterized
	public static void produceFH3( ServiceRegistration serviceRegistration, AbstractionBaseController abstractionBase, UserSettings options, List<ServiceInterface> allInterfaces, ParameterizedServiceOrganizationVisualizer serviceOrganizationVisualizer ) {
		if( allInterfaces == null ) return;//I check the non existence of interfaces without knowing if it has been already checked by the caller to be sure for the future that this function will be functional!
		
//		reset( options.getDelay(), serviceOrganizationVisualizer );//GUI!

		List<ServiceInterface> unRepresentedInterfaces = new ArrayList<ServiceInterface>();

		for( int i = 0; allInterfaces != null && i < allInterfaces.size(); ++i ) unRepresentedInterfaces.add( allInterfaces.get( i ) );

		Map<Integer, ServiceInterface> interfaces = initializeInterfacesHashTable( allInterfaces );//Construct the hash table of the interfaces!

//		loadExecutionLabel( options.getDelay(), serviceOrganizationVisualizer );//GUI!


		//Initialization phase.
		LongHolder millis = new LongHolder( 0 );


		int initialNumOfInterfaces = allInterfaces.size();

		Map<String, InterfacesMapping> allInterfacesMappings = calculateDistancesAndMappings( millis, options, interfaces );//Construct the hash table of the mapped interfaces!


//		int initialPairs = ( allInterfaces.size() * ( allInterfaces.size() - 1 ) ) / 2;

//		if( allInterfacesMappings != null ) loadInitializationCompleted( options.getDelay(), initialPairs, allInterfacesMappings.size(), serviceOrganizationVisualizer );//GUI!

//		else loadInitializationCompleted( options.getDelay(), initialPairs, 0, serviceOrganizationVisualizer );//GUI!


		Long startTime = System.currentTimeMillis();

		Integer functionalAbstractionID = initialNumOfInterfaces - 1;//If there is no initial mapping, then it used only for visualization purposes.


		if( allInterfacesMappings != null ){

			printAllInterfaceMapping( allInterfacesMappings );

			IntHolder member1OfPair = new IntHolder( -1 ), member2OfPair = new IntHolder( -1 );

			do {//Clustering interfaces and extracting abstractions phase.

				member1OfPair.value = -1;
				member2OfPair.value = -1;

				InterfacesMapping minInterfacesMapping = findMin( allInterfacesMappings, member1OfPair, member2OfPair );

				PrintMessages.print( PrintMessages.organizationPrint, "Min distance has the pair of interfaces: (" + minInterfacesMapping.getServiceInterfaces().get( 0 ).getName() + ", " + minInterfacesMapping.getServiceInterfaces().get( 1 ).getName() + ") - id1 = " + member1OfPair.value + " - id2 = " + member2OfPair.value );


				//Extract functional abstraction!
				++functionalAbstractionID;

				unRepresentedInterfaces.remove( minInterfacesMapping.getServiceInterfaces().get( 0 ) );
				unRepresentedInterfaces.remove( minInterfacesMapping.getServiceInterfaces().get( 1 ) );

				ServiceInterface interfaceOfAbstraction = abstractionBase.addAbstraction( options.getCollectionID(), functionalAbstractionID, member1OfPair.value, member2OfPair.value, minInterfacesMapping, unRepresentedInterfaces );


				millis.value = millis.value + System.currentTimeMillis() - startTime;//Pause!

//				loadFunctionalHierarchy( options.getDelay(), abstractionBase, options.getCollectionID(), serviceOrganizationVisualizer );//GUI!


				if( interfaceOfAbstraction == null ) break;

//				loadProducedFALabels( options.getDelay(), functionalAbstractionID - initialNumOfInterfaces + 1, interfaceOfAbstraction.getName(), serviceOrganizationVisualizer );//GUI!


				startTime = System.currentTimeMillis();


				interfaces.remove( member1OfPair.value );//Update the contents of the 'allInterfacesMapping' hash table and 'interfaces' hash table.
				interfaces.remove( member2OfPair.value );

				allInterfacesMappings = removeAllInterfaceMappings( allInterfacesMappings, member1OfPair.value, member2OfPair.value );

				millis.value = millis.value + System.currentTimeMillis() - startTime;//Pause!


				allInterfacesMappings = addToAllInterfacesMappings( millis, options, functionalAbstractionID, interfaceOfAbstraction, interfaces, allInterfacesMappings );

				interfaces.put( functionalAbstractionID, interfaceOfAbstraction );


//				resetLabelsOfFA( options.getDelay(), serviceOrganizationVisualizer );//GUI!


				startTime = System.currentTimeMillis();


				printAllInterfaceMapping( allInterfacesMappings );


			} while( ! allInterfacesMappings.isEmpty() && interfaces.size() > 1);
//			loadFunctionalHierarchy( options.getDelay(), abstractionBase, options.getCollectionID(), serviceOrganizationVisualizer );//GUI!
			System.out.println("Time taken: " + (float)millis.value/1000 + " seconds");
		}


//		resetLabelsOfFA( options.getDelay(), serviceOrganizationVisualizer );//GUI!
//
//		loadProducedLabel( options.getDelay(), functionalAbstractionID - initialNumOfInterfaces + 1, serviceOrganizationVisualizer );//GUI!
//
//		loadTerminationLabel( options.getDelay(), ElapsedTimeCalculator.getTime( millis.value ), serviceOrganizationVisualizer );//GUI!
//
//
		loadFunctionalHierarchy( options.getDelay(), abstractionBase, options.getCollectionID(), serviceOrganizationVisualizer );//GUI!

		enableStoreButton( options.getDelay(), serviceOrganizationVisualizer );//GUI!
	}
	
	public static void produceFH1( ServiceRegistration serviceRegistration, AbstractionBaseController abstractionBase, UserSettings options, List<ServiceInterface> allInterfaces, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){//Mode = 1.

		if( allInterfaces == null ) return;//I check the non-existence of interfaces without knowing if it has been already checked by the caller to be sure for the future that this function will be functional!

		reset( options.getDelay(), serviceOrganizationVisualizer  );//GUI!

		List<ServiceInterface> unRepresentedInterfaces = new ArrayList<ServiceInterface>();

		for( int i = 0; allInterfaces != null && i < allInterfaces.size(); ++i ) unRepresentedInterfaces.add( allInterfaces.get( i ) );

		Map<Integer, ServiceInterface> interfaces = initializeInterfacesHashTable( allInterfaces );//Construct the hashtable of the interfaces!

		loadExecutionLabel( options.getDelay(), serviceOrganizationVisualizer );//GUI!


		//Initialization phase.
		LongHolder millis = new LongHolder( 0 );

		int initialNumOfInterfaces = allInterfaces.size();

		Map<String, InterfacesMapping> allInterfacesMappings = calculateDistancesAndMappings( millis, options, interfaces, serviceOrganizationVisualizer );//Construct the hashtable of the mapped interfaces!

		int initialPairs = ( allInterfaces.size() * ( allInterfaces.size() - 1 ) ) / 2;

		if( allInterfacesMappings != null ) loadInitializationCompleted( options.getDelay(), initialPairs, allInterfacesMappings.size(), serviceOrganizationVisualizer );//GUI!

		else loadInitializationCompleted( options.getDelay(), initialPairs, 0, serviceOrganizationVisualizer );//GUI!


		if( allInterfacesMappings == null ) return;

		else{

			if( options.getDelay() ) loadHierarchy( options.getDelay(), abstractionBase, options.getCollectionID(), serviceOrganizationVisualizer );//GUI!

			Integer functionalAbstractionID = initialNumOfInterfaces - 1;
			Integer totalNonFunctionalAbstractionID = 0;


			//Clustering interfaces and extracting abstractions phase.
			do {

				IntHolder member1OfPair = new IntHolder( -1 ), member2OfPair = new IntHolder( -1 );

				InterfacesMapping minInterfacesMapping = findMin( allInterfacesMappings, member1OfPair, member2OfPair );

				//JOptionPane.showMessageDialog(null, "Min distance has the pair of interfaces: (" + minInterfacesMapping.getInterfaces().get(0).getName() + ", " + minInterfacesMapping.getInterfaces().get(1).getName() + ") - id1 = " + member1OfPair.value + " - id2 = " + member2OfPair.value );

				//Extract functional abstraction!
				++functionalAbstractionID;

				unRepresentedInterfaces.remove( minInterfacesMapping.getServiceInterfaces().get( 0 ) );
				unRepresentedInterfaces.remove( minInterfacesMapping.getServiceInterfaces().get( 1 ) );

				ServiceInterface interfaceOfAbstraction = abstractionBase.addAbstraction( options.getCollectionID(), functionalAbstractionID, member1OfPair.value, member2OfPair.value, minInterfacesMapping, unRepresentedInterfaces );

				if( interfaceOfAbstraction == null ) return;

				if( options.getDelay() ) loadProducedFALabels( options.getDelay(), functionalAbstractionID - initialNumOfInterfaces, interfaceOfAbstraction.getName(), serviceOrganizationVisualizer );//GUI!

				//Update the contents of the 'allInterfacesMapping' hashtable and 'interfaces' hashtable.
				interfaces.remove( member1OfPair.value );
				interfaces.remove( member2OfPair.value );

				allInterfacesMappings = removeAllInterfaceMappings( allInterfacesMappings, member1OfPair.value, member2OfPair.value );

				allInterfacesMappings = addToAllInterfaceMappings( millis, options, functionalAbstractionID, interfaceOfAbstraction, interfaces, allInterfacesMappings, serviceOrganizationVisualizer );

				interfaces.put( functionalAbstractionID, interfaceOfAbstraction );

				if( options.getDelay() ) resetLabelsOfFA( options.getDelay(), serviceOrganizationVisualizer );//GUI!

				//Construct nested non-functional hierarchy.
				List<ServiceInterface> representedInterfaces = abstractionBase.getRepresentedInterfaces( options.getCollectionID(), functionalAbstractionID );

				totalNonFunctionalAbstractionID += NonFunctionalOrganization.produceNFH1( serviceRegistration, abstractionBase, options, functionalAbstractionID, representedInterfaces, serviceOrganizationVisualizer );

				if( options.getDelay() ) loadHierarchy( options.getDelay(), abstractionBase, options.getCollectionID(), serviceOrganizationVisualizer );//GUI!

			} while( ! allInterfacesMappings.isEmpty() && interfaces.size() > 1 );

			loadHierarchy( options.getDelay(), abstractionBase, options.getCollectionID(), serviceOrganizationVisualizer );//GUI!

			loadProducedLabel( options.getDelay(), functionalAbstractionID - initialNumOfInterfaces + 1, totalNonFunctionalAbstractionID, serviceOrganizationVisualizer );//GUI!
		}

		//TODO --> time: if( options.getDelay() ) loadTerminationLabel( serviceOrganizationVisualizer );//GUI!

		//TODO --> time: else loadTerminationLabel( ElapsedTimeCalculator.getTime( startTime, endTime ), serviceOrganizationVisualizer );//GUI!

		enableStoreButton( options.getDelay(), serviceOrganizationVisualizer );//GUI!
	}


	public static int produceFH2( ServiceRegistration serviceRegistration, AbstractionBaseController abstractionBase, UserSettings options, Integer nonFunctionalAbstractionID, List<ServiceInterface> allInterfaces, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){//Mode = 2 (I have assured in the function that calls this function that the 'interfaces' list is not NULL or empty!).

		if( allInterfaces == null ) return 0;//However, I check it to be sure for the future that this function will be functional!

		ArrayList<ServiceInterface> unRepresentedInterfaces = new ArrayList<ServiceInterface>();

		for( int i = 0; i < allInterfaces.size(); ++i ) unRepresentedInterfaces.add( allInterfaces.get( i ) );

		Map<Integer, ServiceInterface> interfaces = initializeInterfacesHashTable( allInterfaces );//Construct the hashtable of the interfaces!

		if( options.getDelay() ) loadExecutionLabel( options.getDelay(), serviceOrganizationVisualizer );//GUI!


		//Initialization phase.
		LongHolder millis = new LongHolder( 0 );

		int initialNumOfInterfaces = allInterfaces.size();

		Map<String, InterfacesMapping> allInterfacesMappings = calculateDistancesAndMappings( millis, options, interfaces, serviceOrganizationVisualizer );//Construct the hashtable of the mapped interfaces!

		int initialPairs = ( allInterfaces.size() * ( allInterfaces.size() - 1 ) ) / 2;

		if( allInterfacesMappings != null ) loadInitializationCompleted( options.getDelay(), initialPairs, allInterfacesMappings.size(), serviceOrganizationVisualizer );//GUI!

		else loadInitializationCompleted( options.getDelay(), initialPairs, 0, serviceOrganizationVisualizer );//GUI!


		if( allInterfacesMappings == null ) return 0;

		else{

			Integer functionalAbstractionID = initialNumOfInterfaces - 1;


			//Clustering interfaces and extracting abstractions phase.
			do {

				IntHolder member1OfPair = new IntHolder( -1 ), member2OfPair = new IntHolder( -1 );

				InterfacesMapping minInterfacesMapping = findMin( allInterfacesMappings, member1OfPair, member2OfPair );

				//JOptionPane.showMessageDialog(null, "Min distance has the pair of interfaces: (" + minInterfacesMapping.getInterfaces().get(0).getName() + ", " + minInterfacesMapping.getInterfaces().get(1).getName() + ") - id1 = " + member1OfPair.value + " - id2 = " + member2OfPair.value );

				//Extract functional abstraction!
				++functionalAbstractionID;

				unRepresentedInterfaces.remove( minInterfacesMapping.getServiceInterfaces().get(0) );
				unRepresentedInterfaces.remove( minInterfacesMapping.getServiceInterfaces().get(1) );

				ServiceInterface interfaceOfAbstraction = abstractionBase.addAbstraction( options.getCollectionID(), nonFunctionalAbstractionID, functionalAbstractionID, member1OfPair.value, member2OfPair.value, minInterfacesMapping, unRepresentedInterfaces );
				
				if( options.getDelay() ) loadProducedFLabelsOfNestedLevel( options.getDelay(), functionalAbstractionID - initialNumOfInterfaces, interfaceOfAbstraction.getName(), serviceOrganizationVisualizer );//GUI!

				//Update the contents of the 'allInterfacesMapping' hash table and 'interfaces' hash table.
				interfaces.remove( member1OfPair.value );
				interfaces.remove( member2OfPair.value );

				allInterfacesMappings = removeAllInterfaceMappings( allInterfacesMappings, member1OfPair.value, member2OfPair.value );

				allInterfacesMappings = addToAllInterfaceMappings( millis, options, functionalAbstractionID, interfaceOfAbstraction, interfaces, allInterfacesMappings, serviceOrganizationVisualizer );

				interfaces.put( functionalAbstractionID, interfaceOfAbstraction );

				resetLabelsOfFA( options.getDelay(), serviceOrganizationVisualizer );

				if( options.getDelay() ) loadHierarchy( options.getDelay(), abstractionBase, options.getCollectionID(), serviceOrganizationVisualizer );//GUI!

			} while( ! allInterfacesMappings.isEmpty() && interfaces.size() > 1 );


			return functionalAbstractionID - initialNumOfInterfaces;
		}
	}

	private static Map<Integer, ServiceInterface> initializeInterfacesHashTable( List<ServiceInterface> interfaces ){

		Hashtable<Integer, ServiceInterface> hashTableOfInterfaces = new Hashtable<Integer, ServiceInterface>();

		for( int i = 0; i < interfaces.size(); ++i ) hashTableOfInterfaces.put( i , interfaces.get( i ) );


		return hashTableOfInterfaces;
	}

	private static Map<String, InterfacesMapping> calculateDistancesAndMappings( LongHolder millis, UserSettings options, Map<Integer, ServiceInterface> interfaces, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		Long startTime = System.currentTimeMillis();


		Map<String, InterfacesMapping> allInterfacesMappings = null;

		int pairNumber = 0;

		for( Enumeration<Integer> e1 = ( (Hashtable<Integer, ServiceInterface>) interfaces).keys(); e1.hasMoreElements();){

			Integer i = e1.nextElement();

			for( Enumeration<Integer> e2 = ( (Hashtable<Integer, ServiceInterface>) interfaces).keys(); e2.hasMoreElements();){

				Integer j = e2.nextElement();


				if( i < j ){

					InterfacesMapping interfacesMapping = InterfacesDistanceCalculator.calculate( options.getInterfacesDistanceCalculatorAlgorithm(), options.getNamesDistanceCalculatorAlgorithm(), options.getSchemaRepresentationModel(), interfaces.get( i ), interfaces.get( j ) );

					pairNumber++;

					if( interfacesMapping != null ){

						if( allInterfacesMappings == null ) allInterfacesMappings = new Hashtable<String, InterfacesMapping>();

						allInterfacesMappings.put( i + "," + j, interfacesMapping );


						millis.value = millis.value + System.currentTimeMillis() - startTime;

						loadInitizializationLabelsAfterCalculation( options.getDelay(), pairNumber, interfaces.get( i ).getName(), interfaces.get( j ).getName(), interfacesMapping.getDistance(), serviceOrganizationVisualizer );//GUI!

						startTime = System.currentTimeMillis();
					}

					else{

						millis.value = millis.value + System.currentTimeMillis() - startTime;

						loadInitizializationLabelsAfterCalculation( options.getDelay(), pairNumber, interfaces.get(i).getName(), interfaces.get(j).getName(), 1.0, serviceOrganizationVisualizer );//GUI!

						startTime = System.currentTimeMillis();
					}
				}
			}
		}


		return allInterfacesMappings;//This function returns null when all the interfaces do not match!
	}
	
	private static Map<String, InterfacesMapping> calculateDistancesAndMappings( LongHolder millis, UserSettings options, Map<Integer, ServiceInterface> interfaces ){

		Long startTime = System.currentTimeMillis();


		Map<String, InterfacesMapping> allInterfacesMappings = null;


		for( Enumeration<Integer> e1 = ( (Hashtable<Integer, ServiceInterface>) interfaces).keys(); e1.hasMoreElements();){

			Integer i = e1.nextElement();

			for( Enumeration<Integer> e2 = ( (Hashtable<Integer, ServiceInterface>) interfaces).keys(); e2.hasMoreElements();){

				Integer j = e2.nextElement();


				if( i < j ){

					InterfacesMapping interfacesMapping = InterfacesDistanceCalculator.calculate( options.getInterfacesDistanceCalculatorAlgorithm(), options.getNamesDistanceCalculatorAlgorithm(), options.getSchemaRepresentationModel(), options.getAbstractionDepth(), options.getSimilarityLintel(), interfaces.get( i ), interfaces.get( j ) );

					if( interfacesMapping != null ){

						if( allInterfacesMappings == null ) allInterfacesMappings = new Hashtable<String, InterfacesMapping>();

						allInterfacesMappings.put( i + "," + j, interfacesMapping );


						millis.value = millis.value + System.currentTimeMillis() - startTime;

						startTime = System.currentTimeMillis();
					}

					else{

						millis.value = millis.value + System.currentTimeMillis() - startTime;

						startTime = System.currentTimeMillis();
					}
				}
			}
		}


		return allInterfacesMappings;//This function returns null when all the interfaces do not match!
	}

	private static void printAllInterfaceMapping( Map<String, InterfacesMapping> allInterfacesMappings ){

		for( Enumeration<String> e = ( (Hashtable<String, InterfacesMapping>) allInterfacesMappings ).keys(); e.hasMoreElements();){

			String i = e.nextElement();

			PrintMessages.stdPrintLn( PrintMessages.organizationPrint, "interfaceMapping= " + i + " = ( " + allInterfacesMappings.get( i ).getServiceInterfaces().get( 0 ).getTag() + ", " + allInterfacesMappings.get( i ).getServiceInterfaces().get( 1 ).getTag() + " )" );
		}
	}

	private static InterfacesMapping findMin( Map<String, InterfacesMapping> allInterfacesMappings, IntHolder member1OfPair, IntHolder member2OfPair ){

		InterfacesMapping minInterfacesMapping = null;

		Double min = Double.MAX_VALUE;


		for( Enumeration<String> e = ( (Hashtable<String,InterfacesMapping>) allInterfacesMappings).keys(); e.hasMoreElements();){

			String pair = e.nextElement();

			if( allInterfacesMappings.get( pair ).getDistance() < min ){

				min = allInterfacesMappings.get( pair ).getDistance();

				minInterfacesMapping = allInterfacesMappings.get( pair );

				member1OfPair.value = Integer.parseInt( pair.substring( 0, pair.lastIndexOf( "," ) ) );
				member2OfPair.value = Integer.parseInt( pair.substring( pair.lastIndexOf( "," ) + 1, pair.length() ) );
			}
		}


		return minInterfacesMapping;
	}

	private static Map<String, InterfacesMapping> removeAllInterfaceMappings( Map<String, InterfacesMapping> allInterfacesMappings, int member1OfPair, int member2OfPair ){

		for( Enumeration<String> e = ( (Hashtable<String,InterfacesMapping>) allInterfacesMappings).keys(); e.hasMoreElements();){

			String pair = e.nextElement();

			int cMember1OfPair = Integer.parseInt( pair.substring( 0, pair.lastIndexOf( "," ) ) );
			int cMember2OfPair = Integer.parseInt( pair.substring( pair.lastIndexOf( "," ) + 1, pair.length() ) );

			if( cMember1OfPair == member1OfPair || cMember1OfPair == member2OfPair || cMember2OfPair == member1OfPair || cMember2OfPair == member2OfPair ) allInterfacesMappings.remove( pair );
		}


		return allInterfacesMappings;
	}

	private static Map<String, InterfacesMapping> addToAllInterfaceMappings( LongHolder millis, UserSettings options, Integer functionalAbstractionID, ServiceInterface interfaceOfAbstraction, Map<Integer, ServiceInterface> interfaces, Map<String, InterfacesMapping> allInterfacesMappings, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		Long startTime = System.currentTimeMillis();


		for( Enumeration<Integer> e = ( (Hashtable<Integer,ServiceInterface>) interfaces).keys(); e.hasMoreElements();){

			Integer i = e.nextElement();

			millis.value = millis.value + System.currentTimeMillis() - startTime;

			loadDistanceLabelForAbstractionBeforeCalculation( options.getDelay(), interfaceOfAbstraction.getName(), interfaces.get(i).getName(), serviceOrganizationVisualizer );//GUI!

			startTime = System.currentTimeMillis();


			InterfacesMapping interfacesMapping = InterfacesDistanceCalculator.calculate( options.getInterfacesDistanceCalculatorAlgorithm(), options.getNamesDistanceCalculatorAlgorithm(), options.getSchemaRepresentationModel(), interfaceOfAbstraction, interfaces.get(i) );

			if( interfacesMapping != null ){

				allInterfacesMappings.put( i + "," + functionalAbstractionID, interfacesMapping );

				PrintMessages.stdPrintLn( PrintMessages.organizationPrint, "added InterfaceMapping= " + i + "," + functionalAbstractionID + " = ( " + interfacesMapping.getServiceInterfaces().get( 0 ).getTag() + ", " + interfacesMapping.getServiceInterfaces().get( 1 ).getTag() + " )" );


				millis.value = millis.value + System.currentTimeMillis() - startTime;

				loadDistanceLabelForAbstractionAfterCalculation( options.getDelay(), interfaceOfAbstraction.getName(), interfaces.get(i).getName(), interfacesMapping.getDistance(), serviceOrganizationVisualizer );//GUI!

				startTime = System.currentTimeMillis();
			}

			else{

				millis.value = millis.value + System.currentTimeMillis() - startTime;

				loadDistanceLabelForAbstractionAfterCalculation( options.getDelay(), interfaceOfAbstraction.getName(), interfaces.get(i).getName(), 1.0, serviceOrganizationVisualizer );//GUI!

				startTime = System.currentTimeMillis();
			}
		}


		return allInterfacesMappings;
	}

	private static Map<String, InterfacesMapping> addToAllInterfacesMappings( LongHolder millis, UserSettings options, Integer functionalAbstractionID, ServiceInterface interfaceOfAbstraction, Map<Integer, ServiceInterface> interfaces, Map<String, InterfacesMapping> allInterfacesMappings ){

		Long startTime = System.currentTimeMillis();


		for( Enumeration<Integer> e = ( (Hashtable<Integer,ServiceInterface>) interfaces).keys(); e.hasMoreElements();){

			Integer i = e.nextElement();

			millis.value = millis.value + System.currentTimeMillis() - startTime;

//			loadDistanceLabelForAbstractionBeforeCalculation( options.getDelay(), interfaceOfAbstraction.getName(), interfaces.get(i).getName(), serviceOrganizationVisualizer );//GUI!

			startTime = System.currentTimeMillis();


			InterfacesMapping interfacesMapping = InterfacesDistanceCalculator.calculate( options.getInterfacesDistanceCalculatorAlgorithm(), options.getNamesDistanceCalculatorAlgorithm(), options.getSchemaRepresentationModel(),options.getAbstractionDepth(), options.getSimilarityLintel(), interfaceOfAbstraction, interfaces.get(i) );

			if( interfacesMapping != null ){

				allInterfacesMappings.put( i + "," + functionalAbstractionID, interfacesMapping );


				millis.value = millis.value + System.currentTimeMillis() - startTime;

//				loadDistanceLabelForAbstractionAfterCalculation( options.getDelay(), interfaceOfAbstraction.getName(), interfaces.get(i).getName(), interfacesMapping.getDistance(), serviceOrganizationVisualizer );//GUI!

				startTime = System.currentTimeMillis();
			}

			else{

				millis.value = millis.value + System.currentTimeMillis() - startTime;

//				loadDistanceLabelForAbstractionAfterCalculation( options.getDelay(), interfaceOfAbstraction.getName(), interfaces.get(i).getName(), 1.0, serviceOrganizationVisualizer );//GUI!

				startTime = System.currentTimeMillis();
			}
		}


		return allInterfacesMappings;
	}
	
	private static void enableStoreButton( Boolean delay, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){
		
		serviceOrganizationVisualizer.getButtonOfStoringHierarchy().setEnabled(true);
		serviceOrganizationVisualizer.repaint();
		
		Delay.delay( delay );
	}
	
	private static void enableStoreButton( Boolean delay, ParameterizedServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.getButtonOfStoringHierarchy().setEnabled(true);
		serviceOrganizationVisualizer.repaint();

		Delay.delay( delay );
	}

	private static void loadExecutionLabel( Boolean delay, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.getLabelOfAlgorithmExecution().setText("Algorithm execution...");
		serviceOrganizationVisualizer.getLabelOfInitialization().setText("1. Initialization...");
		serviceOrganizationVisualizer.repaint();

		Delay.delay( delay );
	}

	private static void loadTerminationLabel( Boolean delay, String time, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.getLabelOfAlgorithmExecution().setText( "Algorithm terminated ( " + time + " )." );
		serviceOrganizationVisualizer.repaint();

		Delay.delay( delay );
	}

	private static void loadInitizializationLabelsAfterCalculation( Boolean delay, int pairNumber, String name1, String name2, double distance, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		if( delay ){

			serviceOrganizationVisualizer.getLabelOfCalculatingDistances().setText( "Calculating the distance of the pair" + pairNumber + " of interfaces:" );
			serviceOrganizationVisualizer.getLabelOfPairOfInterfaces().setText( "d (" + name1 + ", " + name2 + ") = " + DecimalPrecision.setFourDecimalsPrecisionHalfDown( distance ) );
			serviceOrganizationVisualizer.repaint();
		}

		Delay.delay( delay );
	}

	private static void loadDistanceLabelForAbstractionBeforeCalculation( Boolean delay, String name1, String name2, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		if( delay ){

			serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions1().setText(" Calculating the distances of the abstraction interface:" );
			serviceOrganizationVisualizer.get3rdLevelOfProducingAbstractions().setText( "d (" + name1 + ", " + name2 + ") = ..." );
			serviceOrganizationVisualizer.get1stLevelOfProducingAbstractions1().setForeground( Color.black );
			serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions2().setForeground( Color.black );

			serviceOrganizationVisualizer.repaint();
		}

		Delay.delay( delay );
	}

	private static void loadDistanceLabelForAbstractionAfterCalculation( Boolean delay, String name1, String name2, double distance, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		if( delay ){

			serviceOrganizationVisualizer.get3rdLevelOfProducingAbstractions().setText( "d (" + name1 + ", " + name2 + ") = " + DecimalPrecision.setFourDecimalsPrecisionHalfDown( distance ) );

			serviceOrganizationVisualizer.repaint();
		}

		Delay.delay( delay );
	}

	private static void resetLabelsOfFA( Boolean delay, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions1().setText(" ");
		serviceOrganizationVisualizer.get3rdLevelOfProducingAbstractions().setText(" ");
		serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions2().setText(" ");
		serviceOrganizationVisualizer.repaint();

		Delay.delay( delay );
	}

	private static void reset( Boolean delay, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.getLabelOfAlgorithmExecution().setText("");
		serviceOrganizationVisualizer.getLabelOfInitialization().setText("");
		serviceOrganizationVisualizer.getLabelOfCalculatingDistances().setText("");
		serviceOrganizationVisualizer.getLabelOfPairOfInterfaces().setText("");
		serviceOrganizationVisualizer.getLabelOfProducing1stAbstractions().setText("");
		serviceOrganizationVisualizer.get1stLevelOfProducingAbstractions1().setText("");
		serviceOrganizationVisualizer.getLabelOfInitialNonFunctionalAbstraction().setText("");
		serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions1().setText("");
		serviceOrganizationVisualizer.get3rdLevelOfProducingAbstractions().setText("");
		serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions2().setText("");

		serviceOrganizationVisualizer.getScrollPanelOfHierarchy().setViewportView( null );

		serviceOrganizationVisualizer.repaint();

		Delay.delay( delay );
	}

	private static void loadProducedFALabels( Boolean delay, int numOfFA, String nameOfFA, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		if( delay ){

			serviceOrganizationVisualizer.getLabelOfProducing1stAbstractions().setText( "2. Producing..." );
			serviceOrganizationVisualizer.get1stLevelOfProducingAbstractions1().setText( "#functional abstractions: " + numOfFA + " (name= " + nameOfFA + ")" );

			serviceOrganizationVisualizer.repaint();
		}

		Delay.delay( delay );
	}

	private static void loadProducedFLabelsOfNestedLevel( Boolean delay, int numOfFA, String nameOfFA, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		int label = numOfFA + 1;

		serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions2().setText("#functional abstractions: " + label + " (name= " + nameOfFA + ")");
		serviceOrganizationVisualizer.repaint();

		Delay.delay( delay );
	}

	private static void loadInitializationCompleted( Boolean delay, int initialPairs, int calculatedPairs, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.getLabelOfInitialization().setText( "1. Initialization completed." );

		if( calculatedPairs == 1 ) serviceOrganizationVisualizer.getLabelOfCalculatingDistances().setText( "The distance of " + calculatedPairs + " (out of " + initialPairs + ") pair of interfaces was calculated." );

		else if( calculatedPairs > 1 ) serviceOrganizationVisualizer.getLabelOfCalculatingDistances().setText( "The distances of " + calculatedPairs + " (out of " + initialPairs + ") pairs of interfaces were calculated." );

		serviceOrganizationVisualizer.getLabelOfCalculatingDistances().setForeground( new Color(0, 200, 80) );

		if( calculatedPairs >= 1 ) serviceOrganizationVisualizer.getLabelOfPairOfInterfaces().setText( "" );

		else{

			serviceOrganizationVisualizer.getLabelOfPairOfInterfaces().setText( "None of the interfaces matches to each other!" );
			serviceOrganizationVisualizer.getLabelOfPairOfInterfaces().setForeground( new Color(200, 0, 40) );
		}

		serviceOrganizationVisualizer.repaint();

		Delay.delay( delay );
	}

	private static void loadProducedLabel( Boolean delay, int numOfFA, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.getLabelOfProducing1stAbstractions().setText( "2. The produced hierarchy consists of" );

		serviceOrganizationVisualizer.get1stLevelOfProducingAbstractions1().setText( "#functional abstractions: " + numOfFA );
		serviceOrganizationVisualizer.get1stLevelOfProducingAbstractions1().setForeground( new Color(0, 200, 80) );
		serviceOrganizationVisualizer.get1stLevelOfProducingAbstractions1().setFont( new java.awt.Font("Tahoma", 0, 10) );
		serviceOrganizationVisualizer.repaint();

		Delay.delay( delay );
	}

	private static void loadProducedLabel( Boolean delay, int numOfFA, int numOfNFA, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		serviceOrganizationVisualizer.getLabelOfProducing1stAbstractions().setText( "2. The produced hierarchy consists of" );

		serviceOrganizationVisualizer.get1stLevelOfProducingAbstractions1().setText("#functional abstractions: " + numOfFA);
		serviceOrganizationVisualizer.get1stLevelOfProducingAbstractions1().setForeground(new Color(0, 200, 80));
		serviceOrganizationVisualizer.get1stLevelOfProducingAbstractions1().setFont(new java.awt.Font("Tahoma", 1, 10));

		if( numOfNFA > 0 ){

			serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions1().setText("#nested non-functional abstractions: " + numOfNFA);
			serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions1().setForeground(new Color(0, 200, 80));
			serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions1().setFont(new java.awt.Font("Tahoma", 0, 10));
		}

		else serviceOrganizationVisualizer.get2ndLevelOfProducingAbstractions1().setText("");

		serviceOrganizationVisualizer.repaint();

		Delay.delay( delay );
	}

	private static void loadHierarchy( Boolean delay, AbstractionBaseController abstractionBase, Integer collectionID, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		DefaultMutableTreeNode root = abstractionBase.visualize( collectionID );

		if( root == null ) return;

		serviceOrganizationVisualizer.setTreeOfHierarchy( root );
		serviceOrganizationVisualizer.getTreeOfHierarchy().addTreeSelectionListener( selectAbstractionActionListener( serviceOrganizationVisualizer ) );
		serviceOrganizationVisualizer.getScrollPanelOfHierarchy().setViewportView( serviceOrganizationVisualizer.getTreeOfHierarchy() );

		expandAll(serviceOrganizationVisualizer.getTreeOfHierarchy(), new TreePath(root));

		serviceOrganizationVisualizer.repaint();

		Delay.delay( delay );
	}

	private static void loadFunctionalHierarchy( Boolean delay, AbstractionBaseController abstractionBase, Integer collectionID, ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		String constraintViolation = null;

		if( ( constraintViolation = abstractionBase.getFinalConstraintViolationFH( collectionID ) ) != null ){

			JTextArea jTextArea = new JTextArea( constraintViolation );

			jTextArea.setForeground( Color.RED );

			serviceOrganizationVisualizer.getScrollPanelOfHierarchy().setViewportView( new JScrollPane( jTextArea ) );
		}

		else{

			DefaultMutableTreeNode root = abstractionBase.visualizeFH( collectionID );
	
			if( root == null ) return;
	
			serviceOrganizationVisualizer.setTreeOfHierarchy( root );
			serviceOrganizationVisualizer.getTreeOfHierarchy().addTreeSelectionListener( selectFunctionalAbstractionOrInterfaceActionListener( serviceOrganizationVisualizer ) );
			serviceOrganizationVisualizer.getScrollPanelOfHierarchy().setViewportView(serviceOrganizationVisualizer.getTreeOfHierarchy());
	
			expandAll( serviceOrganizationVisualizer.getTreeOfHierarchy(), new TreePath( root) );
		}

		serviceOrganizationVisualizer.repaint();

		Delay.delay( delay );
	}

//	Parameterized
	private static void loadFunctionalHierarchy( Boolean delay, AbstractionBaseController abstractionBase, Integer collectionID, ParameterizedServiceOrganizationVisualizer serviceOrganizationVisualizer ){

		String constraintViolation = null;

		if( ( constraintViolation = abstractionBase.getFinalConstraintViolationFH( collectionID ) ) != null ){

			JTextArea jTextArea = new JTextArea( constraintViolation );

			jTextArea.setForeground( Color.RED );

			serviceOrganizationVisualizer.getScrollPanelOfHierarchy().setViewportView( new JScrollPane( jTextArea ) );
		}

		else{

			DefaultMutableTreeNode root = abstractionBase.visualizeFH( collectionID );
	
			if( root == null ) return;
	
			serviceOrganizationVisualizer.setTreeOfHierarchy( root );
//			serviceOrganizationVisualizer.getTreeOfHierarchy().addTreeSelectionListener( selectFunctionalAbstractionOrInterfaceActionListener( serviceOrganizationVisualizer ) );
			serviceOrganizationVisualizer.getScrollPanelOfHierarchy().setViewportView(serviceOrganizationVisualizer.getTreeOfHierarchy());
	
			expandAll( serviceOrganizationVisualizer.getTreeOfHierarchy(), new TreePath( root) );
		}

		serviceOrganizationVisualizer.repaint();

		Delay.delay( delay );
	}

	private static TreeSelectionListener selectFunctionalAbstractionOrInterfaceActionListener( final ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

        return new TreeSelectionListener(){

        	public void valueChanged(TreeSelectionEvent ew){

        		serviceOrganizationVisualizer.setPath1 ( ew.getPath() );

        		int pathCount = ew.getPath().getPathCount();
            	String selection = ew.getPath().getPathComponent( pathCount - 1 ).toString();

            	if( selection != null && ( selection.contains( "Functional abstraction" ) && ! selection.contains( "Functional abstractions" )) || selection.contains( "Service interface" ) )
            		serviceOrganizationVisualizer.getButtonOfVisualizingFunctional().setEnabled( true );
            	
            	else 
            		serviceOrganizationVisualizer.getButtonOfVisualizingFunctional().setEnabled( false );
        	}
        };
    }

	private static TreeSelectionListener selectAbstractionActionListener( final ServiceOrganizationVisualizer serviceOrganizationVisualizer ){

        return new TreeSelectionListener(){

        	public void valueChanged(TreeSelectionEvent ew){

        		serviceOrganizationVisualizer.setPath1 ( ew.getPath() );

        		int pathCount = ew.getPath().getPathCount();
            	String selection = ew.getPath().getPathComponent( pathCount - 1 ).toString();

            	if( selection != null && selection.contains( "Functional abstraction" ) && ! selection.contains( "Functional abstractions" ) ) serviceOrganizationVisualizer.getButtonOfVisualizingNonFunctional().setEnabled( true );
            	
            	else serviceOrganizationVisualizer.getButtonOfVisualizingNonFunctional().setEnabled( false );
        	}
        };
    }

	@SuppressWarnings("unchecked")
	private static void expandAll( JTree tree, TreePath root ){

        TreeNode node = (TreeNode)root.getLastPathComponent();

        if( node.getChildCount() >= 0 ){

            for( Enumeration e = node.children(); e.hasMoreElements(); ) {

                TreeNode n = (TreeNode)e.nextElement();
                TreePath path = root.pathByAddingChild(n);

                expandAll(tree, path);
            }
        }

        tree.expandPath(root);
    }

	public void updateFH() {

	}
}