package org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service;

import java.util.ArrayList;
import java.util.List;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.measurements.number.DecimalPrecision;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


@SuppressWarnings("serial")
public class InterfacesMapping implements java.io.Serializable {


	//Variables.
	private double distance;

	private List<ServiceInterface> interfaces;

	private List<OperationsMapping> operationsMappings;//This is not null. I hold only the mapped operations!

	private int dbAbstractionKey;
	
	
	//Empty constructor used for xml transformation
	public InterfacesMapping(){
				
	}


	//Setters.
	public void add( ServiceInterface i ){

		if( interfaces == null ) interfaces = new ArrayList<ServiceInterface>();

		interfaces.add( i );
	}

	public void set( double cDistance ){

		distance = cDistance;
	}

	public void set( List<OperationsMapping> cOperationsMappings ){

		operationsMappings = cOperationsMappings;
	}
	
	public void setDBAbstractionKey( int cDBKey ){

		dbAbstractionKey = cDBKey;
	}
	
	public void setInterfaces(List<ServiceInterface> representedInterfaces){

		interfaces = representedInterfaces;
	}


	//Getters.
	public List<ServiceInterface> getServiceInterfaces(){

		return interfaces;
	}

	public double getDistance(){

		return distance;
	}

	public List<OperationsMapping> getOperationsMappings(){

		return operationsMappings;
	}
	
	public int getDBAbstractionKey(){

		return dbAbstractionKey;
	}

	public ServiceInterface getServiceInterface( String serviceInterfaceName ){//It retrieves the service interface object which has name the parameter 'serviceInterfaceName'.

		ServiceInterface serviceInterface = null;


		for( int i = 0; interfaces != null && i < interfaces.size(); ++i ){

			if( interfaces.get( i ).getName().equals( serviceInterfaceName ) ){

				serviceInterface = interfaces.get( i );

				break;
			}
		}


		return serviceInterface;
	}

	public ServiceInterface getMappedServiceInterface( String serviceInterfaceName ){//It returns randomly one of them mapped interfaces.

		ServiceInterface serviceInterface = null;

		List<Integer> candidateIndices = null;


		boolean exist = false;

		for( int i = 0; interfaces != null && i < interfaces.size(); ++i ){

			if( interfaces.get( i ).getName().equals( serviceInterfaceName ) ) exist = true;

			else{

				if( candidateIndices == null ) candidateIndices = new ArrayList<Integer>();

				candidateIndices.add( i );
			}
		}


		if( exist ){

			int randomIndex = ListGetters.getElementRandomly( candidateIndices );

			serviceInterface = interfaces.get( randomIndex ) ;
		}


		return serviceInterface;
	}


	//Visualization.
	public DefaultMutableTreeNode visualize( DefaultMutableTreeNode node ){

		DefaultMutableTreeNode child1 = new DefaultMutableTreeNode( "Interfaces mapping" );

		node.add( child1 );


		for( int i = 0; i < interfaces.size(); ++i ){

			Integer label = i + 1;

			DefaultMutableTreeNode child11 = null;


			if( interfaces.get( i ).hasServiceInstances() ) child11 = new DefaultMutableTreeNode( "Service Interface (" + label + "): " + interfaces.get( i ).getName() );

			else child11 = new DefaultMutableTreeNode( "Abstraction Interface (" + label + "): " + interfaces.get( i ).getName() );


			child1.add( child11 );
		}


		DefaultMutableTreeNode child12 = new DefaultMutableTreeNode( "Distance = " + DecimalPrecision.setFourDecimalsPrecisionHalfDown( distance ) );

		child1.add( child12 );


		DefaultMutableTreeNode child13 = new DefaultMutableTreeNode( "Operations mappings" );

		child1.add( child13 );


		for( int k = 0; operationsMappings != null &&  k < operationsMappings.size(); ++k ){

			Integer label = k + 1;

			child13 = operationsMappings.get( k ).visualize( label, child13 );
		}


    	return node;
	}
	
	//Setters used for xml transformation
	
	public void setxDistance( double cDistance ){
		distance = cDistance;
	}
		
	public void setxInterfaces( List<ServiceInterface> cInterfaces ){
		interfaces = cInterfaces;
	}
		
	public void setxOperationsMappings( List<OperationsMapping> cOperationsMappings ){
		operationsMappings = cOperationsMappings;
	}
		
	public void setxDBAbstractionKey( int cDBAbstractionKey ){
		dbAbstractionKey = cDBAbstractionKey;
	}
		
	//Getters used for xml transformation
		
	public double getxDistance(){
		return distance;
	}
			
	public List<ServiceInterface> getxInterfaces(){
		return interfaces;
	}
			
	public List<OperationsMapping> getxOperationsMappings(){
		return operationsMappings;
	}
			
	public int getxDBAbstractionKey(){
		return dbAbstractionKey;
	}
}
