package org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service;

import java.util.ArrayList;
import java.util.List;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.schema.ComponentsAndStructuresMapping;
import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.measurements.number.DecimalPrecision;
import org.ow2.choreos.serviceRepresentation.functional.MessageType;


public class MessageTypesMapping extends ComponentsAndStructuresMapping implements java.io.Serializable {


	//Variables.
	private static final long serialVersionUID = 3060762847339911567L;

	private double distance;

	private String targetSchemaRepresentationModel;//It is the schema representation model from which the mappings have calculated. It is not null if also the 'componentsMapping' is not null.

	private List<MessageType> messageTypes;//It is not null!

	private ComponentsAndStructuresMapping componentsAndStructuresMapping;//It is null if the mapped messageTypes are built-in types.

	private int dbAbstractionKey;
	
	
	//Empty constructor used for xml transformation
	public MessageTypesMapping(){
				
	}


	//Setters.
	public void add( MessageType messageType ){

		if( messageTypes == null ) messageTypes = new ArrayList<MessageType>();

		messageTypes.add( messageType );
	}

	public void set( double cDistance ){

		distance = cDistance;
	}

	public void set( String schemaRepresentationModel ){

		targetSchemaRepresentationModel = schemaRepresentationModel;
	}

	public void set( ComponentsAndStructuresMapping cComponentsMapping ){

		componentsAndStructuresMapping = cComponentsMapping;
	}

	public void setDBAbstractionKey( int cDBKey ){

		dbAbstractionKey = cDBKey;
	}

	public void setMessageTypes(List<MessageType> representedMessageTypes){

		messageTypes = representedMessageTypes;
	}

	//Getters.
	public List<MessageType> getMessageTypes(){

		return messageTypes;
	}

	public double getDistance(){

		return distance;
	}

	public String getSchemaRepresentationModel(){

		return targetSchemaRepresentationModel;
	}

	public ComponentsAndStructuresMapping getComponentsAndStructuresMapping(){

		return componentsAndStructuresMapping;
	}
	
	public int getDBAbstractionKey(){

		return dbAbstractionKey;
	}

	public MessageType getMessage( String messageTypeName ){//It retrieves the message object which has name the parameter 'messageTypeName'.

		MessageType messageType = null;


		for( int i = 0; messageTypes != null && i < messageTypes.size(); ++i ){

			if( messageTypes.get( i ).getName().equals( messageType ) ){

				messageType = messageTypes.get( i );

				break;
			}
		}


		return messageType;
	}

	public MessageType getMappedMessage( String messageTypeName ){//It returns randomly one of them mapped messageTypes.

		MessageType messageType = null;

		List<Integer> candidateIndices = null;


		boolean exist = false;

		for( int i = 0; messageTypes != null && i < messageTypes.size(); ++i ){

			if( messageTypes.get( i ).getName().equals( messageTypeName ) ) exist = true;

			else{

				if( candidateIndices == null ) candidateIndices = new ArrayList<Integer>();

				candidateIndices.add( i );
			}
		}


		if( exist ){

			int randomIndex = ListGetters.getElementRandomly( candidateIndices );

			messageType = messageTypes.get( randomIndex ) ;
		}


		return messageType;
	}


	//Visualization.
	public DefaultMutableTreeNode visualize( Integer label, DefaultMutableTreeNode node ){

		DefaultMutableTreeNode child1 = new DefaultMutableTreeNode( "Message types mapping (" + label + ")" );

		node.add( child1 );


		DefaultMutableTreeNode child12 = new DefaultMutableTreeNode( "Distance = " + DecimalPrecision.setFourDecimalsPrecisionHalfDown( distance ) );

		child1.add( child12 );


		for( int i = 0; i < messageTypes.size(); ++i ){

			label = i + 1;

			DefaultMutableTreeNode child13 = new DefaultMutableTreeNode( "Message type (" + label + "): " + messageTypes.get( i ).getName() );

			child1.add( child13 );
		}


		if( componentsAndStructuresMapping != null ) child1 = componentsAndStructuresMapping.visualize( true, child1, targetSchemaRepresentationModel );


    	return node;
	}
	
	//Setters used for xml transformation
	
	public void setxDistance( double cDistance ){
		distance = cDistance;
	}
			
	public void setxTargetSchemaRepresentationModel( String cTargetSchemaRepresentationModel ){
		targetSchemaRepresentationModel = cTargetSchemaRepresentationModel;
	}
			
	public void setxMessageTypes( List<MessageType> cMessageTypes ){
		messageTypes = cMessageTypes;
	}
			
	public void setxComponentsAndStructuresMapping( ComponentsAndStructuresMapping cComponentsAndStructuresMapping ){
		componentsAndStructuresMapping = cComponentsAndStructuresMapping;
	}
			
	public void setxDBAbstractionKey( int cDBAbstractionKey ){
		dbAbstractionKey = cDBAbstractionKey;
	}
	
	//Getters used for xml transformation
	
	public double getxDistance(){
		return distance;
	}
			
	public String getxTargetSchemaRepresentationModel(){
		return targetSchemaRepresentationModel;
	}
			
	public List<MessageType> getxMessageTypes(){
		return messageTypes;
	}
			
	public ComponentsAndStructuresMapping getxComponentsAndStructuresMapping(){
		return componentsAndStructuresMapping;
	}
			
	public int getxDBAbstractionKey(){
		return dbAbstractionKey;
	}
}
