package org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.interfaceMappingsToJTrees;

import java.awt.Frame;
import java.util.ArrayList;

import javax.swing.JOptionPane;

import org.ow2.choreos.abstractionBase.AbstractionBaseController;
import org.ow2.choreos.abstractionBase.FunctionalAbstraction;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.InterfacesMapping;
import org.ow2.choreos.gui.interfaceMappingsVisualizer.InterfaceMappingsVisualizer;
import org.ow2.choreos.gui.interfaceMappingsVisualizer.listeners.InterfaceMappingsVisualizerListener;
import org.ow2.choreos.gui.visualComponentConnection.VisualComponentConnection;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


/**
 * 
 * @author dathanas
 * TODO
 */
public class InterfaceMappingsToJTreesControllerImplV1 implements InterfaceMappingsToJTreesController {


	//Variables.
	private FunctionalAbstraction functionalAbstraction;

	private ServiceInterface interfaceA;
	private ServiceInterface interfaceB;
	
	private InterfacesMapping interfacesMapping;
	
	private JTreeProducer treeProducer;
	
	private VisualComponentConnectionsProducer connectionsProducer;
	
	private InterfaceMappingsVisualizer visualizer;

	private Frame visualizerOwner;
	private boolean visualizerModality;

	//Constructor.
	public InterfaceMappingsToJTreesControllerImplV1( Frame visualizerOwner, boolean visualizerModality, AbstractionBaseController abstractionBase, Integer collectionID, Integer functionalAbstractionID, Integer numOfInterfaces){
			this.visualizerOwner = visualizerOwner;
			this.visualizerModality = visualizerModality;
			functionalAbstraction = abstractionBase.get(collectionID).get( functionalAbstractionID + numOfInterfaces - 1 );
			interfacesMapping = functionalAbstraction.getInterfacesMapping();
			interfaceA = interfacesMapping.getServiceInterfaces().get(0);
			interfaceB = interfacesMapping.getServiceInterfaces().get(1);
			
			/*System.out.println("Functional Abstraction ID: " + functionalAbstraction.getID());
			System.out.println("Functional Abstraction Interface: " + functionalAbstraction.getInterface().getName());
			for(ServiceInterface i: functionalAbstraction.getRepresentedInterfaces())
			System.out.println("Functional Abstraction Represented Interface: " + i.getName());
			System.out.println("InterfaceA: " + interfaceA.getName());
			System.out.println("InterfaceB: " + interfaceB.getName());*/
	}


	//Getters.
	public FunctionalAbstraction getFunctionalAbstraction() {
		return functionalAbstraction;
	}

	public ServiceInterface getInterfaceA() {
		return interfaceA;
	}

	public ServiceInterface getInterfaceB() {
		return interfaceB;
	}

	public InterfacesMapping getInterfacesMapping() {
		return interfacesMapping;
	}

	public InterfaceMappingsVisualizer getVisualizer() {
		return visualizer;
	}


	//Transformation.
	public void transform() {

		treeProducer = new JTreeProducerImplV1();

		visualizer = new InterfaceMappingsVisualizer(visualizerOwner,visualizerModality,treeProducer.produceJTree(interfaceA),
				treeProducer.produceJTree(interfaceB), 
				new InterfaceMappingsVisualizerListener(){
					public void addMapping(VisualComponentConnection mapping){
						if (mapping.getDestPath().getLastPathComponent().toString().split("=")[0]
				            	.equals(mapping.getSourcePath().getLastPathComponent().toString().split("=")[0])){
							visualizer.getMappingsPanel().addToAddedMappings(mapping);
							visualizer.getMappingsPanel().repaint();
				        }
					}

					public void deleteMapping(VisualComponentConnection mapping) {
						visualizer.getMappingsPanel().addToDeletedMappings(mapping);
					}

					public void deleteMappings(ArrayList<VisualComponentConnection> mappings) {
				        for (int i = 0; i < mappings.size(); i++) {
				            visualizer.getMappingsPanel().addToDeletedMappings(mappings.get(i));
				        }
					}

					public void complete() {}
				}
		);
		visualizer.setViewLabels(interfaceA.getName(), interfaceB.getName());
		visualizer.setTitle("Mappings between Service Interfaces \"" + interfaceA.getName()+"\" and \"" + interfaceB.getName()+"\"");
		connectionsProducer = new VisualComponentConnectionsProducerImplV1(visualizer);
		
		connectionsProducer.produceVisualComponentConnectionsFromMapping(interfacesMapping);

		visualizer.setVisible(true);
		ArrayList<VisualComponentConnection> addedMappings = visualizer.getMappingsPanel().getAddedMappings();
		ArrayList<String> mappedInterfaces = new ArrayList<String>();
		ArrayList<String> mappedOperations = new ArrayList<String>();
		ArrayList<String> mappedInputMessages = new ArrayList<String>();
		ArrayList<String> mappedOutputMessages = new ArrayList<String>();
		ArrayList<String> mappedMessageTypes = new ArrayList<String>();
		
		String source;
		String dest;
		for(VisualComponentConnection mapping:addedMappings){
			source = mapping.getSourcePath().getLastPathComponent().toString();
			dest = mapping.getDestPath().getLastPathComponent().toString();
			
			if(source.startsWith("service interface")){
				mappedInterfaces.add(extractNameAttribute(source));
				mappedInterfaces.add(extractNameAttribute(dest));
			}
			else if(source.startsWith("operation")){
				mappedOperations.add(extractNameAttribute(source));
				mappedOperations.add(extractNameAttribute(dest));
				
			}
			else if(source.startsWith("Input message")){
				mappedInputMessages.add(extractNameAttribute(source));
				mappedInputMessages.add(extractNameAttribute(dest));
				
			}
			else if(source.startsWith("Output message")){
				mappedOutputMessages.add(extractNameAttribute(source));
				mappedOutputMessages.add(extractNameAttribute(dest));
				
			}
			else if(source.startsWith("messageType")){
				mappedMessageTypes.add(extractNameAttribute(source));
				mappedMessageTypes.add(extractNameAttribute(dest));				
			}
			else JOptionPane.showMessageDialog(visualizer, "WTF man!!");
		}
		System.out.println("Mapped Interfaces:");
		for (String string : mappedInterfaces) {
			System.out.println(string);
		}
		System.out.println("Mapped Operations:");
		for (String string : mappedOperations) {
			System.out.println(string);
		}
		System.out.println("Mapped Input Messages:");
		for (String string : mappedInputMessages) {
			System.out.println(string);
		}
		System.out.println("Mapped Output Messages:");
		for (String string : mappedOutputMessages) {
			System.out.println(string);
		}
		System.out.println("Mapped Message types:");
		for (String string : mappedMessageTypes) {
			System.out.println(string);
		}
	}
	
	private String extractNameAttribute(String text){
		if(text.contains("name= "))
			return text.substring(text.indexOf("name= ")).split("\'")[1];
		else
			return "";
	}
}
