package org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.interfaceMappingsToJTrees;

import java.util.ArrayList;

import javax.swing.JTree;
import javax.swing.tree.TreePath;

import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.InterfacesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.MessageTypesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.MessagesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.OperationsMapping;
import org.ow2.choreos.gui.interfaceMappingsVisualizer.InterfaceMappingsVisualizer;
import org.ow2.choreos.gui.visualComponentConnection.VisualComponentConnection;
import org.ow2.choreos.serviceRepresentation.functional.Message;
import org.ow2.choreos.serviceRepresentation.functional.MessageType;
import org.ow2.choreos.serviceRepresentation.functional.Operation;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;

/**
 * This class is used for producing a set({@link ArrayList}) of {@link VisualComponentConnection}
 * objects from a given set of Mappings. This class needs an {@link InterfaceMappingsVisualizer}
 * object, so that the produced {@code VisualComponentConnection} objects have the correct visual bounds.
 *
 */
public class VisualComponentConnectionsProducerImplV1 implements VisualComponentConnectionsProducer {
	private InterfaceMappingsVisualizer visualizer;
	private JTree treeA;
	private JTree treeB;
	
	
/**
 * Constructs a new instance of VisualComponentConnectionsProducerImplV1. {@link InterfaceMappingsVisualizer visualizer
 * is needed so that when producing {@link VisualComponentConnection}s	for {@code visualizer} they need
 * to have correct bounds.
 * @param visualizer An instance of {@link InterfaceMappingsVisualizer} from which produced 
 * VisualComponentConnections are going to get their bounds
 */
	public VisualComponentConnectionsProducerImplV1(InterfaceMappingsVisualizer visualizer) {
		this.visualizer = visualizer;
		this.treeA = visualizer.getSourceTree();
		this.treeB = visualizer.getDestTree();
	}

	public ArrayList<VisualComponentConnection> produceVisualComponentConnectionsFromMappings(
			ArrayList<Object> mappings) {
		
		return null;
	}

	public ArrayList<VisualComponentConnection> produceVisualComponentConnectionsFromMapping(Object mapping) {
		InterfacesMapping iMapping = (InterfacesMapping)mapping;
		ArrayList<VisualComponentConnection> list = produceVisualComponentConnectionsFromInterfacesMapping(iMapping);
		visualizer.addExistingConnections(list);
		return null;
	}
	
	private ArrayList<VisualComponentConnection> produceVisualComponentConnectionsFromInterfacesMapping(InterfacesMapping mapping){
		ServiceInterface interfaceA = mapping.getServiceInterfaces().get(0);
		ServiceInterface interfaceB = mapping.getServiceInterfaces().get(1);
		
		ArrayList<String> namesA = new ArrayList<String>();
		namesA.add(interfaceA.getTag());
		ArrayList<String> namesB = new ArrayList<String>();
		namesB.add(interfaceB.getTag());
		
		TreePath pathA = TreeUtils.findTreePathByName(treeA, namesA);
		TreePath pathB = TreeUtils.findTreePathByName(treeB, namesB);
		
		ArrayList<VisualComponentConnection> connections = new ArrayList<VisualComponentConnection>();
		VisualComponentConnection connection = visualizer.getMappingsPanel().createNewMapping(pathA, pathB);
		if(connection != null)
			connections.add(connection);
		
		if (mapping.getOperationsMappings() != null) {
			for (OperationsMapping oM : mapping.getOperationsMappings())
				connections.addAll(produceVisualComponentConnectionsFromOperationsMapping(
								oM, namesA, namesB));
		}
		return connections;
	}
	
	private ArrayList<VisualComponentConnection> produceVisualComponentConnectionsFromOperationsMapping(
			OperationsMapping mapping, ArrayList<String> ancestorNamesA, ArrayList<String> ancestorNamesB){
		Operation operationA = mapping.getOperations().get(0);
		Operation operationB = mapping.getOperations().get(1);
		
		ArrayList<String> namesA = new ArrayList<String>();
		namesA.addAll(ancestorNamesA);
		namesA.add(operationA.getTag());
		
		ArrayList<String> namesB = new ArrayList<String>();
		namesB.addAll(ancestorNamesB);
		namesB.add(operationB.getTag());

		TreePath pathA = TreeUtils.findTreePathByName(treeA, namesA);
		TreePath pathB = TreeUtils.findTreePathByName(treeB, namesB);

		ArrayList<VisualComponentConnection> connections = new ArrayList<VisualComponentConnection>();
		VisualComponentConnection connection = visualizer.getMappingsPanel().createNewMapping(pathA, pathB);
		if(connection != null)
			connections.add(connection);

		if( mapping != null ){

			connections.addAll(produceVisualComponentConnectionsFromMessagesMapping(mapping.getInputMappings(),"Input",namesA,namesB));
			connections.addAll(produceVisualComponentConnectionsFromMessagesMapping(mapping.getOutputMappings(),"Output",namesA,namesB));
		}
		
		return connections;
	}
	
	private ArrayList<VisualComponentConnection> produceVisualComponentConnectionsFromMessagesMapping(
			MessagesMapping mapping,String text, ArrayList<String> ancestorNamesA, ArrayList<String> ancestorNamesB){

		ArrayList<VisualComponentConnection> connections = null;

		if( mapping.getMessages() != null ){

			Message messageA = mapping.getMessages().get(0);
			
			ArrayList<String> namesA = new ArrayList<String>();
			namesA.addAll(ancestorNamesA);
			if(messageA != null)
				namesA.add(text + " " + messageA.getTag());
			else namesA.add("Undefined " + text);
			
	
			Message messageB = mapping.getMessages().get(1);
			
			ArrayList<String> namesB = new ArrayList<String>();
			namesB.addAll(ancestorNamesB);
			if(messageB != null)
				namesB.add(text + " " + messageB.getTag());
			else namesB.add("Undefined " + text);
			
			TreePath pathA = TreeUtils.findTreePathByName(treeA, namesA);
			TreePath pathB = TreeUtils.findTreePathByName(treeB, namesB);
	
			connections = new ArrayList<VisualComponentConnection>();
			VisualComponentConnection connection = visualizer.getMappingsPanel().createNewMapping(pathA, pathB);
			if(connection != null)
				connections.add(connection);
	
			if( mapping.getMessageTypesMapping() != null ) 
				for( MessageTypesMapping tM:mapping.getMessageTypesMapping() ) connections.addAll(produceVisualComponentConnectionsFromComponentMapping(tM,namesA,namesB));
		}


		return connections;
	}
	
	private ArrayList<VisualComponentConnection> produceVisualComponentConnectionsFromComponentMapping(
			MessageTypesMapping mapping,ArrayList<String>ancestorNamesA,ArrayList<String> ancestorNamesB){
		
		MessageType typeA = mapping.getMessageTypes().get(0);
		MessageType typeB = mapping.getMessageTypes().get(1);
		
		ArrayList<String> namesA = new ArrayList<String>();
		namesA.addAll(ancestorNamesA);
		namesA.add(typeA.getTag());
		
		ArrayList<String> namesB = new ArrayList<String>();
		namesB.addAll(ancestorNamesB);
		namesB.add(typeB.getTag());
		
		TreePath pathA = TreeUtils.findTreePathByName(treeA, namesA);
		TreePath pathB = TreeUtils.findTreePathByName(treeB, namesB);

		ArrayList<VisualComponentConnection> connections = new ArrayList<VisualComponentConnection>();
		
		VisualComponentConnection connection = visualizer.getMappingsPanel().createNewMapping(pathA, pathB);
		if(connection != null)
			connections.add(connection);
		
		return connections;
	}
//Getters
	public InterfaceMappingsVisualizer getVisualizer() {
		return visualizer;
	}

}
