package org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.JOptionPane;

import org.ow2.choreos.gui.ServiceOrganizationVisualizer;
import org.ow2.choreos.measurements.time.Delay;


/**
 * 
 * @author dathanas
 *
 */
public class JointGroupsCalculatorImplV1 implements JointGroupsCalculator {


	//Variables.
	private ServiceOrganizationVisualizer guiForHierarchyProducer;
	private boolean sleep;


	//Constructor.
	public JointGroupsCalculatorImplV1( ServiceOrganizationVisualizer guiForHierarchyProducer ){

		//JOptionPane.showMessageDialog(null, "New joint group calculator v1 object!");
	}


	//Methods.
	@SuppressWarnings("unchecked")
	public List<JointGroup> produce( Boolean stepByStepExecution, Integer collectionID, List<ArrayList<PropertyBasedGroup>> propertiesBasedGroups, ServiceOrganizationVisualizer guiForHierarchyProducer ){//This function implements the cartesian product of the previously produced property-based groups.

		sleep = stepByStepExecution;

		this.guiForHierarchyProducer = guiForHierarchyProducer;

        Set[] sets = init( propertiesBasedGroups );

        Set<Set<Object>> product = findCartesianProduct( sets );

        if( product == null ) return null;

        ArrayList<JointGroup> jointGroups = formJointGroups( collectionID, product );

        resetLabels();//GUI!
		
		return jointGroups;
	}

	@SuppressWarnings("unchecked")
	private Set[] init( List<ArrayList<PropertyBasedGroup>> propertiesBasedGroups ){

		Set[] sets = new Set[propertiesBasedGroups.size()];

        for( int i = 0 ; i < propertiesBasedGroups.size(); ++i){

        	Set set = new HashSet();

        	for( int j = 0 ; j < propertiesBasedGroups.get(i).size(); ++j) set.add( propertiesBasedGroups.get(i).get(j) );

        	sets[i] = set;
        }

        return sets;
	}

	private Set<Set<Object>> findCartesianProduct(Set<?>... sets) {

        if ( sets.length < 2 ){
        	JOptionPane.showMessageDialog(null, "set size = " + sets.length  +" - Can't calcualte cartesian product of fewer than two sets");

        	return null;
        }

        return findCartesianProduct( 0,sets );
    }

    private Set<Set<Object>> findCartesianProduct( int index, Set<?>... sets ) {
        Set<Set<Object>> ret = new HashSet<Set<Object>>();

        if( sets.length == index ) ret.add( new HashSet<Object>() );

        else{
            for (Object obj : sets[index]) {

                for (Set<Object> set : findCartesianProduct( index + 1, sets ) ) {

                    set.add(obj);
                    ret.add(set);
                }
            }
        }

        return ret;
    }

	@SuppressWarnings("unchecked")
	private ArrayList<JointGroup> formJointGroups( Integer collectionID, Set<Set<Object>> product ){

		ArrayList<JointGroup> jointGroups = new ArrayList<JointGroup>();

		Iterator iterator = ((HashSet) product).iterator();

		int num = 0;

        while( iterator.hasNext() ) {

            Set group = (Set) iterator.next();

            JointGroup jointGroup = formJointGroup( collectionID, group );

            jointGroups.add( jointGroup );

            loadLabelsForJointGroups( ++num );//GUI!
        }

		return jointGroups;
	}

	@SuppressWarnings("unchecked")
	private JointGroup formJointGroup( Integer collectionID, Set group ){

		JointGroup jointGroup = new JointGroup();

		Iterator iterator = group.iterator();

        while (iterator.hasNext()) {

        	PropertyBasedGroup propertyBasedGroup = (PropertyBasedGroup) iterator.next();

        	jointGroup.addQualifier( propertyBasedGroup.getProperty(), propertyBasedGroup.getQualifier() );

        	jointGroup.addMappingsToRanges( propertyBasedGroup.getProperty(), propertyBasedGroup.getRange() );

        	jointGroup.addRepresentedServiceInstances( propertyBasedGroup.getProperty(), propertyBasedGroup.getRepresentedServiceInstances() );
        }

        //jointGroup.print();

		return jointGroup;
	}

	private void loadLabelsForJointGroups( int num ){
		guiForHierarchyProducer.getLabelOfJointGroups().setText( "Producing joint groups: " + num );

		guiForHierarchyProducer.repaint();

		Delay.delay( sleep );
	}

	private void resetLabels(){

		guiForHierarchyProducer.getLabelOfJointGroups().setText("");

		guiForHierarchyProducer.repaint();
	}
}
