package org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.propertyAndJointBasedGroups;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.partitionalClustering.kMeans.DataPointForServices;
import org.ow2.choreos.clusteringAlgorithms.Clustering;
import org.ow2.choreos.clusteringAlgorithms.partitionalClustering.kMeans.Cluster;
import org.ow2.choreos.clusteringAlgorithms.partitionalClustering.kMeans.DataPoint;
import org.ow2.choreos.clusteringAlgorithms.partitionalClustering.kMeans.Kmeans;
import org.ow2.choreos.gui.ServiceOrganizationVisualizer;
import org.ow2.choreos.serviceRegistration.ServiceRegistration;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;
import org.ow2.choreos.serviceRepresentation.nonFunctional.QualityProperties;


/**
 * 
 * @author dathanas
 *
 */
public class PropertiesBasedGroupsCalculatorImplV1 implements PropertiesBasedGroupsCalculator {

	//Variables.
	private ServiceRegistration serviceRegistration;
	private ServiceOrganizationVisualizer guiForHierarchyProducer;
	private Clustering kmeans;
	private boolean sleep;


	//Constructor.
	public PropertiesBasedGroupsCalculatorImplV1( ServiceRegistration serviceRegistration ){

		this.serviceRegistration = serviceRegistration;
		this.kmeans = new Kmeans();

		//JOptionPane.showMessageDialog(null, "New property-based group calculator v1 object!");
	}


	//Methods.
	public List<ArrayList<PropertyBasedGroup>> produce( Boolean stepByStepExecution, boolean init, Integer collectionID, QualityProperties Q, Map<Integer, ServiceInstance> representedServiceInstances, ServiceOrganizationVisualizer guiForHierarchyProducer ){

		sleep = stepByStepExecution;

		this.guiForHierarchyProducer = guiForHierarchyProducer;

		ArrayList<ArrayList<PropertyBasedGroup>> propertiesBasedGroups = new ArrayList<ArrayList<PropertyBasedGroup>>();

		for( int i = 0; i < Q.getNumOfProperties(); ++i ){

			List<PropertyBasedGroup> propertyBasedGroups = null;

			if( init == true ) propertyBasedGroups = producePropertyBasedGroups( collectionID, Q.getProperty( i ), representedServiceInstances );

			else propertyBasedGroups = producePropertyBasedGroups( collectionID, Q.getProperty( i ), Q.getDomain( i ), representedServiceInstances );


			if( propertyBasedGroups == null ) return null;

			propertiesBasedGroups.add( (ArrayList<PropertyBasedGroup>) propertyBasedGroups );
		}

		resetLabels();//GUI!

		return propertiesBasedGroups;
	}

	private List<PropertyBasedGroup> producePropertyBasedGroups( Integer collectionID, String property, Map<Integer, ServiceInstance> representedServiceInstances ){//This function is used only for the first non-functional abstraction.

		loadLabelsForPropertyBasedGroups( property );//GUI!

		List<PropertyBasedGroup> propertyBasedGroups = new ArrayList<PropertyBasedGroup>();

		List<Double> values = new ArrayList<Double>();

		Map<Integer, ServiceInstance> filteredRepresentedServiceInstances = new Hashtable<Integer, ServiceInstance>();//These are the service instances that provide values for this property.

		for( Enumeration<Integer> e = ( (Hashtable<Integer,ServiceInstance>) representedServiceInstances).keys(); e.hasMoreElements(); ){

			Integer i = e.nextElement();

			Double value = representedServiceInstances.get(i).getValue( serviceRegistration.getRef2SourceOfServiceInstance( collectionID, representedServiceInstances.get(i) ), property );

			if( value != null ){

				filteredRepresentedServiceInstances.put( filteredRepresentedServiceInstances.size(), representedServiceInstances.get(i));

				values.add( value );
			}
		}

		if( values.isEmpty() ){

			//JOptionPane.showMessageDialog(null, "---------- Property: " + property + " -- Empty values.");

			PropertyBasedGroup propertyBasedGroup = new PropertyBasedGroup( property );

			propertyBasedGroups.add( propertyBasedGroup );
		}

		else{

			PropertyBasedGroup propertyBasedGroup = new PropertyBasedGroup( values, property, filteredRepresentedServiceInstances );//The ranges are constructed based on the values that are inside the non-functional decsriptions of the representedServiceInstances.

			propertyBasedGroups.add( propertyBasedGroup );
		}

		return propertyBasedGroups;
	}

	private List<PropertyBasedGroup> producePropertyBasedGroups( Integer collectionID, String property, List<String> domain, Map<Integer, ServiceInstance> representedServiceInstances ){//This function uses the K-means algorithm!

		loadLabelsForPropertyBasedGroups( property );//GUI!

		ArrayList<PropertyBasedGroup> propertyBasedGroups = new ArrayList<PropertyBasedGroup>();

		Vector<DataPoint> dataPoints = new Vector<DataPoint>();//These 'dataPoints' objects are given as input to the K-means algorithm.

		for( Enumeration<Integer> e = ( (Hashtable<Integer,ServiceInstance>) representedServiceInstances).keys(); e.hasMoreElements(); ){

			Integer i = e.nextElement();

			Double value = representedServiceInstances.get(i).getValue( serviceRegistration.getRef2SourceOfServiceInstance( collectionID, representedServiceInstances.get(i) ), property );

			if( value != null ) dataPoints.add( new DataPointForServices( value.doubleValue(), 0.0, representedServiceInstances.get(i).getName(), representedServiceInstances.get(i) ) );
		}

		if( dataPoints.isEmpty() ){

			//JOptionPane.showMessageDialog(null, "---------- Property: " + property + " -- Empty dataPoints.");

			for( int i = 0; i < domain.size(); ++i ) propertyBasedGroups.add( new PropertyBasedGroup( property, domain.get(i) ) );
		}

		else{
			int numOfClusters = domain.size();

			((Kmeans) kmeans).init( numOfClusters, 1000, dataPoints );

		    kmeans.cluster();

		    Cluster[] clusters = kmeans.getClusters();

		    //Sorting the centroids of clusters.
		    clusters = sort( clusters );
	
		    for( int i = 0; clusters != null && i < clusters.length; i++ ){
	
		    	Vector<DataPoint> cluster = clusters[i].getDataPoints();
	
		    	//JOptionPane.showMessageDialog(null, "---------- Property: " + property + " -- Cluster " + i + " ( " + domain.get(i) + " ) ---------");
	
		    	ArrayList<Double> values = new ArrayList<Double>();
	
		    	Hashtable<Integer, ServiceInstance> filteredRepresentedServiceInstances = new Hashtable<Integer, ServiceInstance>();//These are the service instances that provide values for this property.
	
		    	Iterator<DataPoint> iter = cluster.iterator();
	
		    	while( iter.hasNext() ){
	
		    		DataPointForServices dataPoint = (DataPointForServices) iter.next();
	
		    		values.add( dataPoint.getX() );
	
		    		//JOptionPane.showMessageDialog(null, "---------- Property: " + property + " -- Cluster " + i + " --------- dataPoint = " + dataPoint.getX());
	
		    		filteredRepresentedServiceInstances.put( filteredRepresentedServiceInstances.size(), dataPoint.getServiceInstance() );
		    	}
	
		    	if( values.isEmpty() ){

					//JOptionPane.showMessageDialog(null, "---------- Property: " + property + " -- Empty cluster.");

					propertyBasedGroups.add( new PropertyBasedGroup( property, domain.get(i) ) );
				}

		    	else propertyBasedGroups.add( new PropertyBasedGroup( values, property, domain.get(i), filteredRepresentedServiceInstances ) );//The ranges are constructed based on the values that are inside the non-functional decsriptions of the representedServiceInstances.
		    }
		}
		
		return propertyBasedGroups;
	}

	private Cluster[] sort( Cluster[] clusters ){

		boolean sorted = false;

		for( int top = clusters.length - 1; top > 0 && !sorted; top-- ){

			sorted = true;

			for( int i = 0; i < top; i++ ){

				if ( clusters[i].getCentroid().getCx() > clusters[i+1].getCentroid().getCx() ){

					sorted = false;

					Cluster temp = clusters[i];
					clusters[i] = clusters[i+1];
					clusters[i+1] = temp;
				}
			}
		}

		return clusters;
	}

	private void loadLabelsForPropertyBasedGroups( String property ){

		guiForHierarchyProducer.getLabelOfPropertyGroups().setText("Producing groups for the property: " + property);

		guiForHierarchyProducer.repaint();

		sleep();
	}

	private void resetLabels(){

		guiForHierarchyProducer.getLabelOfPropertyGroups().setText("");

		guiForHierarchyProducer.repaint();
	}

	void sleep(){
		if( sleep )
			//try {Thread.sleep(2000);}
			try {Thread.sleep(0);}
			catch (InterruptedException e) {}
	}
}
