package org.ow2.choreos.abstractionGenerator;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;

import org.ow2.choreos.schemaRepresentation.xml.XMLBuiltInTypes;
import org.ow2.choreos.serviceRepresentation.functional.Message;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


public class AbstractionGenerator 
{
	ArrayList<String> imports;
	ArrayList<String> operationsPrototype;
	ArrayList<String> inputClassBody;
	ArrayList<String> outputClassBody;
	
	
	
	
	public AbstractionGenerator()
	{
		imports =     new ArrayList<String>();
		operationsPrototype =  new ArrayList<String>();
		inputClassBody =  new ArrayList<String>() ;
		outputClassBody = new ArrayList<String>();
		
	}
	
	
	public void createAbstraction(ServiceInterface serviceInterface, String URL)
	{
            
            imports(serviceInterface);//creation of appropriate imports
            
            for (int j=0;j<serviceInterface.getOperations().size();j++)
    		{
            	String op;
            	op="public "+firstCharacterToUpperCase(serviceInterface.getOperations().get(j).getOutMessage().getName())+"Out";
            	op=op.concat(" "+serviceInterface.getOperations().get(j).getName() +" ( ");
            	op=op.concat(firstCharacterToUpperCase(serviceInterface.getOperations().get(j).getInMessage().getName())+"In input )");
            	
            	//creation of input class
            	Message messageIn=serviceInterface.getOperations().get(j).getInMessage();
            	inputClassBody.add(createClassBody(messageIn,"In"));
            	
            	//creation of output class
    			Message messageOut=serviceInterface.getOperations().get(j).getOutMessage();
            	outputClassBody.add(createClassBody(messageOut,"Out"));
            	
            	operationsPrototype.add(op);
    		}
            removeDuplicates();
            
           
            
            createFiles(serviceInterface, URL);   
	}
	
	//Creation of java files!!
	public void createFiles(ServiceInterface serviceInterface, String URL)
	{
		createAbstractionPublisher(serviceInterface, URL);
		try
        {
			File directory=new  File("src/main/java/org/ow2/choreos/abstractionGenerator/abstractionImpl/"+serviceInterface.getName().toLowerCase());
			directory.mkdirs();
            FileWriter fstream = new FileWriter("src/main/java/org/ow2/choreos/abstractionGenerator/abstractionImpl/"+serviceInterface.getName().toLowerCase()+"/"+serviceInterface.getName() + "Interface.java");
            BufferedWriter out = new BufferedWriter(fstream);
            
            FileWriter fstreamImpl = new FileWriter("src/main/java/org/ow2/choreos/abstractionGenerator/abstractionImpl/"+serviceInterface.getName().toLowerCase() +"/"+serviceInterface.getName() + "Impl.java");
            BufferedWriter outImpl = new BufferedWriter(fstreamImpl);
            
            out.write("package org.ow2.choreos.abstractionGenerator.abstractionImpl."+serviceInterface.getName().toLowerCase()+";\n");
            outImpl.write("package org.ow2.choreos.abstractionGenerator.abstractionImpl."+serviceInterface.getName().toLowerCase()+";\n");
            
            for(int i=0;i<imports.size();i++)
            {
            	out.write(imports.get(i)+"\n");
            	outImpl.write(imports.get(i)+"\n");
            }
            
            out.write("@WebService\n"+"public interface " + serviceInterface.getName() + "Interface {\n");
            outImpl.write("@WebService()\n public class " + serviceInterface.getName() + "Impl extends BaseAbstractionImplService implements "+serviceInterface.getName()+"Interface {\n");
       
            outImpl.write("\tpublic "+serviceInterface.getName()+"Impl () \n\t{\n\t\tsuper(\""+serviceInterface.getName()+"\");\n\t}\n\n");
            
            
            for(int i=0;i<inputClassBody.size();i++)
            	generateClasses(serviceInterface ,inputClassBody,serviceInterface.getOperations().get(i).getInMessage(),i,"In");
            
            for(int i=0;i<outputClassBody.size();i++)
            	generateClasses(serviceInterface ,outputClassBody,serviceInterface.getOperations().get(i).getOutMessage(),i,"Out");
            
            for(int i=0;i<operationsPrototype.size();i++)
            {
            	out.write("\t@WebMethod "+operationsPrototype.get(i)+";\n\n");
            	outImpl.write("\t"+operationsPrototype.get(i)+"\n\t{\n\n");	
            	outImpl.write("\t\t"+firstCharacterToUpperCase(serviceInterface.getOperations().get(i).getOutMessage().getName())+"Out result = new " +
            			firstCharacterToUpperCase(serviceInterface.getOperations().get(i).getOutMessage().getName())+"Out();\n\n");
            	outImpl.write("\t\tLoadVariables();\n\n");
            	outImpl.write("\t\tServiceInterface abstractionServiceInterface = interfaceMapping.getServiceInterface(abstractionName);\n\n"); 
            	outImpl.write("\t\tObject output = invokeConcreteService(input,\""+serviceInterface.getOperations().get(i).getName()+"\", abstractionServiceInterface);\n\n");
            	outImpl.write("\t\tsetOutPut (output, abstractionServiceInterface, \""+serviceInterface.getOperations().get(i).getName()+"\" , result);\n\n");
            	outImpl.write("\t\treturn result;\n\n");
            	
            	outImpl.write("\t}\n\n");
            }
            
            out.write("}");
            outImpl.write("}");
            out.close();
            outImpl.close();
            fstream.close();
            fstreamImpl.close();
        }
		catch (Exception e)
        {
            System.err.println("Error: " + e.getMessage());
        }	    
		System.out.println(serviceInterface.getServiceInstances().get(0).getName());
		
		ClassCreation.createClassesFromWsdl ( serviceInterface.getName() , serviceInterface.getServiceInstances().get(0).getURI() , true );
		
		 try {
			Thread.sleep(20000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		ClassCreation.correctPackage(serviceInterface.getName(), true);
		 
		
	}
	
	private void createAbstractionPublisher(ServiceInterface serviceInterface, String URL)
	{
		FileWriter fstream;
		try {
			File directory=new  File("src/main/java/org/ow2/choreos/abstractionGenerator/abstractionImpl/"+serviceInterface.getName().toLowerCase()+"/Publisher/");
			directory.mkdirs();
			fstream = new FileWriter("src/main/java/org/ow2/choreos/abstractionGenerator/abstractionImpl/"+serviceInterface.getName().toLowerCase()+"/Publisher/"+serviceInterface.getName()+"AbstractionPublisher.java");
			BufferedWriter publisher = new BufferedWriter(fstream);
			publisher.write("package org.ow2.choreos.abstractionGenerator.abstractionImpl."+serviceInterface.getName().toLowerCase()+".Publisher"+";\n\n");
			publisher.write("import javax.xml.ws.Endpoint;\nimport org.ow2.choreos.abstractionGenerator.abstractionImpl."+serviceInterface.getName().toLowerCase()+"."+serviceInterface.getName()+"Impl;\n");
			
			
			publisher.write("public class "+serviceInterface.getName()+"AbstractionPublisher\n{\n");
			publisher.write("\tpublic static void main ( String [] args ) \n\t{\n");
			publisher.write("\t\tEndpoint.publish(\""+URL+"\",new "+serviceInterface.getName() + "Impl ());\n\t}");
			publisher.write("\n}");
			publisher.close();
			fstream.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
	}
	
	
	
	private String createClassBody(Message message,String param)
	{
		 
		String classBody="";
		classBody=classBody.concat("public class "+firstCharacterToUpperCase(message.getName())+param+"\n{\n");
		if(message.getMessageTypes()!= null)
		{
			for(int i=0;i<message.getMessageTypes().size();i++)
				classBody=classBody.concat("\tpublic "+firstCharacterToUpperCase(XMLBuiltInTypes.transformToJavaBuiltInType( message.getMessageTypes().get(i).getReferencedComponent().getType() ) )+" "+message.getMessageTypes().get(i).getName()+";\n");
		}
		classBody=classBody.concat("}");
		return classBody;
	}
	 
	public void generateClasses(ServiceInterface serviceInterface ,ArrayList <String> body,Message message,int i,String param)
	{

        FileWriter classCreator;
		try {
			classCreator = new FileWriter("src/main/java/org/ow2/choreos/abstractionGenerator/abstractionImpl/"+serviceInterface.getName().toLowerCase()+"/"+firstCharacterToUpperCase(message.getName()) +param+ ".java");
			BufferedWriter createClass = new BufferedWriter(classCreator);
		    createClass.write("package org.ow2.choreos.abstractionGenerator.abstractionImpl."+serviceInterface.getName().toLowerCase()+";\n\n");
		    createClass.write("import org.ow2.choreos.abstractionGenerator.abstractionImpl."+serviceInterface.getName().toLowerCase()+"."+
		    		serviceInterface.getName().toLowerCase()+".*;\n");
		    for(int j=0; j<imports.size();j++)
		    	createClass.write(imports.get(j)+"\n");
		    createClass.write(body.get(i));
		    createClass.close();
		    classCreator.close();
		} 
		catch (IOException e) {
				// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	
	public void imports(ServiceInterface serviceInterface)
	{
		for(int i=0;i<serviceInterface.getOperations().size();i++)
		{
			Message typeIn=serviceInterface.getOperations().get(i).getInMessage();
			createImports(typeIn);
			Message typeOut=serviceInterface.getOperations().get(i).getOutMessage();
			createImports(typeOut);
		}
		
	}
	
	
	//creation of appropriate imports 
	public void createImports(Message type)
	{
        imports.add("import javax.jws.WebMethod;");
        imports.add("import javax.jws.WebService;");
        imports.add("import org.ow2.choreos.abstractionGenerator.InputOutputDataTranslator;");
        imports.add("import org.ow2.choreos.abstractionGenerator.Serialiazer;");
        imports.add("import org.ow2.choreos.abstractionGenerator.BaseAbstractionImplService;");
        imports.add("import org.ow2.choreos.abstractionGenerator.ClassCreation;");
        imports.add("import org.ow2.choreos.abstractionGenerator.MappingsManipulation;");
        imports.add("import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;");
      if(type.getMessageTypes()!= null){
		for(int k=0; k<type.getMessageTypes().size();k++)
		{
			if(type.getMessageTypes().get(k).getReferencedComponent().getType().equals("integer"))
				imports.add("import java.math.BigInteger;");
			else if(type.getMessageTypes().get(k).getReferencedComponent().getType().equals("decimal"))
				imports.add("import java.math.BigDemical;");
			else if(type.getMessageTypes().get(k).getReferencedComponent().getType().equals("QName"))
				imports.add("import javax.xml.namespace.QName;");
			else if(type.getMessageTypes().get(k).getReferencedComponent().getType().equals("dataTime"))
				imports.add("import javax.xml.datatype.XMLGregorianCalendar;");
			else if(type.getMessageTypes().get(k).getReferencedComponent().getType().equals("time"))
				imports.add("import javax.xml.datatype.XMLGregorianCalendar;");
			else if(type.getMessageTypes().get(k).getReferencedComponent().getType().equals("Date"))
				imports.add("import java.util.Date;");			
		}
      }
	}	
	

	
	public void createConstructor(BufferedWriter outImpl,String serviceInterfaceName)
	{
		try {
			outImpl.write("\tpublic "+serviceInterfaceName+"Impl () \n\t{\n\t\tsuper(\""+serviceInterfaceName+"\");\n\t}\n\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void removeDuplicates()
	{
		HashSet<String> hashSet = new HashSet<String>(imports);
		ArrayList<String> arrayList = new ArrayList<String>(hashSet) ;
		imports=arrayList;
	}
	
	public String firstCharacterToUpperCase(String name)
	{
		char[] temp = name.toCharArray();
		temp[0] = Character.toUpperCase(temp[0]);
		name = new String(temp);
		return name;
	}
	
}
