package org.ow2.choreos.basicAlgorithms;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.omg.CORBA.IntHolder;
import org.ow2.choreos.dataStructures.array.arrayInstantiation.ArrayInstantiation;
import org.ow2.choreos.dataStructures.list.listInsertion.ListInsertion;
import org.ow2.choreos.dataStructures.list.listRemoval.ListRemoval;
import org.ow2.choreos.dataStructures.map.mapInsertion.MapInsertion;
import org.ow2.choreos.dataStructures.map.mapInstantiation.MapInstantiation;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapRemoval.MapRemoval;
import org.ow2.choreos.gui.PrintMessages;


/**
 * 
 * @author dathanas
 *
 */
public class CartesianProduct {


	//Variables.
	private static int repetitions = 0;


	static public Object[][] calculateCartesianPoduct( List<Object> orderingOfKeys, Map<Object, List<Object>> values, Map<Object, Object> orderedSets ){//The first map contains keys that index a list of objects. The cartesian product of the latter objects is calculated.
																														   							//The second map is used for determining which list of objects are ordered.

		return calculateCartesianPoductCommonCode( orderingOfKeys, values, orderedSets );
	}

	static public Object[][] calculateCartesianPoduct( List<Object> orderingOfKeys, Map<Object, List<Object>> values ){//The map contains keys that index a list of objects. The cartesian product of the latter object is calculated.

		return calculateCartesianPoductCommonCode( orderingOfKeys, values, null );
	}

	static private Object[][] calculateCartesianPoductCommonCode( List<Object> orderingOfKeys, Map<Object, List<Object>> values, Map<Object, Object> orderedSets ){

		Object[][] tuples = null;


		int numberOfSets = values != null ? values.size() : 0;

		Map<Object, List<Object>> copyOfValues = numberOfSets > 0 ? MapInstantiation.newMapOfLists() : null;

		int[] sizeOfSets = ArrayInstantiation.newIntArray( numberOfSets );

		boolean[] isOrderedSet = ArrayInstantiation.newbooleanArray( numberOfSets );

		int[] partialProductsOfSizeOfSets = ArrayInstantiation.newIntArray( numberOfSets );


		getPartialProductsOfSizeOfSets( false, values, copyOfValues, orderedSets, orderingOfKeys, sizeOfSets, isOrderedSet, partialProductsOfSizeOfSets );


		int sizeOfCartesianProduct = partialProductsOfSizeOfSets != null ? partialProductsOfSizeOfSets[ partialProductsOfSizeOfSets.length - 1 ] : 0;

		tuples = partialProductsOfSizeOfSets != null ? ArrayInstantiation.newArrayOfArray( sizeOfCartesianProduct, partialProductsOfSizeOfSets.length ) : null;

		boolean[] settedTuples = ArrayInstantiation.newbooleanArray( sizeOfCartesianProduct );


		for( int tupleCounter = 0; tupleCounter < sizeOfCartesianProduct; ++tupleCounter ) setTuple( orderingOfKeys, copyOfValues, tupleCounter, partialProductsOfSizeOfSets, sizeOfSets, tuples, settedTuples );


		System.out.println( "\n" + tuples.length + " tuple(s) in " + repetitions + " repetitions:" );

		printTuples( tuples );


		return tuples;
	}

	static public void getPartialProductsOfSizeOfSets( boolean isUsedForProducingCombinations, Map<Object, List<Object>> values, Map<Object, List<Object>> copyOfValues, Map<Object, Object> orderedSets, List<Object> orderingOfKeys, int[] sizeOfSets, boolean[] isOrderedSet, int[] partialProductsOfSizeOfSets ){

		int numOfMetOrderedSets = 0;

		int commonSizeOfOrderedSets = 0;

		int commonSizeOfSets = 0;

		int lastOrderedSetIndex = 0, partialProductOfLastOrderedSet = 0;


		for( int i = 0; orderingOfKeys != null && i < orderingOfKeys.size(); ++i ){ ++repetitions;

			Object key = orderingOfKeys.get( i );

			List<Object> valuesOfAKey = MapGetter.getValueOfMapOfList( key, values );


			List<Object> newListOfValuesOfAKey = ListInsertion.addAll( valuesOfAKey, null );

			copyOfValues = MapInsertion.putInMapOfList( key, newListOfValuesOfAKey, copyOfValues, true );


			sizeOfSets[ i ] = valuesOfAKey.size();

			if( isUsedForProducingCombinations ){

				if( i == 0 ) commonSizeOfSets = valuesOfAKey.size();

				else if( commonSizeOfSets != valuesOfAKey.size() ){

					PrintMessages.print( "Cartesian product for combinations: There is one or more sets that have different size to each other." );

					System.exit( 0 );
				}
			}


			isOrderedSet[ i ] = MapGetter.exists( key, orderedSets ) ? (Boolean) MapGetter.getValue( key, orderedSets ) : false;

			if( isOrderedSet[ i ] ) ++numOfMetOrderedSets;

			if( isOrderedSet[ i ] ){

				if( numOfMetOrderedSets == 1 ) commonSizeOfOrderedSets = valuesOfAKey.size();

				else if( commonSizeOfOrderedSets != valuesOfAKey.size() ){

					PrintMessages.print( "Ordered cartesian product: There is one or more ordered that have different size to each other." );

					System.exit( 0 );
				}
			}


			if( i == 0 ) partialProductsOfSizeOfSets[ i ] = isOrderedSet[ i ] ? 1 : valuesOfAKey.size();

			else partialProductsOfSizeOfSets[ i ] = isOrderedSet[ i ] ? partialProductsOfSizeOfSets[ i - 1 ] : valuesOfAKey.size() * partialProductsOfSizeOfSets[ i - 1 ]; 


			if( isOrderedSet[ i ] ){

				partialProductOfLastOrderedSet = partialProductsOfSizeOfSets[ i ];

				lastOrderedSetIndex = i;
			}
		}


		for( int i = lastOrderedSetIndex; numOfMetOrderedSets > 0 && partialProductsOfSizeOfSets != null && i < partialProductsOfSizeOfSets.length; ++i ){ ++repetitions;

			partialProductOfLastOrderedSet = partialProductsOfSizeOfSets[ lastOrderedSetIndex ] * commonSizeOfOrderedSets;

			if( ! isOrderedSet[ i ] ) partialProductsOfSizeOfSets[ i ] = partialProductsOfSizeOfSets[ i ] * commonSizeOfOrderedSets;
		}


		for( int i = 0; numOfMetOrderedSets > 0 && partialProductsOfSizeOfSets != null && i < partialProductsOfSizeOfSets.length; ++i ){ ++repetitions;

			if( isOrderedSet[ i ] ) partialProductsOfSizeOfSets[ i ] = partialProductOfLastOrderedSet;
		}
	}

	public static Object getValue( int keyIndex, List<Object> orderingOfKeys, int valueIndex, Map<Object, List<Object>> values ){

		Object key = orderingOfKeys != null && keyIndex >=0 && keyIndex < orderingOfKeys.size() ? orderingOfKeys.get( keyIndex ) : null;

		List<Object> valuesOfAKey = key != null ? MapGetter.getValueOfMapOfList( key, values ) : null;


		return valuesOfAKey != null && ! valuesOfAKey.isEmpty() && valueIndex >= 0 && valueIndex < valuesOfAKey.size() ? valuesOfAKey.get( valueIndex ) : null;
	}

	private static List<Object> getValues( int keyIndex, List<Object> orderingOfKeys, Map<Object, List<Object>> values ){

		Object key = orderingOfKeys != null && keyIndex >=0 && keyIndex < orderingOfKeys.size() ? orderingOfKeys.get( keyIndex ) : null;

		return key != null ? MapGetter.getValueOfMapOfList( key, values ) : null;
	}



	static public Map<Object, Object> combine( List<Object> orderingOfKeys, Map<Object, List<Object>> values, Map<Object, Object> orderedSets, boolean checkExistenceOfTuples ){//It combines the results of the cartesian product of a number of sets.

		return combineCommonCode( orderingOfKeys, values, orderedSets, checkExistenceOfTuples, -1 );
	}

	static public Map<Object, Object> combine( List<Object> orderingOfKeys, Map<Object, List<Object>> values, Map<Object, Object> orderedSets, int maximumSizeOfSets ){//It combines the results of the cartesian product of a number of sets.

		return combineCommonCode( orderingOfKeys, values, orderedSets, true, maximumSizeOfSets );
	}

	static public Map<Object, Object> combine( List<Object> orderingOfKeys, Map<Object, List<Object>> values, Map<Object, Object> orderedSets ){//It combines the results of the cartesian product of a number of sets.

		return combineCommonCode( orderingOfKeys, values, orderedSets, false, -1 );
	}

	static public Map<Object, Object> combine( List<Object> orderingOfKeys, Map<Object, List<Object>> values, boolean checkExistenceOfTuples ){//It combines the results of the cartesian product of a number of sets.

		return combineCommonCode( orderingOfKeys, values, null, checkExistenceOfTuples, -1 );
	}

	static public Map<Object, Object> combine( List<Object> orderingOfKeys, Map<Object, List<Object>> values ){//It combines the results of the cartesian product of a number of sets.

		return combineCommonCode( orderingOfKeys, values, null, false, -1 );
	}


	static private Map<Object, Object> combineCommonCode( List<Object> orderingOfKeys, Map<Object, List<Object>> values, Map<Object, Object> orderedSets, boolean checkExistenceOfTuples, int maximumSizeOfSets ){

		Map<Object, Object> allCombinations = MapInstantiation.newMap();


		int numberOfSets = values != null ? values.size() : 0;

		Map<Object, List<Object>> copyOfValues = numberOfSets > 0 ? MapInstantiation.newMapOfLists() : null;

		int[] sizeOfSets = ArrayInstantiation.newIntArray( numberOfSets );

		boolean[] isOrderedSet = ArrayInstantiation.newbooleanArray( numberOfSets );

		int[] partialProductsOfSizeOfSets = ArrayInstantiation.newIntArray( numberOfSets );


		getPartialProductsOfSizeOfSets( true, values, copyOfValues, orderedSets, orderingOfKeys, sizeOfSets, isOrderedSet, partialProductsOfSizeOfSets );


		int commonSizeOfSets = sizeOfSets[ 0 ];

		if( maximumSizeOfSets > 0 && maximumSizeOfSets < commonSizeOfSets ){

			Object[] combination = greedyCalculationOfCombination( orderingOfKeys, copyOfValues, numberOfSets, commonSizeOfSets, isOrderedSet );

			allCombinations = MapInsertion.put( combination, true, allCombinations, true );

			printCombinations( "\n\n" + allCombinations.size() + " final combination(s) in " + repetitions + " repetitions:", allCombinations );
		}

		else{

			int sizeOfCartesianProduct = partialProductsOfSizeOfSets != null ? partialProductsOfSizeOfSets[ partialProductsOfSizeOfSets.length - 1 ] : 0;

			int lastKeyIndexRepetition = partialProductsOfSizeOfSets != null ? partialProductsOfSizeOfSets[ partialProductsOfSizeOfSets.length - 1 ] / sizeOfSets [ partialProductsOfSizeOfSets.length - 1 ] : 0;

			Object[][] tuples = partialProductsOfSizeOfSets != null ? ArrayInstantiation.newArrayOfArray( sizeOfCartesianProduct, partialProductsOfSizeOfSets.length ) : null;

			boolean[] settedTuples = ArrayInstantiation.newbooleanArray( sizeOfCartesianProduct );

			Boolean[] existingTuples = ArrayInstantiation.newBooleanArray( sizeOfCartesianProduct );

			double[] tuplesConfidence = ArrayInstantiation.newdoubleArray( sizeOfCartesianProduct );


			boolean[] existingInitializedCombination = ArrayInstantiation.newbooleanArray( lastKeyIndexRepetition );


			for( int currentLastSetValueRepetition = 0; currentLastSetValueRepetition < lastKeyIndexRepetition; ++currentLastSetValueRepetition ){

				Map<Object, Object> currentCombinations = MapInstantiation.newMap();

				Map<Object, Object> usedValueIndexesOfAllSets = MapInstantiation.newMap();

				IntHolder numberOfMetOrderedSets = new IntHolder( 0 );


				existingInitializedCombination[ currentLastSetValueRepetition ] = initializeCombination( currentLastSetValueRepetition, numberOfSets, orderingOfKeys, values, partialProductsOfSizeOfSets, sizeOfSets, numberOfMetOrderedSets, isOrderedSet, tuples, settedTuples, existingTuples, tuplesConfidence, usedValueIndexesOfAllSets, currentCombinations, checkExistenceOfTuples );


				enumerateCombinations( numberOfSets, orderingOfKeys, values, sizeOfSets, partialProductsOfSizeOfSets, numberOfMetOrderedSets, isOrderedSet, tuples, settedTuples, existingTuples, usedValueIndexesOfAllSets, currentCombinations, allCombinations );
			}


			checkCombinations( allCombinations, existingTuples, checkExistenceOfTuples, tuplesConfidence );


			printTuples( tuples, existingTuples );

			printCombinations( "", allCombinations.size() + " final combination(s) in " + repetitions + " repetitions:", allCombinations, tuples );
		}


		return allCombinations;
	}


	//Greedy calculation of combinations.
	private static Object[] greedyCalculationOfCombination( List<Object> orderingOfKeys, Map<Object, List<Object>> values, int numberOfSets, int commonSizeOfSets, boolean[] isOrderedSet ){

		Object[] combination = ArrayInstantiation.newArray( commonSizeOfSets );

		int numberOfMetOrderedSets = 0;

		double maxMultipartiteConfidence = 1;

		for( int combinationElement = 0; maxMultipartiteConfidence != 0 && combinationElement < commonSizeOfSets; ++combinationElement ){ System.out.println( "\ncombination element = " + combinationElement );//TODO

			Object[] tuple = ArrayInstantiation.newArray( numberOfSets );


			maxMultipartiteConfidence = 1;

			for( int setIndex = 0; maxMultipartiteConfidence != 0 && setIndex < numberOfSets; ++setIndex ){ System.out.println( "\n\tset = " + setIndex );//TODO

				List<Object> valuesOfASet = getValues( setIndex, orderingOfKeys, values );

				if( isOrderedSet[ setIndex ] ) ++numberOfMetOrderedSets;


				maxMultipartiteConfidence = 0;

				int valueIndexOfMaxMultipartiteConfidence = 0;


				if( isOrderedSet[ setIndex ] && numberOfMetOrderedSets > 1 ){

					int valueIndex = 0;

					System.out.println( "\n\t\tcandidate value = " + getValue( setIndex, orderingOfKeys, valueIndex, values ) );//TODO

					maxMultipartiteConfidence = calculateMultipartiteConfidence( setIndex, tuple );

					valueIndexOfMaxMultipartiteConfidence = valueIndex;
				}

				else{

					for( int valueIndex = 0; valuesOfASet != null && valueIndex < valuesOfASet.size(); ++valueIndex ){ System.out.println( "\n\t\tcandidate value = " + getValue( setIndex, orderingOfKeys, valueIndex, values ) );//TODO

						double multipartiteConfidence = calculateMultipartiteConfidence( setIndex, tuple );

						if( multipartiteConfidence > maxMultipartiteConfidence ){

							maxMultipartiteConfidence = multipartiteConfidence;

							valueIndexOfMaxMultipartiteConfidence = valueIndex;
						}


						if( setIndex == 0 ) break;
					}
				}


				System.out.println( "\n\t\tmax multipartite confidence " + maxMultipartiteConfidence + " by value = " + getValue( setIndex, orderingOfKeys, valueIndexOfMaxMultipartiteConfidence, values ) );//TODO

				if( maxMultipartiteConfidence > 0 ){

					tuple[ setIndex ] = getValue( setIndex, orderingOfKeys, valueIndexOfMaxMultipartiteConfidence, values );


					ListRemoval.remove( valueIndexOfMaxMultipartiteConfidence, valuesOfASet );

					values = MapInsertion.putInMapOfList( orderingOfKeys.get( setIndex ), valuesOfASet, values, true );
				}
			}


			combination [ combinationElement ] = tuple;

			if( maxMultipartiteConfidence == 0 ) combination = null;
		}


		return combination;
	}

	private static double calculateMultipartiteConfidence( int setIndex, Object[] tuple ){

		double multipartiteConfidence = 1;

		for( int previousSetIndex = 0; multipartiteConfidence != 0 && previousSetIndex < setIndex; ++previousSetIndex ){

			System.out.println( "\n\t\t\tprevious value = " + tuple[ previousSetIndex ] );//TODO

			double bipartiteConfidence = (double) Dice.getRandomNumber( 11 ) / (double) 10;//Dummy check!

			//double bipartiteConfidence = 1;

			multipartiteConfidence *= bipartiteConfidence;

			System.out.println( "\n\t\t\t\tbipartite confidence " + bipartiteConfidence + " - multipartiteConfidence = " + multipartiteConfidence );//TODO

			++repetitions;
		}


		return multipartiteConfidence;
	}


	//Exhaustive calculation of combinations.
	private static boolean initializeCombination( int currentLastSetValueRepetition, int numberOfSets, List<Object> orderingOfKeys, Map<Object, List<Object>> values, int[] partialProductsOfSizeOfSets, int[] sizeOfSets, IntHolder numberOfMetOrderedSets, boolean[] isOrderedSet, Object[][] tuples, boolean[] settedTuples, Boolean[] existingTuples, double[] initializedTuplesConfidence, Map<Object, Object> usedValueIndexesOfAllSets, Map<Object, Object> currentCombinations, boolean checkExistenceOfTuples ){

		int commonSizeOfSets = sizeOfSets[ 0 ];

		Object[] combination = ArrayInstantiation.newArray( commonSizeOfSets );


		for( int setIndex = numberOfSets - 1; setIndex >= 0; --setIndex ){ //System.out.println( "\n\tset = " + setIndex );//TODO

			int repetition = partialProductsOfSizeOfSets[ setIndex ] / sizeOfSets [ setIndex ];

			int valueIndex = ( currentLastSetValueRepetition / repetition ) % sizeOfSets [ setIndex ];

			Object value = getValue( setIndex, orderingOfKeys, valueIndex, values );


			tuples[ currentLastSetValueRepetition ][ setIndex ] = value;

			settedTuples[ currentLastSetValueRepetition ] = true;


			if( setIndex == numberOfSets - 1 ){

				if( isOrderedSet[ setIndex ] ) numberOfMetOrderedSets.value++;


				Map<Object, Object> usedValueIndexesOfASet = null;

				for( int candidateValueIndex = 0; candidateValueIndex < sizeOfSets [ setIndex ]; ++candidateValueIndex ){ ++repetitions;

					combination[ candidateValueIndex ] = candidateValueIndex == 0 ? currentLastSetValueRepetition : candidateValueIndex * repetition;

					usedValueIndexesOfASet = MapInsertion.put( candidateValueIndex, true, usedValueIndexesOfASet, true );
				}

				usedValueIndexesOfAllSets = MapInsertion.put( setIndex, usedValueIndexesOfASet, usedValueIndexesOfAllSets, true );
			}

			else{ ++repetitions;

				Map<Object, Object> usedValueIndexesOfASet = MapInsertion.put( valueIndex, true, null, true );

				usedValueIndexesOfAllSets = MapInsertion.put( setIndex, usedValueIndexesOfASet, usedValueIndexesOfAllSets, true );
			}
		}


		boolean exist = true;

		if( checkExistenceOfTuples ){

			initializedTuplesConfidence[ currentLastSetValueRepetition ] = (double) Dice.getRandomNumber( 11 ) / (double) 10;//Dummy check!

			exist = (double) initializedTuplesConfidence[ currentLastSetValueRepetition ] == 0.0 ? false : true;//Dummy check!
		}

		existingTuples[ currentLastSetValueRepetition ] = exist;

		if( checkExistenceOfTuples ) System.out.println( "Tuple " + currentLastSetValueRepetition + " exists = " + exist + " - confidence = " + initializedTuplesConfidence[ currentLastSetValueRepetition ] );

		if( exist ) currentCombinations = MapInsertion.put( combination, usedValueIndexesOfAllSets, currentCombinations, true );


		return exist;
	}

	@SuppressWarnings("unchecked")
	private static void enumerateCombinations( int numberOfSets, List<Object> orderingOfKeys, Map<Object, List<Object>> values, int[] sizeOfSets, int[] partialProductsOfSizeOfSets, IntHolder numberOfMetOrderedSets, boolean[] isOrderedSet, Object[][] tuples, boolean[] settedTuples, Boolean[] existingTuples, Map<Object, Object> usedValueIndexesOfAllSets, Map<Object, Object> currentCombinations, Map<Object, Object> allCombinations ){

		int commonSizeOfSets = sizeOfSets[ 0 ];


		for( int setIndex = numberOfSets - 2; setIndex >= 0; --setIndex ){

			//System.out.println( "\n\tset = " + setIndex );//TODO

			if( isOrderedSet[ setIndex ] ) ++numberOfMetOrderedSets.value;

			int repetition = partialProductsOfSizeOfSets[ setIndex ] / sizeOfSets [ setIndex ];


			for( int combinationMember = 1; combinationMember < commonSizeOfSets; ++combinationMember ) {

				//System.out.println( "\n\tcombination member = " + combinationMember );//TODO

				Map<Object, Object> newCombinations = null;

				Iterator<Object> currentCombinationsIterator = currentCombinations != null && currentCombinations.keySet() != null ? currentCombinations.keySet().iterator() : null;


				while( currentCombinationsIterator != null && currentCombinationsIterator.hasNext() ){

					Object[] currentCombination = (Object[]) currentCombinationsIterator.next();

					//printCombination( "\n\t\t", "Removed combination: ", currentCombination, tuples );//TODO


					usedValueIndexesOfAllSets = (Map<Object, Object>) MapGetter.getValue( currentCombination, currentCombinations );

					Map<Object, Object> usedValueIndexesOfASet = (Map<Object, Object>) MapGetter.getValue( setIndex, usedValueIndexesOfAllSets );


					for( int candidateValueIndex = 0; candidateValueIndex < sizeOfSets [ setIndex ]; ++candidateValueIndex ){

						if( ! MapGetter.exists( candidateValueIndex, usedValueIndexesOfASet ) ){

							//System.out.println( "\n\t\tcandidate value index = " + candidateValueIndex );//TODO

							Object[] newCombination = ArrayInstantiation.newArray( commonSizeOfSets );

							System.arraycopy( currentCombination, 0, newCombination, 0, currentCombination.length );

							Map<Object, Object> newUsedValueIndexesOfAllSets = MapInsertion.putAll( usedValueIndexesOfAllSets, null );

							Map<Object, Object> newUsedValueIndexesOfASet = MapInsertion.putAll( usedValueIndexesOfASet, null );


							newCombination[ combinationMember ] = numberOfMetOrderedSets.value > 1 && isOrderedSet[ setIndex ] ? (Integer) currentCombination[ combinationMember ] : (Integer) currentCombination[ combinationMember ] + candidateValueIndex * repetition;


							newUsedValueIndexesOfASet = MapInsertion.put( candidateValueIndex, true, newUsedValueIndexesOfASet, true );

							newUsedValueIndexesOfAllSets = MapInsertion.put( setIndex, newUsedValueIndexesOfASet, newUsedValueIndexesOfAllSets, true );

							newCombinations = MapInsertion.put( newCombination, newUsedValueIndexesOfAllSets, newCombinations, true );

							//printCombinations( "\t\t\t\t", "newCombinations:", newCombinations, tuples );//TODO


							setTuple( orderingOfKeys, values, (Integer) newCombination[ combinationMember ], partialProductsOfSizeOfSets, sizeOfSets, tuples, settedTuples );
						}

						else ++repetitions;
					}
				}

				MapRemoval.clear( currentCombinations );

				currentCombinations = MapInsertion.putAll( newCombinations, currentCombinations );
			}
		}


		allCombinations = MapInsertion.putAll( currentCombinations, allCombinations );
	}

	public static void setTuple( List<Object> orderingOfKeys, Map<Object, List<Object>> values, int tupleCounter, int[] partialProductsOfSizeOfSets, int[] sizeOfSets, Object[][] tuples, boolean[] settedTuples ){

		//System.out.println( "tuple = " + tuple );//TODO

		for( int setIndex = partialProductsOfSizeOfSets.length - 1; settedTuples != null && ! settedTuples[ tupleCounter ] && partialProductsOfSizeOfSets != null && setIndex >= 0; --setIndex ){ ++repetitions;

			int repetition = partialProductsOfSizeOfSets[ setIndex ] / sizeOfSets [ setIndex ]; //System.out.println( "\tsetIndex = " + setIndex + " - partialProductsOfSizeOfSets = " + partialProductsOfSizeOfSets[ setIndex ] + " - repetition = " + repetition );//TODO

			int valueIndex = ( tupleCounter / repetition ) % sizeOfSets [ setIndex ];

			Object value = getValue( setIndex, orderingOfKeys, valueIndex, values );


			tuples[ tupleCounter ][ setIndex ] = value;
		}

		settedTuples[ tupleCounter ] = true;
	}

	private static void checkCombinations( Map<Object, Object> combinations, Boolean[] existingTuples, boolean checkExistenceOfTuples, double[] tuplesConfidence ){

		Map<Object, Object> removedCombinations = null;


		if( checkExistenceOfTuples && ! combinations.isEmpty() ) System.out.println( "\n\n---- START - Combinations have been calculated and now are checked ----" );//TODO

		int checkRepetitions = 0, combinationCounter = 0;

		Iterator<Object> combinationsIterator = combinations != null && combinations.keySet() != null ? combinations.keySet().iterator() : null;

		while( combinationsIterator != null && combinationsIterator.hasNext() ){

			Object[] currentCombination = (Object[]) combinationsIterator.next(); if( checkExistenceOfTuples ) System.out.println( "Combination " + combinationCounter++ );//TODO


			boolean exist = true;

			for( int i = 0; exist && i < currentCombination.length; ++i ){ ++checkRepetitions;

				int tupleNumber = (Integer) currentCombination[ i ];


				if( existingTuples[ tupleNumber ] == null ){

					if( checkExistenceOfTuples ){

						tuplesConfidence[ tupleNumber ] = (double) Dice.getRandomNumber( 11 ) / (double) 10;//Dummy check!

						existingTuples[ tupleNumber ] = (double) tuplesConfidence[ tupleNumber ] == 0.0 ? false : true;//Dummy check!
					}

					else existingTuples[ tupleNumber ] = true;


					if( checkExistenceOfTuples ) System.out.println( "\ttupleNumber " + tupleNumber + " unchecked, now is " + existingTuples[ tupleNumber ] );//TODO
				}

				else if( checkExistenceOfTuples ) System.out.println( "\ttupleNumber " + tupleNumber + " exists = " + existingTuples[ tupleNumber ] );


				exist = existingTuples[ tupleNumber ];
			}

			if( checkExistenceOfTuples ) System.out.println( "\tcombination " + combinationCounter + " exist = " + exist );

			if( ! exist ) removedCombinations = MapInsertion.put( currentCombination, true, removedCombinations, true );
		}


		Iterator<Object> removedCombinationsIterator = removedCombinations != null && removedCombinations.keySet() != null ? removedCombinations.keySet().iterator() : null;

		while( removedCombinationsIterator != null && removedCombinationsIterator.hasNext() ){

			Object currentCombination = removedCombinationsIterator.next();

			MapRemoval.remove( currentCombination, combinations );
		}


		if( checkExistenceOfTuples && ! combinations.isEmpty() ) System.out.println( "---- END - Combinations have been in " + checkRepetitions + " repetitions ----" );//TODO
	}


	//Visualization-Greedy calculation of combinations.
	private static void printCombinations( String msg, Map<Object, Object> combinations ){

		System.out.println( msg );


		Iterator<Object> combinationsIterator = combinations != null && combinations.keySet() != null ? combinations.keySet().iterator() : null;

		while( combinationsIterator != null && combinationsIterator.hasNext() ){

			Object[] combination = (Object[]) combinationsIterator.next();

			printCombination( combination );
		}
	}

	private static void printCombination( Object[] combination ){

		for( int i = 0; combination != null && i < combination.length; ++i ){

			printTuple( (Object[]) combination[ i ] );

			System.out.print( " " );
		}
	}

	private static void printTuple( Object[] tuple ){

		System.out.print( "(" );

		for( int i = 0; tuple != null && i < tuple.length; ++i ){

			if( i == tuple.length - 1 ) System.out.print( tuple[ i ] );

			else System.out.print( tuple[ i ] + " " );
		}

		System.out.print( ")" );
	}


	//Visualization-Exhaustive calculation of combinations.
	private static void printTuples( Object[][] tuples, Boolean[] existingTuples ){

		printTuplesCommonCode( tuples, existingTuples );
	}

	private static void printTuples( Object[][] tuples ){

		printTuplesCommonCode( tuples, null );
	}

	private static void printTuplesCommonCode( Object[][] tuples, Boolean[] existingTuples ){

		System.out.println( "\n" );

    	for( int i = 0; tuples != null && i < tuples.length; ++i ){

    		if( existingTuples == null || ( existingTuples != null && existingTuples[ i ] != null && existingTuples[ i ] ) ){

    			System.out.print( i + " : " );

    			for( int j = 0; tuples[ i ] != null && j < tuples[ i ].length; ++j ) System.out.print( tuples[ i ][ j ] );

    			System.out.println();
    		}
    	}

    	System.out.println( "\n" );
	}

	private static void printCombination( String tabs, String msg, Object[] combination, Object[][] tuples ){

		System.out.print( tabs + msg );

		for( int i = 0; combination != null && i < combination.length; ++i ){

			System.out.print( combination[ i ] );

			printTupleOfCartesianProduct( (Integer) combination[ i ], tuples );

			System.out.print( " " );
		}

		System.out.println();
	}

	private static void printTupleOfCartesianProduct( int tupleNumber, Object[][] tuples ){

		System.out.print( "(" );

		for( int i = 0; tuples != null && tuples[ tupleNumber ] != null && i < tuples[ tupleNumber ].length; ++i ){

			if( i == tuples[ tupleNumber ].length - 1 ) System.out.print( tuples[ tupleNumber ][ i ] );

			else System.out.print( tuples[ tupleNumber ][ i ] + " " );
		}

		System.out.print( ")" );
	}

	@SuppressWarnings("unchecked")
	private static void printCombinations( String tabs, String msg, Map<Object, Object> combinations, Object[][] tuples ){

		System.out.println( tabs + msg );


		int combinationCounter = 0;

		Iterator<Object> combinationsIterator = combinations != null && combinations.keySet() != null ? combinations.keySet().iterator() : null;

		while( combinationsIterator != null && combinationsIterator.hasNext() ){

			Object[] currentCombination = (Object[]) combinationsIterator.next();

			printCombination( "\n\n" + tabs + "\t", combinationCounter++ + " : ", currentCombination, tuples );


			System.out.print( tabs + "\t\t" + "Used value indexes: " );


			Map<Object, Object> usedValueIndexesOfAllSets = (Map<Object, Object>) MapGetter.getValue( currentCombination, combinations );

			Iterator<Object> usedValueIndexesOfAllSetsIterator = usedValueIndexesOfAllSets != null && usedValueIndexesOfAllSets.keySet() != null ? usedValueIndexesOfAllSets.keySet().iterator() : null;

			while( usedValueIndexesOfAllSetsIterator != null && usedValueIndexesOfAllSetsIterator.hasNext() ){

				Integer setIndex = (Integer) usedValueIndexesOfAllSetsIterator.next(); System.out.print( "\n" + tabs + "\t\t" + setIndex + " --> " );

				Map<Object, Object> usedValueIndexesOfASet = (Map<Object, Object>) MapGetter.getValue( setIndex, usedValueIndexesOfAllSets );


				Iterator<Object> usedValueIndexesOfASetIterator = usedValueIndexesOfASet != null && usedValueIndexesOfASet.keySet() != null ? usedValueIndexesOfASet.keySet().iterator() : null;

				while( usedValueIndexesOfASetIterator != null && usedValueIndexesOfASetIterator.hasNext() ){

					Integer useValueIndex = (Integer) usedValueIndexesOfASetIterator.next();

					System.out.print( useValueIndex + " " );
				}
			}
		}
	}
}
