package org.ow2.choreos.clusteringAlgorithms.partitionalClustering.kMeans;

import java.util.Vector;


public class Cluster {

	//Variables.
    private String mName;
    private Centroid mCentroid;
    private double mSumSqr;
    private Vector<DataPoint> mDataPoints;

    //Constructor.
    Cluster(String name) {
        this.mName = name;
        this.mCentroid = null; //will be set by calling setCentroid()
        this.mDataPoints = new Vector<DataPoint>();
    }

    void setCentroid(Centroid c) {
        mCentroid = c;
    }

    public Centroid getCentroid() {

        return mCentroid;
    }

    void addDataPoint(DataPoint dp) { //called from CAInstance
        dp.setCluster(this); //initiates a inner call to calcEuclideanDistance() in DP.
        this.mDataPoints.addElement(dp);
        calcSumOfSquares();
    }

    void removeDataPoint(DataPoint dp) {
        this.mDataPoints.removeElement(dp);
        calcSumOfSquares();
    }

    int getNumDataPoints() {
        return this.mDataPoints.size();
    }

    DataPoint getDataPoint(int pos) {
        return (DataPoint) this.mDataPoints.elementAt(pos);
    }

    void calcSumOfSquares() { //called from Centroid
        int size = this.mDataPoints.size();
        double temp = 0;
        for (int i = 0; i < size; i++) {
            temp = temp + ((DataPoint)
            this.mDataPoints.elementAt(i)).getCurrentEuDt();
        }
        this.mSumSqr = temp;
    }

    double getSumSqr() {
        return this.mSumSqr;
    }

    public String getName() {
        return this.mName;
    }

    public Vector<DataPoint> getDataPoints() {
        return this.mDataPoints;
    }
}