package org.ow2.choreos.gui;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Map;

import javax.swing.JOptionPane;

import org.ow2.choreos.classInfo.ClassInfo;
import org.ow2.choreos.dataStructures.map.mapInsertion.MapInsertion;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.writer.freeTextDocuments.TxtWriter;


/**
 * 
 * @author dathanas
 *
 */
@SuppressWarnings("serial")
public class PrintMessages {


	//Variables.
	private final static String defaultLogFileName = "log.txt";

	private final static String defaultLogPrintFilePath = "src/main/resources/" + defaultLogFileName;

	public final static String xmlPrint = "xml";
	public final static String guiPrint = "gui";
	public final static String wsdlPrint = "wsdl";
	public final static String pluginPrint = "plugin";
	public final static String parserPrint = "parser";
	public final static String mappingPrint = "mapping";
	public final static String xmlParserPrint = "xmlParser";
	public final static String wsdlParserPrint = "wsdlParser";
	public final static String behavioralPrint = "behavioral";
	public final static String controllerPrint = "controller";
	public final static String matchingPrint = "matchingPrint";
	public final static String abstractionPrint = "abstraction";
	public final static String registrationPrint = "registration";
	public final static String organizationPrint = "organization";
	public final static String transformationPrint = "transformation";
	public final static String schemaMatchingIndexing = "schemaMatchingIndexing";
	public final static String schemaRegistrationPrint = "schemaRegistrationPrint";
	public final static String connectionXMLWithXMLPrint = "connectionXMLWithXMLPrint";
	public final static String connectionWSDLWithXMLPrint = "connectionWSDLWithXMLPrint";
	

	private static Map<Object, Object> printKinds = new Hashtable<Object, Object>() { {   put( xmlPrint, false );
																					 	  put( guiPrint, true );
																					 	  put( wsdlPrint, false );
																					 	  put( parserPrint, false );
																					 	  put( pluginPrint, false );
																					 	  put( mappingPrint, false );
																					 	  put( xmlParserPrint, false );
																					 	  put( behavioralPrint, false );
																					 	  put( controllerPrint, false );
																					 	  put( wsdlParserPrint, false );
																					 	  put( abstractionPrint, false );
																					 	  put( organizationPrint, false );
																					 	  put( registrationPrint, false );
																					 	  put( transformationPrint, false );
																					 	  put( schemaMatchingIndexing, true );
																					 	  put( schemaRegistrationPrint, true );
																					 	  put( connectionXMLWithXMLPrint, false );
																					 	  put( connectionWSDLWithXMLPrint, false );
																					} };


	//Getters.
	static public boolean toPrint( Object printKind ){

		return MapGetter.exists( printKind, printKinds ) ? (Boolean) MapGetter.getValue( printKind, printKinds ) : false;
	}


	//Setters.
	static public void addPrintKind( Object key, Object value ){

		printKinds = MapInsertion.put( key, value, printKinds, false );
	}


	//Methods.
	static public void print( String msg ){

		JOptionPane.showMessageDialog( null, msg );
	}

	static public void print( String printKind, boolean hasConstraintViolation, Object object ){

		if( toPrint( printKind ) && ! hasConstraintViolation ) JOptionPane.showMessageDialog( null, "New '" + ClassInfo.getClass( object ) + "' object." );
	}

	static public void print( String printKinds[], boolean hasConstraintViolation, Object object ){

		if( print( printKinds ) && ! hasConstraintViolation ) JOptionPane.showMessageDialog( null, "New '" + ClassInfo.getClass( object ) + "' object." );
	}

	static public void print( String printKinds[], Object object ){

		if( print( printKinds ) ) JOptionPane.showMessageDialog( null, "New '" + ClassInfo.getClass( object ) + "' object." );
	}

	static public void print( String printKind, Object object ){

		if( toPrint( printKind ) ) JOptionPane.showMessageDialog( null, "New '" + ClassInfo.getClass( object ) + "' object." );
	}

	static public void print( String printKinds[], Object object, String msg ){

		if( print( printKinds ) ) JOptionPane.showMessageDialog( null, "New '" + ClassInfo.getClass( object ) + " ( " + msg + " )' object." );
	}

	static public void print( String printKind, Object object, String msg ){

		if( toPrint( printKind ) ) JOptionPane.showMessageDialog( null, "New '" + ClassInfo.getClass( object ) + " ( " + msg + " )' object." );
	}

	static public void print( String[] printKinds, String msg ){

		if( print( printKinds ) ) JOptionPane.showMessageDialog( null, msg );
	}

	static public void stdPrint( String[] printKinds, String msg ){

		if( print( printKinds ) ) System.out.println( msg );
	}

	static public void print( String printKind, String msg ){

		if( toPrint( printKind ) ) JOptionPane.showMessageDialog( null, msg );
	}

	static public void stdPrintLn( String printKind, String msg ){

		if( toPrint( printKind ) ) System.out.println( msg );
	}

	static public void stdPrintLn( String msg ){

		System.out.println( msg );
	}

	static public void stdPrint( String printKind, String msg ){

		if( toPrint( printKind ) ) System.out.print( msg );
	}

	static public void stdPrint( String msg ){

		System.out.print( msg );
	}


	static public String resetLogFile( String printKind ){

		if( toPrint( printKind ) ){

			TxtWriter.write( "", defaultLogPrintFilePath );

			return defaultLogPrintFilePath;
		}


		return null;
	}

	static public String resetLogFile( String printKind, String logFilePath ){

		if( toPrint( printKind ) ){

			if( logFilePath != null ){

				String logFile = logFilePath + "\\" + defaultLogFileName;

				TxtWriter.write( "", logFile );

				System.out.println( "logFilePath = " + logFile );

				return logFile;
			}

			else{

				TxtWriter.write( "", defaultLogPrintFilePath );

				return defaultLogPrintFilePath;
			}
		}


		return null;
	}

	static public void appendLnInLogFile( String printKind, String msg ){

		if( toPrint( printKind ) ) TxtWriter.append( msg + "\n", defaultLogPrintFilePath );
	}

	static public void appendLnInLogFile( String printKind, String logFilePath, String msg ){

		if( toPrint( printKind ) ){

			if( logFilePath != null ) TxtWriter.append( msg + "\n", logFilePath + "\\" + defaultLogFileName );

			else TxtWriter.append( msg + "\n", defaultLogPrintFilePath );
		}
	}

	static public void appendInLogFile( String printKind, String msg ){

		if( toPrint( printKind ) ) TxtWriter.append( msg, defaultLogPrintFilePath );
	}

	static public void appendInLogFile( String printKind, String logFilePath, String msg ){

		if( toPrint( printKind ) ){

			if( logFilePath != null ) TxtWriter.append( msg, logFilePath + "\\" + defaultLogFileName );

			else TxtWriter.append( msg, defaultLogPrintFilePath );
		}
	}


	static public void openLogFile( String printKind, String logFilePath ){

		if( toPrint( printKind ) )

			if( logFilePath != null ){

				try{ Runtime.getRuntime().exec( new String[]{ "rundll32", "url.dll,FileProtocolHandler", logFilePath + "\\" + defaultLogFileName }); }

				catch( IOException ex ){}
			}

			else{

				try{ Runtime.getRuntime().exec( new String[]{ "rundll32", "url.dll,FileProtocolHandler", defaultLogPrintFilePath }); }

				catch( IOException ex ){}
			}
	}


	public static boolean print( String cPrintKinds[] ){

		for( int i = 0; cPrintKinds != null && i < cPrintKinds.length; ++i ) if( toPrint( cPrintKinds[ i ] ) ) return true;


		return false;
	}
}
