package org.ow2.choreos.gui.interfaceMappingsVisualizer;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Hashtable;
import java.util.List;

import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.SwingConstants;
import javax.swing.GroupLayout.Alignment;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.abstractionBase.AbstractionBaseController;
import org.ow2.choreos.abstractionBase.Hierarchy;
import org.ow2.choreos.abstractionDrivenServiceOrganization.ServiceOrganizationController;
import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.names.NamesDistanceController;
import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.serviceRepresentation.functional.service.DistanceCalculatorController;
import org.ow2.choreos.abstractionDrivenServiceOrganization.functionalOrganization.FunctionalOrganizationController;
import org.ow2.choreos.abstractionDrivenServiceOrganization.nonFunctionalOrganization.NonFunctionalOrganizationController;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.gui.UserSettings;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.serviceBase.ServiceBaseController;
import org.ow2.choreos.serviceRegistration.ServiceRegistration;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;
import org.ow2.choreos.userAuthentication.UserAuthenticationChecker;

public class ParameterizedServiceOrganizationVisualizer extends JFrame {


	private static final long serialVersionUID = -8995144219218422831L;

	private JPanel contentPane;
	private JSlider thresholdSlider;
	private JSlider depthSlider;
	private JLabel thresholdValueLabel;
	private JLabel depthValueLabel;
	
	private Hashtable<Integer, JLabel> depthLabelTable;
	
	private ServiceRegistration serviceRegistration;
	private AbstractionBaseController abstractionBase;
	private ServiceOrganizationController serviceOrganization;
	private UserSettings userSettings;	
	private JScrollPane treePane;
	private JTree tree;
	private JButton storeHierarchiesButton;

	public ParameterizedServiceOrganizationVisualizer(ServiceRegistration serviceRegistration,
			AbstractionBaseController abstractionBase,
			ServiceOrganizationController serviceOrganization,
			UserSettings userSettings) throws HeadlessException {
		this.serviceRegistration = serviceRegistration;
		this.abstractionBase = abstractionBase;
		this.serviceOrganization = serviceOrganization;
		this.userSettings = userSettings;
		
		this.userSettings.setAbstractionDepth("Message Types");
		this.userSettings.setSimilarityLintel(1.0);

		initComponents();
		
		setVisible( true );
	}

	private void initComponents() {
		setMinimumSize(new Dimension(800, 600));
		setTitle("TestFrame");
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setBounds(100, 100, 800, 600);
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		
		JPanel sliderPanel = new JPanel();
		sliderPanel.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		
		treePane = new JScrollPane();
		
		tree = null;

		GroupLayout gl_contentPane = new GroupLayout(contentPane);
		gl_contentPane.setHorizontalGroup(
			gl_contentPane.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_contentPane.createSequentialGroup()
					.addContainerGap()
					.addGroup(gl_contentPane.createParallelGroup(Alignment.LEADING)
						.addGroup(gl_contentPane.createSequentialGroup()
							.addComponent(sliderPanel, GroupLayout.PREFERRED_SIZE, 274, GroupLayout.PREFERRED_SIZE)
							.addPreferredGap(ComponentPlacement.RELATED)
							.addComponent(treePane, GroupLayout.DEFAULT_SIZE, 474, Short.MAX_VALUE))
						.addComponent(buttonPanel, GroupLayout.DEFAULT_SIZE, 754, Short.MAX_VALUE))
					.addContainerGap())
		);
		gl_contentPane.setVerticalGroup(
			gl_contentPane.createParallelGroup(Alignment.TRAILING)
				.addGroup(gl_contentPane.createSequentialGroup()
					.addContainerGap()
					.addGroup(gl_contentPane.createParallelGroup(Alignment.LEADING)
						.addComponent(treePane, GroupLayout.DEFAULT_SIZE, 486, Short.MAX_VALUE)
						.addComponent(sliderPanel, GroupLayout.DEFAULT_SIZE, 486, Short.MAX_VALUE))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addComponent(buttonPanel, GroupLayout.PREFERRED_SIZE, 38, GroupLayout.PREFERRED_SIZE)
					.addContainerGap())
		);

		
		thresholdValueLabel = new JLabel();
		thresholdValueLabel.setHorizontalAlignment(SwingConstants.CENTER);
		thresholdValueLabel.setToolTipText("Similarity threshold value");
		thresholdValueLabel.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		
		depthValueLabel = new JLabel();
		depthValueLabel.setHorizontalAlignment(SwingConstants.CENTER);
		depthValueLabel.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		depthValueLabel.setToolTipText("Abstraction depth value");
		
		thresholdSlider = new JSlider();
		thresholdSlider.setMinorTickSpacing(10);
		thresholdSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				thresholdSliderStateChanged(e);
			}
		});
		thresholdSlider.setPaintTicks(true);
		Hashtable<Integer, JLabel> thresholdLabelTable = new Hashtable<Integer,JLabel>();
		thresholdLabelTable.put(new Integer(100), new JLabel("Dissimilar"));
		thresholdLabelTable.put(new Integer(0), new JLabel("Identical"));
		thresholdSlider.setLabelTable(thresholdLabelTable);
		thresholdSlider.setPaintLabels(true);
		thresholdSlider.setValue(100);
		thresholdSlider.setOrientation(SwingConstants.VERTICAL);
		
		depthSlider = new JSlider();
		depthSlider.setMinorTickSpacing(1);
		depthSlider.setSnapToTicks(true);
		depthSlider.setMaximum(3);
		depthSlider.setValue(3);
		depthSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				depthSliderStateChanged(e);
			}
		});
		depthSlider.setPaintTicks(true);
		depthLabelTable = new Hashtable<Integer,JLabel>();
		depthLabelTable.put(new Integer(0), new JLabel("Interfaces"));
		depthLabelTable.put(new Integer(1), new JLabel("Operations"));
		depthLabelTable.put(new Integer(2), new JLabel("Messages"));
		depthLabelTable.put(new Integer(3), new JLabel("Message Types"));
		depthSlider.setLabelTable(depthLabelTable);
		depthSlider.setPaintLabels(true);
		depthSlider.setOrientation(SwingConstants.VERTICAL);
		
		depthValueLabel.setText(getAbstractionDepth());
		
		JLabel thresholdLabel = new JLabel("<html>Similarity<br />Threshold: </html>");
		
		JLabel algorithmdepthLabel = new JLabel("<html>Abstraction<br />Depth:</html>");
		GroupLayout gl_sliderPanel = new GroupLayout(sliderPanel);
		gl_sliderPanel.setHorizontalGroup(
			gl_sliderPanel.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_sliderPanel.createSequentialGroup()
					.addContainerGap()
					.addGroup(gl_sliderPanel.createParallelGroup(Alignment.TRAILING)
						.addComponent(thresholdLabel)
						.addGroup(Alignment.LEADING, gl_sliderPanel.createParallelGroup(Alignment.TRAILING, false)
							.addComponent(thresholdSlider, Alignment.LEADING, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
							.addComponent(thresholdValueLabel, Alignment.LEADING, GroupLayout.DEFAULT_SIZE, 68, Short.MAX_VALUE)))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addGroup(gl_sliderPanel.createParallelGroup(Alignment.TRAILING)
						.addComponent(depthValueLabel, Alignment.LEADING, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addComponent(algorithmdepthLabel, Alignment.LEADING, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addComponent(depthSlider, GroupLayout.DEFAULT_SIZE, 70, Short.MAX_VALUE))
					.addContainerGap())
		);
		gl_sliderPanel.setVerticalGroup(
			gl_sliderPanel.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_sliderPanel.createSequentialGroup()
					.addContainerGap()
					.addGroup(gl_sliderPanel.createParallelGroup(Alignment.TRAILING)
						.addComponent(thresholdLabel, GroupLayout.PREFERRED_SIZE, 30, GroupLayout.PREFERRED_SIZE)
						.addComponent(algorithmdepthLabel))
					.addPreferredGap(ComponentPlacement.UNRELATED)
					.addGroup(gl_sliderPanel.createParallelGroup(Alignment.LEADING)
						.addComponent(thresholdSlider, GroupLayout.DEFAULT_SIZE, 225, Short.MAX_VALUE)
						.addComponent(depthSlider, GroupLayout.DEFAULT_SIZE, 225, Short.MAX_VALUE))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addGroup(gl_sliderPanel.createParallelGroup(Alignment.LEADING, false)
						.addComponent(depthValueLabel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addComponent(thresholdValueLabel, GroupLayout.DEFAULT_SIZE, 31, Short.MAX_VALUE))
					.addContainerGap())
		);
		sliderPanel.setLayout(gl_sliderPanel);
		
		JButton zoomInButton = new JButton("Zoom In");
		zoomInButton.setEnabled(false);
		zoomInButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
					
			}
		});
		buttonPanel.add(zoomInButton);
		
		JButton zoomOutButton = new JButton("Zoom Out");
		zoomOutButton.setEnabled(false);
		zoomOutButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				
			}
		});
		buttonPanel.add(zoomOutButton);
		
		JButton editButton = new JButton("Edit");
		editButton.setEnabled(false);
		buttonPanel.add(editButton);
		
		JButton validateButton = new JButton("Validate");
		validateButton.setEnabled(false);
		buttonPanel.add(validateButton);
		
		JButton produceHierarchiesButton = new JButton("Produce Hierarchies");
		produceHierarchiesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTreeOfHierarchy(new DefaultMutableTreeNode());
				getScrollPanelOfHierarchy().setViewportView(getTreeOfHierarchy());
				produce( userSettings.getMode(), false ); 
			}
		});
		produceHierarchiesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				userSettings.setThreshold(1); 
			}
		});
		buttonPanel.add(produceHierarchiesButton);
		
		storeHierarchiesButton = new JButton("Store Hierarchies");
		storeHierarchiesButton.setEnabled(false);
		storeHierarchiesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				
				if( ! UserAuthenticationChecker.isCurator( userSettings.getUserCategory() ) ) PrintMessages.print( PrintMessages.guiPrint, "You do not have the priviledge to use the store functionality." );

				else ServiceBaseController.storeAbstractions( abstractionBase, "" );
			}
		});
		buttonPanel.add(storeHierarchiesButton);
		contentPane.setLayout(gl_contentPane);
	}

	protected void depthSliderStateChanged(ChangeEvent e) {
		if(!depthSlider.getValueIsAdjusting()){
			String value = ((JLabel)depthLabelTable.get(depthSlider.getValue())).getText();
			
			depthValueLabel.setText(value);
			setAbstractionDepth(value);
			System.out.println(getAbstractionDepth());
		}
	}

	protected void thresholdSliderStateChanged(ChangeEvent e) {
		float value = (float)thresholdSlider.getValue()/100;
		
		thresholdValueLabel.setText(value + "");
		setSimilarityLintel(value);
	}

	public double getSimilarityLintel() {
		return userSettings.getSimilarityLintel();
	}

	public void setSimilarityLintel(double similarityLintel) {
		this.userSettings.setSimilarityLintel(similarityLintel);
	}

	public String getAbstractionDepth() {
		return userSettings.getAbstractionDepth();
	}

	public void setAbstractionDepth(String abstractionDepth) {
		userSettings.setAbstractionDepth(abstractionDepth);
	}

	public JScrollPane getScrollPanelOfHierarchy() {
		return treePane;
	}

	public void setScrollPanelOfHierarchy(JScrollPane treePane) {
		this.treePane = treePane;
	}

	public JTree getTreeOfHierarchy() {
		return tree;
	}

	public void setTreeOfHierarchy(JTree tree) {
		this.tree = tree;
	}
	
	public void setTreeOfHierarchy( DefaultMutableTreeNode root ){
				
	   	if( root != null )
    		tree = new JTree( root );
    	else tree = null;
    }
	
    public JButton getButtonOfStoringHierarchy(){
    	return storeHierarchiesButton;
    }
	
	public Hierarchy produce( int mode, boolean waitProductionThreads ){

    	Hierarchy hierarchy = null;

    	if( mode == 3 ) return produceIndependentHierarchies( waitProductionThreads );


    	return hierarchy;
    }
    
	private Hierarchy produceIndependentHierarchies( boolean waitProductionThreads ) {

    	Hierarchy hierarchy = null;


//    	if( thread != null && thread.isAlive() ) thread.stop();

    	//if( options.getNamesDistanceCalculatorAlgorithm() == null || ( options.getNamesDistanceCalculatorAlgorithm() != null && options.getNamesDistanceCalculatorAlgorithm() == 0 ) ) JOptionPane.showMessageDialog(null, "You have to select the type of the naming distance metric!");

    	//else if( options.getSchemaRepresentationModel() == null ) JOptionPane.showMessageDialog(null, "You have to select whether the type of the messages distance!");

    	if( userSettings.getThreshold() == null && userSettings.existNonFuncDescriptions() ) JOptionPane.showMessageDialog(null, "You have to select the maximum number of instances contained in a non-functional abstraction!");

    	else{


    		if( userSettings.getDelay() == null ) userSettings.setDelay( false );

    		userSettings.setFunctionalOrganizationAlgorithm( FunctionalOrganizationController.getHierarchicalAlgorithm() );

			userSettings.setInterfacesDistanceCalculatorAlgorithm( DistanceCalculatorController.getHungarianAlgorithm() );

			userSettings.setNonFunctionalOrganizationAlgorithm( NonFunctionalOrganizationController.getKmeans() );

			userSettings.setSchemaRepresentationModel( XMLSchemaRepresentationController.getCompletelyUnstructuredSchemaRepresentation() );

			userSettings.setNamesDistanceCalculatorAlgorithm( NamesDistanceController.getEditDistanceCalculator() );


			serviceRegistration.transform( userSettings.getSchemaRepresentationModel() );


			if( ! serviceRegistration.hasTransformedCollections( userSettings.getSchemaRepresentationModel() ) ){

				String constraintViolation = serviceRegistration.getFinalConstraintViolation();

				JTextArea jTextArea = new JTextArea( constraintViolation );

				jTextArea.setForeground( Color.RED );

				treePane.setViewportView( new JScrollPane( jTextArea ) );
			}

			else{

				List<ServiceInterface> allInterfaces = serviceRegistration.getAllInterfaces( userSettings.getCollectionID() );

				PrintMessages.print( PrintMessages.organizationPrint, "The transformation of the message types of the services in this collection to the schema representation model '" + userSettings.getSchemaRepresentationModel() + "' has been completed.");


				//TestMemory.test();

				hierarchy = serviceOrganization.produce( waitProductionThreads, serviceRegistration, new AbstractionBaseController(serviceRegistration), allInterfaces, userSettings, this );


				//TestMemory.test();


				if( hierarchy != null && hierarchy.hasFinalConstraintViolation() ){

					String constraintViolation = hierarchy.getFinalConstraintViolation();

					JTextArea jTextArea = new JTextArea( constraintViolation );

					jTextArea.setForeground( Color.RED );

					treePane.setViewportView( new JScrollPane( jTextArea ) );
				}
			}
    	}


    	return hierarchy;
    }
}