package org.ow2.choreos.gui.visualComponentConnection;


import java.awt.Color;

/**
 * This class represents the line properties of a {@link VisualComponentConnection VisualComponentConnection}.
 * <ul>These properties are:
 * <li>The direction of the arrow(if any)<li>The line type<li>The color of the line</ul>
 */
public class VisualComponentConnectionLineProperties {

    /**An integer indicating the <code>simple line</code> property of a VisualComponentConnection*/
    public static final int CONNECT_LINE_TYPE_SIMPLE = 0;
    /**An integer indicating the <code>rectangular line</code> property of a VisualComponentConnection*/
    public static final int CONNECT_LINE_TYPE_RECTANGULAR = 1;

    /**An integer indicating at which endpoint of a {@code VisualComponentConnection} an Arrow may appear.
     * <ul>Valid Values are:
     * <li>ConnectLine.LINE_ARROW_NONE<li>ConnectLine.LINE_ARROW_SOURCE
     * <li>ConnectLine.LINE_ARROW_DEST<li>ConnectLine.LINE_ARROW_BOTH</ul>
     *Default value is set to {@code ConnectLine.LINE_ARROW_NONE}*/
    protected int lineArrow = ConnectLine.LINE_ARROW_NONE;

    /**An integer indicating the {@code line type} property of a {@code VisualComponentConnection}.
     * <ul>Valid Values are: <li>CONNECT_LINE_TYPE_SIMPLE<li>CONNECT_LINE_TYPE_RECTANGULAR</ul>
     *Default value is set to {@code CONNECT_LINE_TYPE_RECTANGULAR}*/
    protected int lineType = CONNECT_LINE_TYPE_RECTANGULAR;

    /**An instance of {@code Color} indicating the {@code lineColor} property of a {@code VisualComponentConnection}*/
    protected Color lineColor;

    /**
     * Constructs an instance of {@code VisualComponentConnectionLineProperties} class with the given properties
     * @param lineArrow An integer indicating the line arrow property
     * @param lineType An integer indicating the line type property
     * @param lineColor An instance of {@link Color Color} indicating the line color property
     */
    public VisualComponentConnectionLineProperties(int lineArrow, int lineType, Color lineColor) {
        this.lineArrow = lineArrow;
        this.lineType = lineType;
        this.lineColor = lineColor;
    }

    /*Getters & setters*/
    public int getLineArrow() {
        return lineArrow;
    }

    public void setLineArrow(int lineArrow) {
        this.lineArrow = lineArrow;
    }

    public Color getLineColor() {
        return lineColor;
    }

    public void setLineColor(Color lineColor) {
        this.lineColor = lineColor;
    }

    public int getLineType() {
        return lineType;
    }

    public void setLineType(int lineType) {
        this.lineType = lineType;
    }
}
