package org.ow2.choreos.gui.visualComponentConnection;


/**
 *This class represents the visibility properties of a {@link VisualComponentConnection VisualComponentConnection}.
 * <ul>These properties are:
 * <li>The visibility of the Source<li>The visibility of the Destination
 * <li>The collapsion of the Source<li>The collapsion of the Destination</ul>
 */
public class VisualComponentConnectionVisibilityProperties {

    private boolean visibleSource;
    private boolean visibleDest;
    private boolean collapsedSource;
    private boolean collapsedDest;

    /**
     * Default Constructor. Constructs an instance of {@code VisualComponentConnectionVisibilityProperties}
     * with <strong>all</strong> fields set to true*/
    public VisualComponentConnectionVisibilityProperties(){
        this.visibleSource = true;
        this.visibleDest = true;
        this.collapsedSource = true;
        this.collapsedDest = true;
    }

    /**
     * Constructs an instance of {@code VisualComponentConnectionVisibilityProperties}
     * with given parameters
     * @param visibleSource A boolean indicating the visibility of the source
     * @param visibleDest A boolean indicating the visibility of the destination
     * @param collapsedSource A boolean indicating the collapsion of the source
     * @param collapsedDest A boolean indicating the collapsion of the destination
     */
    public VisualComponentConnectionVisibilityProperties(boolean visibleSource, boolean visibleDest,
            boolean collapsedSource, boolean collapsedDest) {
        this.visibleSource = visibleSource;
        this.visibleDest = visibleDest;
        this.collapsedSource = collapsedSource;
        this.collapsedDest = collapsedDest;
    }

    /*Getters & Setters*/
    public boolean isCollapsedDest() {
        return collapsedDest;
    }

    public void setCollapsedDest(boolean collapsedDest) {
        this.collapsedDest = collapsedDest;
    }

    public boolean isCollapsedSource() {
        return collapsedSource;
    }

    public void setCollapsedSource(boolean collapsedSource) {
        this.collapsedSource = collapsedSource;
    }

    public boolean isVisibleDest() {
        return visibleDest;
    }

    public void setVisibleDest(boolean visibleDest) {
        this.visibleDest = visibleDest;
    }

    public boolean isVisibleSource() {
        return visibleSource;
    }

    public void setVisibleSource(boolean visibleSource) {
        this.visibleSource = visibleSource;
    }

    /**@return {@code true} if <strong>any</strong> of source or destination are collapsed, {@code false} otherwise*/
    public boolean isCollapsed(){
        return (isCollapsedSource() || isCollapsedDest());
    }

    /**Sets the collapsion of <strong>both</strong> source and destination to the
     * value indicated by {@code collapsed}
     * @param collapsed A boolean
     */
    public void setCollapsed(boolean collapsed){
        setCollapsedSource(collapsed);
        setCollapsedDest(collapsed);
    }

    /**@return {@code true} if <strong>both</strong> of source or destination are visible, {@code false} otherwise*/
    public boolean isVisible(){
        return (isVisibleSource() && isVisibleDest());
    }

    /**Sets the visibility of <strong>both</strong> source and destination to the
     * value indicated by {@code visible}
     * @param visible A boolean
     */
    public void setVisible(boolean visible){
        setVisibleSource(visible);
        setVisibleDest(visible);
    }
}
