package org.ow2.choreos.matchingProcess.schemaRepresentation.concreteElementIndexing;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;

import org.ow2.choreos.dataStructures.list.listInsertion.ListInsertion;
import org.ow2.choreos.dataStructures.map.mapInsertion.MapInsertion;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.schemaRepresentation.KindDescr;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;


/**
 * 
 * @author dathanas
 *
 */
public class AbstractElement {


	//Variables.
	private String prefix = "Abstract element";

	private int abstractElementID;


	private List<Object> rules;

	private Map<Object, Object> kindDescrs;//The first 'Object' is the string description of the 'KindDescr' object while the second is the 'KindDescr' object.

	private Boolean canBeRoot;


	private List<Object> orderingOfRefs2DocOfSchemas;//This is the ordering of the schemas (user-provided) to which the concrete elements of the abstract element belong.

	private Map<Object, Object> concreteElements;//The first 'Object' is the 'ref2DocOfSchemas' while the second is the data structure that keeps the schema elements (list or interval tree)

	private String concreteElementsDataStructureKind = ConcreteElementsDataStructureController.list;//default.


	private int totalSchemaElementsNumber = 0;


	//Constructors.
	protected AbstractElement( int ID, Object cKindDescr, String cSchemaElementsDataStructureKind ){//It is used in case an 'AbstractElement' object consists of only an 'kindDescr'.

		abstractElementID = ID;

		concreteElementsDataStructureKind = cSchemaElementsDataStructureKind;


		addKindDescr( cKindDescr );
	}

	protected AbstractElement( int ID, Object[] cKindDescrs, String cSchemaElementsDataStructureKind ){

		abstractElementID = ID;

		concreteElementsDataStructureKind = cSchemaElementsDataStructureKind;


		addKindDescrs( cKindDescrs );
	}

	public AbstractElement( int ID, String cSchemaElementsDataStructureKind ){//It is used by the 'Pattern' class. It is the intermediate object for querying pattern-growth abstract elements the indexing data structure. Note that it uses the default assignment for the data structure object.

		abstractElementID = ID;

		concreteElementsDataStructureKind = cSchemaElementsDataStructureKind;
	}


	//Setters.
	protected void addRule( Object cRule ){

		if( SchemaRepresentationController.isRule( cRule ) ) rules = ListInsertion.add( cRule, rules );
	}

	protected void addKindDescr( Object cKindDescr ){

		if( SchemaRepresentationController.isKindDescr( cKindDescr ) ) kindDescrs = MapInsertion.put( ((KindDescr) cKindDescr).getKindDescrToString(), cKindDescr, kindDescrs, false );


		boolean tmpCanBeRoot = SchemaRepresentationController.canBeRootKindDescr( cKindDescr );

		if( canBeRoot != null && canBeRoot != tmpCanBeRoot ) throw new RuntimeException( getKindDescrsToString() + " two kind descriptions cannot be concurrently root elements." );

		canBeRoot = tmpCanBeRoot;
	}

	protected void addKindDescrs( Object[] cKindDescrs ){

		for( int i = 0; cKindDescrs != null && i < cKindDescrs.length; ++i ) addKindDescr( cKindDescrs[ i ] );
	}


	public void addConcreteElement( String ref2DocOfSchema, Object currentConcreteElementInASchema ){

		++totalSchemaElementsNumber;


		Object existingDataStructure = MapGetter.getValue( ref2DocOfSchema, concreteElements );


		if( existingDataStructure == null ) orderingOfRefs2DocOfSchemas = ListInsertion.add( ref2DocOfSchema, orderingOfRefs2DocOfSchemas );


		existingDataStructure = ConcreteElementsDataStructureController.add( concreteElementsDataStructureKind, existingDataStructure, currentConcreteElementInASchema );


		concreteElements = MapInsertion.put( ref2DocOfSchema, existingDataStructure, concreteElements, true );
	}


	//Getters.
	String getLabel(){

		return prefix + " " + abstractElementID;
	}

	public int getAbstractElementID(){

		return abstractElementID;
	}

	List<Object> getRules(){

		return rules;
	}

	Map<Object, Object> getKindDescrs(){

		return kindDescrs;
	}

	public List<Object> getOrderingOfRefs2DocOfSchemas(){

		return orderingOfRefs2DocOfSchemas;
	}

	public Map<Object, Object> getConcreteElements(){

		return concreteElements;
	}

	public String getSchemaElementsDataStructureKind(){

		return concreteElementsDataStructureKind;
	}

	int getFrequency(){

		return concreteElements != null ? concreteElements.size() : 0;
	}

	public int getSchemaElementNumber(){

		return totalSchemaElementsNumber;
	}

	public String getKindDescrsToString(){

		String allKindDescrs = "";

		Iterator<Object> i = kindDescrs != null && kindDescrs.keySet() != null ? kindDescrs.keySet().iterator() : null;

		while( i != null && i.hasNext() ){

			String kindDescrToString = (String) i.next();

			allKindDescrs += " (" + kindDescrToString + ")";
		}


		String retValue = getLabel() + " : " + allKindDescrs + " (size = " + totalSchemaElementsNumber + ")";


		return canBeRoot != null ? retValue + " - canBeRoot = " + canBeRoot : retValue;
	}

	protected boolean isFrequentAbstractElement( int minFrequency ){

		return getFrequency() >= minFrequency;
	}

	protected boolean canBeRoot(){

		return canBeRoot;
	}


	//Visualization.
	public Object visualize( String logFilePath ){

		boolean filePrint = logFilePath != null ? true : false;

		DefaultMutableTreeNode root = null;


		if( filePrint ){

			if( getKindDescrsToString() != null ) PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t" + getKindDescrsToString() );
		}

		else root = new DefaultMutableTreeNode( getKindDescrsToString() );


		for( int i = 0; orderingOfRefs2DocOfSchemas != null && i < orderingOfRefs2DocOfSchemas.size(); ++i ){

			Object ref2DocOfSchemas = orderingOfRefs2DocOfSchemas.get( i );

			DefaultMutableTreeNode ref2DocOfSchemasNode = null;


			if( filePrint ) PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, "\t\t\t" + ref2DocOfSchemas );

			else{

				ref2DocOfSchemasNode = new DefaultMutableTreeNode( ref2DocOfSchemas );

				root.add( ref2DocOfSchemasNode );
			}


			Object existingDataStructure = MapGetter.getValue( ref2DocOfSchemas, concreteElements );


			List<Object> concreteElementNodes = ConcreteElementsDataStructureController.visualize( concreteElementsDataStructureKind, existingDataStructure, logFilePath );


			for( int j = 0; ! filePrint && concreteElementNodes != null && j < concreteElementNodes.size(); ++j ) ref2DocOfSchemasNode.add( (MutableTreeNode) concreteElementNodes.get( j ) );
		}


    	return root;
    }
}
