package org.ow2.choreos.matchingProcess.schemaRepresentation.concreteElementIndexing;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;

import org.omg.CORBA.IntHolder;
import org.ow2.choreos.dataStructures.array.arrayInstantiation.ArrayInstantiation;
import org.ow2.choreos.dataStructures.array.arrayManipulation.ArraySorting;
import org.ow2.choreos.dataStructures.interval.Interval1D;
import org.ow2.choreos.dataStructures.list.listInsertion.ListInsertion;
import org.ow2.choreos.dataStructures.list.listRemoval.ListRemoval;
import org.ow2.choreos.dataStructures.map.mapInsertion.MapInsertion;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapRemoval.MapRemoval;
import org.ow2.choreos.gui.UserSettings;
import org.ow2.choreos.gui.guiManipulation.JFrameManipulation;
import org.ow2.choreos.schemaRepresentation.KindDescr;
import org.ow2.choreos.schemaRepresentation.Rule;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;


/**
 * 
 * @author dathanas
 * 
 */
public class ConcreteElementsIndex {


	//Variables.
	private long duration = 0;


	private int totalSchemaElementsNumber = 0, lastAbstractElementID = 0;


	Map<Object, Object> schemaSizes;//The 1st object is the 'ref2DocOfSchemas' and the 2nsd object is the size of the schema, not the original but the one stored in the index (some elements have been discarded, e.g., annotation). 

	private Map<Object, Object> abstractElementKindDescrs;//The 1st 'Object' is the string description of a 'KindDescr' object while the 2nd 'Object' is an 'AbstractElement' object. This 'Map' is used for searching for a 'KindDescr' in a constant time: function 'lookUp( kindDescr )' in journal paper.

	protected Map<Object, Object> abstractElements;//The 1st 'Object' is the id (int) of a 'AbstractElement' object while the 2nd 'Object' is the 'AbstractElement' object. This 'Map' is used in conjunction with the 'index' data structure.


	//Constructors.
	public ConcreteElementsIndex( UserSettings userSettings ){

		initializeIndex( userSettings, null, null );
	}

	public ConcreteElementsIndex( UserSettings userSettings, Object guiLabel, Object guiProgressBar ){

		initializeIndex( userSettings, guiLabel, guiProgressBar );
	}

	private void initializeIndex( UserSettings userSettings, Object guiLabel, Object guiProgressBar ){

		String prevGUILabel = JFrameManipulation.getLabel( guiLabel );

		JFrameManipulation.appendLabel( guiLabel, " (" + getLabelOfConstruction() + ")" );

		JFrameManipulation.resetProgressBarLabel( guiProgressBar );


		long startTime = System.currentTimeMillis();


		Object[] rules = userSettings.getSimilarityRulesForElementKinds();

		for( int i = 0; rules != null && i < rules.length; ++i ){

			if( SchemaRepresentationController.isRule( rules[ i ] ) ){

				Rule rule = (Rule) rules[ i ];

				AbstractElement abstractElement = addKindDescrs( rule.getKindDescrs(), userSettings.getSchemaElementsDataStructureKind() );

				abstractElement.addRule( rule );
			}
		}


		populateAbstractElements( userSettings.getDocsOfSchema(), userSettings.getTargetSchemaRepresentationModel(), userSettings.getSchemaElementsDataStructureKind(), guiProgressBar );


		duration = System.currentTimeMillis() - startTime;


		JFrameManipulation.setLabel( guiLabel, prevGUILabel );
	}

	private AbstractElement addKindDescrs( KindDescr[] cKindDescrs, String concreteElementsDataStructureKind ){

		KindDescr foundKindDescr = null;

		for( int i = 0; cKindDescrs != null && i < cKindDescrs.length; ++i ) if( MapGetter.exists( cKindDescrs[ i ].getKindDescrToString(), abstractElementKindDescrs ) ) foundKindDescr = cKindDescrs[ i ];


		return getAbstractElement( foundKindDescr, cKindDescrs, concreteElementsDataStructureKind );
	}

	private AbstractElement getAbstractElement( KindDescr foundKindDescr, KindDescr[] cKindDescrs, String concreteElementsDataStructureKind ){

		AbstractElement abstractElement = foundKindDescr != null ? addKindDescrs( foundKindDescr, cKindDescrs ) : createAbstractElement( cKindDescrs, concreteElementsDataStructureKind );


		for( int i = 0; cKindDescrs != null && i < cKindDescrs.length; ++i ) abstractElementKindDescrs = MapInsertion.put( cKindDescrs[ i ].getKindDescrToString(), abstractElement, abstractElementKindDescrs, false );


		return abstractElement;
	}

	private AbstractElement addKindDescrs( KindDescr foundKindDescr, KindDescr[] cKindDescrs ){

		AbstractElement abstractElement = (AbstractElement) MapGetter.getValue( foundKindDescr.getKindDescrToString(), abstractElementKindDescrs );

		abstractElement.addKindDescrs( cKindDescrs );


		return abstractElement;
	}

	private AbstractElement createAbstractElement( Object[] cKindDescrs, String concreteElementsDataStructureKind ){

		AbstractElement abstractElement = new AbstractElement( ++lastAbstractElementID, cKindDescrs, concreteElementsDataStructureKind );

		abstractElements = MapInsertion.put( abstractElement.getAbstractElementID(), abstractElement, abstractElements, false );


		return abstractElement;
	}


	private void populateAbstractElements( Object[] docsOfSchema, String schemaRepresentationModel, String concreteElementsDataStructureKind, Object guiProgressBar ){

		IntHolder concreteElementsNumber = new IntHolder( 0 );


		for( int i = 0 ; docsOfSchema != null && i < docsOfSchema.length; ++i ){

			String ref2DocOfSchemas = SchemaRepresentationController.getRef2DocOfSchemas( docsOfSchema[ i ] ); JFrameManipulation.setProgressBarLabel( guiProgressBar, i + 1, docsOfSchema.length );


			if( ref2DocOfSchemas != null ){

				concreteElementsNumber.value = 0;

				traverseDFS( new IntHolder( -1 ), docsOfSchema[ i ], null, ref2DocOfSchemas, schemaRepresentationModel, concreteElementsDataStructureKind, concreteElementsNumber, guiProgressBar );

				schemaSizes = MapInsertion.put( ref2DocOfSchemas, concreteElementsNumber.value, schemaSizes, false ); System.out.println( "schema '" + ref2DocOfSchemas + "' has size = " + concreteElementsNumber.value );
			}
		}
	}

	private int traverseDFS( IntHolder globalDFSID, Object indexedObject, Object parentOfIndexedObject, String ref2DocOfSchemas, String schemaRepresentationModel, String concreteElementsDataStructureKind, IntHolder concreteElementsNumberPerDocOfSchemas, Object guiProgressBar ){

		int myRightmostChildDFSID = -1;

		KindDescr kindDescr = SchemaRepresentationController.getKindDescr( indexedObject );

		int myDFSID = ! SchemaRepresentationController.isSchemaTagName( ((KindDescr) kindDescr).getKind() ) ? ++globalDFSID.value : 0;


		List<?> children = SchemaRepresentationController.getChildrenWithoutAnnotation( indexedObject, schemaRepresentationModel );


		if( children == null || ( children != null && children.isEmpty() ) ) myRightmostChildDFSID = myDFSID;

		else{

			int[] keys = ArrayInstantiation.newIntArray( children.size() );

			Map<Object, List<Object>> map = null;


			for( int i = 0; i < children.size(); ++i ){

				kindDescr = SchemaRepresentationController.getKindDescr( children.get( i ) );

				AbstractElement abstractElement = addKindDescrs( new KindDescr[] { kindDescr }, concreteElementsDataStructureKind );

				keys[ i ] = abstractElement.getAbstractElementID();

				map = MapInsertion.putInMapOfList( keys[ i ], i, map );//I keep the keys and their positions 'i' (the positions may be more than one).
			}


			ArraySorting.sortIntegers( keys );


			for( int i = 0; i < keys.length; ++i ) {

				List<Object> positions = MapGetter.getValueOfMapOfList( keys[ i ], map );

				Object child = children.get( (Integer) positions.get( 0 ) );


				ListRemoval.remove( 0, positions );

				myRightmostChildDFSID = traverseDFS( globalDFSID, child, indexedObject, ref2DocOfSchemas, schemaRepresentationModel, concreteElementsDataStructureKind, concreteElementsNumberPerDocOfSchemas, guiProgressBar );
			}
		}


		kindDescr = SchemaRepresentationController.getKindDescr( indexedObject );

		if( ! SchemaRepresentationController.isSchemaTagName( ((KindDescr) kindDescr).getKind() ) ){

			AbstractElement abstractElement = addKindDescrs( new KindDescr[] { kindDescr }, concreteElementsDataStructureKind );


			Interval1D scope = new Interval1D( myDFSID, myRightmostChildDFSID );


			SchemaRepresentationController.setScope( indexedObject, scope );

			SchemaRepresentationController.setParent( indexedObject, parentOfIndexedObject );


			abstractElement.addConcreteElement( ref2DocOfSchemas, indexedObject );


			++totalSchemaElementsNumber;

			++concreteElementsNumberPerDocOfSchemas.value;
		}


		return myRightmostChildDFSID;
	}


	//Setters.
	public Map<Object, Object> lookUpFrequentAbstractElements( int minFrequency, boolean root, Object guiLabel, Object guiProgressBar ){

		String prevGUILabel = JFrameManipulation.getLabel( guiLabel );

		JFrameManipulation.appendLabel( guiLabel, " (" + getLabelOfIdentifyingFrequentElements() + ")" );

		JFrameManipulation.resetProgressBarLabel( guiProgressBar );


		long startTime = System.currentTimeMillis();


		Map<Object, Object> frequentAbstractElements = null;

		List<Object> nonFrequentAbstractElements = null;


		int counter = 0;

		Iterator<Object> iterator = abstractElements != null && abstractElements.keySet() != null ? abstractElements.keySet().iterator() : null;

		while( iterator != null && iterator.hasNext() ){

			int abstractElementNumber = (Integer) iterator.next();

			AbstractElement abstractElement = (AbstractElement) abstractElements.get( abstractElementNumber );

			if( ! abstractElement.isFrequentAbstractElement( minFrequency ) ) nonFrequentAbstractElements = ListInsertion.add( abstractElement, nonFrequentAbstractElements );

			else if( ( root && abstractElement.canBeRoot() ) || ! root ) frequentAbstractElements = MapInsertion.put( abstractElement.getAbstractElementID(), abstractElement, frequentAbstractElements , false );


			JFrameManipulation.setProgressBarLabel( guiProgressBar, ++counter, abstractElements.size() );
		}


		for( int i = 0; nonFrequentAbstractElements != null && i < nonFrequentAbstractElements.size(); ++i ) removeAbstractElement( (AbstractElement) nonFrequentAbstractElements.get( i ) );


		duration = System.currentTimeMillis() - startTime;


		JFrameManipulation.setLabel( guiLabel, prevGUILabel );


		return frequentAbstractElements;
	}

	private void removeAbstractElement( AbstractElement abstractElement ){

		int abstractElementNumber = abstractElement.getAbstractElementID();

		totalSchemaElementsNumber -= abstractElement.getSchemaElementNumber();

		MapRemoval.remove( abstractElementNumber, abstractElements );


		Map<Object, Object> kindDescrs = abstractElement.getKindDescrs();

		Iterator<Object> iterator = kindDescrs != null && kindDescrs.keySet() != null ? kindDescrs.keySet().iterator() : null;

		while( iterator != null && iterator.hasNext() ){

			Object kindDescr = iterator.next();

			MapRemoval.remove( kindDescr, abstractElementKindDescrs );
		}
	}


	//Getters.
	public Map<Object, Object> getAbstractElements(){

		return abstractElements;
	}

	public Object lookUpAbstractElement( int abstractElementID ){

		return abstractElements != null ? MapGetter.getValue( abstractElementID, abstractElements ) : null;
	}

	public int getNumberOfAbstractElements(){

		return abstractElements != null ? abstractElements.size() : 0;
	}

	public int getTotalNumberOfSchemaElements(){

		return totalSchemaElementsNumber;
	}

	public Map<Object, Object> getSchemaSizes(){

		return schemaSizes;
	}

	public long getDuration(){

		return duration;
	}

	//Getters-Verifiers.
	public boolean isComponentDeclaration( int abstractElementID ){

		AbstractElement abstractElement = (AbstractElement) lookUpAbstractElement( abstractElementID );


		return false;
	}

	//Getters-Visualization.
	static public String getLabelOfConstruction(){

		return "index construction";
	}

	static public String getLabelOfDuration(){

		return "Indexing duration";
	}

	static public String getLabelOfIdentifyingFrequentElements(){

		return "Identifying frequent elements";
	}

	static public String getLabelOfIdentifyingFrequentElementsDuration(){

		return "Identifying frequent elements duration";
	}


	//Visualization.
	static public String getLabelOfVisualization(){

		return "index visualization";
	}

	static public String getVisualizationRootLabelForAllElements(){

		return "Data structure for indexing schema elements";
	}

	static public String getVisualizationRootLabelForFrequentElements(){

		return "Data structure for indexing frequent schema elements";
	}

	public Object visualize( String rootLabel, String logFilePath ){

    	return visualize( null, null, rootLabel, logFilePath );
    }

	public Object visualize( Object guiLabel, Object guiProgressBar, String rootLabel, String logFilePath ){

		String prevGUILabel = JFrameManipulation.getLabel( guiLabel );

		JFrameManipulation.appendLabel( guiLabel, " (" + getLabelOfVisualization() + ")" );

		JFrameManipulation.resetProgressBarLabel( guiProgressBar );


		boolean logFile = logFilePath != null ? true : false;

		DefaultMutableTreeNode root = null;


		if( logFile ) System.out.println( rootLabel );

		else root = new DefaultMutableTreeNode( rootLabel );


		int counter = 0;

		Iterator<Object> i = abstractElements != null && abstractElements.keySet() != null ? abstractElements.keySet().iterator() : null;

		while( i != null && i.hasNext() ){

			int abstractElementNumber = (Integer) i.next();

			AbstractElement abstractElement = ((AbstractElement) abstractElements.get( abstractElementNumber ) );


			Object retValue = abstractElement.visualize( logFilePath );

			if( ! logFile ) root.add( (MutableTreeNode) retValue );


			JFrameManipulation.setProgressBarLabel( guiProgressBar, ++counter, abstractElements.size() );
		}


		JFrameManipulation.setLabel( guiLabel, prevGUILabel );


    	return root;
    }
}