package org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching.pattern;

import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.interval.Interval1D;
import org.ow2.choreos.dataStructures.list.listInsertion.ListInsertion;
import org.ow2.choreos.dataStructures.map.mapInsertion.MapInsertion;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;


/**
 * 
 * @author dathanas
 *
 */
public class PatternAbstractElement {


	//Variables.
	private int ID;

	private Interval1D scope;

	private List<Object> children;

	private Map<Object, List<Object>> nonLeafSuccessiveChildren, leafChildrenInGroupsOfSuccessiveSiblingOfSimilarKinds;


	//Constructors.
	protected PatternAbstractElement( int cID ){

		ID = cID;

		scope = new Interval1D( 0, 0 );
	}

	protected PatternAbstractElement( int cID, int leftEndpoint, int rightEndpoint ){

		ID = cID;

		scope = new Interval1D( leftEndpoint, rightEndpoint );
	}

	protected PatternAbstractElement( PatternAbstractElement patternAbstractElement ){//Copy constructor: it is used by the class 'Pattern' for creating a deep copy of a pattern.

		if( patternAbstractElement != null ){

			ID = patternAbstractElement.ID;

			scope = new Interval1D( patternAbstractElement.scope.getIntLeftEndpoint(), patternAbstractElement.scope.getIntRightEndpoint() );


			for( int i = 0; patternAbstractElement.children != null && i < patternAbstractElement.children.size(); ++i ){

				PatternAbstractElement copyChildPatternAbstractElement = new PatternAbstractElement( (PatternAbstractElement) patternAbstractElement.children.get( i ) );

				children = ListInsertion.add( copyChildPatternAbstractElement, children );
			}
		}

		else throw new RuntimeException( "PatternAbstractElement: Illegal (null) copy pattern abstract element." );
	}


	//Setters.
	void addChild( PatternAbstractElement patternAbstractElement ){

		increaseRightEndpoint();

		children = ListInsertion.add( patternAbstractElement, children );
	}

	void increaseRightEndpoint(){

		scope.setRightEndpoint( scope.getRightEndpoint() + 1 );
	}


	//Getters.
	List<Object> getChildren(){

		return children;
	}

	Map<Object, List<Object>> getLeafChildrenInGroupsOfSuccessiveSiblingOfSimilarKinds(){

		if( children != null && leafChildrenInGroupsOfSuccessiveSiblingOfSimilarKinds == null && nonLeafSuccessiveChildren == null ) divideChildrenInGroups();


		return leafChildrenInGroupsOfSuccessiveSiblingOfSimilarKinds;		
	}

	Map<Object, List<Object>> getNonLeafSuccessiveChildren(){

		if( children != null && leafChildrenInGroupsOfSuccessiveSiblingOfSimilarKinds == null && nonLeafSuccessiveChildren == null ) divideChildrenInGroups();


		return nonLeafSuccessiveChildren;
	}

	private void divideChildrenInGroups(){

		int groupNumber = -1;


		for( int i = 0; children != null && i < children.size(); ++i ){

			boolean isCurrentGroupOfNonLeafChildren = MapGetter.existsInMapOfList( groupNumber, nonLeafSuccessiveChildren );


			PatternAbstractElement child = (PatternAbstractElement) children.get( i );

			if( child.children != null ){

				if( isCurrentGroupOfNonLeafChildren ){

					List<Object> currentGroupOfNonLeafChildren = MapGetter.getValueOfMapOfList( groupNumber, nonLeafSuccessiveChildren );

					currentGroupOfNonLeafChildren = ListInsertion.add( child, currentGroupOfNonLeafChildren );

					nonLeafSuccessiveChildren = MapInsertion.putInMapOfList( groupNumber, currentGroupOfNonLeafChildren, nonLeafSuccessiveChildren, true );
				}

				else{

					List<Object> currentGroupOfNonLeafChildren = ListInsertion.add( child, null );

					nonLeafSuccessiveChildren = MapInsertion.putInMapOfList( ++groupNumber, currentGroupOfNonLeafChildren, nonLeafSuccessiveChildren, true );
				}
			}

			else{

				if( isCurrentGroupOfNonLeafChildren ){

					List<Object> currentGroupOfLeafChildren = ListInsertion.add( child, null );

					leafChildrenInGroupsOfSuccessiveSiblingOfSimilarKinds = MapInsertion.putInMapOfList( ++groupNumber, currentGroupOfLeafChildren, leafChildrenInGroupsOfSuccessiveSiblingOfSimilarKinds, true );
				}

				else{

					List<Object> currentGroupOfLeafChildren = MapGetter.getValueOfMapOfList( groupNumber, leafChildrenInGroupsOfSuccessiveSiblingOfSimilarKinds );

					int commonPatternabstractElementID = currentGroupOfLeafChildren != null ? ((PatternAbstractElement) currentGroupOfLeafChildren.get( 0 )).ID : groupNumber;

					if( child.ID == commonPatternabstractElementID ){

						currentGroupOfLeafChildren = ListInsertion.add( child, currentGroupOfLeafChildren );

						leafChildrenInGroupsOfSuccessiveSiblingOfSimilarKinds = MapInsertion.putInMapOfList( groupNumber, currentGroupOfLeafChildren, leafChildrenInGroupsOfSuccessiveSiblingOfSimilarKinds, true );
					}

					else{

						List<Object> newCurrentGroupOfLeafChildren = ListInsertion.add( child, null );

						leafChildrenInGroupsOfSuccessiveSiblingOfSimilarKinds = MapInsertion.putInMapOfList( ++groupNumber, newCurrentGroupOfLeafChildren, leafChildrenInGroupsOfSuccessiveSiblingOfSimilarKinds, true );
					}
				}
			}
		}
	}

	PatternAbstractElement getRightmostChild(){

		return children != null ? (PatternAbstractElement) children.get( children.size() - 1 ) : null;
	}

	public int getPatternAbstractElementID(){

		return ID;
	}

	public Interval1D getScope(){

		return scope;
	}


	//Visualization.
	protected DefaultMutableTreeNode visualize( String logFilePath ){

		boolean logFile = logFilePath != null ? true : false;

		DefaultMutableTreeNode root = null;


		String msg = "\t\tPattern abstract element " + ID + " (scope = " + scope.getIntEndpoints() + ")";

		if( logFile ) PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, logFilePath, msg );

		else root = new DefaultMutableTreeNode( msg );


		for( int i = 0; children != null && i < children.size(); ++i ){

			if( logFile ) ((PatternAbstractElement) children.get( i )).visualize( logFilePath );

			else root.add( ((PatternAbstractElement) children.get( i )).visualize( logFilePath ) );
		}


		return root;
    }
}
