package org.ow2.choreos.matchingProcess.schemaRepresentation.patternBasedMatching.pattern;

import java.util.Iterator;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;

import org.ow2.choreos.dataStructures.interval.Interval1D;
import org.ow2.choreos.dataStructures.map.mapInsertion.MapInsertion;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;


/**
 * 
 * @author dathanas
 *
 */
class PatternInstantiation {


	//Variables.
	private int patternInstantiationID;

	private double density;


	private Object rootConcreteElement, rightmostConcreteElement;


	private Map<Object, Object> mapOfConcreteElements;//Auxiliary map1: this map stores all the the schema elements of the pattern-instantiation tree. This 1st object is the schema element and the 2nd is a boolean object (artificial) whose value is always true.
													  //It helps the checking of the existence of a schema element in constant time.

	private Map<Object, Object> mapOfConcreteElementScopes;//Auxiliary map2: this map stores all the schema elements of the pattern-instantiation tree. This 1st object is the left endpoint of the schema element and the 2nd is the schema element.
														   //It helps the retrieval of a schema element based on the left endpoint of its scope in constant time.

	private Map<Object, Object> mapOfRightmostChildScopeRightEndpoints;//Auxiliary map3: this map stores for each schema element (node) of the pattern-instantiation tree, the right endpoint of the scope of its rightmost child in the original schema tree. Note that this map has two levels artificially. The first object is the schema element and the second level corresponds to a pair <concreteElement, rightEndpoint> of the same schema element.
	 													  			   //It helps the retrieval of the scope of the rightmost child of a schema element constant time while constructing a querying abstract element.


	//Constructors.
	protected PatternInstantiation( int cPatternInstantiationID, int concreteElementScopeLeftEndpoint, Object concreteElement ){//It is used for initializing singleton patterns.

		patternInstantiationID = cPatternInstantiationID;

		rootConcreteElement = concreteElement;


		add( concreteElementScopeLeftEndpoint, concreteElement, null );
	}

	protected PatternInstantiation( int cPatternInstantiationID, PatternInstantiation patternInstantiation ){//Copy constructor: it is used by the class 'Pattern' for creating a deep copy of a pattern.

		if( patternInstantiation != null ){

			patternInstantiationID = cPatternInstantiationID;

			rootConcreteElement = patternInstantiation.rootConcreteElement;

			rightmostConcreteElement = patternInstantiation.rightmostConcreteElement;


			add( patternInstantiation.mapOfConcreteElements, patternInstantiation.mapOfConcreteElementScopes, patternInstantiation.mapOfRightmostChildScopeRightEndpoints );
		}

		else throw new RuntimeException( "PatternInstantiation: Illegal (null) copy pattern instantiation." );
	}


	//Setters.
	void add( int concreteElementScopeLeftEndpoint, Object concreteElement, Object parentSchemaElement ){//The 'concreteElementScope' object corresponds to the scope of the schema element which is the same with the scope of its abstract element in the pattern tree.  

		rightmostConcreteElement = concreteElement;


		mapOfConcreteElements = MapInsertion.put( concreteElement, true, mapOfConcreteElements, false );

		mapOfConcreteElementScopes = MapInsertion.put( concreteElementScopeLeftEndpoint, concreteElement, mapOfConcreteElementScopes, false );


		mapOfRightmostChildScopeRightEndpoints = MapInsertion.put( concreteElement, MapInsertion.put( concreteElement, 0, null, false ), mapOfRightmostChildScopeRightEndpoints, true );

		mapOfRightmostChildScopeRightEndpoints = MapInsertion.put( parentSchemaElement, MapInsertion.put( parentSchemaElement, SchemaRepresentationController.getScope( concreteElement ).getIntRightEndpoint(), null, false ), mapOfRightmostChildScopeRightEndpoints, true );
	}

	private void add( Map<Object, Object> cMapOfSchemaElements, Map<Object, Object> cMapOfSchemaElementScopes, Map<Object, Object> cMapOfRightmostChildScopes ){

		mapOfConcreteElements = MapInsertion.putAll( cMapOfSchemaElements, mapOfConcreteElements );

		mapOfConcreteElementScopes = MapInsertion.putAll( cMapOfSchemaElementScopes, mapOfConcreteElementScopes );

		mapOfRightmostChildScopeRightEndpoints = MapInsertion.putAll( cMapOfRightmostChildScopes, mapOfRightmostChildScopeRightEndpoints );
	}


	//Getters.
	int getPatternInstantiationID(){

		return patternInstantiationID;
	}

	protected Object getConcreteElement( int concreteElementScopeLeftEndpoint ){

		return MapGetter.getValue( concreteElementScopeLeftEndpoint, mapOfConcreteElementScopes );
	}

	protected Interval1D getRightmostChildScope( Object concreteElement ){

		return MapGetter.exists( concreteElement, mapOfRightmostChildScopeRightEndpoints ) ? (Interval1D) MapGetter.getValue( concreteElement, mapOfRightmostChildScopeRightEndpoints ) : null;
	}

	@SuppressWarnings("unchecked")
	protected Map<Object, Object> getPairOfConcreteElementAndRightmostChildScopeRightEndpoint( Object concreteElement ){

		return MapGetter.exists( concreteElement, mapOfRightmostChildScopeRightEndpoints ) ? (Map<Object, Object>) MapGetter.getValue( concreteElement, mapOfRightmostChildScopeRightEndpoints ) : null;
	}

	protected int getRightmostChildScopeRightEndpoint( Object concreteElement ){

		Map<Object, Object> pairOfSchemaElementAndRightmostChildScopeRightEndpoint = getPairOfConcreteElementAndRightmostChildScopeRightEndpoint( concreteElement );

		return (Integer) MapGetter.getValue( concreteElement, pairOfSchemaElementAndRightmostChildScopeRightEndpoint );
	}

	protected double getDensity(){

		return density;
	}

	protected double getDensity( int schemaSize, boolean isUpperBound ){

		int patternInstantiationSize = mapOfConcreteElements != null ? mapOfConcreteElements.size() : 0;


		if( ! isUpperBound ) density = (double) patternInstantiationSize / (double) getNumberOfInternalConcreteElements( schemaSize, isUpperBound );

		else density = (double) ( patternInstantiationSize + getNumberOfGrowthConcreteElements( schemaSize ) ) / (double) getNumberOfInternalConcreteElements( schemaSize, isUpperBound );


		return density;
	}

	private int getNumberOfInternalConcreteElements( int schemaSize, boolean isUpperBound ){

		if( ! isUpperBound ) return getNumberOfInternalConcreteElements();

		else return getNumberOfInternalConcreteElements() + getNumberOfGrowthConcreteElements( schemaSize );
	}

	private int getNumberOfInternalConcreteElements(){

		return SchemaRepresentationController.getScope( rightmostConcreteElement ).getIntLeftEndpoint() - SchemaRepresentationController.getScope( rootConcreteElement ).getIntLeftEndpoint() + 1;
	}

	private int getNumberOfGrowthConcreteElements( int schemaSize ){

		return schemaSize - SchemaRepresentationController.getScope( rightmostConcreteElement ).getIntLeftEndpoint() - 1;
	}

	protected double getCoverage( int schemaSize, boolean isUpperBound ){

		int patternInstantiationSize = mapOfConcreteElements != null ? mapOfConcreteElements.size() : 0;

		if( ! isUpperBound ) return (double) patternInstantiationSize / (double) schemaSize;

		else return (double) ( patternInstantiationSize + getNumberOfGrowthConcreteElements( schemaSize ) ) / (double) schemaSize;
	}


	//Visualization.
	public Object visualize( String printFilePath ){

		boolean filePrint = printFilePath != null ? true : false;

		DefaultMutableTreeNode root = null;


		String msg = "\t\t\t\tPattern instantiation " + patternInstantiationID;

		if( filePrint ) PrintMessages.appendLnInLogFile( PrintMessages.matchingPrint, printFilePath, msg );

		else root = new DefaultMutableTreeNode( msg );


		Iterator<Object> i = mapOfConcreteElementScopes != null && mapOfConcreteElementScopes.keySet() != null ? mapOfConcreteElementScopes.keySet().iterator() : null;

		while( i != null && i.hasNext() ){

			int leftEndpointOfConcreteElementScope = (Integer) i.next();

			Object concreteElement = mapOfConcreteElementScopes.get( leftEndpointOfConcreteElementScope );

			int rightmostChildScopeRightEndpoint = getRightmostChildScopeRightEndpoint( concreteElement );


			Object retValue = SchemaRepresentationController.visualize( concreteElement, leftEndpointOfConcreteElementScope, rightmostChildScopeRightEndpoint, printFilePath );

			if( ! filePrint ) root.add( (MutableTreeNode) retValue );
		}


    	return root;
	}
}
