package org.ow2.choreos.queryEngine;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.abstractionBase.FunctionalAbstraction;
import org.ow2.choreos.abstractionBase.FunctionalHierarchy;
import org.ow2.choreos.abstractionBase.NonFunctionalAbstraction;
import org.ow2.choreos.abstractionBase.NonFunctionalHierarchy;
import org.ow2.choreos.gui.UserSettings;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;


/**
 * 
 * @author dathanas
 *
 * This class initiates a QueryParser and a QueryExecuter object and use their methods to create the query, execute it and reconstruct 
 * the abstraction object corresponding to the query executed
 */

public class QueryEngineController{
	private QueryParser queryParse;
	private QueryExecutor queryExec;
	private ArrayList <FunctionalAbstraction> retrievedFunctionalAbstractions;
	private ArrayList <NonFunctionalAbstraction> retrievedNonFunctionalAbstractions;
	private FunctionalHierarchy functionalHierarchyToVisualize;
	private NonFunctionalHierarchy nonFunctionalHierarchyToVisualize;
	private ArrayList <Integer> functionalRoots;
	private ArrayList <Integer> nonFunctionalRoots;
	private int resultSetType;
	
	//constructor
	public QueryEngineController(){
		queryExec = new QueryExecutor();
		queryParse = new QueryParser();
		retrievedFunctionalAbstractions = new ArrayList <FunctionalAbstraction>();
		retrievedNonFunctionalAbstractions = new ArrayList <NonFunctionalAbstraction>();
		functionalRoots =  new ArrayList <Integer>();
		nonFunctionalRoots =  new ArrayList <Integer>();
		functionalHierarchyToVisualize = null;
		nonFunctionalHierarchyToVisualize = null;
		resultSetType = -1;
	}
	
	//Getters
	public ArrayList <FunctionalAbstraction> getRetrievedFunctionalAbstractions(){
		return retrievedFunctionalAbstractions;
	}
	
	public ArrayList <NonFunctionalAbstraction> getRetrievedNonFunctionalAbstractions(){
		return retrievedNonFunctionalAbstractions;
	}
	
	public ArrayList <Integer> getFunctionalRoots(){
		return functionalRoots;
	}
	
	public ArrayList <Integer> getNonFunctionalRoots(){
		return nonFunctionalRoots;
	}
	
	public FunctionalHierarchy getFunctionalHierarchyToVisualize(){
		return functionalHierarchyToVisualize;
	}
	
	public NonFunctionalHierarchy getNonFunctionalHierarchyToVisualize(){
		return nonFunctionalHierarchyToVisualize;
	}
	
	public int getResultSetType(){
		return resultSetType;
	}
	
	//this method creates and executes the query defined in the 'filename' parameter and reconstructs the abstractions
	//returned in the result set of the query
	public void executeLookupQuery(String dBase, String fileContent){
		Connection dbConnection = null;
		//System.out.println(fileContent);
		
		if (dBase.equals("")){
    		dBase = "localhost/default";
			//dBase = "localhost/dump_data_full";
    	}
		
		try{
			Class.forName("com.mysql.jdbc.Driver").newInstance(); // load SQL driver to connect to DBMS
			//open connection to database
   	  	 	dbConnection = DriverManager.getConnection("jdbc:mysql://" + dBase,"root", "root");
			//dbConnection = DriverManager.getConnection("jdbc:mysql://" + "gatepc73.cs.uoi.gr:3306/default","root", "root");
   	  	 	String query = queryParse.createQuery(fileContent);
   	  	 	resultSetType = queryParse.getResultSetType();
   	  	 	
   	  	 	queryExec.executeLookupQuery(dbConnection, query);
   	  	 	queryExec.retrieveAbstractions(dbConnection, resultSetType);
   	  	 	 
   	  	 	retrievedFunctionalAbstractions = queryExec.getRetrievedFunctionalAbstractions();
   	  	 	retrievedNonFunctionalAbstractions = queryExec.getRetrievedNonFunctionalAbstractions();
   	  	
   	  	 	if (retrievedFunctionalAbstractions.size() != 0){
   	  	 		UserSettings options = new UserSettings(0);
   	  	 		options.setNamesDistanceCalculatorAlgorithm(retrievedFunctionalAbstractions.get(0).getNamesDistanceCalculatorAlgorithm());
   	  	 		// ZAS comment functionalHierarchyToVisualize = new FunctionalHierarchy(options, retrievedFunctionalAbstractions.get(retrievedFunctionalAbstractions.size()-1).getRepresentedInterfaces());
   	  	 		// ZAS Seems that in the constructor the list of the unrepresented interfaces should be null 
   	  	 		functionalHierarchyToVisualize = new FunctionalHierarchy(options, null);
   	  	 	}
   	  	 	
   	  	 	if (retrievedNonFunctionalAbstractions.size() != 0){
	  	 		nonFunctionalHierarchyToVisualize = new NonFunctionalHierarchy( 0, 3, 100 );
	  	 	}
   	  	 	
   	  	 	//choose the roots of the hierarchies of the retrieved functional abstractions
   	  	 	for (int i = 0; i < retrievedFunctionalAbstractions.size(); i++){
   	  	 		
   	  	 		if (!retrievedFunctionalAbstractions.get(i).getDescendants().isEmpty()){
   	  	 			
   	  	 			for (int k = 0; k < retrievedFunctionalAbstractions.get(i).getDescendants().size(); k++){
   	  	 				FunctionalAbstraction tmpDescandent = retrievedFunctionalAbstractions.get(i).getDescendants().get(k);
   	  	 				for (int j = 0; j < retrievedFunctionalAbstractions.size(); j++){
   	  	 					if (retrievedFunctionalAbstractions.get(j).getID() == tmpDescandent.getID()){
   	  	 						retrievedFunctionalAbstractions.get(j).setAncestorId(retrievedFunctionalAbstractions.get(i).getID());
   	  	 						break;
   	  	 					}
   	  	 				}
   	  	 			}
   	  	 			
   	  	 		}
   	  	 	}
   	  	 	
   	  	 	/*for (int i = 0; i < retrievedFunctionalAbstractions.size(); i++){
	  	 		System.out.println("Abstr:" + retrievedFunctionalAbstractions.get(i).getID() + "--ancestor:" + retrievedFunctionalAbstractions.get(i).getAncestorId());
	  	 	}*/
	  	 	
   	  	 	for (int i = 0; i < retrievedFunctionalAbstractions.size(); i++){
   	  	 		if (retrievedFunctionalAbstractions.get(i).getAncestorId() == -1){
   	  	 			functionalRoots.add(i);
   	  	 		}
   	  	 	}
   	  	 	
   	  	 	/*for (int i = 0; i < functionalRoots.size(); i++){
   	  	 		System.out.println(functionalRoots.get(i));
   	  	 	}*/
   	  	 	
   	  	 	//choose the roots of the hierarchies of the retrieved non - functional abstractions
   	  	 	for (int i = 0; i < retrievedNonFunctionalAbstractions.size(); i++){
	  	 		//if (!retrievedNonFunctionalAbstractions.get(i).getDescendants().isEmpty()){
   	  	 		if (retrievedNonFunctionalAbstractions.get(i).getDescendants() != null){
	  	 			System.err.println("ZAS: making non-functional hierarchy 1 ");
	  	 			for (int k = 0; k < retrievedNonFunctionalAbstractions.get(i).getDescendants().size(); k++){
	  	 				//System.out.println("Abstr:");
	  	 				NonFunctionalAbstraction tmpDescandent = retrievedNonFunctionalAbstractions.get(i).getDescendants().get(k);
	  	 				for (int j = 0; j < retrievedNonFunctionalAbstractions.size(); j++){
	  	 					if (retrievedNonFunctionalAbstractions.get(j).getNonFunctionalAbstractionID() == tmpDescandent.getNonFunctionalAbstractionID()){
	  	 						retrievedNonFunctionalAbstractions.get(j).setAncestor(retrievedNonFunctionalAbstractions.get(i));
	  	 						System.err.println("ZAS: making non-functional hierarchy 2");
	  	 						break;
	  	 					}
	  	 				}
	  	 			}
	  	 			
	  	 		}
	  	 	}
   	  	 	
   	  	 	for (int i = 0; i < retrievedNonFunctionalAbstractions.size(); i++){
	  	 		if (retrievedNonFunctionalAbstractions.get(i).getAncestor() == null){
	  	 			nonFunctionalRoots.add(i);
	  	 		}
	  	 	}
   	  	 	for (int i = 0; i < retrievedNonFunctionalAbstractions.size(); i++){
	   	  		if (retrievedNonFunctionalAbstractions.get(i).isEmpty()){
	   	  			System.out.println(retrievedNonFunctionalAbstractions.get(i).getNonFunctionalAbstractionID());
	   	  		}
   	  	 	}
   	  	 	
   	  	 	// ZAS method to clean up abstractions
   	   	  	produceFunctionalNonFunctionalIntersection();
   	   	  	
   	  	/*for (int i = 0; i < retrievedNonFunctionalAbstractions.size(); i++){
   	  		if (retrievedNonFunctionalAbstractions.get(i).getAncestor() != null){
   	  			System.out.println("Abstr:" + retrievedNonFunctionalAbstractions.get(i).getNonFunctionalAbstractionID() + "--ancestor:" + retrievedNonFunctionalAbstractions.get(i).getAncestor().getNonFunctionalAbstractionID());
   	  		}
   	  		else{
   	  			System.out.println("Abstr:" + retrievedNonFunctionalAbstractions.get(i).getNonFunctionalAbstractionID() + "--ancestor:" + " null");
   	  		}
   	  	}*/	
	  	 	//System.out.println(retrievedFunctionalAbstractions.get(i).getID());
	  	 	//hierarchyToVisualize.addAbstraction(retrievedFunctionalAbstractions.get(i), retrievedFunctionalAbstractions.get(i).getID(), null, null, retrievedFunctionalAbstractions.get(i).getInterfacesMapping(), null);
	  	 	//System.out.println("Abstraction - " + retrievedFunctionalAbstractions.get(i).getID() + "added!");
	  	 	//System.out.println(retrievedFunctionalAbstractions.get(i).getID());
	  	 	//retrievedFunctionalAbstractions.get(i).visualize();
   	  	 	
		}
		catch(Exception e) {
            System.err.println("Exception executeLookupQuery (Controller): " + e.getMessage());
          } finally {
            try {
              if(dbConnection != null)
              	dbConnection.close();
            } catch(SQLException e) {}
          }
	}
	
	/*
	 * Zas added to clean up the query results in case the query concerned 
	 * both functional and non functional properties
	 */
	private void produceFunctionalNonFunctionalIntersection(){
		/* 
		 * Remove from retrieved functional abstractions
		 * instances that do not belong to required non functional abstractions
		 * e.g. hotels that do not have high availability
		 */
		if(retrievedNonFunctionalAbstractions != null && retrievedNonFunctionalAbstractions.size() != 0){
			
			for(int i=0; retrievedFunctionalAbstractions!= null && i < retrievedFunctionalAbstractions.size(); i++){
				
				FunctionalAbstraction retrievedFAbstraction = retrievedFunctionalAbstractions.get(i);
				List<ServiceInterface> retrievedInterfaces = retrievedFAbstraction.getRepresentedInterfaces();
				
				List<ServiceInterface> intersectedInterfaces = new ArrayList<ServiceInterface>();
				
				for(int k=0; k < retrievedInterfaces.size(); k++){
					
					ServiceInterface retrievedInterface = retrievedInterfaces.get(k);
					List<ServiceInstance> retrievedFInstances = retrievedInterface.getServiceInstances();
					
					List<ServiceInstance> intersectedInstances = new ArrayList<ServiceInstance>();
					
					for(int j=0; j < retrievedFInstances.size(); j++){
						
						boolean found = false;
						ServiceInstance retrievedFInstance = retrievedFInstances.get(j);
						
						for(int l=0; l < retrievedNonFunctionalAbstractions.size(); l++){
							
							NonFunctionalAbstraction retrievedNFAbstraction = retrievedNonFunctionalAbstractions.get(l);
							Map<Integer, ServiceInstance> retrievedNFInstancesMap = retrievedNFAbstraction.getRepresentedServiceInstances();
							ArrayList<ServiceInstance> retrievedNFInstances = new ArrayList<ServiceInstance>(retrievedNFInstancesMap.values());
							
							for(int m=0; m < retrievedNFInstances.size(); m++){
								
								ServiceInstance retrievedNFInstance = retrievedNFInstances.get(m);
								
								if(retrievedNFInstance.getURI().equals(retrievedFInstance.getURI())){
									found = true;
									break;
								}
							}
						}
						
						if(found){
							
							intersectedInstances.add(retrievedFInstance);
						}
					}
					
					if(intersectedInstances.size() != 0){
						ServiceInterface intersectedInterface = new ServiceInterface(retrievedInterface);
						intersectedInterface.setxServiceInstances(intersectedInstances);
						intersectedInterfaces.add(intersectedInterface);
					}
				}
				
				retrievedFAbstraction.setxRepresentedInterfaces(intersectedInterfaces);
			}
		}
	}

}