package org.ow2.choreos.queryEngine;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.ow2.choreos.abstractionBase.FunctionalAbstraction;
import org.ow2.choreos.abstractionBase.NonFunctionalAbstraction;
import org.ow2.choreos.serviceBase.AbstractionsHierarchyRetriever;


/**
 * 
 * @author dathanas
 *
 * This class contains methods used to execute the query produced by the QueryParser and reconstruct the Abstraction objects
 * corresponding to the executed query 
 */

public class QueryExecutor{
	
	private Hashtable <Integer, Integer> retrievedAbstractionIds;	//the ids of the Abstractions retrieved are stored in this Hashtable
	private ArrayList <FunctionalAbstraction> retrievedFunctionalAbstractions;
	private ArrayList <NonFunctionalAbstraction> retrievedNonFunctionalAbstractions;
	
	//Constructor
	QueryExecutor(){
		retrievedAbstractionIds = new Hashtable <Integer, Integer>();
		retrievedFunctionalAbstractions = new ArrayList <FunctionalAbstraction>();
		retrievedNonFunctionalAbstractions = new ArrayList <NonFunctionalAbstraction>();
	}
	
	//Getters
	public Hashtable <Integer, Integer> getRetrievedAbstractionIds(){
		return retrievedAbstractionIds;
	}
	
	public ArrayList <FunctionalAbstraction> getRetrievedFunctionalAbstractions(){
		return retrievedFunctionalAbstractions;
	}
	
	public ArrayList <NonFunctionalAbstraction> getRetrievedNonFunctionalAbstractions(){
		return retrievedNonFunctionalAbstractions;
	}
	
	//this method executes the query it takes as parameter and stores the ids of the abstractions returned in the result set
	protected void executeLookupQuery(Connection dbConnection, String query){

		try {
		    Statement statement = dbConnection.createStatement();
		    ResultSet rs = null;
		    		    
		    if (!(query.trim().equals(""))){
  	  			rs = statement.executeQuery(query);
		    }
		    
		    if (rs != null){
		    	ResultSetMetaData rsmd = rs.getMetaData();
		        int numberOfColumns = rsmd.getColumnCount();
		    	
  	  			while (rs.next()) {
  	  				int abstractionId1 = rs.getInt(1);
  	  				int abstractionId2 = -1;
  	  				if (numberOfColumns == 4){
  	  					abstractionId2 = rs.getInt(3);
  	  					if (!retrievedAbstractionIds.containsKey(abstractionId2)){
  	  						if (rs.getInt(4) == -1){	//non Functional Abstraction
  	  							retrievedAbstractionIds.put(abstractionId2, 1);	// 1 means that this abstractionId corresponds to NFA
  	  						}
  	  						else{
  	  							retrievedAbstractionIds.put(abstractionId2, 0);	// 0 means that this abstractionId corresponds to FA
  	  						}
  	  					}
  	  				}
  	  				
  	  				if (!retrievedAbstractionIds.containsKey(abstractionId1)){
  	  					if (rs.getInt(2) == -1){	//non Functional Abstraction
  	  						retrievedAbstractionIds.put(abstractionId1, 1);	// 1 means that this abstractionId corresponds to NFA
  	  					}
  	  					else{
  	  						retrievedAbstractionIds.put(abstractionId1, 0);	// 0 means that this abstractionId corresponds to FA
  	  					}
  	  				}
  	  			
  	  			}//end of while
  	  			
		    }
		}
		catch(Exception e) {
	        System.err.println("Exception executeLookupQuery: " + e.getMessage());
	    }
		
	}
	
	//this method retrieves and reconstructs the Abstraction objects (functional and/or non functional) corresponding to the 
	//abstraction ids contained in hashtable 'retrievedAbstractionIds'
	@SuppressWarnings("rawtypes")
	protected void retrieveAbstractions(Connection dbConnection, int resultSetType){
		AbstractionsHierarchyRetriever queryAbstractionRetriever = new AbstractionsHierarchyRetriever();
		
		try {
		    
			if (retrievedAbstractionIds.size() != 0){
				Set abstractionIdsSet = retrievedAbstractionIds.entrySet();
			    Iterator abstractionIdsIterator = abstractionIdsSet.iterator();
			    
			    //for each id in retrievedAbstractionIds, retrieve the corresponding 'Abstraction' object
			    while (abstractionIdsIterator.hasNext()) {
					Map.Entry entry = (Map.Entry) abstractionIdsIterator.next();
			    	if ((Integer)entry.getValue() == 0){		//this is a FunctionalAbstraction
			    		String whereClause = new String("and A_ID = " + entry.getKey().toString());
			    		ArrayList <FunctionalAbstraction> temp = new ArrayList <FunctionalAbstraction>();	
			    		temp = queryAbstractionRetriever.retrieveFunctionalAbstractions(dbConnection, whereClause, resultSetType);
			    		retrievedFunctionalAbstractions.add(temp.get(0));
			    	}
			    	else{		//this is a NonFunctionalAbstraction
			    		String whereClause = new String("and A_ID = " + entry.getKey().toString());
			    		ArrayList <NonFunctionalAbstraction> temp = new ArrayList <NonFunctionalAbstraction>();	
			    		if (queryAbstractionRetriever.retrieveNonFunctionalAbstractions(dbConnection, whereClause, null, resultSetType).get(0).isEmpty()){
			    			System.out.println(queryAbstractionRetriever.retrieveNonFunctionalAbstractions(dbConnection, whereClause, null, resultSetType).get(0).getNonFunctionalAbstractionID());
			    		}
			    		temp = queryAbstractionRetriever.retrieveNonFunctionalAbstractions(dbConnection, whereClause, null, resultSetType);
			    		if (temp.get(0).isEmpty()){
	  			  			System.out.println(temp.get(0).getNonFunctionalAbstractionID());
	  			  		}
			    		retrievedNonFunctionalAbstractions.add(temp.get(0));
			    	}
			    	
			    }//end of while
			}
			
		}
		catch(Exception e) {
	        System.err.println("Exception retrieveAbstractions: " + e.getMessage());
	    }
	}
	
}