package org.ow2.choreos.queryEngine;

import java.io.*;
import java.util.ArrayList;
import java.util.Hashtable;

/**
 * 
 * @author dathanas
 *
 * This class is used to produce the final query (to the DBMS) from a given text file
 */

public class QueryParser{
	private String dbase;
	private int resultSetType = -1; // 0 for representative info, 1 for single object, -1 for full object
	
	//constructor
	public QueryParser(){
		dbase = new String("localhost/default");
	}
	
	/*void printFK(ArrayList<ArrayList<String>> foreignKeys){
		for (int i = 0; i < foreignKeys.size(); i++){
			System.out.println(foreignKeys.get(i).get(0) + "--" + foreignKeys.get(i).get(1) + "--" + foreignKeys.get(i).get(2));
		}
	}*/

	// this method finds and returns the foreign key of the table 'table' to the table 'fkFor' 
	String findFK(String table, String fkFor, ArrayList<ArrayList<String>> foreignKeys){
		for (int i = 0; i < foreignKeys.size(); i++){
			if (foreignKeys.get(i).get(0).equals(table) && foreignKeys.get(i).get(1).equals(fkFor)){
				return foreignKeys.get(i).get(2);
			}
		}
		return null;
	}

	// this method is used to parse the string passed as parameter ('fileContent') and produce the final query (returned)
	@SuppressWarnings("unchecked")
	protected String createQuery(String fileContent) {
		String query = new String("");
		File file = new File("query.txt");
		FileWriter filestream = null;
		try {
			filestream = new FileWriter("query.txt");
			BufferedWriter out = new BufferedWriter(filestream);
			out.write(fileContent);
			out.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		}			
		
		try{
			
			String fkSubstring = new String(" ");
			FileInputStream fstream = new FileInputStream("query.txt");
			//get the object of DataInputStream
			DataInputStream in = new DataInputStream(fstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String strLine;
			ArrayList<String> abstractionVrbl = new ArrayList<String>();
			Hashtable <String, String> vrblXToTableName = new Hashtable <String, String>();
			
			boolean letFlag = false;
			boolean forFlag = false;
			boolean whereFlag = false;
			boolean returnFlag = false;
			boolean first = false;
			
			boolean forFlagFirst = true;
			
			//for each table used in the query language, we must define the database primary and foreign keys 
			//these keys will be used during query production, as the file is parsed
			Hashtable <String, String> primaryKeys = new Hashtable <String, String>();
			primaryKeys.put("hierarchies","H_ID");
			primaryKeys.put("servicecollections","C_ID");
			primaryKeys.put("operations","OP_ID");
			primaryKeys.put("parameters","P_ID");
			primaryKeys.put("inmsgs","INM_ID");
			primaryKeys.put("outmsgs","OUTM_ID");
			primaryKeys.put("abstractions","A_ID");
			primaryKeys.put("serviceinterfaces","SI_ID");
			primaryKeys.put("messages","MS_ID");
			primaryKeys.put("messagestypes","MT_ID");
			primaryKeys.put("representedinterfaces","RI_ID");
			primaryKeys.put("representedinstances","RIS_ID");
			primaryKeys.put("representativeinterfaces","RSI_ID");
			primaryKeys.put("representativeoperations","ROP_ID");
			primaryKeys.put("representativemessages","RMS_ID");
			primaryKeys.put("representativemessagetypes","RMT_ID");
				
			ArrayList<ArrayList<String>> foreignKeys = new ArrayList<ArrayList<String>>();
			ArrayList<String> temp = new ArrayList<String>();
			ArrayList<String> temp2 = new ArrayList<String>();
			temp.add("servicesources");
			temp.add("servicecollections");
			temp.add("ss_c_id");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
			
			temp.add("hierarchies");
			temp.add("servicecollections");
			temp.add("H_C_ID");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
		
			temp.add("abstractions");
			temp.add("hierarchies");
			temp.add("A_H_ID");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
			
			temp.add("representedinstances");
			temp.add("abstractions");
			temp.add("ris_nfa_id");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
			
			temp.add("representedinterfaces");
			temp.add("abstractions");
			temp.add("ri_a_id");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
			
			temp.add("nfaqp_v");
			temp.add("abstractions");
			temp.add("nfa_id");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
			
			temp.add("qproperty");
			temp.add("abstractions");
			temp.add("nfa_id");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
						
			temp.add("representativeinterfaces");
			temp.add("abstractions");
			temp.add("rsi_a_id");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
			
			temp.add("representativeoperations");
			temp.add("representativeinterfaces");
			temp.add("rop_rsi_id");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
			
			temp.add("representativemessages");
			temp.add("representativeoperations");
			temp.add("rms_rop_id");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
			
			temp.add("representativemessagetypes");
			temp.add("representativemessages");
			temp.add("rmt_rms_id");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
			
			temp.add("operations");
			temp.add("serviceinterfaces");
			temp.add("OP_SI_ID");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
			
			temp.add("messages");
			temp.add("operations");
			temp.add("ms_op_id");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
						
			temp.add("messagetypes");
			temp.add("messages");
			temp.add("mt_ms_id");
			temp2 = (ArrayList<String>)temp.clone();
			temp.clear();
			foreignKeys.add(temp2);
					
			//read the file line by line
			while ((strLine = br.readLine()) != null) {
				
				
				//---------------  code that fixes the << like '%for%' >> problem  (added at 19/2/2013)
				
				/*previous code
				
				int letIndex = strLine.indexOf("let");
				int forIndex = strLine.indexOf("for");
				int whereIndex = strLine.indexOf("where");
				int returnIndex = strLine.indexOf("return");*/
				
				
				int letIndex = -1;
				int forIndex = -1;
				int whereIndex = -1;
				int returnIndex = -1;
				
				String strLineTrimmed = strLine.trim();
				boolean startsWithLet = strLineTrimmed.startsWith("let");
				boolean startsWithFor = strLineTrimmed.startsWith("for");
				boolean startsWithWhere = strLineTrimmed.startsWith("where");
				boolean startsWithReturn = strLineTrimmed.startsWith("return");
				
				if(startsWithLet)
					letIndex = strLine.indexOf("let");
				else if(startsWithFor)
					forIndex = strLine.indexOf("for");
				else if(startsWithWhere)
					whereIndex = strLine.indexOf("where");
				else if(startsWithReturn)
					returnIndex = strLine.indexOf("return");
				
				//----------------- end of code that fixes the <<like '%for%' problem
				
				
				
				
				
				//identify the reserved words 'let', 'for', 'where', 'return' and update the 'query' string
				if (letIndex != -1){
					letFlag = true;
				}
				
				if (forIndex != -1){
					forFlag = true;
					letFlag = false;
					if (forFlagFirst){
						query = query.concat(" \n\n from ");
						forFlagFirst = false;
					}
					else{
						query = query.concat(" \n");
					}
				}
				
				if (whereIndex != -1){
					whereFlag = true;
					first = true;
					forFlag = false;
					query = query.concat(" \n\n where ");
				}
				
				if (returnIndex != -1){
					returnFlag = true;
					whereFlag = false;
					first = true;
				}
				
				//depending on which part of the file is parsing (defined by the flags), the query is updating properly
				if (letFlag){
					String[] dbSplit = strLine.split("'");
					if (dbSplit.length < 2){
						System.out.println("Error in syntax: " + strLine);
						query = "";
						break;
					}
					else{
						dbase = dbSplit[1];
					}
				}
				
				if (forFlag){
					int stringSize = strLine.length();
					char[] table = new char[stringSize + 1];
					table = strLine.toCharArray();
					
					int ind1 = strLine.indexOf('$');
					if (ind1 == -1){
						System.out.println("Error in syntax: " + strLine);
						query = "";
						break;
					}
					
					//find vrblX
					int i = 0;
					ind1++;
					char[] varTable1 = new char[strLine.length() - ind1];
					while (table[ind1] != ' '){
						varTable1[i] = table[ind1];
						i++;
						ind1++;
					}
					String vrblX = new String(varTable1);
					vrblX = vrblX.trim();
					
					//find vrblA
					i = 0;
					ind1 = strLine.lastIndexOf('$');
					ind1++;
					char[] varTable2 = new char[strLine.length() - ind1];
					while (table[ind1] != '/'){
						varTable2[i] = table[ind1];
						i++;
						ind1++;
					}
					String vrblA = new String(varTable2);
					vrblA = vrblA.trim();
					
					String[] hops = strLine.split("/");
					String[] aliases = new String[hops.length-1];
					if (vrblXToTableName.get(vrblX) == null){
						vrblXToTableName.put(vrblX, hops[hops.length-1]);
						
						if (hops[hops.length-1].equals("abstractions")){
							abstractionVrbl.add(vrblX);
							
						}
					}
					
					i = 0;
					//create 'from' part of the query
					for (int j = 1; j <hops.length ; j++){
						
						if (j == hops.length - 1 ){
							if (hops[j].trim().equals("qproperty")){
								query = query.concat("nfaqp_v" + " " + vrblX + ",");
								aliases[i] = vrblX;
								i++;
							}
							else{
								query = query.concat(hops[j].trim() + " " + vrblX + ",");
								aliases[i] = vrblX;
								i++;
							}
						}
						else {
							if (hops[j].trim().equals("qproperty")){
								aliases[i] = " _" + vrblX + "_" + "nfaqp_v";
								i++;
								query = query.concat("nfaqp_v" + " _" + vrblX + "_" + "nfaqp_v" + ",");
							}
							else{
								aliases[i] = " _" + vrblX + "_" + hops[j].trim();
								i++;
								query = query.concat(hops[j].trim() + " _" + vrblX + "_" + hops[j].trim() + ",");
							}
						}
					}//end of for
					
					
					//create second part of the 'where clause' -- link the foreign keys and store it to 'fkSubstring'
					for (int j = 0; j < hops.length - 1; j++){
					
						if (j == 0 ){
							String tmpFK = new String ();
							
							if (!vrblA.equals("db")){
								tmpFK = findFK(hops[j+1].trim(), vrblXToTableName.get(vrblA), foreignKeys);
							} //end of if
							else{
								tmpFK = findFK(hops[j+1].trim(), vrblXToTableName.get(vrblA), foreignKeys);
							}
							
							if (!vrblA.equals("db")){
								if(tmpFK != null){
									fkSubstring = fkSubstring.concat("\n AND " + vrblA + "." + primaryKeys.get(vrblXToTableName.get(vrblA)) + " = " + aliases[j] + "."  + tmpFK );
									
								}
								else{
									System.out.println("Error in syntax: One or more of the table(s) defined does not exist! \n" + strLine);
									query = "";
									return query;
								}
							}
						}//end of if j==0
						
						else {
							String tmpFK = new String();
							
							tmpFK = findFK(hops[j+1].trim(), hops[j].trim(), foreignKeys);
							
							if (tmpFK != null){
								fkSubstring = fkSubstring.concat("\n AND " + aliases[j-1] + "." + primaryKeys.get(hops[j]) + " = " + aliases[j] + "."  + tmpFK );
								
							}
							else{
								System.out.println("Error in syntax: One or more of the table(s) defined does not exist! \n" + strLine);
								query = "";
								return query;
							}
						}
					}//end of for
					
				}
				
				//create 'where' part of the query
				if (whereFlag){
					if (first){
						first = false;
						char[] tmp = query.toCharArray();
						int ind1 = query.lastIndexOf(',');
						if (ind1 != -1){
							tmp[ind1] = ' ';
						}
						String tmpQuery = new String(tmp);
						query = tmpQuery;
					}
					else{
						String tmp = strLine.replace('$', ' ');
						tmp = tmp.replace('/', '.');
						
						tmp = tmp.trim();
						
						query = query.concat(tmp + " \n");
					}
				}
				
				if (returnFlag){
					if (first){
						first = false;
						char[] tmp = query.toCharArray();
						String tmpQuery = new String(tmp);
						query = tmpQuery;
					}
					
					else{
						strLine = strLine.trim();
						String[] returnType = strLine.split("\\.");
						if (returnType.length != 2){
							System.out.println("Error in syntax: " + strLine);
							query = "";
							return query;
						}
						else{
							String selectQuery = new String(" select distinct ");
							for (int i = 0; i < abstractionVrbl.size(); i++){
								selectQuery =  selectQuery.concat(abstractionVrbl.get(i) + ".a_id, " + abstractionVrbl.get(i) + ".a_fa_id, ");
							}
							char[] tmp = selectQuery.toCharArray();
							int ind1 = selectQuery.lastIndexOf(',');
							if (ind1 != -1){
								tmp[ind1] = ' ';
							}
							String tmpQuery = new String(tmp);
							selectQuery = tmpQuery;
							
							if (returnType[0].equals("F-Abstractions")){
								query = query.concat(" and A_NFA_ID = -1 \n");
							}
							else if (returnType[0].equals("NF-Abstractions")){
								query = query.concat(" and A_FA_ID = -1 \n");
							}
							else if (returnType[0].equals("Abstractions")){

							}
							else{
								System.out.println("Error in syntax: " + strLine);
								query = "";
								return query;
							}
							
							if (returnType[1].equals("representativeInfo")){
								resultSetType = 0;
							}
							else if (returnType[1].equals("fullObject")){
								resultSetType = 1;
							}
							
							query = selectQuery.concat(query);
						}//end of else
					}//end of else
				}
			}
			
			query = query.concat(fkSubstring);
			//System.out.println(query);
						
			in.close();
		
		}catch (Exception e){//Catch exception if any
			System.err.println("Error QueryParser: " + e.getMessage());
		}
		
		file.delete();
		return query;
	}
	
	protected String getDbase(){
		return dbase;
	}
	
	protected int getResultSetType(){
		return resultSetType;
	}
	
}