/**
 * The Lts2SbqlGanarator is a utility for generating SBQL queries 
 * from LTS related information
 */
package org.ow2.choreos.queryEngineFacade;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;
import java.util.Map;


/**
 * @author zarras
 * @author pilios
 */
public class Lts2SbqlGenerator {
	
	/* 
	 * Given a list of operations names the method generates a corresponding SBQL query 
	 */
	public static String generateSBQLQuery(List<String> operations){
		
		String variableDeclarations = 
			"let \t\t $db = db('localhost/mySB')\n" +
			"for \t\t $c in $db/servicecollections\n" +
			"for \t\t $fa in $c/hierarchies/abstractions\n" +
			"for \t\t $ri in $fa/representativeinterfaces\n";
		
		String whereClause = "where\n";
		
		String resultClause = "return \n\t\tAbstractions.fullObject\n";
		
		for(int i = 0; i < operations.size(); i++){
			variableDeclarations = variableDeclarations +
			"for \t\t $rop" + i + " in $ri/representativeoperations\n";
		}
		
		for(int i = 0; i < operations.size(); i++){
			whereClause = whereClause +
			"\t\t $rop" + i + "/rop_name like '%" +
			operations.get(i) +
			"%'";
			if(i != operations.size()-1)
				whereClause = whereClause + " and";
			whereClause = whereClause + "\n";
		}				
		
		String sbqlQuery = variableDeclarations + whereClause + resultClause;
		
		return sbqlQuery;
	}
	
	public static String generateSBQLQuery(AdvancedQuery advancedQuery){
		
		List<OperationConstraints> operationsConstraints = advancedQuery.getOperationsConstraints();
		int operationsConstraintsSize = operationsConstraints.size();
		
		List<QualityPropertyConstraints> qualityPropertiesConstraints = advancedQuery.getQualityPropertiesConstraints();
		int qualityPropertiesConstraintsSize = 0;
		
		if(qualityPropertiesConstraints != null)
			qualityPropertiesConstraintsSize = qualityPropertiesConstraints.size();
		
		String RQs = advancedQuery.getRQs();
		
		String variableDeclarations = 
				"let \t\t $db = db('localhost/mySB')\n" +
				"for \t\t $c in $db/servicecollections\n" +
				"for \t\t $fa in $c/hierarchies/abstractions\n" +
				"for \t\t $ri in $fa/representativeinterfaces\n";
			
		String whereClause = "where\n";
			
		String resultClause = "return \n\t\tAbstractions.fullObject\n";
		
		for(int i = 0; i < operationsConstraintsSize; i++){
			
			OperationConstraints operationConstraints = operationsConstraints.get(i);
			variableDeclarations += "for \t\t $rop" + i + " in $ri/representativeoperations\n";
				
			String opName = operationConstraints.getName();
			
			if(opName != null){
				whereClause += "\t\t $rop" + i + "/rop_name like '%" + opName + "%' and\n";
			}
			
			List<String> inParamNames = operationConstraints.getInParamNames();
			List<String> inParamTypes = operationConstraints.getInParamTypes();
			
			if(inParamNames != null){
				
				variableDeclarations += "for \t\t $rms" + i + "_1 in $rop" + i + "/representativemessages\n";
				whereClause += "\t\t $rms" + i + "_1/rms_type like '%IN%' and\n";
				
				int inParamNamesSize = inParamNames.size();
				
				for(int j = 0; j < inParamNamesSize; j++){
					
					variableDeclarations += "for \t\t $rmt" + i + "_1_" + j + " in $rms" + i + "_1/representativemessagetypes\n";
					
					String name = inParamNames.get(j);
					String type = inParamTypes.get(j);
					
					if(name != null){
						whereClause += "\t\t $rmt" + i + "_1_" + j + "/rmt_name like '%" + name + "%' and\n";
					}
					
					if(type != null){
						whereClause += "\t\t $rmt" + i + "_1_" + j + "/rmt_type like '%" + type + "%' and\n";
					}
				}
			}
			
			List<String> outParamNames = operationConstraints.getOutParamNames();
			List<String> outParamTypes = operationConstraints.getOutParamTypes();
			
			if(outParamNames != null){
				
				variableDeclarations += "for \t\t $rms" + i + "_2 in $rop" + i + "/representativemessages\n";
				whereClause += "\t\t $rms" + i + "_2/rms_type like '%OUT%' and\n";
				
				int outParamNamesSize = outParamNames.size();
				
				for(int j = 0; j < outParamNamesSize; j++){
					
					variableDeclarations += "for \t\t $rmt" + i + "_2_" + j + " in $rms" + i + "_2/representativemessagetypes\n";
					
					String name = outParamNames.get(j);
					String type = outParamTypes.get(j);
					
					if(name != null){
						whereClause += "\t\t $rmt" + i + "_2_" + j + "/rmt_name like '%" + name + "%' and\n";
					}
					
					if(type != null){
						whereClause += "\t\t $rmt" + i + "_2_" + j + "/rmt_type like '%" + type + "%' and\n";
					}
				}
			}
				
		}
		
		if(qualityPropertiesConstraintsSize != 0)
			variableDeclarations += "for \t\t $nfa in $c/hierarchies/abstractions\n";
		
		for(int i = 0; i < qualityPropertiesConstraintsSize; i++){
			
			variableDeclarations += "for \t\t $qp" + i + " in $nfa/qproperty\n";
			
			QualityPropertyConstraints qualityPropertyConstraints = qualityPropertiesConstraints.get(i);
			
			String name = qualityPropertyConstraints.getName();
			whereClause += "\t\t $qp" + i + "/qp_name like '%" + name + "%' and\n";
			
			boolean hasValueDescription = qualityPropertyConstraints.getHasValueDescription();
			
			if(hasValueDescription){
				whereClause += "\t\t $qp" + i + "/qp_value like '%" + qualityPropertyConstraints.getValueDescription() + "%' and\n";
			}
			else {
				whereClause += "\t\t $qp" + i + "/value_from <= " + qualityPropertyConstraints.getValueFrom() + " and\n";
				whereClause += "\t\t $qp" + i + "/value_to >= " + qualityPropertyConstraints.getValueTo() + " and\n";
			}	
									
		}
		
		if(whereClause.endsWith(" and\n")){
			int lastIndexOfAnd = whereClause.lastIndexOf(" and\n");
			whereClause = whereClause.substring(0, lastIndexOfAnd) + "\n";
		}
			
		String sbqlQuery = variableDeclarations + whereClause + resultClause;
		
		return sbqlQuery;		
	}
	
	
	/* 
	 * Given an LTS identifier the method generates a corresponding SBQL query 
	 */
	public static String generateSQBLQuery(String participantLTS){
		LTSUtility ltsUtility = new LTSUtility();
		
		if(participantLTS == null){
			System.err.println("Error: The participant LTS must not be null");
			return null;
		}
			
		List<String> requiredOperations = ltsUtility.getOperations(participantLTS);
		
		if(requiredOperations == null){
			System.err.println("Error: The operations produced by the LTS parser must not be null");
			return null;
		}
		
		String sbqlQuery = generateSBQLQuery(requiredOperations);
		
		return sbqlQuery;
	}
	
	/* 
	 * Given an LTS identifier the method generates a corresponding SBQL query 
	 * The query is stored in "LTSParticipant-SBQL.txt"
	 */
	public static void generateSQBLQueryFile(String participantLTS, String targetPath){
			
		String sbqlQuery = Lts2SbqlGenerator.generateSQBLQuery(participantLTS);
				
		try {
			FileWriter sbqlFile = new FileWriter(targetPath + participantLTS + "-SBQL.txt");
			PrintWriter sbqlFilePrintWriter = new PrintWriter(sbqlFile);
			sbqlFilePrintWriter.print(sbqlQuery);
			sbqlFilePrintWriter.close();
		} catch (IOException ioEx) {
			System.err.println(ioEx.getMessage());
		}
	}
}