package org.ow2.choreos.queryEngineFacade;

import java.io.Serializable;
import java.util.Map;
import java.util.List;

/**
 * <pre>
 * Contains operation constraints, thus requirements about a service operation.
 * These are:
 * 
 * -	Name.
 * -	A list of names of input parameters.
 * -	A list of types of input parameters.
 * -	A list of names of output parameters.
 * -	A list of types of output parameters.	
 * </pre>
 * 
 * @author pilios
 *
 */
public class OperationConstraints implements Serializable{

	private String name;
	private List<String> inParamNames;
	private List<String> inParamTypes;
	private List<String> outParamNames;
	private List<String> outParamTypes;
	
	/**
	 * Creates a complete OperationConstraints object.
	 * 
	 * @param name Operation name.
	 * @param inParamNames Names of input parameters.
	 * @param inParamTypes Types of input parameters.
	 * @param outParamNames Names of output parameters.
	 * @param outParamTypes Types of output parameters.
	 */
	public OperationConstraints(String name, List<String> inParamNames, List<String> inParamTypes, List<String> outParamNames, List<String> outParamTypes){
		this.name = name;
		this.inParamNames = inParamNames;
		this.inParamTypes = inParamTypes;
		this.outParamNames = outParamNames;
		this.outParamTypes = outParamTypes;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 */
	public OperationConstraints(){
		
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets operation name.
	 * 
	 * @param name Operation name.
	 */
	public void setName(String name){
		this.name = name;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets names of input parameters.
	 * 
	 * @param inParamNames Names of input parameters.
	 */
	public void setInParamNames(List<String> inParamNames){
		this.inParamNames = inParamNames;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets types of input parameters. 
	 * 
	 * @param inParamTypes Types of input parameters. 
	 */
	public void setInParamTypes(List<String> inParamTypes){
		this.inParamTypes = inParamTypes;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets names of output parameters.
	 * 
	 * @param outParamNames Names of output parameters.
	 */
	public void setOutParamNames(List<String> outParamNames){
		this.outParamNames = outParamNames;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets types of output parameters.
	 * 
	 * @param outParamTypes Types of output parameters.
	 */
	public void setOutParamTypes(List<String> outParamTypes){
		this.outParamTypes = outParamTypes;
	}
	
	/**
	 * Returns operation name.
	 * 
	 * @return Operation name.
	 */
	public String getName(){
		return name;
	}
	
	/**
	 * Returns names of input parameters.
	 * 
	 * @return A list of names of input parameters.
	 */
	public List<String> getInParamNames(){
		return inParamNames;
	}
	
	/**
	 * Returns types of input parameters.
	 * 
	 * @return A list of types of input parameters.
	 */
	public List<String> getInParamTypes(){
		return inParamTypes;
	}
	
	/**
	 * Returns names of output parameters.
	 * 
	 * @return A list of names of output parameters.
	 */
	public List<String> getOutParamNames(){
		return outParamNames;
	}
	
	/**
	 * Returns types of output parameters.
	 * 
	 * @return A list of types of output parameters.
	 */
	public List<String> getOutParamTypes(){
		return outParamTypes;
	}
	
}
