package org.ow2.choreos.queryEngineFacade;

import java.io.Serializable;

/**
 * <pre>
 * Contains requirements about a quality property.
 * These are:
 * 
 * -	Name.
 * -	Value description.That is a human friendly description of property's value.
 * -	ValueFrom, the low bound of the domain that the quality property's value must be in.
 * -	ValueTo, the high bound of the domain that the quality property's value must be in.
 * 
 * When ValueFrom and ValueTo are specified, this means that the quality property's value must
 * be in the domain [ValueFrom, ValueTo].
 * </pre>
 * 
 * @author pilios
 *
 */
public class QualityPropertyConstraints implements Serializable{

	private String name;
	
	private boolean hasValueDescription;
	private String valueDescription;
	
	private double valueFrom;
	private double valueTo;
	
	/**
	 * Creates a QualityPropertyConstraints object with name and value description filled.
	 * 
	 * @param name The quality property's name.
	 * @param valueDescription The quality property's value description.
	 */
	public QualityPropertyConstraints(String name, String valueDescription){
		this.name = name;
		this.valueDescription = valueDescription;
		hasValueDescription = true;
	}
	
	/**
	 * Creates a QualityPropertyConstraints object with name, valueFrom and valueTo filled.
	 * 
	 * @param name The quality property's name.
	 * @param valueFrom The low bound of the domain that the quality property's value must be in.
	 * @param valueTo The high bound of the domain that the quality property's value must be in.
	 */
	public QualityPropertyConstraints(String name, double valueFrom, double valueTo){
		this.name = name;
		hasValueDescription = false;
		this.valueFrom = valueFrom;
		this.valueTo = valueTo;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 */
	public QualityPropertyConstraints(){
		
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets quality property name.
	 * 
	 * @param name The quality property name.
	 */
	public void setName(String name){
		this.name = name;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets quality property value description.
	 * 
	 * @param valueDescription The quality property value description.
	 */
	public void setValueDescription(String valueDescription){
		this.valueDescription = valueDescription;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets whether the quality property has a value description or not (true/false).
	 * 
	 * @param hasValueDescription Whether the quality property has a value description or not (true/false).
	 */
	public void setHasValueDescription(boolean hasValueDescription){
		this.hasValueDescription = hasValueDescription;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets the low bound of the domain that the quality property's value must be in.
	 * 
	 * @param valueFrom The low bound of the domain that the quality property's value must be in.
	 */
	public void setValueFrom(double valueFrom){
		this.valueFrom = valueFrom;
	}
	
	/**
	 * <b>Special use:</b> information exchange over web.Should not be used for other purposes.
	 * Sets the high bound of the domain that the quality property's value must be in.
	 * 
	 * @param valueTo The high bound of the domain that the quality property's value must be in.
	 */
	public void setValueTo(double valueTo){
		this.valueTo = valueTo;
	}
	
	/**
	 * Returns the quality property's name.
	 * 
	 * @return The quality property's name.
	 */
	public String getName(){
		return name;
	}
	
	/**
	 * Returns the quality property's value description.
	 * 
	 * @return The quality property's value description.
	 */
	public String getValueDescription(){
		return valueDescription;
	}
	
	/**
	 * Returns whether the quality property has a value description or not (true/false).
	 * 
	 * @return Whether the quality property has a value description or not (true/false).
	 */
	public boolean getHasValueDescription(){
		return hasValueDescription;
	}
	
	/**
	 * Returns the low bound of the domain that the quality property's value must be in.
	 * 
	 * @return The low bound of the domain that the quality property's value must be in.
	 */
	public double getValueFrom(){
		return valueFrom;
	}
	
	/**
	 * Returns the high bound of the domain that the quality property's value must be in.
	 * 
	 * @return The high bound of the domain that the quality property's value must be in.
	 */
	public double getValueTo(){
		return valueTo;
	}
}
