/**
 * The QueryEngineFacade is a Facade for the AoSBM query engine 
 * that serves for executing LST related queries
 */
package org.ow2.choreos.queryEngineFacade;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

import javax.swing.JOptionPane;

import org.ow2.choreos.abstractionBase.FunctionalAbstraction;
import org.ow2.choreos.abstractionBase.NonFunctionalAbstraction;
import org.ow2.choreos.queryEngine.QueryEngineController;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;
import org.ow2.choreos.serviceRepresentation.functional.Operation;
import org.ow2.choreos.serviceRepresentation.functional.Message;
import org.ow2.choreos.serviceRepresentation.functional.MessageType;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;

import org.ow2.choreos.serviceRepresentation.nonFunctional.QualityProperties;
import org.ow2.choreos.serviceRepresentation.nonFunctional.NonFuncDescription;

/**
 * @author zarras
 * @author pilios
 */

public class QueryEngineFacade {
	

	public List<ConcreteService> executeSimpleQueryOverAbstractions(String dBase, SimpleQuery simpleQuery) throws QueryEngineFacadeException{
		
		if(dBase == null){
			throw(new QueryEngineFacadeException("SimpleQuery over abstractions error: no database specified"));
		}
		if(simpleQuery == null){
			throw(new QueryEngineFacadeException("SimpleQuery over abstractions error: no SimpleQuery object specified"));
		}
		
		if (dBase.equals(""))
    		dBase = "localhost/default";
		
		try{
			simpleQuery.checkWellFormedNess();
		} catch (QueryEngineFacadeException qefEx) {
			throw(new QueryEngineFacadeException("SimpleQuery over abstractions error: " + qefEx.getMessage()));
		}
		
		List<ConcreteService> concreteServices = retrieveConcreteServicesUsingSBQL(dBase, simpleQuery);
			
		return concreteServices;		
	}
	
	public List<ConcreteService> executeSimpleQueryOverInstances(String dBase, SimpleQuery simpleQuery) throws QueryEngineFacadeException{
		
		if(dBase == null){
			throw(new QueryEngineFacadeException("SimpleQuery over instances error: no database specified"));
		}
		if(simpleQuery == null){
			throw(new QueryEngineFacadeException("SimpleQuery over instances error: no SimpleQuery object specified"));
		}
		
		if (dBase.equals(""))
    		dBase = "localhost/default";
		
		try{
			simpleQuery.checkWellFormedNess();
		} catch (QueryEngineFacadeException qefEx) {
			throw(new QueryEngineFacadeException("SimpleQuery over instances error: " + qefEx.getMessage()));
		}
			
        List<ConcreteService> concreteServices = retrieveConcreteServicesUsingSQL(dBase, simpleQuery);
        	
    	return concreteServices;	
	}
	
	public List<ConcreteService> executeAdvancedQueryOverAbstractions(String dBase, AdvancedQuery advancedQuery) throws QueryEngineFacadeException{
		
		if(dBase == null){
			throw(new QueryEngineFacadeException("AdvancedQuery over abstractions error: no database specified"));
		}
		if(advancedQuery == null){
			throw(new QueryEngineFacadeException("AdvancedQuery over abstractions error: no AdvancedQuery object specified"));
		}
		
		if (dBase.equals(""))
    		dBase = "localhost/default";
		
		try{
			advancedQuery.checkWellFormedNess();
		} catch (QueryEngineFacadeException qefEx) {
			throw(new QueryEngineFacadeException("AdvancedQuery over abstractions error: " + qefEx.getMessage()));
		}
		
		List<ConcreteService> concreteServices = retrieveConcreteServicesUsingSBQL(dBase, advancedQuery);
			
		return concreteServices;
	}
	
	public List<ConcreteService> executeAdvancedQueryOverInstances(String dBase, AdvancedQuery advancedQuery) throws QueryEngineFacadeException{
		
		if(dBase == null){
			throw(new QueryEngineFacadeException("AdvancedQuery over instances error: no database specified"));
		}
		if(advancedQuery == null){
			throw(new QueryEngineFacadeException("AdvancedQuery over instances error: no AdvancedQuery object specified"));
		}
		
		if (dBase.equals(""))
    		dBase = "localhost/default";
		
		try{
			advancedQuery.checkWellFormedNess();
		} catch (QueryEngineFacadeException qefEx) {
			throw(new QueryEngineFacadeException("AdvancedQuery over instances error: " + qefEx.getMessage()));
		}
		
		List<QualityPropertyConstraints> qualityPropertiesConstraints = advancedQuery.getQualityPropertiesConstraints();
		
		if(qualityPropertiesConstraints != null){
			
			int qualityPropertiesConstraintsSize = qualityPropertiesConstraints.size();
			
			for(int i = 0; i < qualityPropertiesConstraintsSize; i++){
				QualityPropertyConstraints qualityPropertyConstraints = qualityPropertiesConstraints.get(i);
				if(qualityPropertyConstraints.getHasValueDescription()){
					throw(new QueryEngineFacadeException("AdvancedQuery over instances error: value descriptions are not allowed"));
				}
			}
		}
		
		List<ConcreteService> concreteServices = retrieveConcreteServicesUsingSQL(dBase, advancedQuery);
    	
    	return concreteServices;
	}
	
	/*
	 * Retrieves AoSBM service interfaces that match a given list of operations (simpleQuery), using sbql
	 */
	private List<ServiceInterface> retrieveInterfacesUsingSBQL(String dBase, SimpleQuery simpleQuery){
		
		List<String> queryOpNames = simpleQuery.getOperationNames();
		
		String sbqlQuery = Lts2SbqlGenerator.generateSBQLQuery(queryOpNames);

		List<ServiceInterface> retrievedInterfaces = executeSBQLQuery(dBase, sbqlQuery);
		
		return retrievedInterfaces;
	}
	
	/*
	 * Retrieves AoSBM service interfaces that match the given operations constraints (advancedQuery), using sbql
	 */
	private List<ServiceInterface> retrieveInterfacesUsingSBQL(String dBase, AdvancedQuery advancedQuery){
		
		String sbqlQuery = Lts2SbqlGenerator.generateSBQLQuery(advancedQuery);

		List<ServiceInterface> retrievedInterfaces = executeSBQLQuery(dBase, sbqlQuery);
		
		return retrievedInterfaces;
	}
	
	/*
	 * Retrieves AoSBM service interfaces using sql
	 */
	private List<ServiceInterface> retrieveInterfacesUsingSQL(String dBase, String sqlQuery) throws QueryEngineFacadeException{
		
		ResultSet interfacesSet;
    	ResultSet instancesSet;
    	ResultSet operationsSet;
    	ResultSet serviceTypeSet;
    	ResultSet messagesSet;
    	ResultSet messageTypesSet;
    	
    	ResultSet instanceRecommendersSet;
    	ResultSet instanceRecommenderQPSet;
    	ResultSet instanceRecommenderQPNamesSet;
    	ResultSet instanceRecommenderNonFuncDescrSet;
    	ResultSet instanceRecommenderQPValueSet;
    	
    	List<ServiceInterface> retrievedInterfaces = new ArrayList<ServiceInterface>();
    	
		try{
			
			interfacesSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
			
			while( interfacesSet.next() ) {
				
	    		int interfaceID = interfacesSet.getInt("SI_ID");
	        	String interfaceName = interfacesSet.getString("SI_NAME");
	        	String interfaceProfile = interfacesSet.getString("SI_PROFILE"); 
	        	String interfaceBahav = interfacesSet.getString("SI_BEHAV");
	        	String interfaceConstr = interfacesSet.getString("SI_CONSTR");
	        	int interfaceSTID = interfacesSet.getInt("SI_ST_ID");
	        
	        	sqlQuery = "SELECT * " +
						  	   "FROM operations OP " +
						       "WHERE OP.OP_SI_ID = " + interfaceID;
	        	
	        	operationsSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
	        	
	        	List<Operation> operationList = new ArrayList<Operation>();
	        	List<ServiceInstance> instancesList = new ArrayList<ServiceInstance>();
	        	
	        	while( operationsSet.next() ) {
	        		
	        		int operationID = operationsSet.getInt("OP_ID");
	        		String operationName = operationsSet.getString("OP_NAME");
	        		String operationProfile = operationsSet.getString("OP_PROFILE");
	        		String operationPre = operationsSet.getString("OP_PRE");
	        		String operationPost = operationsSet.getString("OP_POST");
	        		
	        		Message messageIn = null;
	        		Message messageOut = null;
	        		
	        		sqlQuery = "SELECT * " +
						  	   "FROM messages MS " +
						       "WHERE MS.MS_OP_ID = " + operationID;
	        		
	        		messagesSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
	        		
	        		while( messagesSet.next() ) {
	        			
	        			int messageID = messagesSet.getInt("MS_ID");
	        			String messageName = messagesSet.getString("MS_NAME");
	        			int messageHasMessTypes = messagesSet.getInt("MS_HAS_MESS_TYPES");
	        			String messageMsType = messagesSet.getString("MS_TYPE");
	        			
	        			List<MessageType> messageTypes = new ArrayList<MessageType>();
	        			
	        			sqlQuery = "SELECT * " +
	    					  	   "FROM messagetypes MT " +
	    					       "WHERE MT.MT_MS_ID = " + messageID;
	        			
	        			messageTypesSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
	        			
	        			while( messageTypesSet.next() ) {
	        				
	            			String messageTypeName = messageTypesSet.getString("MT_NAME");
	            			String messageTypeTypeKind = messageTypesSet.getString("MT_TYPEKIND");
	            			String messageTypeType = messageTypesSet.getString("MT_TYPE");
	            			
	            			messageTypes.add(new MessageType(messageTypeName, messageTypeType, messageTypeTypeKind));
	        			}
	  
	        			int messageTypesSize = messageTypes.size();
	        			
	        			Message message = new Message(messageName);
	        			
	        			for(int i = 0; i < messageTypesSize; i++){
	        				message.add(messageTypes.get(i));
	        			}
	        			message.setInOrOut(messageMsType);
	        			
	        			if(messageMsType.equals("IN"))
	        				messageIn = message;
	        			else
	        				messageOut = message;
	        		}
	        		
	        		Operation operation = new Operation(operationName, messageIn, messageOut);
	        		operation.setxP(operationProfile);
	        		operation.setxPre(operationPre);
	        		operation.setxPost(operationPost);
	        		
	        		operationList.add(operation);
	        		
	        	}
	        	
	        	sqlQuery = "SELECT * " +
						  	   "FROM serviceinstances SIS " +
						       "WHERE SIS.SIS_SI_ID = " + interfaceID;
	        	
	        	instancesSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
	        	
	        	while( instancesSet.next() ) {
	        		
	        		int instanceID = instancesSet.getInt("SIS_ID");
	        		String instanceName = instancesSet.getString("SIS_NAME");
	        		String instanceProto = instancesSet.getString("SIS_PROTO");
	        		String instanceURI = instancesSet.getString("SIS_URI");
	        		String instanceImpDet = instancesSet.getString("SIS_IMP_DET");
	        		String instanceReqs = instancesSet.getString("SIS_REQS");
	        		
	        		ServiceInstance serviceInstance = new ServiceInstance(instanceName, instanceProto, instanceURI);
	        		serviceInstance.setDBKey(instanceID);
	        		
	        		QualityProperties qualityProperties = new QualityProperties();
	        		NonFuncDescription nonFuncDescription;
	        		String ref2DocOfFuncsProps = null;
					String sTypeName = null;
					String sInstanceName = null;
	        		
	        		
	        		sqlQuery = "SELECT V.recommender " +
						  	   "FROM si_qp_descr_v V " +
						       "WHERE V.sis_id = " + instanceID;
	        		
	        		instanceRecommendersSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
	        		
	        		while( instanceRecommendersSet.next() ) {
	        			
	        			String recommender = instanceRecommendersSet.getString("recommender");
	        			
	        			sqlQuery = "SELECT DISTINCT V.qp_name " +
	    					  	   "FROM si_qp_descr_v V " +
	    					       "WHERE V.sis_id = " + instanceID + " and V.recommender like '" + recommender.replaceAll("\\\\", "\\\\\\\\\\\\\\\\")  + "'";
	        			
	        			instanceRecommenderQPNamesSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
	        			
	        			while( instanceRecommenderQPNamesSet.next() ) {
	        				
	        				String qpName = instanceRecommenderQPNamesSet.getString("qp_name");
	        				
	        				sqlQuery = "SELECT * " +
	     					  	   "FROM si_qp_descr_v V " +
	     					       "WHERE V.sis_id = " + instanceID  + " and " +
	     					       "V.recommender like '" + recommender.replaceAll("\\\\", "\\\\\\\\\\\\\\\\")  + "' and " +
	     					       "V.qp_name like '" + qpName + "'";
	        				
	        				instanceRecommenderQPSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
	        				ArrayList<String> domain = new ArrayList<String>();
	        				
	        				while( instanceRecommenderQPSet.next() ) {
	        					domain.add(instanceRecommenderQPSet.getString("ranking"));
	        				}
	        				
	        				qualityProperties.add(qpName, domain);
	        			}
	        			
	        			sqlQuery = "SELECT * " +
	  					  	   "FROM nonfuncdescriptions NF " +
	  					       "WHERE NF.NF_SIS_ID = " + instanceID  + " and " +
	  					       "NF.NF_REF2SOURCE_ID like '" + recommender.replaceAll("\\\\", "\\\\\\\\\\\\\\\\")  + "'";
	        			
	        			instanceRecommenderNonFuncDescrSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
	        			
	        			while( instanceRecommenderNonFuncDescrSet.next() ) {
	        				
	        				ref2DocOfFuncsProps = instanceRecommenderNonFuncDescrSet.getString("NF_REF2DOCOFFUNCPROPS");
	        				sTypeName = instanceRecommenderNonFuncDescrSet.getString("NF_STYPE_NAME");
	        				sInstanceName = instanceRecommenderNonFuncDescrSet.getString("NF_SINSTANCE_NAME");
	        			}
	        			
	        			sqlQuery = "SELECT * " +
	   					  	   "FROM si_qp_values_v V " +
	   					       "WHERE V.NF_SIS_ID = " + instanceID  + " and " +
	   					       "V.recommender like '" + recommender.replaceAll("\\\\", "\\\\\\\\\\\\\\\\")  + "'";
	        			
	        			instanceRecommenderQPValueSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
	        			
	        			nonFuncDescription = new NonFuncDescription(recommender, ref2DocOfFuncsProps, sTypeName, sInstanceName);
	        			
	        			while( instanceRecommenderQPValueSet.next() ) {
	        				
	        				String qpName = instanceRecommenderQPValueSet.getString("qp_name");
	        				double qpValue = instanceRecommenderQPValueSet.getDouble("val");
	        				
	        				nonFuncDescription.addPropertyAndValue(qpName, qpValue);
	        			}
	        			
	  					serviceInstance.set(nonFuncDescription, qualityProperties);      
	        		}
	        		
	        		serviceInstance.setxD(instanceImpDet);
	        		serviceInstance.setxR(instanceReqs);
	        		
	        		instancesList.add(serviceInstance);
	        	
	        	}
	        	
	        	ServiceInterface serviceInterface = new ServiceInterface(interfaceName);
	        	
	        	sqlQuery = "SELECT ST.ST_NAME " +
					  	   "FROM servicetypes ST " +
					       "WHERE ST.ST_ID = " + interfaceSTID;
	        	
	        	serviceTypeSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
	        	
	        	while( serviceTypeSet.next() ) {
	        		String serviceTypeName = serviceTypeSet.getString("ST_NAME");
	        		serviceInterface.setServiceTypeName(serviceTypeName);
	        	}
	        	
	        	int operationListSize = operationList.size();
	        	
	        	for(int i = 0; i < operationListSize; i++) {
	        		serviceInterface.add(operationList.get(i));
	        	}
	        	
	        	int instancesListSize = instancesList.size();
	        	for(int i = 0; i < instancesListSize; i++) {
	        		serviceInterface.addServiceInstance(instancesList.get(i));
	        	}
	        	
	        	serviceInterface.setxP(interfaceProfile);
	        	serviceInterface.seteBehavioralSpecification(interfaceBahav);
	        	serviceInterface.setConstraintViolation(interfaceConstr);
	        	
	        	retrievedInterfaces.add(serviceInterface);
	    	}
		
		} catch (SQLException e) {
			throw(new QueryEngineFacadeException("Error at retrieving interfaces using sql: " + e.getMessage()));
		} catch (QueryEngineFacadeException qefEx) {
			throw(new QueryEngineFacadeException("Error at retrieving interfaces using sql: " + qefEx.getMessage()));
		}
		
		return retrievedInterfaces;
	}
	
	/*
	 * Retrieves AoSBM service interfaces that match the given operations constraints (simpleQuery) using sbql, 
	 * keeps only those that match RQs (simpleQuery) and
	 * creates concrete services
	 */
	private List<ConcreteService> retrieveConcreteServicesUsingSBQL(String dBase, SimpleQuery simpleQuery){
		
		List<String> queryOpNames = simpleQuery.getOperationNames();
		String RQs = simpleQuery.getRQs();
	
		List<ServiceInterface> retrievedInterfaces = retrieveInterfacesUsingSBQL(dBase, simpleQuery);
		
		List<ConcreteService> concreteServices = filterInterfaces(retrievedInterfaces, RQs, queryOpNames);
		
		return concreteServices;
	}
	
	/*
	 * Retrieves AoSBM service interfaces that match the given operations constraints (advancedQuery) using sbql, 
	 * keeps only those that match RQs (advancedQuery) and
	 * creates concrete services
	 */
	private List<ConcreteService> retrieveConcreteServicesUsingSBQL(String dBase, AdvancedQuery advancedQuery){
	
		List<ServiceInterface> retrievedInterfaces = retrieveInterfacesUsingSBQL(dBase, advancedQuery);
		
		//retrievedInterfaces = filterInterfacesOverAbstractions(retrievedInterfaces, dBase, advancedQuery.getQualityPropertiesConstraints());
		
		String RQs = advancedQuery.getRQs();
		
		List<OperationConstraints> operationsConstraints = advancedQuery.getOperationsConstraints();
		int operationsConstraintsSize = operationsConstraints.size();
		List<String> queryOpNames = new ArrayList<String>();
		
		for(int i = 0; i < operationsConstraintsSize; i++){
			
			OperationConstraints opConstrs = operationsConstraints.get(i);
			queryOpNames.add(opConstrs.getName());
		}
		
		List<ConcreteService> concreteServices = filterInterfaces(retrievedInterfaces, RQs, queryOpNames);
		
		return concreteServices;
	}
	
	/*
	 * Retrieves AoSBM service interfaces that match the given operations constraints (simpleQuery) using sql, 
	 * keeps only those that match RQs (simpleQuery) and
	 * creates concrete services
	 */
	private List<ConcreteService> retrieveConcreteServicesUsingSQL(String dBase, SimpleQuery simpleQuery) throws QueryEngineFacadeException{
		
		List<String> queryOpNames = simpleQuery.getOperationNames();
		String RQs = simpleQuery.getRQs();
    	
    	String sqlQuery = 	 "SELECT * " +
        					 "FROM serviceinterfaces SI " +
        					 "WHERE SI.SI_ID IN " +
        					 		"(SELECT SI_ID1 " +
        							" FROM ";

        int queryOpNamesSize = queryOpNames.size();
        for(int i = 0; i < queryOpNamesSize; i++){
        	
        	if( i > 0){
        		sqlQuery += " INNER JOIN ";
        	}
        	
        	sqlQuery += 
						"(SELECT OP.OP_SI_ID SI_ID" + (i + 1) + " " +
						" FROM operations OP " +
						" WHERE OP.OP_NAME like '%" + queryOpNames.get(i) + "%') J" + (i + 1) + " ";
        		
        	if( i > 0){
        		sqlQuery += " ON J1.SI_ID1 = J" + (i + 1) + ".SI_ID" + (i + 1) + " ";
        	}
        	
        	
        }
        								
        sqlQuery += ")";
        							
        List<ServiceInterface> retrievedInterfaces;
        
        try{
        	retrievedInterfaces = retrieveInterfacesUsingSQL(dBase, sqlQuery);
        } catch (QueryEngineFacadeException qefEx) {
        	throw(new QueryEngineFacadeException("Error at retrieving concrete services using sql: " + qefEx.getMessage()));
        }
		
		List<ConcreteService> concreteServices = filterInterfaces(retrievedInterfaces, RQs, queryOpNames);
		
		return concreteServices;
	}
	
	/*
	 * Retrieves AoSBM service interfaces that match the given operations constraints (advancedQuery) using sql, 
	 * keeps only those that match RQs (advancedQuery) and
	 * creates concrete services
	 */
	private List<ConcreteService> retrieveConcreteServicesUsingSQL(String dBase, AdvancedQuery advancedQuery) throws QueryEngineFacadeException{
		
		List<OperationConstraints> operationConstraints = advancedQuery.getOperationsConstraints();
		int operationConstraintsSize = operationConstraints.size();
		
		List<QualityPropertyConstraints> qualityPropertiesConstraints = advancedQuery.getQualityPropertiesConstraints();
		
		String sqlQuery = "SELECT * " + "\n" +
				  		  "FROM serviceinterfaces SI " + "\n" +
				  		  "WHERE SI.SI_ID IN " + "\n\n" +
				  		  "\t(SELECT OP_SI_ID1" + "\n" +
				  		  "\tFROM" + "\n\n";
		
		for(int i = 0; i < operationConstraintsSize; i++){
			
			OperationConstraints opConstrs = operationConstraints.get(i);
			
			String operationName = opConstrs.getName();
			List<String> inParamNames = opConstrs.getInParamNames();
			List<String> inParamTypes = opConstrs.getInParamTypes();
			List<String> outParamNames = opConstrs.getOutParamNames();
			List<String> outParamTypes = opConstrs.getOutParamTypes();
			
			if(i > 0)
				sqlQuery += "\n\n" + "\t\tINNER JOIN" + "\n\n";
			
			sqlQuery += "\t\t(SELECT OP.OP_SI_ID OP_SI_ID" + (i + 1) + "\n" +
						"\t\tFROM" + "\n";
			
			if(inParamNames == null && outParamNames == null){
				
				sqlQuery += "\t\t\toperations OP" + "\n" +
							"\t\tWHERE OP.OP_NAME like '%" + operationName + "%') JJJ" + (i + 1);
			} else {
				
				if(inParamNames != null){
					
					sqlQuery += "\t\t\t(SELECT MS.MS_OP_ID OP_ID_IN" + "\n" +
								"\t\t\tFROM" + "\n\n";
					
					int inParamNamesSize = inParamNames.size();
					
					for(int j = 0; j < inParamNamesSize; j++){
						
						String name = inParamNames.get(j);
						String type = inParamTypes.get(j);
						
						if(j > 0)
							sqlQuery += "\n\n" + "\t\t\t\tINNER JOIN" + "\n\n";
						
						sqlQuery += "\t\t\t\t(SELECT MT.MT_MS_ID MS_ID" + (j + 1) + "\n" +
									"\t\t\t\tFROM messagetypes MT" + "\n" +
									"\t\t\t\tWHERE ";
						
						if(name != null && type != null){
							sqlQuery += "MT.MT_NAME like '%" + name + "%' ";
							sqlQuery += "and MT.MT_TYPE like '%" + type + "%' " + "\n";	
						}
						else if(name != null)
							sqlQuery += "MT.MT_NAME like '%" + name + "%' " + "\n";
						else if(type != null)
							sqlQuery += "MT.MT_TYPE like '%" + type + "%' " + "\n";
						
						sqlQuery += "\t\t\t\tGROUP BY MT.MT_MS_ID) J" + (j + 1);
						
						if(j > 0)
							sqlQuery += "\n\n\t\t\t\tON J1.MS_ID1 = J" + (j + 1) + ".MS_ID" + (j + 1);
						
					}
					
					
					sqlQuery += ", messages MS" + "\n\n" +
								"\t\t\tWHERE MS_ID1 = MS.MS_ID and MS.MS_TYPE like '%IN%') JJ_IN";
				}
				
				if(outParamNames != null){
					
					if(inParamNames != null)
						sqlQuery += "\n\n" + "\t\t\tINNER JOIN" + "\n\n";
					
					sqlQuery += "\t\t\t(SELECT MS.MS_OP_ID OP_ID_OUT" + "\n" +
								"\t\t\tFROM" + "\n\n";
					
					int outParamNamesSize = outParamNames.size();
					
					for(int j = 0; j < outParamNamesSize; j++){
						
						String name = outParamNames.get(j);
						String type = outParamTypes.get(j);
						
						if(j > 0)
							sqlQuery += "\n\n" + "\t\t\t\tINNER JOIN" + "\n\n";
						
						sqlQuery += "\t\t\t\t(SELECT MT.MT_MS_ID MS_ID" + (j + 1) + "\n" +
									"\t\t\t\tFROM messagetypes MT" + "\n" +
									"\t\t\t\tWHERE ";
						
						if(name != null && type != null){
							sqlQuery += "MT.MT_NAME like '%" + name + "%' ";
							sqlQuery += "and MT.MT_TYPE like '%" + type + "%' " + "\n";	
						}
						else if(name != null)
							sqlQuery += "MT.MT_NAME like '%" + name + "%' " + "\n";
						else if(type != null)
							sqlQuery += "MT.MT_TYPE like '%" + type + "%' " + "\n";
						
						sqlQuery += "\t\t\t\tGROUP BY MT.MT_MS_ID) J" + (j + 1);
						
						if(j > 0)
							sqlQuery += "\n\t\t\t\tON J1.MS_ID1 = J" + (j + 1) + ".MS_ID" + (j + 1);
						
					}
					
					sqlQuery += ", messages MS" + "\n\n" +
								"\t\t\tWHERE MS_ID1 = MS.MS_ID and MS.MS_TYPE like '%OUT%') JJ_OUT";
				}
				
				if(inParamNames != null && outParamNames != null)
					sqlQuery += "\n\n\t\t\tON JJ_IN.OP_ID_IN = JJ_OUT.OP_ID_OUT, operations OP" + "\n\n" +
								"\t\tWHERE OP_ID_IN" + " = OP.OP_ID and OP.OP_NAME like '%" + operationName + "%') JJJ" + (i + 1);
				else if(inParamNames != null)
					sqlQuery += ", operations OP" + "\n\n" +
								"\t\tWHERE OP_ID_IN" + " = OP.OP_ID and OP.OP_NAME like '%" + operationName + "%') JJJ" + (i + 1);	
				else if(outParamNames != null)
					sqlQuery += ", operations OP" + "\n\n" +
								"\t\tWHERE OP_ID_OUT" + " = OP.OP_ID and OP.OP_NAME like '%" + operationName + "%') JJJ" + (i + 1);
				
			}
		
			if(i > 0)
				sqlQuery += "\n\n\t\tON JJJ1.OP_SI_ID1 = JJJ" + (i + 1) + ".OP_SI_ID" + (i + 1);
				
		}
		
		sqlQuery += ")";
		
		System.out.println(sqlQuery);
		
		List<ServiceInterface> retrievedInterfaces;
		
		try {
			retrievedInterfaces = retrieveInterfacesUsingSQL(dBase, sqlQuery);
		} catch (QueryEngineFacadeException qefEx) {
			throw(new QueryEngineFacadeException("Error at retrieving concrete services using sql: " + qefEx.getMessage()));
		}
		
		String RQs = advancedQuery.getRQs();
		
		List<OperationConstraints> operationsConstraints = advancedQuery.getOperationsConstraints();
		int operationsConstraintsSize = operationsConstraints.size();
		List<String> queryOpNames = new ArrayList<String>();
		
		for(int i = 0; i < operationsConstraintsSize; i++){
			
			OperationConstraints opConstrs = operationsConstraints.get(i);
			queryOpNames.add(opConstrs.getName());
		}
		
		List<ConcreteService> concreteServices = filterInterfaces(filterInterfaces(retrievedInterfaces, qualityPropertiesConstraints), RQs, queryOpNames);
		
		return concreteServices;
	}
	
	
	/* 
	 * Finds out what instances of given interfaces match given RQs and returns new (resulted) interfaces that represent
	   these instances
	*/
	private List<ConcreteService> filterInterfaces(List<ServiceInterface> serviceInterfaces, String RQs, List<String> queryOpNames){
		
		if(serviceInterfaces == null)
			return null;
		
		List<ServiceInterface> resultedInterfaces = new ArrayList<ServiceInterface>();
		
		if(RQs == null){
			
			resultedInterfaces = serviceInterfaces;
		}
		else{
			
			int serviceInterfacesSize = serviceInterfaces.size();
			
			for(int i = 0; i < serviceInterfacesSize; i++){
				
				List<ServiceInstance> resultedInstances = new ArrayList<ServiceInstance>();
				
				ServiceInterface serviceInterface = serviceInterfaces.get(i);
				List<ServiceInstance> allInstances = serviceInterface.getServiceInstances();
				
				int allInstancesSize = allInstances.size();
					
				for(int j = 0; j < allInstancesSize; j++){
					ServiceInstance instance = allInstances.get(j);
					String instanceRQs = instance.getRequirements();
					
					if( equalRQs(RQs, instanceRQs) ){
						resultedInstances.add(instance);
					}
				}
				
				if(resultedInstances.size() != 0){
					ServiceInterface modifiedInterface = new ServiceInterface(serviceInterface);
					modifiedInterface.setxServiceInstances(resultedInstances);
					
					resultedInterfaces.add(modifiedInterface);
				}
					
			}
		}
		
		int resultedInterfacesSize = resultedInterfaces.size();
		List<ConcreteService> concreteServices = new ArrayList<ConcreteService>();
		
		for(int i = 0; i < resultedInterfacesSize; i++)
			concreteServices.add(new ConcreteService(queryOpNames, resultedInterfaces.get(i)));
    	
    	return concreteServices;
		
	}
	
	private boolean equalRQs(String RQs1, String RQs2){
		
		String [] reqs1 = RQs1.split(";");
		String [] reqs2 = RQs2.split(";");
		
		Boolean areEqual = true;
		
		for(int i = 0; i < reqs1.length; i++){
			
			String req = reqs1[i].replaceAll("\\s", "");
			Boolean equalFound = false;
			
			for(int j = 0; j < reqs2.length; j++){
				
				if( reqs2[j].replaceAll("\\s", "").equals(req) ){
					equalFound = true;
					break;
				}
			}
			
			if(equalFound == false){
				areEqual = false;
				break;
			}
		
		}
		
		return areEqual;
	}
	
	private List<ServiceInterface> filterInterfaces(List<ServiceInterface> serviceInterfaces, List<QualityPropertyConstraints> qualityPropertiesConstraints){
		
		if(serviceInterfaces == null)
			return null;
		
		List<ServiceInterface> resultedInterfaces = new ArrayList<ServiceInterface>();
		
		if(qualityPropertiesConstraints == null){
			
			resultedInterfaces = serviceInterfaces;
		}
		else{
			
			int qualityPropertiesConstraintsSize = qualityPropertiesConstraints.size();
			
			int serviceInterfacesSize = serviceInterfaces.size();
			
			for(int i = 0; i < serviceInterfacesSize; i++){
				
				List<ServiceInstance> resultedInstances = new ArrayList<ServiceInstance>();
				
				ServiceInterface serviceInterface = serviceInterfaces.get(i);
				List<ServiceInstance> allInstances = serviceInterface.getServiceInstances();
				
				int allInstancesSize = allInstances.size();
					
				for(int j = 0; j < allInstancesSize; j++){
					
					ServiceInstance instance = allInstances.get(j);
					
					Map<String, QualityProperties> qualityProperties = instance.getQualityProperties();
					Map<String, NonFuncDescription> nonFuncDescriptions = instance.getNonFuncDescriptions();
					
					boolean instanceSatisfiesQPConstraints = true;
					
					for(int k = 0; k < qualityPropertiesConstraintsSize; k++){
						
						QualityPropertyConstraints qualityPropertyConstraints = qualityPropertiesConstraints.get(k);
						
						String qpName = qualityPropertyConstraints.getName();
						boolean hasValueDescription = qualityPropertyConstraints.getHasValueDescription();
						
						boolean qpConstrSatisfied = false;
						
						if(hasValueDescription){
								
							String qpValueDescription = qualityPropertyConstraints.getValueDescription();
							
							qpConstrSatisfied = true;
						}
						else {
							
							double qpValueFrom = qualityPropertyConstraints.getValueFrom();
							double qpValueTo = qualityPropertyConstraints.getValueTo();
							
							for (Map.Entry<String, NonFuncDescription> entry : nonFuncDescriptions.entrySet()) {
							    
							    NonFuncDescription nfDescription = entry.getValue();
							    Hashtable<String, Double> propertiesValues = nfDescription.getPropertiesValues();
							    
							    for (Map.Entry<String, Double> entry2 : propertiesValues.entrySet()) {
								    
								    if(entry2.getKey().replaceAll("\\s","").toLowerCase().contains(qpName.replaceAll("\\s","").toLowerCase())){
								    	
								    	if(entry2.getValue() >= qpValueFrom && entry2.getValue() <= qpValueTo){
								    		qpConstrSatisfied = true;
								    		
								    		break;
								    	}
								    }
								}
							    
							    if(qpConstrSatisfied)
							    	break;
							}
						}
						
						if(!qpConstrSatisfied){
							instanceSatisfiesQPConstraints = false;
							break;
						}
					}
					
					if(instanceSatisfiesQPConstraints)
						resultedInstances.add(instance);
					
				}
				
				if(resultedInstances.size() != 0){
					ServiceInterface modifiedInterface = new ServiceInterface(serviceInterface);
					modifiedInterface.setxServiceInstances(resultedInstances);
					
					resultedInterfaces.add(modifiedInterface);
				}
					
			}
		}
		
    	return resultedInterfaces;
		
	}
	
	/*private List<ServiceInterface> filterInterfacesOverAbstractions(List<ServiceInterface> serviceInterfaces, String dBase, List<QualityPropertyConstraints> qualityPropertiesConstraints){
		
		if(serviceInterfaces == null)
			return null;
		
		List<ServiceInterface> resultedInterfaces = new ArrayList<ServiceInterface>();
		
		if(qualityPropertiesConstraints == null){
			
			resultedInterfaces = serviceInterfaces;
		}
		else{
			
			int qualityPropertiesConstraintsSize = qualityPropertiesConstraints.size();
			
			int serviceInterfacesSize = serviceInterfaces.size();
			
			for(int i = 0; i < serviceInterfacesSize; i++){
				
				List<ServiceInstance> resultedInstances = new ArrayList<ServiceInstance>();
				
				ServiceInterface serviceInterface = serviceInterfaces.get(i);
				List<ServiceInstance> allInstances = serviceInterface.getServiceInstances();
				
				int allInstancesSize = allInstances.size();
					
				for(int j = 0; j < allInstancesSize; j++){
					
					ServiceInstance instance = allInstances.get(j);
					
					String sqlQuery =	
			  				  			"SELECT S.nfa_id " + "\n" +
			  				  			"FROM " + "\n" +
			  				  			"\t(SELECT N.nfa_id, COUNT(*) C " + "\n" +
			  				  			"\tFROM nfaqp_v N" + "\n" +
			  				  			"\tWHERE " + "\n";
					
					for(int k = 0; k < qualityPropertiesConstraintsSize; k++){
						
						QualityPropertyConstraints qualityPropertyConstraints = qualityPropertiesConstraints.get(k);
						
						String qpName = qualityPropertyConstraints.getName();
						boolean hasValueDescription = qualityPropertyConstraints.getHasValueDescription();
						
						if(hasValueDescription){
								
							String qpValueDescription = qualityPropertyConstraints.getValueDescription();
							
							sqlQuery += "\t\tN.qp_name like '%" + qpName + "%' and N.qp_value like '%" + qpValueDescription + "%'\n";
						}
						else {
							
							double qpValueFrom = qualityPropertyConstraints.getValueFrom();
							double qpValueTo = qualityPropertyConstraints.getValueTo();
							
							sqlQuery += "\t\tN.qp_name like '%" + qpName + "%' and N.value_from <= " + qpValueFrom + " and N.value_to >= " + qpValueTo + "\n";
						}
						
						if(k < (qualityPropertiesConstraintsSize - 1))
							sqlQuery += "\t\tor\n";
					}
					
					sqlQuery += "\tGROUP BY N.nfa_id) S" + "\n" +
								"WHERE S.C = " + qualityPropertiesConstraintsSize;
					
					try{
						
						ResultSet nfaKeysSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
						ArrayList<Integer> nfaKeys = new ArrayList<Integer>();
						ArrayList<Integer> resultedNfaKeys = new ArrayList<Integer>();
								
						while( nfaKeysSet.next() ) {
							
							nfaKeys.add(nfaKeysSet.getInt("nfa_id"));	
						}
						
						int nfaKeysSize = nfaKeys.size();
						
						for(int n = 0; n < nfaKeysSize; n++){
							
							int key = nfaKeys.get(n);
							
							for(int p = 0; p < nfaKeysSize; p++){
								
								int anotherKey = nfaKeys.get(p);
								
								sqlQuery = "SELECT * FROM hier_edges H WHERE H.AD_ANC_ID = " + anotherKey + " and H.AD_DES_ID = " + key;
								ResultSet hierEdgesSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
								if(!hierEdgesSet.next())
									resultedNfaKeys.add(key);	
							}
						}
						
						sqlQuery = "SELECT RIS.RIS_ID FROM representedinstances RIS WHERE RIS.RIS_NFA_ID IN (";
						
						int resultedNfaKeysSize = resultedNfaKeys.size();
						
						for(int n = 0; n < resultedNfaKeysSize; n++){
							
							sqlQuery += resultedNfaKeys.get(n);
							
							if(n < (resultedNfaKeysSize - 1))
								sqlQuery += ", ";
						}
						
						sqlQuery += ")";
						
						ResultSet instancesKeysSet = MySQLTransaction.executeQuery(dBase, sqlQuery);
						
						while( instancesKeysSet.next() ) {
							
							int instanceKey = instancesKeysSet.getInt("RIS_ID");
							if(instance.getDBKey() == instanceKey)
								resultedInstances.add(instance);
						}
						
					} catch(SQLException e) {
						e.printStackTrace();
						JOptionPane.showMessageDialog(null, e.getMessage(), "error", JOptionPane.ERROR_MESSAGE);
					    return null;
					}
								
				}
				
				if(resultedInstances.size() != 0){
					ServiceInterface modifiedInterface = new ServiceInterface(serviceInterface);
					modifiedInterface.setxServiceInstances(resultedInstances);
					
					resultedInterfaces.add(modifiedInterface);
				}
					
			}
		}
		
    	return resultedInterfaces;
		
	}*/
	
	
	/*
	 * Retrieves AoSBM service interfaces that match a given LTS spec.
	 */	
	private List<ServiceInterface> retrieveData(String dBase, String participantLTS){
		
		List<ServiceInterface> result;
		
		if(dBase == null || participantLTS == null){
			System.err.println("retrieveData parameters must be != null");
			return null;
		}
		
		String sbqlQuery = Lts2SbqlGenerator.generateSQBLQuery(participantLTS);

		result = executeSBQLQuery(dBase, sbqlQuery);
		return result;
	}
	
	/*
	 * Retrieves AoSBM service interfaces that match a given SBQL query.
	 */
	private List<ServiceInterface> executeSBQLQuery(String dBase, String sbqlQuery) {
		
		List<ServiceInterface> result = new ArrayList<ServiceInterface>();
		
		QueryEngineController queryEngineController = new QueryEngineController();
		queryEngineController.executeLookupQuery(dBase, sbqlQuery);
		
		List<FunctionalAbstraction> faQueryResult = queryEngineController.getRetrievedFunctionalAbstractions();
			
		for(int i = 0; i < faQueryResult.size(); i++){
			FunctionalAbstraction fa = faQueryResult.get(i);
			if(fa.getAncestorId() == -1)
				result.addAll(fa.getRepresentedInterfaces());
		}
		return result;
	}
	
}




