package org.ow2.choreos.queryEngineServiceDeployment;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;

import org.ow2.choreos.abstractionBase.FunctionalAbstraction;
import org.ow2.choreos.abstractionBase.NonFunctionalAbstraction;
import org.ow2.choreos.queryEngineFacade.AdvancedQuery;
import org.ow2.choreos.queryEngineFacade.ConcreteService;
import org.ow2.choreos.queryEngineFacade.OperationConstraints;
import org.ow2.choreos.queryEngineFacade.QualityPropertyConstraints;
import org.ow2.choreos.queryEngineFacade.QueryEngineFacadeException;
import org.ow2.choreos.queryEngineFacade.SimpleQuery;
import org.ow2.choreos.serviceRepresentation.functional.Operation;

/**
 * @author pilios
 * 
 */

public class ConcreteExample {
	
	public static void main(String [] args){
		
		//Start a server thread which serves the query engine (RESTful) service
		//Server's characteristics, like where it is addressed, are defined inside QueryEngineServiceServer class
		
		QueryEngineServiceServer.start();
		
		//Create a URL address and set up a client, which will consume operations of the query engine service
		//The URL address is the address of the server thread that is running
		
		String port = "8080";
	    String url = "http://localhost:" + port + "/servicedeployer/";
		QueryEngineServiceClient client = new QueryEngineServiceClient(url);
		client.run();
		
		
		//Now you can:
		//Create a query object, execute query using the appropriate client's method, 
		//get the response and, eventually, the QueryEngineServiceResponse object.
		
		
		
		String results;
		String underLine = "\n______________________________________\n";
		
		
		
		//---------------------------------------------------- Examples ------------------------------------------------------
		
		
		
		//SBQL query example

		String sbqlQuery = produceSBQLQuery();
		Response responseToSBQLQuery = client.executeSBQLQuery(sbqlQuery);
		QueryEngineServiceResponse qeResponseToSBQLQuery = getQueryEngineServiceResponse(responseToSBQLQuery);
		results = getQueryEngineResponseInfo(qeResponseToSBQLQuery, true, null, null);
		System.out.println(underLine + "SBQLQuery results" + underLine);
		System.out.print(results);
		
		
		
		
		//Simple query example
		
		SimpleQuery simpleQuery = produceSimpleQuery();
		
		Response responseToSimpleQuery = client.executeSimpleQueryOverInstances(simpleQuery);
		QueryEngineServiceResponse qeResponseToSimpleQuery = getQueryEngineServiceResponse(responseToSimpleQuery);
		results = getQueryEngineResponseInfo(qeResponseToSimpleQuery, false, simpleQuery, null);
		System.out.println(underLine + "SimpleQueryOverInstances results" + underLine);
		System.out.print(results);
		
		responseToSimpleQuery = client.executeSimpleQueryOverAbstractions(simpleQuery);
		qeResponseToSimpleQuery = getQueryEngineServiceResponse(responseToSimpleQuery);
		results = getQueryEngineResponseInfo(qeResponseToSimpleQuery, false, simpleQuery, null);
		System.out.println(underLine + "SimpleQueryOverAbstractions results" + underLine);
		System.out.print(results);
		
		
		
		
		//Advanced query example
		
		AdvancedQuery advancedQuery = produceAdvancedQuery();
		
		Response responseToAdvancedQuery = client.executeAdvancedQueryOverInstances(advancedQuery);
		QueryEngineServiceResponse qeResponseToAdvancedQuery = getQueryEngineServiceResponse(responseToAdvancedQuery);
		results = getQueryEngineResponseInfo(qeResponseToSimpleQuery, false, null, advancedQuery);
		System.out.println(underLine + "AdvancedQueryOverInstances results" + underLine);
		System.out.print(results);
		
		responseToAdvancedQuery = client.executeAdvancedQueryOverAbstractions(advancedQuery);
		qeResponseToAdvancedQuery = getQueryEngineServiceResponse(responseToAdvancedQuery);
		results = getQueryEngineResponseInfo(qeResponseToSimpleQuery, false, null, advancedQuery);
		System.out.println(underLine + "AdvancedQueryOverAbstractions results" + underLine);
		System.out.print(results);
		
	}
	
	private static String getQueryEngineResponseInfo(QueryEngineServiceResponse qeResponse, boolean isAResponseToSBQLQuery, SimpleQuery simpleQuery, AdvancedQuery advancedQuery){
		
    	Status status = qeResponse.getHttpStatus();
    	
    	if(status != Status.OK) {
    		JOptionPane.showMessageDialog(null, status, "error", JOptionPane.ERROR_MESSAGE);
    		return "No results";
    	}
    	
    	String errorMessage = qeResponse.getErrorMessage();
    	
    	if(errorMessage != null) {
    		JOptionPane.showMessageDialog(null, errorMessage, "error", JOptionPane.ERROR_MESSAGE);
    		return "No results";
    	}
    	
    	if(isAResponseToSBQLQuery){
    		
    		List<FunctionalAbstraction> functionalAbstractions = qeResponse.getFunctionalAbstractions();
        	List<NonFunctionalAbstraction> nonFunctionalAbstractions = qeResponse.getNonFunctionalAbstractions();
        	
        	String result = "";
        	
        	if(functionalAbstractions != null) {
        		
            	int faListSize = functionalAbstractions.size();
            	
            	if(faListSize != 0) {
            		result = "\n---------------------------------- Functional Abstractions ---------------------------------\n\n";
        	    	for(int i = 0; i < faListSize; i++) {
        	    		FunctionalAbstraction fa = functionalAbstractions.get(i);
        	    		result += (i + 1) + "\tinterface name: " + fa.getInterface().getName() + "\n";
        	    	}
            	}
            	
        	}
        	
        	if(nonFunctionalAbstractions != null) {
            	
        		int nfaListSize = nonFunctionalAbstractions.size();
        		
            	if(nfaListSize != 0) {
            		result += "\n\n---------------------------------- Non Functional Abstractions ----------------------------------\n\n";
        	    	for(int i = 0; i < nfaListSize; i++) {
        	    		NonFunctionalAbstraction nfa = nonFunctionalAbstractions.get(i);
        	    		result += (i + 1) + "\tid: " + nfa.getNonFunctionalAbstractionID() + "\n";
        	    	}
            	}
            	
            	result += "\n\n";
        	}
        	
        	return result;
    	}
    	
    	List<ConcreteService> queryResults = qeResponse.getConcreteServices();
    	
    	if(queryResults == null)
    		return "Number of concrete services found : 0";
    	
    	String result = "";
    	result += "Number of concrete services found : " + queryResults.size() + "\n\n";
    	
		for(int i = 0; i < queryResults.size(); i++){
			ConcreteService concreteService = queryResults.get(i);
			
			result += "\nConcrete Service: " + (i + 1) + "\n\n";
			result += "LTS URI: " + concreteService.getBehavioralSpecificationURI() + "\n";
			result += "ENC URI: " + concreteService.getEnactementURI() + "\n";
			result += "Mappings:" + "\n";
			
			List<String> queryOperations = null;
			
			if(simpleQuery != null)
				queryOperations = simpleQuery.getOperationNames();
			else
				queryOperations = advancedQuery.getOperationNames();
			
			for(int j = 0; j < queryOperations.size(); j++){
				result += "\tQuery operation: " + queryOperations.get(j) + "\n";
				List<Operation> retrievedOperations = concreteService.getRetrievedOperations(queryOperations.get(j));
				
				if(retrievedOperations != null){
					for(int k = 0; k < retrievedOperations.size(); k++){
						result += "\t\t" + retrievedOperations.get(k).getName() + "\n";
					}
				}
			}
			
			List<String> retrievedURIs = concreteService.getServiceInstancesURIs();
			result += "\nService Instances URIs:" + "\n";
			for(int j = 0; j < retrievedURIs.size(); j++)
				result += "\t" + retrievedURIs.get(j) + "\n";
			
			result += "\n";
		}
		
		return result;
	}
	
	private static QueryEngineServiceResponse getQueryEngineServiceResponse(Response response) {
    	
    	QueryEngineServiceResponse qeResponse = null;
    	
    	try {
    		qeResponse = unMarshal(response);
			
		} catch (WebApplicationException e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, e.getMessage(), "error", JOptionPane.ERROR_MESSAGE);
		} catch (JAXBException e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, e.getMessage(), "error", JOptionPane.ERROR_MESSAGE);
		}
    	
    	return qeResponse;
    }
	
	private static QueryEngineServiceResponse unMarshal(Response response) throws WebApplicationException, JAXBException{
    	
    	QueryEngineServiceResponse qeResponse;
    	
    	try {
			InputStream in = (InputStream) response.getEntity();
			
			JAXBContext context = JAXBContext.newInstance(QueryEngineServiceResponse.class);
			Unmarshaller unmarshaller = context.createUnmarshaller();    
			qeResponse = (QueryEngineServiceResponse) unmarshaller.unmarshal(in);
			
		} catch (WebApplicationException e) {
		    throw(e);
		} catch (JAXBException e) {
			throw(e);
		}
    	
    	return qeResponse;
    }
	
	private static String produceSBQLQuery(){
		
		String sbqlQuery = 
				
				"let		$db = db('localhost/mySB')" + "\n" + 
				"for		$c in $db/servicecollections" + "\n" + 
				"for 		$fa in $c/hierarchies/abstractions" + "\n" + 
				"for 		$ri in $fa/representativeinterfaces" + "\n" + 
				"for		$nfa in $c/hierarchies/abstractions" + "\n" + 
				"for		$pr1 in $nfa/qproperty" + "\n" + 
				"where" + "\n" + 
							"$ri/rsi_name like '%orecast%' and" + "\n" + 
							"$pr1/qp_name = 'Availability' and" + "\n" + 
							"$pr1/qp_value = 'High'" + "\n" + 
				"return" + "\n" + 
							"Abstractions.fullObject";
		
		return sbqlQuery;
	}
	
	private static SimpleQuery produceSimpleQuery(){
		
		List<String> operations = new ArrayList<String>();
		
		operations.add("request");
		operations.add("get");
		
		String RQs = null;
		
		SimpleQuery simpleQuery = new SimpleQuery(operations, RQs);
		
		return simpleQuery;
		
	}
	
	private static AdvancedQuery produceAdvancedQuery(){
		
		String opName1 = "request";
		
		List<String> inParamNames1 = new ArrayList<String>();
		inParamNames1.add("parameters");
		
		List<String> inParamTypes1 = new ArrayList<String>();
		inParamTypes1.add("string");
		
		List<String> outParamNames1 = new ArrayList<String>();
		outParamNames1.add("parameters");
		
		List<String> outParamTypes1 = new ArrayList<String>();
		outParamTypes1.add("string");
		
		String opName2 = "get";
		
		List<String> inParamNames2 = new ArrayList<String>();
		inParamNames2.add("parameters");
		
		List<String> inParamTypes2 = new ArrayList<String>();
		inParamTypes2.add("string");
		
		List<String> outParamNames2 = new ArrayList<String>();
		outParamNames2.add("parameters");
		
		List<String> outParamTypes2 = new ArrayList<String>();
		outParamTypes2.add("string");

		OperationConstraints opConstr1 = new OperationConstraints(opName1, inParamNames1, inParamTypes1, outParamNames1, outParamTypes1);
		OperationConstraints opConstr2 = new OperationConstraints(opName2, inParamNames2, inParamTypes2, outParamNames2, outParamTypes2);
		
		List<OperationConstraints> operationsConstraints = new ArrayList<OperationConstraints>();
		operationsConstraints.add(opConstr1);
		operationsConstraints.add(opConstr2);
		
		QualityPropertyConstraints qpConstr1 = new QualityPropertyConstraints("ResponseTime", 700, 800);
		QualityPropertyConstraints qpConstr2 = new QualityPropertyConstraints("Availability", 40, 50);
		
		List<QualityPropertyConstraints> qualityPropertyConstraints = new ArrayList<QualityPropertyConstraints>();
		qualityPropertyConstraints.add(qpConstr1);
		qualityPropertyConstraints.add(qpConstr2);
		
		AdvancedQuery advancedQuery = new AdvancedQuery(operationsConstraints, qualityPropertyConstraints, null);
		
		return advancedQuery;
	
	}

}
