package org.ow2.choreos.schemaRegistration;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;


/**
 * 
 * @author dathanas
 *
 */
class SchemaCollection {


	//Variables.
	private Integer collectionID;

	private List<SchemaSource> sources;
	private boolean hasSources;

	private Map<String, Boolean> hasTransformedSources;//The 'String' is the 'schemaRepresentationmodel'.

	private double sumDuration, depth, sumSize, sumMemory;//Statistics.

	private int numOfParsedSchemaDocs;

	private String constraintViolation;
	private boolean hasConstraintViolation;


	//Constructor.
	protected SchemaCollection( Integer cCollectionID ){

		hasSources = false;

		sumDuration = 0.0;

		sumMemory = 0.0;

		sumSize = 0.0;

		hasConstraintViolation = false;

		collectionID = cCollectionID;


		PrintMessages.print( PrintMessages.registrationPrint, hasConstraintViolation(), this );
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	void add( String ref2Source, List<Object> docsOfSchemas ){

		SchemaSource schemaSource = new SchemaSource( ref2Source, docsOfSchemas );

		add( schemaSource );
	}

	private void add( SchemaSource schemaSource ){

		if( ObjectConstraintsManipulation.setContainmentError( this, schemaSource ) == null ){

			if( ! hasSources ){

				sources = new ArrayList<SchemaSource>();

				hasSources = true;
			}

			sumDuration += schemaSource.getDuration();

			sumMemory += schemaSource.getMemory();

			sumSize += schemaSource.getSize();

			numOfParsedSchemaDocs += schemaSource.getNumOfParsableSchemaDocs();

			sources.add( schemaSource );
		}
	}


	/*void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedSources= " + MapManipulation.getBooleanValueByKey( targetModel, hasTransformedSources ) + " hasConstraintViolation= " + hasConstraintViolation );

		if( hasSources && ! MapManipulation.getBooleanValueByKey( targetModel, hasTransformedSources ) && ! hasConstraintViolation ){

			for( int i = 0; i < sources.size(); ++i ){

				sources.get( i ).transform( targetModel );

				if( ! sources.get( i ).hasTransformedDocsOfFuncProperties( targetModel ) ) return;
			}

			if( hasTransformedSources == null ) hasTransformedSources = new Hashtable<String, Boolean>();

			hasTransformedSources.put( targetModel, true );
		}

		PrintMessages.print( PrintMessages.transformationPrint, "END - transform: " + getTag() + " - hasTransformedSources= " + MapManipulation.getBooleanValueByKey( targetModel, hasTransformedSources ) + " hasConstraintViolation= " + hasConstraintViolation );
	}*/

	public void updateSource(){//TODO
		
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any 'checkConstraint()' method.


		for( int i = 0; sources != null && i < sources.size(); ++i ) if( sources.get( i ).hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, sources.get( i ) );


		return constraintViolation;
	}

	public int getCollectionID(){

		return collectionID;
	}

	public boolean isEmpty(){

		return sources == null || ( sources != null && sources.isEmpty() ) ? true : false;
	}

	public List<SchemaSource> getSources(){

		return sources;
	}

	String getSingleRef2Source(){

		return sources != null && sources.size() == 1 ? sources.get( 0 ).getRef2Source() : null;
	}

	public String getTag(){

		return "schema collection '" + collectionID + "'";
	}

	public boolean hasTransformedSources( String targetModel ){

		return MapGetter.getBooleanValueByKey( targetModel, hasTransformedSources );
	}

	double getDuration(){

		return sources != null && (double) sources.size() != 0 ? sumDuration / (double) sources.size() : 0.0;
	}

	double getMemory(){

		return sources != null && (double) sources.size() != 0 ? sumMemory / (double) sources.size() : 0.0;
	}

	double getSize(){

		return sources != null && (double) sources.size() != 0 ? sumSize / (double) sources.size() : 0.0;
	}

	double getDuration( String ref2Source, String ref2DocOfSchemas ){

		SchemaSource schemaSource = null;


		return ( schemaSource = lookUpSource( ref2Source ) ) != null ? schemaSource.getDuration( ref2DocOfSchemas ) : 0.0;
	}

	int getNumOfParsedSchemaDocs(){

		return numOfParsedSchemaDocs;
	}

	SchemaSource lookUpSource( String ref2Source ){

		for( int i = 0; sources != null && i < sources.size(); ++i ) if( sources.get( i ).getRef2Source().equals( ref2Source ) ) return sources.get( i );


		return null;
	}

	Object lookUpDocOfSchema( String ref2Source, String ref2DocOfSchemas ){

		SchemaSource schemaSource = null;


		return ( schemaSource = lookUpSource( ref2Source ) ) != null ? schemaSource.lookUp( ref2DocOfSchemas ) : null;
	}

	Object lookUpDocOfSchema( String ref2DocOfSchemas ){

		Object docOfSchema = null;


		for( int i = 0; hasSources && i < sources.size(); ++i ){

			docOfSchema = sources.get( i ).lookUp( ref2DocOfSchemas );

			if( docOfSchema != null ) return docOfSchema;
        }


		return docOfSchema;
	}

	List<String> getNamesOfDocOfSchemas(){

		List<String> docsOfSchemas = null;


		for( int i = 0; hasSources && i < sources.size(); ++i ){

			if( docsOfSchemas == null ) docsOfSchemas = new ArrayList<String>();

			docsOfSchemas.addAll( sources.get( i ).getNamesOfDocOfSchemas() );
        }


    	return docsOfSchemas;
    }

	List<String> getRefs2DocsOfSchemas(){

		List<String> refs2DocsOfSchemas = null;


		for( int i = 0; hasSources && i < sources.size(); ++i ){

			if( refs2DocsOfSchemas == null ) refs2DocsOfSchemas = new ArrayList<String>();

			refs2DocsOfSchemas.addAll( sources.get( i ).getRefs2DocsOfSchemas() );
        }


    	return refs2DocsOfSchemas;
    }

	String getSchemaRepresentationModel(){

		String schemaRepresentationModel = null;


		for( int i = 0; hasSources && i < sources.size(); ++i ) if( ( schemaRepresentationModel = sources.get( i ).getSchemaRepresentationModel() ) != null ) return schemaRepresentationModel;


		return schemaRepresentationModel;
    }


	//Visualization.
	DefaultMutableTreeNode visualize( DefaultMutableTreeNode root, boolean withContents ){

		DefaultMutableTreeNode node1 = new DefaultMutableTreeNode( "Collection " + collectionID );

		if( root != null ) root.add( node1 );

		else root = node1;


		if( hasSources && withContents ){

			DefaultMutableTreeNode node2 = new DefaultMutableTreeNode( "Schema Sources" ); node1.add( node2 );

			for( int i = 0; i < sources.size(); ++i ){

				DefaultMutableTreeNode child1 = sources.get( i ).visualize();

				node2.add( child1 );
			}
        }


    	return root;
    }
}
