package org.ow2.choreos.schemaRegistration;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;


/**
 * 
 * @author dathanas
 *
 */
class SchemaSource {


	//Variables.
	private String ref2Source;

	private List<Object> docsOfSchemas;
	private boolean hasDocsOfSchemas;

	private Map<String, Boolean> hasTransformedDocsOfFuncProperties;//The 'String' is the 'schema representation model'.

	private int parsedDocs;
	private int parsableDocs;

	private double sumDuration, depth, sumSize, sumMemory;//Statistics.

	private String constraintViolation;
	private boolean hasConstraintViolation;


	//Constructor.
	protected SchemaSource( String cRef2Source, List<Object> docsOfSchemas ){

		ref2Source = cRef2Source;

		parsedDocs = 0;

		parsableDocs = 0;

		sumDuration = 0.0;

		sumMemory = 0.0;

		sumSize = 0.0;

		hasDocsOfSchemas = false;

		hasConstraintViolation = false;


		for( int i = 0; docsOfSchemas != null && i < docsOfSchemas.size(); ++i ) add( docsOfSchemas.get( i ) );

		sumDuration = parsableDocs != 0 ? sumDuration / (double) parsableDocs : 0.0;

		sumMemory = parsableDocs != 0 ? sumMemory / (double) parsableDocs : 0.0;

		sumSize = parsableDocs != 0 ? sumSize / (double) parsableDocs : 0.0;


		PrintMessages.print( PrintMessages.organizationPrint, hasConstraintViolation(), this );
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}


		return constraintViolation;
	}

	protected void add( Object docOfSchema ){

		if( ObjectConstraintsManipulation.setContainmentError( this, docOfSchema ) == null ){

			if( ! hasDocsOfSchemas ){

				docsOfSchemas = new ArrayList<Object>();

				hasDocsOfSchemas = true;
			}


			++parsedDocs;

			if( ! SchemaRepresentationController.hasConstraintViolation( docOfSchema ) ){

				++parsableDocs;


				sumDuration += SchemaRepresentationController.getDuration( docOfSchema );

				sumMemory += SchemaRepresentationController.getMemory( docOfSchema );

				sumSize += SchemaRepresentationController.getSize( docOfSchema );
			}


			docsOfSchemas.add( docOfSchema );
		}
	}

	/*void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedDocsOfFuncProperties= " + MapManipulation.getBooleanValueByKey( targetModel, hasTransformedDocsOfFuncProperties ) + " hasConstraintViolation= " + hasConstraintViolation );

		if( ! MapManipulation.getBooleanValueByKey( targetModel, hasTransformedDocsOfFuncProperties ) && ! hasConstraintViolation ){

	        for( int i = 0; hasDocsOfSchemas && i < docsOfSchemas.size(); ++i ){

				docsOfSchemas.get( i ).transform( targetModel );

				if( ! docsOfSchemas.get( i ).hasTransformedServiceTypes( targetModel ) ) return;
			}

			if( hasTransformedDocsOfFuncProperties == null ) hasTransformedDocsOfFuncProperties = new Hashtable<String, Boolean>();

			hasTransformedDocsOfFuncProperties.put( targetModel, true );
		}

		PrintMessages.print( PrintMessages.transformationPrint, "END - transform: " + getTag() + " - hasTransformedDocsOfFuncProperties= " + MapManipulation.getBooleanValueByKey( targetModel, hasTransformedDocsOfFuncProperties ) + " hasConstraintViolation= " + hasConstraintViolation );
	}*/


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any 'checkConstraint()' method.


		for( int i = 0; hasDocsOfSchemas && i < docsOfSchemas.size(); ++i ) if( SchemaRepresentationController.hasFinalConstraintViolation( docsOfSchemas.get( i ) ) ) ObjectConstraintsManipulation.setContainmentError( this, docsOfSchemas.get( i ) );


		return constraintViolation;
	}

	public String getTag(){

		return "source '" + getName() + "'";
	}

	public String getName(){

		String name = null;


		int pos = ref2Source.lastIndexOf("\\");

		if( pos > 0 ) name = ref2Source.substring( pos + 1 );


		return name;
	}

	public boolean hasTransformedDocsOfFuncProperties( String targetModel ){

		return MapGetter.getBooleanValueByKey( targetModel, hasTransformedDocsOfFuncProperties );
	}

	public String getRef2Source(){

		return ref2Source;
	}

	double getDuration(){

		return sumDuration;
	}

	double getMemory(){

		return sumMemory;
	}

	double getSize(){

		return sumSize;
	}

	double getDuration( String ref2DocOfSchemas ){

		for( int i = 0; hasDocsOfSchemas && i < docsOfSchemas.size(); ++i ) if( SchemaRepresentationController.getRef2DocOfSchemas( docsOfSchemas.get( i ) ).equals( ref2DocOfSchemas ) ) return SchemaRepresentationController.getDuration( docsOfSchemas.get( i ) );


		return 0.0;
	}

	int getNumOfParsableSchemaDocs(){

		return parsableDocs;
	}

	Object lookUp( String ref2DocOfSchemas ){

		for( int i = 0; hasDocsOfSchemas && i < docsOfSchemas.size(); ++i ) if( SchemaRepresentationController.getRef2DocOfSchemas( docsOfSchemas.get( i ) ).equals( ref2DocOfSchemas ) ) return docsOfSchemas.get( i );


		return null;
	}

	List<String> getNamesOfDocOfSchemas(){

		List<String> ref2DocOfSchemas = null;


		for( int i = 0; hasDocsOfSchemas && i < docsOfSchemas.size(); ++i ){

			if( ref2DocOfSchemas == null ) ref2DocOfSchemas = new ArrayList<String>();

			ref2DocOfSchemas.add( SchemaRepresentationController.getNameOfDocOfSchemas( docsOfSchemas.get( i ) ) );
        }


    	return ref2DocOfSchemas;
    }

	List<String> getRefs2DocsOfSchemas(){

		List<String> ref2DocOfSchemas = null;


		for( int i = 0; hasDocsOfSchemas && i < docsOfSchemas.size(); ++i ){

			if( ref2DocOfSchemas == null ) ref2DocOfSchemas = new ArrayList<String>();

			ref2DocOfSchemas.add( SchemaRepresentationController.getRef2DocOfSchemas( docsOfSchemas.get( i ) ) );
        }


    	return ref2DocOfSchemas;
    }

	String getSchemaRepresentationModel(){

		String schemaRepresentationModel = null;


		for( int i = 0; hasDocsOfSchemas && i < docsOfSchemas.size(); ++i ) if( ( schemaRepresentationModel = SchemaRepresentationController.getSchemaRepresentationModel( docsOfSchemas.get( i ) ) ) != null ) return schemaRepresentationModel;


		return schemaRepresentationModel;
    }


	//Visualization.
	DefaultMutableTreeNode visualize(){//Visualize the content of each source of the current collection. Note that I do not vizualize the content of the documents!

		DefaultMutableTreeNode node = new DefaultMutableTreeNode( ref2Source );


		if( hasDocsOfSchemas ){

			DefaultMutableTreeNode node1 = new DefaultMutableTreeNode( "Schema Documents" ); node.add( node1 );

			for( int i = 0; i < docsOfSchemas.size(); ++i ) node1.add( new DefaultMutableTreeNode( SchemaRepresentationController.getRef2DocOfSchemas( docsOfSchemas.get( i ) ) ) );
		}


    	return node;
    }
}
