package org.ow2.choreos.schemaRepresentation;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.interval.Interval1D;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.freeTextDocuments.Tokenizer;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;


/**
 * 
 * @author dathanas
 *
 */
class DocOfSchemas {


	//Variables.
	public final static String docOfSchemasTagName = "Schemas document";


	private String ref2DocOfSchema;

	private String primarySchemaRepresentationModel;

	private String schemaDefinitionNamespace;

	private List<Object> schemas;
	private boolean hasSchemas;

	private boolean hasUnmaterializedComponents;//Components in each one of the schemas that are not connected with their references types.


	private int myOrderID = 0;//This is the order in which the last schema appears in the document. Its current value is used for setting the order of each schema.


	private long duration;

	private int size;//This is the total number of the building blocks (without following the references).

	private double depth, memory;//on average for all schemas included in a schema document.

	private Interval1D scope;

	private Object parent;


	private String constraintViolation;

	private boolean hasConstraintViolation;


	//Constructor.
	protected DocOfSchemas( String cRef2DocOfSchema ){

		ref2DocOfSchema = cRef2DocOfSchema;

		hasSchemas = false;

		hasUnmaterializedComponents = false;

		hasConstraintViolation = false;

		size = 0;


		PrintMessages.print( PrintMessages.xmlPrint, hasConstraintViolation(), this );
	}


	//Setters.
	String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}


	private void addSchema( Schema schema ){

		schemas.add( schema );


		size += schema.getSize();


		schema.setMyOrderID( ++myOrderID );
	}

	protected void add( Schema schema ){

		if( ObjectConstraintsManipulation.setContainmentError( this, schema ) == null ){

			if( ! hasSchemas ){

				schemas = new ArrayList<Object>();

				hasSchemas = true;


				hasUnmaterializedComponents = true;

				primarySchemaRepresentationModel = schema.getSchemaRepresentationModel();

				schemaDefinitionNamespace = schema.getSchemaDefinitionNamespace();


				addSchema( schema );
			}

			else if( hasSchemas && schema.getSchemaRepresentationModel().equals( primarySchemaRepresentationModel ) ){

				hasUnmaterializedComponents = true;


				addSchema( schema );
			}

			else if( hasSchemas && ! schema.getSchemaRepresentationModel().equals( primarySchemaRepresentationModel ) ) ObjectConstraintsManipulation.setConnectionError ( this, schema, "Its schema does NOT have the SAME schema representation model" );
		}
	}

	/*public void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedServiceTypes= " + MapManipulation.getBooleanValueByKey( targetModel, hasTransformedServiceTypes ) + " hasConstraintViolation= " + hasConstraintViolation );

		if( hasServiceTypes && ! MapManipulation.getBooleanValueByKey( targetModel, hasTransformedServiceTypes ) && ! hasConstraintViolation  ){

			for ( Enumeration<String> i = ( (Hashtable<String, ServiceType>) serviceTypes ).keys(); i.hasMoreElements(); ){

				String serviceType = i.nextElement();


				serviceTypes.get( serviceType ).transform( targetModel );


				if( ! serviceTypes.get( serviceType ).hasTransformedInterfaces( targetModel ) ) return;
			}

			if( hasTransformedServiceTypes == null ) hasTransformedServiceTypes = new Hashtable<String, Boolean>();

			hasTransformedServiceTypes.put( targetModel, true );
		}

		PrintMessages.print( PrintMessages.transformationPrint, "END - transform: " + getTag() + " - hasTransformedServiceTypes= " + MapManipulation.getBooleanValueByKey( targetModel, hasTransformedServiceTypes ) + " hasConstraintViolation= " + hasConstraintViolation );
	}*/

	private Map<String, String> makeIndexQualifierNamespaces( Map<String, String> index ){

		for( int i = 0; hasSchemas && i < schemas.size(); ++i ) index = ( (Schema) schemas.get( i ) ).makeIndexQualifierNamespaces( index );


		String[] printKinds = { PrintMessages.connectionXMLWithXMLPrint, PrintMessages.connectionWSDLWithXMLPrint };

		MapPrinting.stdPrintStringIndex( printKinds, index, "makeIndexQualifierNamespaces" );


		return index;
	}

	private Map<String, String> makeIndexNamespaceQualifiers( Map<String, String> index ){

		for( int i = 0; hasSchemas && i < schemas.size(); ++i ) index = ( (Schema) schemas.get( i ) ).makeIndexNamespaceQualifiers( index );


		String[] printKinds = { PrintMessages.connectionXMLWithXMLPrint, PrintMessages.connectionWSDLWithXMLPrint };

		MapPrinting.stdPrintStringIndex( printKinds, index, "makeIndexNamespaceQualifiers" );


		return index;
	}

	private Map<String, String> add( Map<String, String> index, String key, String value ){

		if( index == null ) index = new Hashtable<String, String>();

		if( key != null && ! index.containsKey( key ) ) index.put( key, value );


		return index;
	}

	private Map<String, Object> makeIndexGlobalComponents( Map<String, Object> indexComponents, String indexKind ){

		for( int i = 0; hasSchemas && i < schemas.size(); ++i ) indexComponents = ( (Schema) schemas.get( i ) ).makeIndexGlobalComponents( indexComponents, indexKind );


		String[] printKinds = { PrintMessages.connectionXMLWithXMLPrint, PrintMessages.connectionWSDLWithXMLPrint };

		MapPrinting.stdPrintObjectsIndex( printKinds, indexComponents, indexKind );


		return indexComponents;
	}

	public void connect(){

		Map<String, String> qualifierNamespaces = null;

		Map<String, Object> indexGlobalComponentsWithoutTargetNamespaces = null;

		Map<String, Object> indexGlobalComponentsWithTargetNamespaces = null;


		PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "START - connectXMLWithXML: " + getTag() + " - hasSchemas= " + hasSchemas + " - hasUnmaterializedComponents= " + hasUnmaterializedComponents );

		if( hasSchemas && hasUnmaterializedComponents && ! hasConstraintViolation ) {

			PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "** START - Make Indices **" );

			if( qualifierNamespaces == null || ( qualifierNamespaces != null && qualifierNamespaces.isEmpty() ) ) qualifierNamespaces = makeIndexQualifierNamespaces( qualifierNamespaces );

			if( indexGlobalComponentsWithoutTargetNamespaces == null || ( indexGlobalComponentsWithoutTargetNamespaces != null && indexGlobalComponentsWithoutTargetNamespaces.isEmpty() ) ) indexGlobalComponentsWithoutTargetNamespaces = makeIndexGlobalComponents( indexGlobalComponentsWithoutTargetNamespaces, "makeIndexGlobalComponentsWithoutTargetNamespaces" );

			if( indexGlobalComponentsWithTargetNamespaces == null || ( indexGlobalComponentsWithTargetNamespaces != null && indexGlobalComponentsWithTargetNamespaces.isEmpty() ) ) indexGlobalComponentsWithTargetNamespaces = makeIndexGlobalComponents( indexGlobalComponentsWithTargetNamespaces, "makeIndexGlobalComponentsWithTargetNamespaces" );

			PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "** END - Make Indices **" );


			int countSize = 0;

			for( int i = 0; i < schemas.size(); ++i ){
	
				Schema schema = (Schema) schemas.get( i );
	
	
				PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "START connection XML with XML in schema= " + schema.getTag() + "**" );
	
				schema.connect( qualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces );
	
	
				if( ! schema.hasUnmaterializedComponents() ) countSize++;

				else if( schema.hasConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, schema );


				PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "END connection XML with XML in schema= " + schema.getTag() + "**" );
	    	}

			if( countSize == schemas.size() ) hasUnmaterializedComponents = false;
		}

		PrintMessages.print( PrintMessages.connectionXMLWithXMLPrint, "END - connectXMLWithXML: " + getTag() + " - hasSchemas= " + hasSchemas + " - hasUnmaterializedComponents= " + hasUnmaterializedComponents );
	}

	void setDuration( long cDuration ){

		duration = cDuration;
	}

	void setMemory( double cMemory ){

		memory = cMemory;
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any 'checkConstraint()' method.


		for( int i = 0; hasSchemas && i < schemas.size(); ++i ) if( ( (Schema) schemas.get( i ) ).hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, schemas.get( i ) );


		return constraintViolation;
	}

	public boolean hasSchemas(){

		return hasSchemas;
	}

	List<Object> getSchemas(){

		return schemas;
	}

	protected String getRef2DocOfSchemas(){

		return ref2DocOfSchema;
	}

	public String getName(){

		return Tokenizer.getSuffix( ref2DocOfSchema, "\\" );
	}

	long getDuration(){

		return duration;
	}

	double getMemory(){

		return memory;
	}

	int getSize(){

		return size;
	}

	public String getSchemaRepresentationModel(){

		return primarySchemaRepresentationModel;
	}

	public String getTagName(){

		return docOfSchemasTagName;
	}

	void setScope( Interval1D cScope ){

		scope = cScope;
	}

	void setParent( Object cParent ){

		parent = cParent;
	}

	private String printScope(){

		//return PrintMessages.get( PrintMessages.xmlPrint ) ? "(orderID= " + getMyOrderID() + ")" : "";
		return scope != null ? scope.getIntEndpoints() + " : " : "";
	}

	public String getTag(){

		return getTagName() + " name= '" + getName() + "'";
	}

	public KindDescr getKindDescr(){

		return hasSchemas ? ( (Schema) schemas.get( 0 ) ).getKindDescr( getTagName() ) : null;
	}

	protected boolean hasContent( Object object ){

		return SchemaRepresentationController.isSchemaKind( object );
	}

	Interval1D getScope(){

		return scope;
	}

	Object getParent(){

		return parent;
	}


	//Visualization.
	public DefaultMutableTreeNode visualize(){//This function is used by the 'Visualizer' class.

        DefaultMutableTreeNode node = new DefaultMutableTreeNode( printScope() + getTag() );


        for( int i = 0; hasSchemas && i < schemas.size(); ++i ) node.add( ( (Schema) schemas.get( i ) ).visualize() );


		return node;
    }
}
