package org.ow2.choreos.schemaRepresentation.modelTransformation;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.classInfo.ClassInfo;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.Structure;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyUnstructured.XMLCompletelyUnstructuredSchemaRepresentationController;
import org.ow2.choreos.unknownClasses.UnknownClassesController;


@SuppressWarnings("serial")
public class SchemaModelTransformation {


	//Variables.
	private static Map<String, Hashtable<String, Hashtable<String, String>>> rootTransformationRules = new Hashtable<String, Hashtable<String, Hashtable<String, String>>>() { {

		put( XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel(), new Hashtable<String, Hashtable<String, String>>() { {

		put( XMLSchemaRepresentationController.getCompletelyUnstructuredSchemaRepresentation(), new Hashtable<String, String>() { {//The flag (true/false) is the result of if the component has built-in type.

																								put( "SimpleType,false", "ComplexType" );
																								put( "ComplexType,false", "ComplexType" );

																								put( "AttributeDefinition,false", "ComplexType" );
																								put( "AttributeDeclaration,true", "BuiltInType" );
																								put( "AttributeDeclaration,false", "ComplexType" );

																								put( "ElementDefinition,false", "ComplexType" );
																								put( "ElementDeclaration,true", "BuiltInType" );
																								put( "ElementDeclaration,false", "ComplexType" );

																								put( "AttributeGroupDefinition,false", "ComplexType" );
																								put( "AttributeGroupDeclaration,true", "BuiltInType" );
																								put( "AttributeGroupDeclaration,false", "ComplexType" );

																								put( "ElementGroupDefinition,false", "ComplexType" );
																								put( "ElementGroupDeclaration,true", "BuiltInType" );
																								put( "ElementGroupDeclaration,false", "ComplexType" );

																								put( "SimpleContent,true", "BuiltInType" );
																								put( "SimpleContent,false", "ComplexType" );

																								put( "ComplexContent,true", "BuiltInType" );
																								put( "ComplexContent,false", "ComplexType" );

																								put( "SimpleTypeRestriction,true", "BuiltInType" );
																								put( "SimpleTypeRestriction,false", "ComplexType" );
																				   				                          			} } 
		   );
																																			 } }
		   );																																							} };

	private static Map<String, Hashtable<String, Hashtable<String, String>>> remainingTransformationRules = new Hashtable<String, Hashtable<String, Hashtable<String, String>>>() { {
		put( XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel(), 		new Hashtable<String, Hashtable<String, String>>() { {
		put( XMLSchemaRepresentationController.getCompletelyUnstructuredSchemaRepresentation(), new Hashtable<String, String>() { {

																								put( "AttributeDeclaration,true", "BuiltInType" );

																								put( "ElementDeclaration,true", "BuiltInType" );

																								put( "AttributeGroupDeclaration,true", "BuiltInType" );

																								put( "ElementGroupDeclaration,true", "BuiltInType" );

																								put( "SimpleContent,true", "BuiltInType" );

																								put( "ComplexContent,true", "BuiltInType" );

																								put( "SimpleTypeRestriction,true", "BuiltInType" );
																				   				                          } } 
		   );
																																			 } }
		   );																																							} };


	//Getters.
	public static Map<String, Hashtable<String, String>> getRootSourceTansformationRules( String sourceModel ){

		return sourceModel != null ? rootTransformationRules.get( sourceModel ) : null;
	}

	public static Map<String, Hashtable<String, String>> getSourceTansformationRules( String sourceModel ){

		return sourceModel != null ? remainingTransformationRules.get( sourceModel ) : null;
	}

	public static Map<String, String> getTargetTansformationRules( Map<String, Hashtable<String, String>> sourceTansformationRules, String targetModel ){

		return sourceTansformationRules != null && targetModel != null ? sourceTansformationRules.get( targetModel ) : null;
	}


	public static Object transform( String sourceModel, String targetModel, Object sourceObject, Map<String, Object> indexOfSourceComponentsPath ){

		List<Object> historyTargetObjects = new ArrayList<Object>();

		List<Map<String, String>> path = null;


		Map<String, String> tag = getTag( sourceObject );

		path = add( path, tag );


		Object targetObject = getTargetObject( path, true, sourceModel, targetModel, sourceObject, true ); PrintMessages.print( PrintMessages.transformationPrint, "Root target object= " + ClassInfo.getClass( targetObject ) );

		add( historyTargetObjects, targetObject );


		if( indexOfSourceComponentsPath == null ) indexOfSourceComponentsPath = new Hashtable<String, Object>();

		indexOfSourceComponentsPath.put( XMLSchemaRepresentationController.getSchemaPath( targetObject ), sourceObject );


		transform( path, sourceModel, targetModel, sourceObject, historyTargetObjects, indexOfSourceComponentsPath );


		return targetObject;
	}

	private static Map<String, String> getTag( Object sourceObject ){

		Map<String, String> transformedTag = null;


		Map<String, String> tag = XMLSchemaRepresentationController.getTagAttributes( sourceObject );


		transformedTag = makeDeepCopy( tag );


		return transformedTag;
	}

	private static Map<String, String> makeDeepCopy( Map<String, String> tag ){

		Map<String, String> transformedTag = new Hashtable<String, String>();


		if( tag != null ){

			for( Enumeration<String> e = ( (Hashtable<String, String>) tag ).keys(); e.hasMoreElements(); ){
	
				String key = e.nextElement();

				if( ! key.equals( SchemaRepresentationController.getTagNameKey() ) ) transformedTag.put( key, tag.get( key ) );
			}
		}


		return transformedTag;
	}

	private static List<Map<String, String>> add( List<Map<String, String>> path, Map<String, String> tag ){

		if( path == null ) path = new ArrayList<Map<String, String>>();

		if( tag != null ) path.add( tag );


		return path;
	}

	private static Object getTargetObject( List<Map<String, String>> path, boolean rootObject, String sourceModel, String targetModel, Object sourceObject, boolean isGlobal ){

		Map<String, Hashtable<String, String>> sourceTansformationRules = null;

		if( rootObject ) sourceTansformationRules = getRootSourceTansformationRules( sourceModel );

		else sourceTansformationRules = getSourceTansformationRules( sourceModel );


		if( sourceTansformationRules == null ) return XMLSchemaRepresentationController.getUnknownSchemaModel( sourceModel, targetModel );


		String sourceObjectKind = ClassInfo.getClass( sourceObject );

		sourceObjectKind += "," + XMLSchemaRepresentationController.isBuiltInType( sourceObject );

		PrintMessages.print( PrintMessages.transformationPrint, "sourceObjectKind= " + sourceObjectKind );


		Map<String, String> targetTansformationRules = getTargetTansformationRules( sourceTansformationRules, targetModel );

		if( targetTansformationRules == null ) return XMLSchemaRepresentationController.getUnknownSchemaModel( sourceModel, targetModel );


		String targetObjectKind = targetTansformationRules.get( sourceObjectKind );

		PrintMessages.print( PrintMessages.transformationPrint, "targetObjectKind= " + targetObjectKind );


		return targetObjectKind != null ? XMLCompletelyUnstructuredSchemaRepresentationController.getObject( path, sourceObjectKind, sourceModel, targetObjectKind, isGlobal ) : null;
	}


	private static void transform( List<Map<String, String>> path, String sourceModel, String targetModel, Object sourceObject, List<Object> historyTargetObjects, Map<String, Object> indexOfSourceComponentsPath ){

		List<Structure> structures = null;

		for( int i = 0; XMLSchemaRepresentationController.hasStructures( sourceObject ) && i < ( structures = XMLSchemaRepresentationController.getStructures( sourceObject ) ).size(); ++i ){
		
			Object structure = structures.get( i ); PrintMessages.print( PrintMessages.transformationPrint, "Structure= " + ClassInfo.getClass( structure ) + " of the object= " + ClassInfo.getClass( sourceObject ) );

			Map<String, String> tag = getTag( structure );

			path = add( path, tag );


			transformAndSet( path, sourceModel, targetModel, historyTargetObjects, structure, indexOfSourceComponentsPath );


			removeLastElement( path );
		}


		List<Component> components = null;

		for( int i = 0; XMLSchemaRepresentationController.hasComponents( sourceObject ) && i < ( components = XMLSchemaRepresentationController.getComponents( sourceObject ) ).size(); ++i ){

			Object component = components.get( i ); PrintMessages.print( PrintMessages.transformationPrint, "Component= " + ClassInfo.getClass( component ) + " of the object= " + ClassInfo.getClass( sourceObject ) );

			Map<String, String> tag = getTag( component );

			path = add( path, tag );


			transformAndSet( path, sourceModel, targetModel, historyTargetObjects, component, indexOfSourceComponentsPath );


			removeLastElement( path );
		}

		for( int i = 0; XMLSchemaRepresentationController.hasMaterializedTypes( sourceObject ) && i < ( components = XMLSchemaRepresentationController.getMaterializedTypes( sourceObject ) ).size(); ++i ){

			Object component = components.get( i ); PrintMessages.print( PrintMessages.transformationPrint, XMLSchemaRepresentationController.getMaterializedTypes( sourceObject ).size() + " types, Materialized type= " + ClassInfo.getClass( component ) + " of the object= " + ClassInfo.getClass( sourceObject ) );

			Map<String, String> tag = getTag( component );

			path = add( path, tag );


			transformAndSet( path, sourceModel, targetModel, historyTargetObjects, component, indexOfSourceComponentsPath );


			removeLastElement( path );
		}
	}

	private static void transformAndSet( List<Map<String, String>> path, String sourceModel, String targetModel, List<Object> historyTargetObjects, Object sourceObject, Map<String, Object> indexOfSourceComponentsPath ){

		Object targetObject = getTargetObject( path, false, sourceModel, targetModel, sourceObject, false );


		if( targetObject != null && ! UnknownClassesController.isUnknownObject( targetObject ) ){

			XMLSchemaRepresentationController.set( targetObject, getLastObject( historyTargetObjects ) ); PrintMessages.print( PrintMessages.transformationPrint, "Set the target object= " + ClassInfo.getClass( targetObject ) + " to the object= " + ClassInfo.getClass( getLastObject( historyTargetObjects ) ) + " that corresponds to the source object= " + ClassInfo.getClass( sourceObject ) );

			add( historyTargetObjects, targetObject );

			indexOfSourceComponentsPath.put( XMLSchemaRepresentationController.getSchemaPath( targetObject ), sourceObject );
		}


		transform( path, sourceModel, targetModel, sourceObject, historyTargetObjects, indexOfSourceComponentsPath );


		if( targetObject != null && ! UnknownClassesController.isUnknownObject( targetObject ) ) remove( historyTargetObjects );
	}

	private static void add( List<Object> historyTargetObjects, Object targetObject ){

		if( historyTargetObjects != null && targetObject != null ) historyTargetObjects.add( targetObject );

		PrintMessages.print( PrintMessages.transformationPrint, "add - historyTargetObjects = " + historyTargetObjects.toString() );
	}

	private static void removeLastElement( List<Map<String, String>> path ){

		if( path != null && ! path.isEmpty() ) path.remove( path.size() - 1 );
	}

	private static void remove( List<Object> historyTargetObjects ){

		if( ! historyTargetObjects.isEmpty() ) historyTargetObjects.remove( historyTargetObjects.size() - 1 );

		PrintMessages.print( PrintMessages.transformationPrint, "remove - historyTargetObjects = " + historyTargetObjects.toString() );
	}

	private static Object getLastObject( List<Object> historyTargetObjects ){

		PrintMessages.print( PrintMessages.transformationPrint, "getLastObject - historyTargetObjects = " + historyTargetObjects );

		return ! historyTargetObjects.isEmpty() ? historyTargetObjects.get( historyTargetObjects.size() - 1 ) : null;
	}
}
