package org.ow2.choreos.schemaRepresentation.xml;

import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.interval.Interval1D;
import org.ow2.choreos.dataStructures.list.listInsertion.ListInsertion;
import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.schemaTagsTransformation.SchemaTags2JavaClassesTransformation;


/**
 * 
 * @author dathanas
 *
 */
public class Annotation {


	//Variables.
	public final static String annotationTagName = "annotation";

	private Map<String, String> tagAttributes;

	private String[] classPath;
	private boolean hasClassPath;


	private int myOrderID = 0;//This is the order in which the schema appears in its schema document.

	private int myChildrenOrderID = 0;//This is the order in which the components of the schema appear in this schema.


	private int size = 1;//This is the total number of the building blocks (without following the references).


	private Documentation documentation;
	private boolean hasDocumentation;


	private Interval1D scope;

	private Object parent;


	private String constraintViolation;
	private boolean hasConstraintViolation;


	//Constructor.
	public Annotation( List<Map<String, String>> path ){

		hasDocumentation = false;

		if( path != null && ! path.isEmpty() ) setClassPath( path );

		else hasClassPath = false;


		Map<String, String> cAttributes = ListGetters.getLastListElement( path );

		if( cAttributes != null && ! cAttributes.isEmpty() ) tagAttributes = cAttributes;


		PrintMessages.print( PrintMessages.xmlPrint, hasConstraintViolation, this );
	}
	
	
	//Empty constructor used for xml transformation
	public Annotation(){
		
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	private void setDocumentation( Documentation cDocumentation ){

		if( ! hasDocumentation ){

			documentation = cDocumentation;

			hasDocumentation = true;


			size += cDocumentation.getSize();


			cDocumentation.setMyOrderID( ++myChildrenOrderID );
		}

		else ObjectConstraintsManipulation.setConnectionError ( this, cDocumentation, "Its documentation has already been set" );
	}

	Object add( Object object ){

		if( ObjectConstraintsManipulation.setContainmentError( this, object ) == null ){

			if( XMLSchemaRepresentationController.isDocumentationKind( object ) ) setDocumentation( (Documentation) object );

			else ObjectConstraintsManipulation.setConnectionError ( this, object, "Valid content: Documentation text." );
		}

		return this;
	}

	private void setClassPath( List<Map<String, String>> path ){

		classPath = SchemaTags2JavaClassesTransformation.transform( path );

		hasClassPath = classPath != null ? true : false;
	}

	protected void setMyOrderID( int id ){

		myOrderID = id;
	}

	void setScope( Interval1D cScope ){

		scope = cScope;
	}

	void setParent( Object cParent ){

		parent = cParent;
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	protected String getConstraintViolation(){

		return constraintViolation;
	}

	private Map<String, String> getAttributes(){

		return tagAttributes;
	}

	public String getName(){

		return getTagName();
	}

	protected String getTagName(){

		return MapGetter.getValueByKeyToLowerCase( SchemaRepresentationController.getTagNameKey(), getAttributes() );
	}

	protected String getTag(){

		return getTagName() + printMyOrderID();
	}

	public String getSchemaKind(){

		return SchemaRepresentationController.getSchemaKind();
	}

	public double getSize(){

		return size;
	}

	protected int getMyOrderID(){

		return myOrderID;
	}

	private String printMyOrderID(){

		//return PrintMessages.get( PrintMessages.xmlPrint ) ? "(orderID= " + getMyOrderID() + ")" : "";
		return " (orderID= " + getMyOrderID() + ")";
	}

	private String printScope(){

		//return PrintMessages.get( PrintMessages.xmlPrint ) ? "(orderID= " + getMyOrderID() + ")" : "";
		return scope != null ? scope.getIntEndpoints() + " : " : "";
	}

	@SuppressWarnings("unchecked")
	public List<?> getChildren(){

		List<?> children = null;


		return ListInsertion.add( documentation, (List<Object>) children );
	}

	Interval1D getScope(){

		return scope;
	}

	Object getParent(){

		return parent;
	}


	//Visualization.
	public DefaultMutableTreeNode visualize(){

		DefaultMutableTreeNode node = new DefaultMutableTreeNode( printScope() + getTag() );


		if( hasDocumentation ) node.add( documentation.visualize() );


		return node;
	}
	
	//Setters used for xml transformation
	
	public void setxTagAttributes( Map<String, String> cTagAttributes ){
		tagAttributes = cTagAttributes;
	}
		
	public void setxClassPath( String [] cClassPath ){
		classPath = cClassPath;
	}
		
	public void setxHasClassPath( boolean cHasClassPath ){
		hasClassPath = cHasClassPath;
	}
		
	public void setxMyOrderID( int cMyOrderID ){
		myOrderID = cMyOrderID;
	}
		
	public void setxMyChildrenOrderID( int cMyChildrenOrderID ){
		myChildrenOrderID = cMyChildrenOrderID;
	}
		
	public void setxSize( int cSize ){
		size = cSize;
	}
		
	public void setxConstraintViolation( String cConstraintViolation ){
		constraintViolation = cConstraintViolation;
	}
		
	public void setxHasConstraintViolation( boolean cHasConstraintViolation ){
		hasConstraintViolation = cHasConstraintViolation;
	}
		
	public void setxDocumentation( Documentation cDocumentation ){
		documentation = cDocumentation;
	}
		
	public void setxHasDocumentation( boolean cHasDocumentation ){
		hasDocumentation = cHasDocumentation;
	}
		
		
	//Getters used for xml transformation
		
	public Map<String, String> getxTagAttributes(){
		return tagAttributes;
	}
			
	public String [] getxClassPath(){
		return classPath;
	}
			
	public boolean getxHasClassPath(){
		return hasClassPath;
	}
			
	public int getxMyOrderID(){
		return myOrderID;
	}
			
	public int getxMyChildrenOrderID(){
		return myChildrenOrderID;
	}
			
	public int getxSize(){
		return size;
	}
			
	public String getxConstraintViolation(){
		return constraintViolation;
	}
			
	public boolean getxHasConstraintViolation(){
		return hasConstraintViolation;
	}
			
	public Documentation getxDocumentation(){
		return documentation;
	}
			
	public boolean getxHasDocumentation(){
		return hasDocumentation;
	}
}
