package org.ow2.choreos.schemaRepresentation.xml;

import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.schemaTagsTransformation.SchemaTags2JavaClassesTransformation;


/**
 * 
 * @author dathanas
 *
 */
class Documentation {


	//Variables.
	private Map<String, String> tagAttributes;

	private String[] classPath;
	private boolean hasClassPath;


	private int myOrderID = 0;//This is the order in which the schema appears in its schema document.


	private int size = 1;//This is the total number of the building blocks (without following the references).


	private String text;
	private boolean hasText;

	private String constraintViolation;
	private boolean hasConstraintViolation;


	//Constructor.
	public Documentation( List<Map<String, String>> path ){

		hasText = false;

		if( path != null && ! path.isEmpty() ) setClassPath( path );

		else hasClassPath = false;


		Map<String, String> cAttributes = ListGetters.getLastListElement( path );

		if( cAttributes != null && ! cAttributes.isEmpty() ) tagAttributes = cAttributes;


		PrintMessages.print( PrintMessages.xmlPrint, hasConstraintViolation, this );
	}
	
	
	//Empty constructor used for xml transformation
		public Documentation(){
			
		}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	protected void setMyOrderID( int id ){

		myOrderID = id;
	}

	Object add( Object object ){

		if( ObjectConstraintsManipulation.setContainmentError( this, object ) == null ){

			if( XMLSchemaRepresentationController.isTextKind( object ) ) text = (String) object;

			else ObjectConstraintsManipulation.setConnectionError ( this, object, "Valid content: Text." );
		}

		return this;
	}

	private void setClassPath( List<Map<String, String>> path ){

		classPath = SchemaTags2JavaClassesTransformation.transform( path );

		hasClassPath = classPath != null ? true : false;
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	protected String getConstraintViolation(){

		return constraintViolation;
	}

	private Map<String, String> getAttributes(){

		return tagAttributes;
	}

	public String getName(){

		return getTagName();
	}

	protected String getTagName(){

		return MapGetter.getValueByKeyToLowerCase( SchemaRepresentationController.getTagNameKey(), getAttributes() );
	}

	protected String getTag(){

		return getTagName() + printMyOrderID();
	}

	public String getSchemaKind(){

		return SchemaRepresentationController.getSchemaKind();
	}

	protected double getSize(){

		return size;
	}

	protected int getMyOrderID(){

		return myOrderID;
	}

	private String printMyOrderID(){

		//return PrintMessages.get( PrintMessages.xmlPrint ) ? "(orderID= " + getMyOrderID() + ")" : "";
		return " (orderID= " + getMyOrderID() + ")";
	}

	public List<?> getChildren(){

		return null;
	}


	//Visualization.
	public DefaultMutableTreeNode visualize(){

		if( hasConstraintViolation() ) return null;


		DefaultMutableTreeNode node = new DefaultMutableTreeNode( getTag() );

		if( hasText ) node.add( new DefaultMutableTreeNode( text ) );


		return node;
	}
	
	//Setters used for xml transformation
	
	public void setxTagAttributes( Map<String, String> cTagAttributes ){
		tagAttributes = cTagAttributes;
	}
			
	public void setxClassPath( String [] cClassPath ){
		classPath = cClassPath;
	}
			
	public void setxHasClassPath( boolean cHasClassPath ){
		hasClassPath = cHasClassPath;
	}
			
	public void setxMyOrderID( int cMyOrderID ){
		myOrderID = cMyOrderID;
	}
			
	public void setxSize( int cSize ){
		size = cSize;
	}
			
	public void setxConstraintViolation( String cConstraintViolation ){
		constraintViolation = cConstraintViolation;
	}
			
	public void setxHasConstraintViolation( boolean cHasConstraintViolation ){
		hasConstraintViolation = cHasConstraintViolation;
	}
			
	public void setxText( String cText ){
		text = cText;
	}
			
	public void setxHasText( boolean cHasText ){
		hasText = cHasText;
	}
			
			
	//Getters used for xml transformation
			
	public Map<String, String> getxTagAttributes(){
		return tagAttributes;
	}
				
	public String [] getxClassPath(){
		return classPath;
	}
				
	public boolean getxHasClassPath(){
		return hasClassPath;
	}
				
	public int getxMyOrderID(){
		return myOrderID;
	}
				
	public int getxSize(){
		return size;
	}
				
	public String getxConstraintViolation(){
		return constraintViolation;
	}
				
	public boolean getxHasConstraintViolation(){
		return hasConstraintViolation;
	}
				
	public String getxText(){
		return text;
	}
				
	public boolean getxHasText(){
		return hasText;
	}
}
