package org.ow2.choreos.schemaRepresentation.xml;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.classInfo.ClassInfo;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.freeTextDocuments.Tokenizer;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.modelTransformation.SchemaModelTransformation;


/**
 * 
 * @author dathanas
 * This class models the 'base'/'type'/'ref'/'element'/'itemType'/'refer' attribute of a tag.
 */
@SuppressWarnings("serial")
public class ReferencedComponent implements java.io.Serializable {


	//Variables.
	private static final long serialVersionUID = 7952761776701599838L;

	public static final Map<String, String> typeKinds = new Hashtable<String, String>() { {  put( "base", "base" );
																							 put( "type", "type" );
																							 put( "ref", "ref" );
																							 put( "element", "element" );
																							 put( "itemType", "itemType" );
																							 put( "refer", "refer" ); } };	

	private String prefix;
	private boolean hasPrefix;

	private String type;
	private boolean hasType;

	private String typeKind;

	private boolean isBuiltInType;

	private boolean belongsToCycle;

	private Component materializedComponent;
	private boolean hasUnmaterializedComponent;

	private Map<String, Boolean> hasTransformedReferencedComponent;//The 'String' is the representation model.
	private Map<String, Component> materializedComponentsInOtherModels;//The 'String' is the name of the 'schemaReprensetationModel'. These are the components that are not built-in types.
	private Map<String, Boolean> belongsToCycleInOtherModels;
	private Map<String, Map<String, Object>> indexOfTransformedObjectsSchemaPath;//The 1st 'String' is the transformed model, the second string is the path of the transformed component/structure in the primary model and the object is the component/structure.
	private boolean hasMaterializedComponentsInOtherModels;

	private String constraintViolation;
	private boolean hasConstraintViolation;


	//Constructor 1: e.g., ref= '...'.
	public ReferencedComponent( String cType, String cTypeKind ){

		if( checkConstraint2( cTypeKind ) == null ){

			typeKind = cTypeKind;

			cTypeKind = cTypeKind.toLowerCase();
		}


		initVariables( cType, true );
	}

	//Constructor 2: e.g., memberTypes in union without 'typeKind'.
	public ReferencedComponent( String cType ){

		initVariables( cType, true );
	}

	//Constructor 3: It is used for the case that we know that this 'referencedComponent' is built-in type, e.g., BuiltInType in 'completelyUnstructuredSchemaRepresenation'.
	public ReferencedComponent( String cType, String cTypeKind, boolean cIisBuiltInType ){

		if( checkConstraint2( cTypeKind ) == null ){

			typeKind = cTypeKind;

			cTypeKind = cTypeKind.toLowerCase();
		}


		initVariables( cType, false );


		isBuiltInType = cIisBuiltInType;

		hasUnmaterializedComponent = ! cIisBuiltInType;
	}

	//Constructor 4: It is used for extraction purposes.
	public ReferencedComponent( String cType, String cTypeKind, Component materializedComponent ){

		if( checkConstraint2( cTypeKind ) == null ){

			typeKind = cTypeKind;

			cTypeKind = cTypeKind.toLowerCase();
		}


		initVariables( cType, true );

		setMaterializedComponent( materializedComponent );
	}
	
	
	//Empty constructor used for xml transformation
	public ReferencedComponent(){
		
	}


	//Method used by constructors.
	private void initVariables( String cType, boolean checkForBuiltInType ){

		hasMaterializedComponentsInOtherModels = false;

		hasConstraintViolation = false;

		hasType = false;

		belongsToCycle = false;


		String cPrefix = null;

		if( checkConstraint1( cType ) == null ){

			cPrefix = Tokenizer.getPrefix( cType, ":" );

			cType = Tokenizer.getSuffix( cType, ":" );
		}

		if( cPrefix != null && ! cPrefix.isEmpty() ){

			prefix = cPrefix;

			hasPrefix = true;
		}

		else hasPrefix = false;


		if( checkConstraint1( cType ) == null ){

			type = cType;

			hasType = true;


			if( checkForBuiltInType ){

				if( ! hasPrefix && XMLBuiltInTypes.isBuiltInType( type ) ){
	
					isBuiltInType = true;
	
					hasUnmaterializedComponent = false;
				}
	
				else{
	
					isBuiltInType = false;
	
					hasUnmaterializedComponent = true;
				}
			}
		}
	}


	//Constraint checkers.
	private String checkConstraint1( String type ){

		return type == null || ( type != null && type.isEmpty() ) ? setConstraintViolation( "Missing value" ) : null;
	}

	private String checkConstraint2( String typeKind ){

		return typeKind == null && typeKinds.get( typeKind ) != null ? setConstraintViolation( "The attribute of the referenced component should be one of the following: 'ref', 'base', 'type', 'element' and 'itemType'" ) : null;
	}


	//Setters.
	protected String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true; 
		}

		return constraintViolation;
	}

	public void setMaterializedComponent( Component component ){//It is used for abstraction extraction purposes.

		if( ObjectConstraintsManipulation.setContainmentError( this, component ) == null ){

			materializedComponent = component;
			
			hasUnmaterializedComponent = false;
		}
	}


	public void replacePrefix( Map<String, String> namespaceQualifiers, String docTargetNamespace ){//This function is used only by the message for changing the URL prefix into qualifier.

		String[] printKinds = { PrintMessages.connectionXMLWithXMLPrint, PrintMessages.connectionWSDLWithXMLPrint };

		PrintMessages.print( printKinds , "START - replacePrefix - ReferencedComponent:" + getTag() + " - hasType= " + hasType + " - isBuiltInType= " + isBuiltInType + " - hasUnmaterializedReferencedTypes= " + hasUnmaterializedComponent + " - hasConstraintViolation= " + hasConstraintViolation + " - prefix= " + prefix + " - docTargetNamespace= " + docTargetNamespace );


		String qualifier = null;


		if( hasPrefix && namespaceQualifiers != null && ( qualifier = namespaceQualifiers.get( prefix ) ) != null ) prefix = qualifier;

		else if( hasPrefix && prefix.equals( docTargetNamespace ) ) prefix = docTargetNamespace;

		else{

			prefix = null;

			hasPrefix = false;
		}

		PrintMessages.print( printKinds , "END - replacePrefix - ReferencedComponent:" + getTag() + " - hasType= " + hasType + " - isBuiltInType= " + isBuiltInType + " - hasUnmaterializedReferencedTypes= " + hasUnmaterializedComponent + " - hasConstraintViolation= " + hasConstraintViolation + " - prefix= " + prefix + " - docTargetNamespace= " + docTargetNamespace );
	}


	public void connect( Object object, Map<String, String> schemaQualifierNamespaces, Map<String, String> tagQualifierNamespaces, Map<String, Object> indexLocalComponentsWithoutTargetNamespaces, Map<String, Object> indexGlobalComponentsWithoutTargetNamespaces, Map<String, Object> indexGlobalComponentsWithTargetNamespaces, String schemaRepresentationModel, String schemaDefinitionNamespace ){//The 'object' can be 'Component', 'Structure' or 'MessageType'. This function materializes globally a referencedComponent.

		String[] printKinds = { PrintMessages.connectionXMLWithXMLPrint, PrintMessages.connectionWSDLWithXMLPrint };


		connect( printKinds, object, schemaQualifierNamespaces, tagQualifierNamespaces, schemaDefinitionNamespace );

		if( indexLocalComponentsWithoutTargetNamespaces != null ) connect( printKinds, object, indexLocalComponentsWithoutTargetNamespaces, schemaRepresentationModel );//The message type does not have local components.

		connect( printKinds, object, schemaQualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces, schemaRepresentationModel );
	}

	private void connect( String[] printKinds, Object object, Map<String, String> schemaQualifierNamespaces, Map<String, String> tagQualifierNamespaces, String schemaDefinitionNamespace ){//This function connect the prefix of the referenced component, it finds the target namespace for its prefix.

		if( hasUnmaterializedComponent && ! hasConstraintViolation ){

			PrintMessages.print( printKinds , "START - connect (prefix) - ReferencedComponent:" + getTag() + " - hasType= " + hasType + " - isBuiltInType= " + isBuiltInType + " - hasUnmaterializedReferencedTypes= " + hasUnmaterializedComponent + " hasConstraintViolation= " + hasConstraintViolation + " - schemaDefinitionNamespace= " + schemaDefinitionNamespace );


			String targetNamespace = null;


			if( schemaDefinitionNamespace == null && XMLBuiltInTypes.isBuiltInType( type ) ){ PrintMessages.print( printKinds, "The type= " + type + " is built-in type" );
	
				hasUnmaterializedComponent = false;
	
				isBuiltInType = true;
	
				return;
			}

			if( hasPrefix && tagQualifierNamespaces != null && ( targetNamespace = tagQualifierNamespaces.get( prefix ) ) != null && targetNamespace.equals( schemaDefinitionNamespace ) ){ PrintMessages.print( printKinds, "The type= " + type + " is built-in type" );

				hasUnmaterializedComponent = false;

				isBuiltInType = true;

				return;
			}

			if( hasPrefix && schemaQualifierNamespaces != null && ( targetNamespace = schemaQualifierNamespaces.get( prefix ) ) == null ){
	
				String msg = "The prefix = '" + prefix + "' is not defined";
	
				ObjectConstraintsManipulation.setErrorAttribute( object, typeKind, msg );
	
				PrintMessages.print( printKinds, msg );
	
				return;
			}
	
			if( hasPrefix && schemaDefinitionNamespace != null && targetNamespace.equals( schemaDefinitionNamespace ) ){ PrintMessages.print( printKinds, "The prefix= " + prefix + " corresponds to namespace of built-in XML types" );
	
				hasUnmaterializedComponent = false;
	
				isBuiltInType = true;
	
				return;
			}

			PrintMessages.print( printKinds , "END - connect (prefix) - ReferencedComponent:" + getTag() + " - hasType= " + hasType + " - isBuiltInType= " + isBuiltInType + " - hasUnmaterializedReferencedTypes= " + hasUnmaterializedComponent + " hasConstraintViolation= " + hasConstraintViolation + " - schemaDefinitionNamespace= " + schemaDefinitionNamespace );
		}
	}

	private void connect( String[] printKinds, Object object, Map<String, Object> indexLocalComponentsWithoutTargetNamespaces, String schemaRepresentationModel ){//This function connects locally a referencedComponent.

		if( hasUnmaterializedComponent && ! hasConstraintViolation ){

			PrintMessages.print( printKinds , "START - connect (with local components) - ReferencedComponent:" + getTag() + " - hasType= " + hasType + " - isBuiltInType= " + isBuiltInType + " - hasUnmaterializedReferencedTypes= " + hasUnmaterializedComponent + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel );


			if( hasPrefix ) PrintMessages.print( printKinds, "Lookup locally for component with prefix= " + prefix + " of the referencedComponent= " + getTag() );

			else PrintMessages.print( printKinds, "Lookup locally for referencedComponent= " + getTag() );


			List<String> compatibleComponentKinds = XMLSchemaRepresentationController.getCompatibleComponentKinds( schemaRepresentationModel, object );


			MapPrinting.stdPrintObjectsIndex( printKinds, indexLocalComponentsWithoutTargetNamespaces, "indexLocalComponentsWithoutTargetNamespaces" );


			for( int i = 0; compatibleComponentKinds != null && i < compatibleComponentKinds.size(); ++i ){
	
				Object selectedComponent = indexLocalComponentsWithoutTargetNamespaces.get( type + "," + compatibleComponentKinds.get( i ) );
	
	
				if( selectedComponent != null ){

					PrintMessages.print( printKinds, "The component " + XMLSchemaRepresentationController.getTag( selectedComponent ) + " is FOUND locally." );
	
					materializedComponent = (Component) selectedComponent;
	
					hasUnmaterializedComponent = false;

					return;
				}
			}
	
			if( compatibleComponentKinds == null ) PrintMessages.print( printKinds, "There are no compatible kinds for the kind= '" + ClassInfo.getClass( object ) + "'" );

			if( materializedComponent == null ) PrintMessages.print( printKinds, "The component is NOT found locally." );


			PrintMessages.print( printKinds , "END - connect (with local components) - ReferencedComponent:" + getTag() + " - hasType= " + hasType + " - isBuiltInType= " + isBuiltInType + " - hasUnmaterializedReferencedTypes= " + hasUnmaterializedComponent + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel );
		}
	}

	private void connect( String[] printKinds, Object object, Map<String, String> schemaQualifierNamespaces, Map<String, Object> indexGlobalComponentsWithoutTargetNamespaces, Map<String, Object> indexGlobalComponentsWithTargetNamespaces, String schemaRepresentationModel ){//This function connects globally a referencedComponent.

		if( hasUnmaterializedComponent && ! hasConstraintViolation ){

			PrintMessages.print( printKinds , "START - connect (with global components) - ReferencedComponent:" + getTag() + " - hasType= " + hasType + " - isBuiltInType= " + isBuiltInType + " - hasUnmaterializedReferencedTypes= " + hasUnmaterializedComponent + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel );


			if( hasPrefix ) PrintMessages.print( printKinds, "Lookup for the component with prefix= " + prefix + " of the referencedComponent= " + getTag() );

			else PrintMessages.print( printKinds, "Lookup for the referencedComponent= " + getTag() );


			String targetNamespace = hasPrefix ? schemaQualifierNamespaces.get( prefix ) : null;


			List<String> compatibleComponentKinds = XMLSchemaRepresentationController.getCompatibleComponentKinds( schemaRepresentationModel, object );

			for( int i = 0; compatibleComponentKinds != null && i < compatibleComponentKinds.size(); ++i ){

				Object selectedComponent = null; PrintMessages.print( printKinds, "Compatible kind= '" + compatibleComponentKinds.get( i ) + "'" );


				if( hasPrefix ) selectedComponent = indexGlobalComponentsWithTargetNamespaces != null ? indexGlobalComponentsWithTargetNamespaces.get( type + "," + compatibleComponentKinds.get( i ) + "," + targetNamespace ) : null;

				else selectedComponent = indexGlobalComponentsWithoutTargetNamespaces != null ? indexGlobalComponentsWithoutTargetNamespaces.get( type + "," + compatibleComponentKinds.get( i ) ) : null;


				if( selectedComponent != null ){

					if( hasPrefix ) PrintMessages.print( printKinds, "The component '" + SchemaRepresentationController.getTag( selectedComponent ) + "' is FOUND in the targetNamespace= " + targetNamespace );

					else PrintMessages.print( printKinds, "The component '" + SchemaRepresentationController.getTag( selectedComponent ) + "' is FOUND globally." );


					belongsToCycle = SchemaRepresentationController.belongsToCycle( (Component) selectedComponent, object );


					if( belongsToCycle ){

						PrintMessages.print( printKinds, "The component '" + SchemaRepresentationController.getTag( this ) + "' creates a cycle due to the reference of its root component '" + SchemaRepresentationController.getTag( selectedComponent ) + "'" );
						//JOptionPane.showMessageDialog( null, "The component '" + SchemaRepresentationController.getTag( object ) + "' creates a cycle due to the reference of its root component '" + SchemaRepresentationController.getTag( selectedComponent ) + "'" );
					}


					materializedComponent = (Component) selectedComponent;

					hasUnmaterializedComponent = false;

					return;
				}
			}


			if( compatibleComponentKinds == null ){

				String msg = "There are no compatible kinds for the kind= '" + ClassInfo.getClass( object ) + "'";

				PrintMessages.print( printKinds, msg );

				ObjectConstraintsManipulation.setErrorAttribute( object, typeKind, msg );
			}

			if( materializedComponent == null ){

				String msg = null;

				if( hasPrefix ) msg = "The component is NOT found in the targetNamespace= '" + targetNamespace + "'";

				else msg = "The component is NOT found";


				PrintMessages.print( printKinds, msg );

				ObjectConstraintsManipulation.setErrorAttribute( object, typeKind, msg );
			}


			PrintMessages.print( printKinds , "END - connect (with global components) - ReferencedComponent:" + getTag() + " - hasType= " + hasType + " - isBuiltInType= " + isBuiltInType + " - hasUnmaterializedReferencedTypes= " + hasUnmaterializedComponent + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel );
		}
	}


	public void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedReferencedType= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedReferencedComponent ) + " hasConstraintViolation= " + hasConstraintViolation );


		if( isBuiltInType && ! MapGetter.getBooleanValueByKey( targetModel, hasTransformedReferencedComponent ) ) add( targetModel );

		else if( ! hasUnmaterializedComponent && ! MapGetter.getBooleanValueByKey( targetModel, hasTransformedReferencedComponent ) ){

			Map<String, Object> indexOfTransformedObjectsSchemaPathForOneModel = new Hashtable<String, Object>();


			Object targetComponent = SchemaModelTransformation.transform( materializedComponent.getSchemaRepresentationModel(), targetModel, materializedComponent, indexOfTransformedObjectsSchemaPathForOneModel );


			add( targetComponent, indexOfTransformedObjectsSchemaPathForOneModel );

			if( hasConstraintViolation ) return;


			add( targetModel );
		}


		//ListManipulation.stdPrint( PrintMessages.transformationPrint, getLeafComponentsAndStructures(), "Leaf components/structures of" + getTag() );


		PrintMessages.print( PrintMessages.transformationPrint, "END - transform: " + getTag() + " - hasTransformedReferencedType= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedReferencedComponent ) + " hasConstraintViolation= " + hasConstraintViolation );
	}

	private void add( String targetModel ){

		if( hasTransformedReferencedComponent == null ) hasTransformedReferencedComponent = new Hashtable<String, Boolean>();

		hasTransformedReferencedComponent.put( targetModel, true );
	}

	private void add( Object component, Map<String, Object> indexOfTransformedObjectsSchemaPathForOneModel ){

		if( ObjectConstraintsManipulation.setContainmentError( this, component ) == null ){

			if( materializedComponentsInOtherModels == null ){

				materializedComponentsInOtherModels = new Hashtable<String, Component>();

				hasMaterializedComponentsInOtherModels = true;
			}

			if( indexOfTransformedObjectsSchemaPath == null ) indexOfTransformedObjectsSchemaPath = new Hashtable<String, Map<String, Object>>();


			materializedComponentsInOtherModels.put( ( (Component) component).getSchemaRepresentationModel(), (Component) component );

			if( indexOfTransformedObjectsSchemaPathForOneModel != null ) indexOfTransformedObjectsSchemaPath.put( ( (Component) component).getSchemaRepresentationModel(), indexOfTransformedObjectsSchemaPathForOneModel );


			String[] printKinds = { PrintMessages.connectionXMLWithXMLPrint, PrintMessages.connectionWSDLWithXMLPrint };

			MapPrinting.stdPrintObjectsIndex( printKinds, indexOfTransformedObjectsSchemaPathForOneModel, "indexOfTransformedObjectsSchemaPathForOneModel" );
		}
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation( Object parentObject ){

		return getFinalConstraintViolation( parentObject ) != null ? true : false;
	}

	public String getFinalConstraintViolation( Object parentObject ){

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any 'checkConstraint()' method.


		if( hasType && hasUnmaterializedComponent ) ObjectConstraintsManipulation.setUnconnectedComponent( parentObject, constraintViolation );

		else if( hasType && ! isBuiltInType && ! hasUnmaterializedComponent && ! belongsToCycle && materializedComponent.hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, materializedComponent );

		else if( hasMaterializedComponentsInOtherModels )

			for( Enumeration<String> enumeration = ( (Hashtable<String, Component>) materializedComponentsInOtherModels ).keys(); enumeration.hasMoreElements(); ){

				String model = enumeration.nextElement();

				Component component = materializedComponentsInOtherModels.get( model );


				if( component.hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, component );
			}


		return constraintViolation;
	}

	public List<Object> getLeafComponentsAndStructures(){

		return getLeafComponentsAndStructures( getSchemaRepresentationModel(), XMLSchemaRepresentationController.getCompletelyUnstructuredSchemaRepresentation(), null, null );
	}

	public List<Object> getLeafComponentsAndStructures( String primarySchemaRepresentationModel, String targetSchemaRepresentationModel, List<Object> leaves, Map<String, Object> indexOfTransformedObjectsPath ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - getLeafComponentsAndStructures: " + getTag() + " - hasTransformedReferencedComponent= " + MapGetter.getBooleanValueByKey( targetSchemaRepresentationModel, hasTransformedReferencedComponent ) + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + targetSchemaRepresentationModel );


		if( ! isBuiltInType ){

			if( targetSchemaRepresentationModel != null && indexOfTransformedObjectsPath == null ) indexOfTransformedObjectsPath = getIndexOfMaterializedComponentsPath( targetSchemaRepresentationModel );
	
	
			if( isThePrimarySchemaRepresentationModel( targetSchemaRepresentationModel ) ) leaves = materializedComponent.getLeafComponentsAndStructures( primarySchemaRepresentationModel, targetSchemaRepresentationModel, leaves, indexOfTransformedObjectsPath );
	
			else if( MapGetter.getBooleanValueByKey( targetSchemaRepresentationModel, hasTransformedReferencedComponent ) ){
	
				Component component = getMaterializedComponent( targetSchemaRepresentationModel );
	
				leaves = component.getLeafComponentsAndStructures( primarySchemaRepresentationModel, targetSchemaRepresentationModel, leaves, indexOfTransformedObjectsPath );
			}
		}


		PrintMessages.print( PrintMessages.transformationPrint, "END - getLeafComponentsAndStructures: " + getTag() + " - hasTransformedReferencedComponent= " + MapGetter.getBooleanValueByKey( targetSchemaRepresentationModel, hasTransformedReferencedComponent ) + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + targetSchemaRepresentationModel );
		

		return leaves;
	}

	public boolean hasUnmaterializedComponent(){

		return hasUnmaterializedComponent;
	}

	public boolean hasMaterializedComponentsInOtherModels(){

		return hasMaterializedComponentsInOtherModels;
	}

	public Map<String, Component> getMaterializedComponentsInOtherModels(){

		return materializedComponentsInOtherModels;
	}

	public boolean hasTransformedReferencedComponent( String targetModel ){

		return MapGetter.getBooleanValueByKey( targetModel, hasTransformedReferencedComponent );
	}

	protected boolean hasPrefix(){

		return hasPrefix;
	}

	public String getPrefix(){

		return prefix;
	}

	public boolean hasType(){

		return hasType;
	}

	public String getType(){

		return type;
	}

	public String getTypeKind(){

		return typeKind;
	}

	public Component getMaterializedComponent(){

		return materializedComponent;
	}

	public Component getMaterializedComponent( String schemaRepresentationModel ){

		if( ! isBuiltInType ){

			if( ! hasUnmaterializedComponent && materializedComponent.getSchemaRepresentationModel().equals( schemaRepresentationModel ) ) return materializedComponent;

			if( ! isBuiltInType && hasMaterializedComponentsInOtherModels ) return materializedComponentsInOtherModels.get( schemaRepresentationModel );
		}


		return null;
	}

	public Map<String, Object> getIndexOfMaterializedComponentsPath( String schemaRepresentationModel ){

		return hasMaterializedComponentsInOtherModels ? indexOfTransformedObjectsSchemaPath.get( schemaRepresentationModel ) : null;
	}

	public boolean isBuiltInType(){

		return isBuiltInType;
	}

	public boolean belongsToCycle(){

		return belongsToCycle;
	}

	public boolean isThePrimarySchemaRepresentationModel( String schemaRepresentationModel ){

		return schemaRepresentationModel != null ? schemaRepresentationModel.equals( getSchemaRepresentationModel() ) : false;
	}

	public String getSchemaRepresentationModel(){

		return ! isBuiltInType() && ! hasUnmaterializedComponent ? materializedComponent.getSchemaRepresentationModel() : null;
	}

	public String getName(){

		return getTagName();
	}

	public String getTagName(){

		return typeKind;
	}

	public String getTag(){

		String label = "";

		if( typeKind != null && ! typeKind.isEmpty() ) label += " " + typeKind + " = '";

		else label += "'";


		if( hasPrefix ) label += prefix + ":";
	
		if( hasType ) label += type + "'";


		return label;
	}


	//Visualization.
	public DefaultMutableTreeNode visualize(){

		DefaultMutableTreeNode node = null;


		if( ! isBuiltInType && ! belongsToCycle ){

			node = new DefaultMutableTreeNode( "The definition of the '" + getTag() + "' in '" + materializedComponent.getSchemaRepresentationModel() + "' is:"  );
	
			node.add( materializedComponent.visualize() );


			//ListManipulation.stdPrint( PrintMessages.transformationPrint, getLeafComponentsAndStructures(), "Leaf components/structures of" + getTag() );


			if( hasMaterializedComponentsInOtherModels )
	
				for( Enumeration<String> i = ( (Hashtable<String, Component>) materializedComponentsInOtherModels ).keys(); i.hasMoreElements(); ){
	
					String model = i.nextElement();
	
					Component component = materializedComponentsInOtherModels.get( model );
	
					String componentName = component.getName();
	
	
					if( componentName.equals( type ) ){
	
						DefaultMutableTreeNode componentNode = new DefaultMutableTreeNode( "The definition of the '" + getTag() + "' in '" + model + "' is:" );

						componentNode.add( component.visualize() );

						node.add( componentNode );
					}
				}
		}


		return node;
	}
	
	
	//Setters used for xml transformation
	
	public void setxPrefix( String cPrefix ){
		prefix = cPrefix;
	}
	
	public void setxHasPrefix( boolean cHasPrefix ){
		hasPrefix = cHasPrefix;
	}
	
	public void setxType( String cType ){
		type = cType;
	}
	
	public void setxHasType( boolean cHasType ){
		hasType = cHasType;
	}
	
	public void setxTypeKind( String cTypeKind ){
		typeKind = cTypeKind;
	}
	
	public void setxIsBuiltInType( boolean cIsBuiltInType ){
		isBuiltInType = cIsBuiltInType;
	}
	
	public void setxBelongsToCycle( boolean cBelongsToCycle ){
		belongsToCycle = cBelongsToCycle;
	}
	
	public void setxMaterializedComponent( Component cMaterializedComponent ){
		materializedComponent = cMaterializedComponent;
	}
	
	public void setxHasUnmaterializedComponent( boolean cHasUnmaterializedComponent ){
		hasUnmaterializedComponent = cHasUnmaterializedComponent;
	}
	
	public void setxHasTransformedReferencedComponent( Map<String, Boolean> cHasTransformedReferencedComponent ){
		hasTransformedReferencedComponent = cHasTransformedReferencedComponent;
	}
	
	public void setxMaterializedComponentsInOtherModels( Map<String, Component> cMaterializedComponentsInOtherModels ){
		materializedComponentsInOtherModels = cMaterializedComponentsInOtherModels;
	}
	
	public void setxBelongsToCycleInOtherModels( Map<String, Boolean> cBelongsToCycleInOtherModels ){
		belongsToCycleInOtherModels = cBelongsToCycleInOtherModels;
	}
	
	/*public void setxIndexOfTransformedObjectsSchemaPath( Map<String, Map<String, Object>> cIndexOfTransformedObjectsSchemaPath ){
		indexOfTransformedObjectsSchemaPath = cIndexOfTransformedObjectsSchemaPath;
	}*/
	
	public void setxHasMaterializedComponentsInOtherModels( boolean cHasMaterializedComponentsInOtherModels ){
		hasMaterializedComponentsInOtherModels = cHasMaterializedComponentsInOtherModels;
	}
	
	public void setxConstraintViolation( String cConstraintViolation ){
		constraintViolation = cConstraintViolation;
	}
	
	public void setxHasConstraintViolation( boolean cHasConstraintViolation ){
		hasConstraintViolation = cHasConstraintViolation;
	}
	
	//Getters used for xml transformation
	
	public String getxPrefix(){
		return prefix;
	}
	
	public boolean getxHasPrefix(){
		return hasPrefix;
	}
	
	public String getxType(){
		return type;
	}
	
	public boolean getxHasType(){
		return hasType;
	}
	
	public String getxTypeKind(){
		return typeKind;
	}
	
	public boolean getxIsBuiltInType(){
		return isBuiltInType;
	}
	
	public boolean getxBelongsToCycle(){
		return belongsToCycle;
	}
	
	public Component getxMaterializedComponent(){
		return materializedComponent;
	}
	
	public boolean getxHasUnmaterializedComponent(){
		return hasUnmaterializedComponent;
	}
	
	public Map<String, Boolean> getxHasTransformedReferencedComponent(){
		return hasTransformedReferencedComponent;
	}
	
	public Map<String, Component> getxMaterializedComponentsInOtherModels(){
		return materializedComponentsInOtherModels;
	}
	
	public Map<String, Boolean> getxBelongsToCycleInOtherModels(){
		return belongsToCycleInOtherModels;
	}
	
	/*public Map<String, Map<String, Object>> getxIndexOfTransformedObjectsSchemaPath(){
		return indexOfTransformedObjectsSchemaPath;
	}*/
	
	public boolean getxHasMaterializedComponentsInOtherModels(){
		return hasMaterializedComponentsInOtherModels;
	}
	
	public String getxConstraintViolation(){
		return constraintViolation;
	}
	
	public boolean getxHasConstraintViolation(){
		return hasConstraintViolation;
	}
}
