package org.ow2.choreos.schemaRepresentation.xml;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.classInfo.ClassInfo;
import org.ow2.choreos.dataStructures.interval.Interval1D;
import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.schemaRepresentation.KindDescr;
import org.ow2.choreos.schemaRepresentation.Schema;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.XMLSchemaCompletelyStructuredRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyUnstructured.XMLCompletelyUnstructuredSchemaRepresentationController;

import org.ow2.choreos.unknownClasses.UnknownClassesController;


/**
 * 
 * @author dathanas
 *
 */
public class XMLSchemaRepresentationController {


	//Variables.
	private static final String xmlSchemaCompletelyStructuredRepresentation = "schemaRepresentation.xml.completelyStructured";//This is the package of the classes that model this schema representation.

	private static final String xmlCompletelyUnstructuredSchemaRepresentation = "completelyUnstructuredSchemaRepresentation.xml.v1";//This is the package of the classes that model this schema representation.

	private static final String unknownSchemaModel = "UnknownSchemaModel";


	//Verifiers.
	public static boolean isSchemaKind( Object object ){

		return object instanceof XMLSchema;
	}

	public static boolean isTextKind( Object object ){

		return object != null ? object.getClass().equals( String.class ) : false;
	}

	public static boolean isReferencedComponentKind( Object object ){

		return object != null ? object.getClass().equals( ReferencedComponent.class ) : false;
	}

	public static boolean isAnnotationKind( Object object ){

		return object != null ? object.getClass().equals( Annotation.class ) : false;
	}

	static boolean isDocumentationKind( Object object ){

		return object != null ? object.getClass().equals( Documentation.class ) : false;
	}

	public static boolean isUnknownSchemaModelKind( Object object ){

		String objectKind = ClassInfo.getClass( object );

		return isUnknownSchemaModel( objectKind );
	}


	private static boolean isAnnotationTagName( String tagName ){

		return tagName != null && tagName.equals( "annotation" ) ? true : false;
	}

	private static boolean isDocumentationTagName( String tagName ){

		return tagName != null && tagName.equals( "documentation" ) ? true : false;
	}

	public static boolean isKnownXMLSchemaRepresentation( String model ){

		return model != null && ( isXMLCompleteSchemaRepresentation( model ) || isCompletelyUnstructuredXMLSchemaRepresentation( model ) ) ? true : false;
	}

	public static boolean isXMLCompleteSchemaRepresentation( String model ){

		return model != null && model.equals( xmlSchemaCompletelyStructuredRepresentation ) ? true : false;
	}

	public static boolean isCompletelyUnstructuredXMLSchemaRepresentation( String model ){

		return model != null && model.equals( xmlCompletelyUnstructuredSchemaRepresentation ) ? true : false;
	}

	public static boolean isUnknownSchemaModel( String objectKind ){

		return objectKind != null && objectKind.equals( unknownSchemaModel ) ? true : false;
	}

	public static boolean isXMLSchemaKind( Object object ){//TODO: to add new models with new conditions...

		if( XMLSchemaCompletelyStructuredRepresentationController.isXMLCompleteSchemaKind( object ) ) return true;


		return false;
	}

	public static List<String> getCompatibleComponentKinds( String model, Object object ){//It returns the compatible component kinds with the 'object' (It is used by an object that does not know the model, e.g. 'MessageType').

		if( isXMLCompleteSchemaRepresentation( model ) ) return XMLSchemaCompletelyStructuredRepresentationController.getCompatibleComponentKinds( object );


		return null;
	}

	static public boolean isComponentKind( Object object ){

		if( XMLSchemaCompletelyStructuredRepresentationController.isComponentKind( object ) ) return true;

		if( XMLCompletelyUnstructuredSchemaRepresentationController.isComponentKind( object ) ) return true;


		return false;
	}

	public static boolean isBuiltInType( Object object ){//TODO: to add new models with new conditions...

		if( isReferencedComponentKind( object ) ) return ( (ReferencedComponent) object ).isBuiltInType();

		if( XMLSchemaCompletelyStructuredRepresentationController.isBuiltInType( object ) ) return true;


		return false;
	}


	public static boolean isStructureKind( Object object ){//TODO: to add new models with new conditions...

		if( XMLSchemaCompletelyStructuredRepresentationController.isStructureKind( object ) ) return true;


		return false;
	}

	public static boolean equalClassPath( String[] classPath, Object object ){

		if( XMLSchemaCompletelyStructuredRepresentationController.isComponentKind( object ) ) return ( (Component) object).equalClassPath( classPath );

		if( XMLSchemaCompletelyStructuredRepresentationController.isStructureKind( object ) ) return ( (Structure) object).equalClassPath( classPath );


		return false;
	}

	public static boolean isSchemaRepresentationKind( Object object ){

		if( isXMLSchemaKind( object ) ) return true;

		if( isAnnotationKind( object ) ) return true;

		if( isDocumentationKind( object ) ) return true;

		if( isReferencedComponentKind( object ) ) return true;

		if( isComponentKind( object ) ) return true;

		if( isStructureKind( object ) ) return true;


		return false;
	}

	public static Boolean isKindDescr( Object kindDescr ){

		return kindDescr != null ? kindDescr instanceof ElementKindDescr : null;
	}

	public static Boolean canBeRootKindDescr( Object kindDescr ){

		return isKindDescr( kindDescr ) ? ComponentTagNames.isComponentTagName( ((ElementKindDescr) kindDescr).getKind() ): null;
	}


	//Getters.
	public static String getCompletelyUnstructuredSchemaRepresentation(){

		return xmlCompletelyUnstructuredSchemaRepresentation;
	}

	public static String getXMLSchemaCompletelyStructuredRepresentationModel(){

		return xmlSchemaCompletelyStructuredRepresentation;
	}

	public static Object getSchema( String tagName, String qualifier, Map<String, String> attributes, String model ){

		if( isXMLCompleteSchemaRepresentation( model ) ) return XMLSchemaCompletelyStructuredRepresentationController.getSchema( tagName, qualifier, attributes );


		return getUnknownSchemaModel( model );
	}

	public static Object getErrorSchema( String tagName, String qualifier, Map<String, String> attributes, String model, String constraintViolation ){

		if( isXMLCompleteSchemaRepresentation( model ) ) return XMLSchemaCompletelyStructuredRepresentationController.getErrorSchema( tagName, qualifier, attributes, constraintViolation );


		return getUnknownSchemaModel( model );
	}

	public static Object getUnknownSchemaModel( String model ){

		return new UnknownSchemaModel( model );
	}

	public static String getUnknownSchemaModel(){

		return unknownSchemaModel;
	}

	public static Object getUnknownSchemaModel( String sourceModel, String targetModel ){

		return new UnknownSchemaModel( sourceModel, targetModel );
	}

	public static Object getTagObject( List<Map<String, String>> path, boolean isGlobal, String model ){//This function is used for parsing purposes.

		String tagName = ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path );


		if( isAnnotationTagName( tagName ) ) return new Annotation( path );

		if( isDocumentationTagName( tagName ) ) return new Documentation( path );


		if( isXMLCompleteSchemaRepresentation( model ) ) return XMLSchemaCompletelyStructuredRepresentationController.getTagObject( path, isGlobal );


		return UnknownClassesController.getUnknownTag( tagName, XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel() );
	}

	public static Object extractObject( Object[] sourceObjects, List<Map<String, String>> path, boolean belongsToInputMessage, boolean isGlobal, String model, String namesDistanceCalculatorAlgorithm ){//This function is used for abstraction extracting purposes.

		if( isXMLCompleteSchemaRepresentation( model ) ) return XMLSchemaCompletelyStructuredRepresentationController.extractObject( sourceObjects, path, belongsToInputMessage, isGlobal, namesDistanceCalculatorAlgorithm );


		return getUnknownSchemaModel( model );
	}

	public static String getUnknownSourceSchemaModel( Object unknownSchemaModel ){//pre-condition: 'unknownSchemaModel' is kind of 'UnknownSchemaModel'.

		return ( (UnknownSchemaModel) unknownSchemaModel).getSourceModel();
	}

	public static String getUnknownTargetSchemaModel( Object unknownSchemaModel ){//pre-condition: 'unknownSchemaModel' is kind of 'UnknownSchemaModel'.

		return ( (UnknownSchemaModel) unknownSchemaModel).getTargetModel();
	}

	public static String getTag( Object object ){

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).getTag();

		if( isAnnotationKind( object ) ) return ( (Annotation) object).getTag();

		if( isDocumentationKind( object ) ) return ( (Documentation) object).getTag();

		if( isReferencedComponentKind( object ) ) return ( (ReferencedComponent) object ).getTag();

		if( isComponentKind( object ) ) return ( (Component) object).getTag();

		if( isStructureKind( object ) ) return ( (Structure) object).getTag();


		return UnknownClassesController.getTagName( object );
	}

	public static String getTagName( Object object ){

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).getTagName();

		if( isAnnotationKind( object ) ) return ( (Annotation) object).getTagName();

		if( isDocumentationKind( object ) ) return ( (Documentation) object).getTagName();

		if( isReferencedComponentKind( object ) ) return ( (ReferencedComponent) object ).getTagName();

		if( isComponentKind( object ) ) return ( (Component) object).getTagName();

		if( isStructureKind( object ) ) return ( (Structure) object).getTagName();


		return UnknownClassesController.getTagName( object );
	}

	public static String getName( Object object ){

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).getName();

		if( isAnnotationKind( object ) ) return ( (Annotation) object).getName();

		if( isDocumentationKind( object ) ) return ( (Documentation) object).getName();

		if( isReferencedComponentKind( object ) ) return ( (ReferencedComponent) object ).getName();

		if( isComponentKind( object ) ) return ( (Component) object).getName();

		if( isStructureKind( object ) ) return ( (Structure) object).getName();


		return UnknownClassesController.getTagName( object );
	}

	public static String getReferencedBuiltInType( Object object ){

		if( isComponentKind( object ) ) return ( (Component) object).getReferenceBuiltInType();

		if( isStructureKind( object ) ) return ( (Structure) object).getReferenceBuiltInType();


		return UnknownClassesController.getTagName( object );
	}

	public static String getSchemaRepresentationModel( Object object ){

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).getSchemaRepresentationModel();

		if( isReferencedComponentKind( object ) ) return ( (ReferencedComponent) object ).getSchemaRepresentationModel();

		if( isComponentKind( object ) ) return ( (Component) object).getSchemaRepresentationModel();

		if( isStructureKind( object ) ) return ( (Structure) object).getSchemaRepresentationModel();

		if( UnknownClassesController.isUnknownTag( object ) ) return UnknownClassesController.getUnknownTagSchemaRepresentationModel( object );

		if( UnknownClassesController.isUnknownObject( object ) ) return UnknownClassesController.getUnknownObjectSourceSchemaRepresentationModel( object );

		if( XMLSchemaRepresentationController.isUnknownSchemaModelKind( object ) ) return XMLSchemaRepresentationController.getUnknownSourceSchemaModel( object );


		return null;
	}

	public static String getConstraintViolation( Object object ){

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).getConstraintViolation();

		if( isReferencedComponentKind( object ) ) return ( (ReferencedComponent) object ).getConstraintViolation();

		if( isAnnotationKind( object ) ) return ( (Annotation) object).getConstraintViolation();

		if( isDocumentationKind( object ) ) return ( (Documentation) object).getConstraintViolation();

		if( isComponentKind( object ) ) return ( (Component) object).getConstraintViolation();

		if( isStructureKind( object ) ) return ( (Structure) object).getConstraintViolation();


		return UnknownClassesController.getUnknownObjectName();
	}

	public static boolean hasConstraintViolation( Object object ){

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).hasConstraintViolation();

		if( isReferencedComponentKind( object ) ) return ( (ReferencedComponent) object ).hasConstraintViolation();

		if( isAnnotationKind( object ) ) return ( (Annotation) object).hasConstraintViolation();

		if( isDocumentationKind( object ) ) return ( (Documentation) object).hasConstraintViolation();

		if( isComponentKind( object ) ) return ( (Component) object).hasConstraintViolation();

		if( isStructureKind( object ) ) return ( (Structure) object).hasConstraintViolation();


		return true;
	}

	public static boolean hasFinalConstraintViolation( Object object ){

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).hasFinalConstraintViolation();

		if( isReferencedComponentKind( object ) ) return ( (ReferencedComponent) object ).hasConstraintViolation();

		if( isAnnotationKind( object ) ) return ( (Annotation) object).hasConstraintViolation();

		if( isDocumentationKind( object ) ) return ( (Documentation) object).hasConstraintViolation();

		if( isComponentKind( object ) ) return ( (Component) object).hasFinalConstraintViolation();

		if( isStructureKind( object ) ) return ( (Structure) object).hasFinalConstraintViolation();


		return true;
	}

	public static DefaultMutableTreeNode visualize( Object object ){

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).visualize();

		if( isReferencedComponentKind( object ) ) return ( (ReferencedComponent) object ).visualize();

		if( isAnnotationKind( object ) ) return ( (Annotation) object).visualize();

		if( isDocumentationKind( object ) ) return ( (Documentation) object).visualize();

		if( isComponentKind( object ) ) return ( (Component) object).visualize();

		if( isStructureKind( object ) ) return ( (Structure) object).visualize();


		return null;
	}

	public static boolean belongsToCycle( Component materializedComponent, Object object ){

		if( isComponentKind( object ) ) return ( (Component) object).belongsToCycle( materializedComponent );

		if( isStructureKind( object ) ) return ( (Structure) object).belongsToCycle( materializedComponent );


		return false;
	}

	public static Map<String, String> getTagAttributes( Object object ){

		if( isComponentKind( object ) ) return ( (Component) object).getTagAttributes();

		if( isStructureKind( object ) ) return ( (Structure) object).getTagAttributes();


		return null;
	}


	public static boolean hasComponents( Object object ){

		if( isComponentKind( object ) ) return ( (Component) object).hasComponents();

		else return ( (Structure) object).hasComponents();
	}

	public static List<Component> getComponents( Object object ){

		if( isComponentKind( object ) ) return ( (Component) object).getComponents();

		else return ( (Structure) object).getComponents();
	}

	public static boolean hasStructures( Object object ){

		if( isComponentKind( object ) ) return ( (Component) object).hasStructures();

		else return ( (Structure) object).hasStructures();
	}

	public static List<Structure> getStructures( Object object ){

		if( isComponentKind( object ) ) return ( (Component) object).getStructures();

		else return ( (Structure) object).getStructures();
	}

	public static List<?> getChildren( Object object, String targetSchemaRepresentationModel ){

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).getChildren();

		if( isComponentKind( object ) ) return ( (Component) object).getChildren( targetSchemaRepresentationModel );

		if( isStructureKind( object ) ) return ( (Structure) object).getChildren( targetSchemaRepresentationModel );

		if( isAnnotationKind( object ) ) return ( (Annotation) object).getChildren();

		if( isDocumentationKind( object ) ) return ( (Documentation) object).getChildren();


		return null;
	}

	public static List<?> getChildrenWithoutAnnotation( Object object, String targetSchemaRepresentationModel ){

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).getChildrenWithoutAnnotation();

		if( isComponentKind( object ) ) return ( (Component) object).getChildrenWithoutAnnotation( targetSchemaRepresentationModel );

		if( isStructureKind( object ) ) return ( (Structure) object).getChildrenWithoutAnnotation( targetSchemaRepresentationModel );


		return null;
	}


	public static String getSchemaPath( Object object ){

		if( isComponentKind( object ) ) return ( (Component) object).getSchemaPathToString();

		else return ( (Structure) object).getSchemaPathToString();
	}

	public static boolean hasMaterializedTypes( Object object ){

		if( isComponentKind( object ) ) return ( (Component) object).hasMaterializedType();

		if( isStructureKind( object ) ) return ( (Structure) object).hasMaterializedTypes();


		return false;
	}

	public static List<Component> getMaterializedTypes( Object object ){

		List<Component> components = null;


		if( isComponentKind( object ) ){

			Component component = ( (Component) object).getMaterializedType();

			components = new ArrayList<Component>();

			components.add( component );
		}


		else if( isStructureKind( object ) ) components = ( (Structure) object).getMaterializedTypes();


		return components;
	}

	public static String getFlatReferencedComponentKinds(){

		return "[ ReferencedComponent ]";
	}

	public static Object newRuleForSimilarElementKinds( String category, String elementKind1, boolean elementKindIsDecl1, boolean elementKindHasBuiltInType1, String elementKind2, boolean elementKindIsDecl2, boolean elementKindHasBuiltInType2, String cQuantitativeSimilarityValue ){

		return new RuleForSimilarElementKinds( category, elementKind1, elementKindIsDecl1, elementKindHasBuiltInType1, elementKind2, elementKindIsDecl2, elementKindHasBuiltInType2, cQuantitativeSimilarityValue );
	}

	public static Object newRuleForSimilarElementKinds( String category, String elementKind1, boolean elementKindIsDecl1, boolean elementKindHasBuiltInType1, String elementKind2, boolean elementKindIsDecl2, boolean elementKindHasBuiltInType2, String cQualitativeSimilarityValue, String cQuantitativeSimilarityValue ){

		return new RuleForSimilarElementKinds( category, elementKind1, elementKindIsDecl1, elementKindHasBuiltInType1, elementKind2, elementKindIsDecl2, elementKindHasBuiltInType2, cQualitativeSimilarityValue, cQuantitativeSimilarityValue );
	}

	public static String[] getTagNames(){

		return XMLSchema.getTagNames();
	}

	public static KindDescr getElementKindDescr( Object object ){

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).getKindDescr();

		if( isComponentKind( object ) ) return ( (Component) object).getKindDescr();

		if( isStructureKind( object ) ) return ( (Structure) object).getKindDescr();


		return null;
	}

	public static boolean isContent( String parentOfCurrentObjectTagName, String currentObjectTagName, String nextObjectTagName, String parentOfNextObjectTagName ){

		return XMLSchema.isContent( parentOfCurrentObjectTagName, currentObjectTagName, nextObjectTagName, parentOfNextObjectTagName );
	}

	static public Interval1D getScope( Object object ){

		if( isAnnotationKind( object ) ) return ( (Annotation) object).getScope();

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).getScope();

		if( isComponentKind( object ) ) return ( (Component) object).getScope();

		if( isStructureKind( object ) ) return ( (Structure) object).getScope();


		return null;
	}

	static public Object getParent( Object object ){

		if( isAnnotationKind( object ) ) return ( (Annotation) object).getParent();

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).getParent();

		if( isComponentKind( object ) ) return ( (Component) object).getParent();

		if( isStructureKind( object ) ) return ( (Structure) object).getParent();


		return null;
	}

	static public boolean hasOrderedChildren( String tagName ){

		return XMLSchema.hasOrderedChildren( tagName );
	}


	//Setters.
	public static Object set( Object internal, Object external ){

		PrintMessages.print( PrintMessages.controllerPrint, "XMLSchemaRepresentationController: Set the internal= " + ClassInfo.getClass( internal ) + " to external= " + ClassInfo.getClass( external ) );


		if( isXMLSchemaKind( external ) ) return ( (Schema) external).add( internal );

		if( isAnnotationKind( external ) ) return ( (Annotation) external).add( internal );


		String model = getSchemaRepresentationModel( external );

		if( isXMLCompleteSchemaRepresentation( model ) ) return XMLSchemaCompletelyStructuredRepresentationController.add( internal, external );

		if( isCompletelyUnstructuredXMLSchemaRepresentation( model ) ) return XMLCompletelyUnstructuredSchemaRepresentationController.add( internal, external );


		return external;
	}

	public static String setConstraintViolation( Object object, String constraintViolation ){

		if( isReferencedComponentKind( object ) ) return ( (ReferencedComponent) object ).setConstraintViolation( constraintViolation );

		if( isXMLSchemaKind( object ) ) return ( (Schema) object).setConstraintViolation( constraintViolation );

		if( isComponentKind( object ) ) return ( (Component) object).setConstraintViolation( constraintViolation );

		if( isStructureKind( object ) ) return ( (Structure) object).setConstraintViolation( constraintViolation );


		return UnknownClassesController.getUnknownTagName();
	}

	public static void setMyOrderID( Object object, int myOrderID ){

		if( isAnnotationKind( object ) ) ( (Annotation) object).setMyOrderID( myOrderID );

		if( isComponentKind( object ) ) ( (Component) object).setMyOrderID( myOrderID );

		if( isStructureKind( object ) ) ( (Structure) object).setMyOrderID( myOrderID );
	}

	static public void setScope( Object object, Interval1D scope ){

		if( isAnnotationKind( object ) ) ( (Annotation) object).setScope( scope );

		if( isXMLSchemaKind( object ) ) ( (Schema) object).setScope( scope );

		if( isComponentKind( object ) ) ( (Component) object).setScope( scope );

		if( isStructureKind( object ) ) ( (Structure) object).setScope( scope );
	}

	static public void setParent( Object object, Object parent ){

		if( isAnnotationKind( object ) ) ( (Annotation) object).setParent( parent );

		if( isXMLSchemaKind( object ) ) ( (Schema) object).setParent( parent );

		if( isComponentKind( object ) ) ( (Component) object).setParent( parent );

		if( isStructureKind( object ) ) ( (Structure) object).setParent( parent );
	}
}
