package org.ow2.choreos.schemaRepresentation.xml.completelyStructured;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.classInfo.ClassInfo;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchema;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.ComponentController;


/**
 * 
 * @author dathanas
 * This class models an XML schema while keeping all the available information that may exist in tag schema!
 */
public class XMLSchemaCompletelyStructured extends XMLSchema {


	//Variables.
	private String attributeFormDefault;
	private boolean hasAttributeFormDefault;

	private List<String> blockDefault;
	private boolean hasBlockDefault;

	private String elementFormDefault;
	private boolean hasElementFormDefault;

	private List<String> finalDefault;
	private boolean hasFinalDefault;

	private String id;
	private boolean hasID;

	private String version;//e.g., v1.0.
	private boolean hasVersion;

	private String xmlLang;//e.g., XSD.
	private boolean hasXmlLang;


	//Constructor.
	public XMLSchemaCompletelyStructured( String qualifier, Map<String, String> attributes ){

		super( qualifier, attributes );


		hasBlockDefault = false;
		hasFinalDefault = false;


		parse( "blockDefault", MapGetter.getValueByKeyToLowerCase( "blockDefault", getAttributes() ) );
		parse( "finalDefault", MapGetter.getValueByKeyToLowerCase( "finalDefault", getAttributes() ) );


		String cAttributeFormDefault = MapGetter.getValueByKeyToLowerCase( "attributeFormDefault", getAttributes() );

		if( cAttributeFormDefault != null && ! cAttributeFormDefault.isEmpty() && checkConstraint2( "attribute", cAttributeFormDefault ) != null ){

			attributeFormDefault = cAttributeFormDefault;

			hasAttributeFormDefault = true;
		}

		else hasAttributeFormDefault = false;


		String cElementFormDefault = MapGetter.getValueByKeyToLowerCase( "elementFormDefault", getAttributes() );

		if( cElementFormDefault != null && ! cElementFormDefault.isEmpty() && checkConstraint2( "element", cElementFormDefault ) != null ){

			elementFormDefault = cElementFormDefault;

			hasElementFormDefault = true;
		}

		else hasElementFormDefault = false;


		String cID =  (String) MapGetter.getValueByStringKey( "id", getAttributes() );

		if( cID != null && ! cID.isEmpty() ){

			id = cID;

			hasID = true;
		}

		else hasID = false;


		String cVersion =  (String) MapGetter.getValueByStringKey( "version", getAttributes() );

		if( cVersion != null && ! cVersion.isEmpty() ){

			version = cVersion;

			hasVersion = true;
		}

		else hasVersion = false;


		String cXmlLang =  (String) MapGetter.getValueByStringKey( "xml:lang", getAttributes() );

		if( cXmlLang != null && ! cXmlLang.isEmpty() ){

			xmlLang = cXmlLang;

			hasXmlLang = true;
		}

		else hasXmlLang = false;
	}


	//Methods used by the constructor.
	void parse( String label, String values ){

		if( values == null || ( values != null && values.isEmpty() ) ) return;
			
		int pos = values.indexOf(" ");

		if( pos <= 0 ){//Only one value exists.

			if( checkConstraint3( label, values ) == null ) add( label, values );
		}

		else{

			while( pos > 0 ){

				String tmpValue = values.substring( 0, pos );

				if( checkConstraint3( label, tmpValue ) == null ) add( label, tmpValue );

				values = values.substring( pos + 1, values.length() );

				pos = values.indexOf(" ");
			}

			if( checkConstraint3( label, values ) == null ) add( label, values );
		}

		checkConstraint4( label );
	}

	private void add( String label, String value ){

		if( label.equals( "blockDefault" ) ){

			if( ! hasBlockDefault ){

				blockDefault = new ArrayList<String>();

				hasBlockDefault = true;
			}

			blockDefault.add( value );
		}

		else{

			if( ! hasFinalDefault ){

				finalDefault = new ArrayList<String>();

				hasFinalDefault = true;
			}

			finalDefault.add( value );
		}
	}


	//Overridden methods.
	protected boolean isComponentKind( Object object ){

		return ComponentController.isComponentKind( object );
	}

	protected String getComponentKinds(){

		return ComponentController.getFlatComponentKinds();
	}

	protected boolean isSchemaKind( Object object ){

		return XMLSchemaCompletelyStructuredRepresentationController.isXMLCompleteSchemaKind( object );
	}

	protected String getSchemaKinds(){

		return "[ " + ClassInfo.getClass( this ) + " ]";
	}

	public String getSchemaRepresentationModel(){

		return XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel();
	}


	//Constraint checkers (constraint1 is checked by the abstract class).
	private String checkConstraint2( String label, String form ){//pre-condition= form != null.

		return ! form.equals( "qualified" ) && ! form.equals( "unqualified" ) ? setConstraintViolation( "'schema': Error value in the '" + label + "FormDefault' attribute ( form= ( qualified | unqualified ) )." ) : null;
	}

	private String checkConstraint3( String label, String value ){

		if( label.equals( "blockDefault" ) ) return ( value != null && ! value.isEmpty() && ! value.equals( "#all" ) && ! value.equals( "extension" ) && ! value.equals( "restriction" ) ) ? ObjectConstraintsManipulation.setErrorAttribute( this, label, label + "= '#all', 'extension' and 'restriction'" ) : null;

		else return ( value != null && ! value.isEmpty() && ! value.equals( "#all" ) && ! value.equals( "extension" ) && ! value.equals( "restriction" ) && ! value.equals( "list" ) ) ? ObjectConstraintsManipulation.setErrorAttribute( this, label, label + "= '#all', 'extension', 'restriction' and 'list" ) : null;
	}

	private String checkConstraint4( String label ){

		int occurrences = 0;

		String message = "The '#all' value cannot appear in a list of values";


		if( label.equals( "blockDefault" ) ){

			for( int i = 0; hasBlockDefault && i < blockDefault.size(); ++i ) if( blockDefault.get( i ).equals( "#all" ) ) ++occurrences;

			return ( hasBlockDefault && blockDefault.size() > 1 && occurrences > 0 ) ? ObjectConstraintsManipulation.setErrorAttribute( this, label, message ) : null;
		}

		else{

			for( int i = 0; hasFinalDefault && i < finalDefault.size(); ++i ) if( finalDefault.get( i ).equals( "#all" ) ) ++occurrences;

			return ( hasFinalDefault && finalDefault.size() > 1 && occurrences > 0 ) ? ObjectConstraintsManipulation.setErrorAttribute( this, label, message ) : null;
		}
	}


	//Getters.
	public String getAttributeFormDefault(){

		return attributeFormDefault;
	}

	public boolean hasBlockDefault(){

		return hasBlockDefault;
	}

	public List<String> getBlockDefault(){

		return blockDefault;
	}

	public String getElementFormDefault(){

		return elementFormDefault;
	}

	public boolean hasFinalDefault(){

		return hasFinalDefault;
	}

	public List<String> getFinalDefault(){

		return finalDefault;
	}

	public boolean hasID(){

		return hasID;
	}

	public String getID(){

		return id;
	}

	public String getTagName(){

		return (String) MapGetter.getValueByStringKey( SchemaRepresentationController.getTagNameKey(), getAttributes() );
	}

	public String getTag(){

		String tag = getTagName() + " (model= " + getSchemaRepresentationModel() + ") : " + printQualifiers();


		if( hasID ) tag += " id= '" + id + "'";

		if( hasXmlLang ) tag += " xml:lang= '" + xmlLang + "'";

		if( hasVersion ) tag += " version= '" + version + "'";

		if( hasAttributeFormDefault && attributeFormDefault.equals( "qualified" ) ) tag += " attributeFormDefault= 'qualified'";

        if( hasBlockDefault ) tag += " block= '" + MapGetter.getValueByKeyToLowerCase( "blockDefault", getAttributes() ) + "'";

        if( hasElementFormDefault && elementFormDefault.equals( "qualified" ) ) tag += " elementFormDefault= 'qualified'";

        if( hasFinalDefault ) tag += " final= '" + MapGetter.getValueByKeyToLowerCase( "finalDefault", getAttributes() ) + "'";


		return tag;
	}
}
