package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.classInfo.ClassInfo;
import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapTransfomation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.attribute.AttributeController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.dataType.DataTypeController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.element.ElementController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.group.GroupController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.identityConstraint.IdentityConstraintController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.structure.StructureController;
import org.ow2.choreos.serviceRepresentation.functional.FunctionalServiceRepresentationController;
import org.ow2.choreos.unknownClasses.UnknownClassesController;


/**
 * 
 * @author dathanas
 *
 */
@SuppressWarnings("serial")
public class ComponentController {


	//Variables.
	private static List<String> compatibleMessageTypeKinds = new ArrayList<String>() { { add( "ElementDefinition" );
																						 add( "ElementDeclaration" );
																						 add( "ComplexType" ); } };

	private static Map<String, ArrayList<String>> compatibleStructureKinds = new Hashtable<String, ArrayList<String>>() { {

		put( "ComplexContentSpecializationKind",  new ArrayList<String>() { { add( "ComplexType" ); } } );
																														} };

	private static Map<String, ArrayList<String>> compatibleComponentKinds = new Hashtable<String, ArrayList<String>>() { {

		put( "ElementDeclaration",  		new ArrayList<String>() { { add( "SimpleType" );
																		add( "ComplexType" );
																		add( "ElementDefinition" );
																		add( "ElementDeclaration" ); } } );
		put( "ElementGroupDeclaration", 	new ArrayList<String>() { { add( "ElementGroupDefinition" ); } } );
		put( "AttributeDeclaration",  		new ArrayList<String>() { { add( "AttributeDeclaration" ); 
																		add( "SimpleType" ); } } );
		put( "AttributeGroupDeclaration",  	new ArrayList<String>() { { add( "AttributeGroupDefinition" ); } } );
		put( "IdentityConstraintKeyRef",  	new ArrayList<String>() { { add( "IdentityConstraint" ); } } );
																														} };


	//Verifiers.
	public static List<String> getCompatibleComponentKinds( Object object ){//It returns the compatible component kinds with the 'object'.

		if( FunctionalServiceRepresentationController.isMessageTypeKind( object ) ) return compatibleMessageTypeKinds;


		String objectKind = ClassInfo.getClass( object );

		if( StructureController.isStructureKind( object ) ) return compatibleStructureKinds.get( objectKind );

		if( isComponentKind( object ) ) return compatibleComponentKinds.get( objectKind );


		return null;
	}

	public static boolean isComponentKind( Object object ){

		if( DataTypeController.isDataTypeKind( object ) ) return true;

		if( AttributeController.isAttributeKind( object ) ) return true;

		if( ElementController.isElementKind( object ) ) return true;

		if( IdentityConstraintController.isIdentityConstraintKind( object ) ) return true;

		if( IdentityConstraintController.isSelectorOrFieldKind( object ) ) return true;

		if( GroupController.isGroupKind( object ) ) return true;


		return false;
	}

	public static boolean isComponentTagName( String tagName ){

		if( ElementController.isElementTagName( tagName ) ) return true;

		if( IdentityConstraintController.isIdentityConstraintTagName( tagName ) ) return true;

		if( IdentityConstraintController.isSelectorOrFieldTagName( tagName ) ) return true;

		if( AttributeController.isAttributeTagName( tagName ) ) return true;

		if( DataTypeController.isDataTypeTagName( tagName ) ) return true;

		if( GroupController.isGroupTagName( tagName ) ) return true;


		return false;
	}


	//Getters.
	public static Object getTagObject( List<Map<String, String>> path, boolean isGlobal ){

		String tagName = ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path );


		if( DataTypeController.isDataTypeTagName( tagName ) ) return DataTypeController.getTagObject( path, isGlobal );

		if( AttributeController.isAttributeTagName( tagName ) ) return AttributeController.getTagObject( path, isGlobal );

		if( ElementController.isElementTagName( tagName ) ) return ElementController.getTagObject( path, isGlobal );

		if( IdentityConstraintController.isIdentityConstraintTagName( tagName ) ) return IdentityConstraintController.getTagObject( path, isGlobal );

		if( IdentityConstraintController.isSelectorOrFieldTagName( tagName ) ) return IdentityConstraintController.getTagObject( path, isGlobal );

		if( GroupController.isGroupTagName( tagName ) ) return GroupController.getTagObject( path, isGlobal );


		return UnknownClassesController.getUnknownTag( tagName, XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel() );
	}

	public static String getFlatComponentKinds(){

		Map<String, String> map1 = MapTransfomation.concat( DataTypeController.getDataTypeKinds(), AttributeController.getAttributeKinds(), ElementController.getElementKinds(), GroupController.getGroupKinds() );

		Map<String, String> map2 = MapTransfomation.concat( IdentityConstraintController.getIdentityConstaintsKinds(), IdentityConstraintController.getSelectorOrFieldKinds() );


		return MapPrinting.printCommaSeparated( MapTransfomation.concat( map1, map2 ) );
	}
}
