package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.attribute;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.Annotation;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.ComponentTagNames;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;


/**
 * 
 * @author dathanas
 *
 */
class AnyAttribute extends Component {


	//Variables.
	private static final long serialVersionUID = 7986428575755591525L;

	private List<String> namespace;
	private boolean hasNamespace;

	private String processContents;
	private boolean hasProcessContents;


	//Constructor.
	AnyAttribute( List<Map<String, String>> path, boolean isGlobal ) {

		super( path, isGlobal );


		checkConstraint3();

		checkConstraint4();


		parse( (String) MapGetter.getValueByStringKey( "namespace", getTagAttributes() ) );


		String cProcessContents = MapGetter.getValueByKeyToLowerCase( "processContents", getTagAttributes() );

		if( cProcessContents != null && checkConstraint2( cProcessContents ) == null ){

			processContents = cProcessContents;

			hasProcessContents = true;
		}

		else hasProcessContents = false;
	}


	//Methods used by the constructor.
	void parse( String values ){

		if( values == null || ( values != null && values.isEmpty() ) ) return;


		int pos = values.indexOf(" ");

		if( pos <= 0 ){//Only one value exists.

			if( checkConstraint1( values ) == null ) add( values );
		}

		else{

			while( pos > 0 ){

				String tmpValue = values.substring( 0, pos );

				if( checkConstraint1( tmpValue ) == null ) add( tmpValue );

				values = values.substring( pos + 1, values.length() );

				pos = values.indexOf(" ");
			}

			if( checkConstraint1( values ) == null ) add(  values );
		}

		checkConstraint1();
	}

	private void add( String value ){

		if( ! hasNamespace ){

			namespace = new ArrayList<String>();

			hasNamespace = true;
		}

		namespace.add( value );
	}


	//Check constraints.
	private String checkConstraint1( String value ){

		return ( value != null && ! value.equals( "##any" ) && ! value.equals( "##other" ) && ! value.equals( "anyURI" ) && ! value.equals( "##targetNamespace" ) && ! value.equals( "##local" ) ) ? ObjectConstraintsManipulation.setErrorAttribute( this, "namespace", "namespace= '##any', '##other', 'anyURI', '##targetNamespace' and '##local'" ) : null;
	}

	private String checkConstraint1(){

		int anyOccurs = 0, otherOccurs = 0;

		for( int i = 0; hasNamespace && i < namespace.size(); ++i ){

			if( namespace.get( i ).equals( "##any") ) anyOccurs++;

			if( namespace.get( i ).equals( "##other") ) otherOccurs++;
		}

		if( hasNamespace && namespace.size() > 1 && anyOccurs > 0 ) return ObjectConstraintsManipulation.setErrorAttribute( this, "namespace", "The '#any' value cannot appear in a list of values" );

		if( hasNamespace && namespace.size() > 1 && otherOccurs > 0 ) return ObjectConstraintsManipulation.setErrorAttribute( this, "namespace", "The '#other' value cannot appear in a list of values" );


		return null;
	}

	private String checkConstraint2( String value ){

		return ( value != null && ! value.equals( "lax" ) && ! value.equals( "skip" ) && ! value.equals( "strict" ) ) ? ObjectConstraintsManipulation.setErrorAttribute( this, "processContents", "processContents= 'lax', 'skip', and 'strict'" ) : null;
	}

	private String checkConstraint3(){

		return ( ! isAnonymous() ) ? ObjectConstraintsManipulation.setErrorAttribute( this, "name", "It cannot have a name" ) : null;
	}

	private String checkConstraint4(){

		return ( isGlobal() ) ? setConstraintViolation( "It cannot be global." ) : null;
	}


	//Setters.
	public Object add( Object object ){

		if( ObjectConstraintsManipulation.setContainmentError( this, object ) == null ){

			if( XMLSchemaRepresentationController.isAnnotationKind( object ) ) setAnnotation( (Annotation) object );

			else ObjectConstraintsManipulation.setConnectionError ( this, object, "Valid content:\n\nAnnotation" );
		}

		return this;
	}


	//Getters.
	public boolean hasNamespace(){

		return hasNamespace;
	}

	public List<String> getNamespace(){

		return namespace;
	}

	public String getProcessContents(){

		return processContents;
	}

	public boolean hasProcessContents(){

		return hasProcessContents;
	}

	public String getSchemaRepresentationModel(){

		return XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel();
	}

	public String getSchemaKind(){

		return SchemaRepresentationController.getSchemaKind();
	}

	public String getTagName(){

		return ComponentTagNames.anyAttributeTagName;
	}

	@SuppressWarnings("serial")
	public String getTag(){

		Map<String, String> exceptedKeys = new Hashtable<String, String>() { { put( SchemaRepresentationController.getTagNameKey(), SchemaRepresentationController.getTagNameKey() );
																			   put( "processContents", "processContents" ); } };


		return getTagName() + MapPrinting.printSpaceSeparated( exceptedKeys, getTagAttributes() ) + " processContents= '" + processContents + printMyOrderID();
	}
}
