package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.dataType;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.names.NamesDistanceCalculator;
import org.ow2.choreos.classInfo.ClassInfo;
import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.ComponentTagNames;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.ComponentController;
import org.ow2.choreos.unknownClasses.UnknownClassesController;


/**
 * 
 * @author dathanas
 *
 */
@SuppressWarnings("serial")
public class DataTypeController {


	//Variables.
	private static Map<String, String> dataTypeKinds = new Hashtable<String, String>() { { put( "SimpleType", "SimpleType" ); put( "ComplexType", "ComplexType" ); } };
	private static Map<String, String> simpleTypeKinds = new Hashtable<String, String>() { { put( "SimpleType", "SimpleType" ); } };


	//Verifiers.
	public static boolean isSimpleTypeKind( Object object ){

		return object != null ? object.getClass().equals( SimpleType.class ) : false;
	}

	public static boolean isComplexTypeKind( Object object ){

		return object != null ? object.getClass().equals( ComplexType.class ) : false;
	}

	public static boolean isDataTypeKind( Object object ){

		return MapGetter.objectNameBelongsToMapStrings( object, dataTypeKinds );
	}

	public static boolean isDataTypeKind( String objectKind ){

		return MapGetter.keyBelongsToMapStrings( objectKind, dataTypeKinds );
	}


	public static boolean isDataTypeTagName( String tagName ){

		return tagName != null && ( tagName.toLowerCase().equals( "simpletype" ) || tagName.toLowerCase().equals( "complextype" ) ) ? true : false;
	}

	public static boolean isSimpleTypeTagName( String tagName ){

		return tagName != null && tagName.toLowerCase().equals( "simpletype" ) ? true : false;
	}

	public static boolean isComplexTypeTagName( String tagName ){

		return tagName != null && tagName.toLowerCase().equals( "complextype" ) ? true : false;
	}


	//Getters.
	public static Object getTagObject( List<Map<String, String>> path, boolean isGlobal ){

		String tagName = ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path );


		if( isSimpleTypeTagName( tagName ) ) return new SimpleType( path, isGlobal );

		if( isComplexTypeTagName( tagName ) ) return new ComplexType( path, isGlobal );


		return UnknownClassesController.getUnknownTag( tagName, XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel() );
	}

	public static Object extractObject( Object[] sourceObjects, String extractedObjectKind, List<Map<String, String>> path, boolean isGlobal, String namesDistanceCalculatorAlgorithm ){

		String[] sourceObjectKinds = null, sourceObjectNames = null, extractedObjectKinds = null;//It is used only by the method 'getUnknownObject()'.


		for( int i = 0; sourceObjects != null && i < sourceObjects.length; ++i ){

			if( sourceObjectKinds == null ) sourceObjectKinds = new String[ sourceObjects.length ];

			if( sourceObjectNames == null ) sourceObjectNames = new String[ sourceObjects.length ];


			sourceObjectKinds[ i ] = ClassInfo.getClass( sourceObjects[ i ] );

			if( ComponentController.isComponentKind( sourceObjects[ i ] ) ) sourceObjectNames[ i ] = ( (Component) sourceObjects[ i ] ).getName();

			else sourceObjectNames[ i ] = "";
		}

		if( sourceObjects != null ){

			extractDataTypeTag( sourceObjectNames, path, namesDistanceCalculatorAlgorithm );


			return new ComplexType( path, isGlobal );
		}

		if( extractedObjectKind != null ){

			String[] tmpExtractedObjectKinds = { extractedObjectKind };

			extractedObjectKinds = tmpExtractedObjectKinds;
		}

		return UnknownClassesController.getUnknownObject( sourceObjectKinds, XMLSchemaRepresentationController.getCompletelyUnstructuredSchemaRepresentation(), extractedObjectKinds, XMLSchemaRepresentationController.getCompletelyUnstructuredSchemaRepresentation() );
	}

	private static void extractDataTypeTag( String[] sourceObjectNames, List<Map<String, String>> path, String namesDistanceCalculatorAlgorithm ){

		String extractedName = NamesDistanceCalculator.extractName( namesDistanceCalculatorAlgorithm, sourceObjectNames );


		Map<String, String> tagAttributes = ListGetters.getLastListElement( path );

		tagAttributes.put( SchemaRepresentationController.getTagNameKey(), ComponentTagNames.complexTypeTagName );

		tagAttributes.put( "name", extractedName );
	}

	public static String getSimpleTypeTagName(){

		return ComponentTagNames.simpleTypeTagName;
	}

	public static Map<String, String> getDataTypeKinds(){

		return dataTypeKinds;
	}

	public static String getFlatDataTypeKinds(){

		return MapPrinting.printCommaSeparated( getDataTypeKinds() );
	}

	public static Map<String, String> getSimpleTypeKinds(){

		return simpleTypeKinds;
	}

	public static String getFlatSimpleTypeKinds(){

		return MapPrinting.printCommaSeparated( getSimpleTypeKinds() );
	}

	public static String getTag( Object object ){

		if( isSimpleTypeKind( object ) ) return ( (SimpleType) object).getTag();

		if( isComplexTypeKind( object ) )  return ( (ComplexType) object).getTag();


		return ClassInfo.getClass( object );
	}
}
