package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.element;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.ComponentTagNames;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.Occurrences;


/**
 * 
 * @author dathanas
 *
 */
abstract class Element extends Component {


	//Variables.
	private static final long serialVersionUID = 7347578014515482464L;

	private boolean isAbstract;

	private List<String> block;
	private boolean hasBlock;

	private String defaultValue;
	protected boolean hasDefaultValue;

	private List<String> finalValues;
	private boolean hasFinal;

	private String fixedValue;
	protected boolean hasFixedValue;

	private String form;
	private boolean hasForm;

	private Occurrences occurs;

	private boolean nillable;


	//Constructor.
	protected Element( List<Map<String, String>> path, boolean isGlobal ){

		super( path, isGlobal );


		hasBlock = false;
		hasFinal = false;


		isAbstract = Boolean.parseBoolean( (String) MapGetter.getValueByStringKey( "abstract", getTagAttributes() ) );


		parse( "block", (String) MapGetter.getValueByStringKey( "block", getTagAttributes() ) );

		checkConstraint2( "block" );


		String cDefaultValue = (String) MapGetter.getValueByStringKey( "default", getTagAttributes() );

		if( cDefaultValue != null && ! cDefaultValue.isEmpty() && checkConstraint3( "default" ) == null ){

			defaultValue = cDefaultValue;

			hasDefaultValue = true;
		}

		else hasDefaultValue = false;


		parse( "final", (String) MapGetter.getValueByStringKey( "final", getTagAttributes() ) );

		checkConstraint2( "final" );


		String cFixedValue = (String) MapGetter.getValueByStringKey( "fixed", getTagAttributes() );

		if( cFixedValue != null && ! cFixedValue.isEmpty() && checkConstraint3( "fixed" ) == null ){

			fixedValue = cFixedValue;

			hasFixedValue = true;
		}

		else hasFixedValue = false;


		String cForm = (String) MapGetter.getValueByStringKey( "form", getTagAttributes() );

		if( cForm != null && ! cForm.isEmpty() && checkConstraint4( cForm ) == null ){

			form = cForm;

			hasForm = true;
		}

		else hasForm = false;


		occurs = new Occurrences( MapGetter.getValueByKeyToLowerCase( "minOccurs", getTagAttributes() ), MapGetter.getValueByKeyToLowerCase( "maxOccurs", getTagAttributes() ) );

		if( occurs.hasConstraintViolation() ) ObjectConstraintsManipulation.setErrorAttribute( this, "minOccurs/maxOccurs", occurs.getConstraintViolation() );


		nillable = Boolean.parseBoolean( (String) MapGetter.getValueByStringKey( "nillable", getTagAttributes() ) );
	}


	//Functions used by the constructor.
	void parse( String label, String values ){

		if( values == null || ( values != null && values.isEmpty() ) ) return;
			
		int pos = values.indexOf(" ");

		if( pos <= 0 ){//Only one value exists.

			if( checkConstraint1( label, values ) == null ) add( label, values );
		}

		else{

			while( pos > 0 ){

				String tmpValue = values.substring( 0, pos );

				if( checkConstraint1( label, tmpValue ) == null ) add( label, tmpValue );

				values = values.substring( pos + 1, values.length() );

				pos = values.indexOf(" ");
			}

			if( checkConstraint1( label, values ) == null ) add( label, values );
		}
	}

	private void add( String label, String value ){

		if( label.equals( "block" ) ){

			if( ! hasBlock ){

				block = new ArrayList<String>();

				hasBlock = true;
			}

			block.add( value );
		}

		else{

			if( ! hasFinal ){

				finalValues = new ArrayList<String>();

				hasFinal = true;
			}

			finalValues.add( value );
		}
	}


	//Constraint checkers.
	private String checkConstraint1( String label, String value ){

		return ( value != null && ! value.equals( "#all" ) && ! value.equals( "extension" ) && ! value.equals( "restriction" ) ) ? ObjectConstraintsManipulation.setErrorAttribute( this, label, label + "= '#all', 'extension' and 'restriction'" ) : null;
	}

	private String checkConstraint2( String label ){

		int occurrences = 0;

		String message = "The '#all' value cannot appear in a list of values";

		if( label.equals( "block" ) ){

			for( int i = 0; hasBlock && i < block.size(); ++i ) if( block.get( i ).equals( "#all" ) ) ++occurrences;

			return ( hasBlock && block.size() > 1 && occurrences > 0 ) ? ObjectConstraintsManipulation.setErrorAttribute( this, label, message ) : null;
		}

		else{

			for( int i = 0; hasFinal && i < finalValues.size(); ++i ) if( finalValues.get( i ).equals( "#all" ) ) ++occurrences;

			return ( hasFinal && finalValues.size() > 1 && occurrences > 0 ) ? ObjectConstraintsManipulation.setErrorAttribute( this, label, message ) : null;
		}
	}

	private String checkConstraint3( String label ){

		return ( ( label.equals( "default" ) && hasFixedValue ) || ( label.equals( "fixed" ) && hasDefaultValue ) ) ? ObjectConstraintsManipulation.setErrorAttribute( this, label, "Values in both 'default' and 'fixed' attributes" ) : null;
	}

	private String checkConstraint4( String value ){

		return ( ! value.equals( "qualified" ) && ! value.equals( "unqualified" ) ) ? ObjectConstraintsManipulation.setErrorAttribute( this, "form", "form= 'qualified' || 'unqualified'" ) : null;
	}


	//Getters.
	protected boolean hasBlock(){

		return hasBlock;
	}

	protected List<String> getBlockValues(){

		return block;
	}

	protected boolean isAbstract(){

		return isAbstract;
	}

	protected boolean hasDefaultValue(){

		return hasDefaultValue;
	}

	protected String getDefaultValue(){

		return defaultValue;
	}

	protected boolean hasFinal(){

		return hasFinal;
	}

	protected List<String> getFinalValues(){

		return finalValues;
	}

	protected boolean hasFixedValue(){

		return hasFixedValue;
	}

	protected String getFixedValue(){

		return fixedValue;
	}

	protected boolean hasForm(){

		return hasForm;
	}

	protected String getForm(){

		return form;
	}

	protected Occurrences getOccurs(){

		return occurs;
	}

	protected boolean nillable(){

		return nillable;
	}

	public String getSchemaRepresentationModel(){

		return XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel();
	}

	public String getSchemaKind(){

		return SchemaRepresentationController.getSchemaKind();
	}

	public String getTagName(){

		return ComponentTagNames.elementTagName;
	}

	@SuppressWarnings("serial")
	public String getTag(){

		Map<String, String> exceptedKeys = new Hashtable<String, String>() { { put( SchemaRepresentationController.getTagNameKey(), SchemaRepresentationController.getTagNameKey() );
																			   put( "minOccurs", "minOccurs" );
																			   put( "maxOccurs", "maxOccurs" ); } };


		return getTagName() + MapPrinting.printSpaceSeparated( exceptedKeys, getTagAttributes() ) + occurs.visualize() + printMyOrderID();
	}
}
