package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.group;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapTransfomation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.unknownClasses.UnknownClassesController;


/**
 * 
 * @author dathanas
 *
 */
@SuppressWarnings("serial")
public class GroupController {


	//Variables.
	private static Map<String, String> elementGroupKinds = new Hashtable<String, String>() { { put( "ElementGroupDefinition", "ElementGroupDefinition" ); put( "ElementGroupDeclaration", "ElementGroupDeclaration" ); } };
	private static Map<String, String> attributeGroupKinds = new Hashtable<String, String>() { { put( "AttributeGroupDefinition", "AttributeGroupDefinition" ); put( "AttributeGroupDeclaration", "AttributeGroupDeclaration" ); } };


	//Verifiers.
	public static boolean isElementGroupKind( Object object ){

		return MapGetter.objectNameBelongsToMapStrings( object, elementGroupKinds );
	}

	public static boolean isGroupKind( Object object ){

		if( isAttributeGroupKind( object ) ) return true;

		if( isElementGroupKind( object ) ) return true;


		return false;
	}

	public static boolean isGroupDefinitionKind( Object object ){

		if( isAttributeGroupDefinitionKind( object ) ) return true;

		if( isElementGroupDefinitionKind( object ) ) return true;


		return false;
	}

	public static boolean isAttributeGroupDefinitionKind( Object object ){

		return object != null ? object.getClass().equals( AttributeGroupDefinition.class ) : false;
	}

	public static boolean isElementGroupDefinitionKind( Object object ){

		return object != null ? object.getClass().equals( ElementGroupDefinition.class ) : false;
	}

	public static boolean isAttributeGroupKind( Object object ){

		return MapGetter.objectNameBelongsToMapStrings( object, attributeGroupKinds );
	}

	public static boolean isGroupTagName( String tagName ){

		return tagName != null && ( tagName.toLowerCase().equals( "attributegroup" ) || tagName.equals( "group" ) ) ? true : false;
	}

	public static boolean isAttributeGroupTagName( String tagName ){

		return tagName != null && tagName.toLowerCase().equals( "attributegroup" ) ? true : false;
	}

	public static boolean isElementGroupTagName( String tagName ){

		return tagName != null && tagName.equals( "group" ) ? true : false;
	}


	//Getters.
	public static Object getTagObject( List<Map<String, String>> path, boolean isGlobal ){

		String tagName = ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path );

		Map<String, String> attributes = ListGetters.getLastListElement( path );


		String ref = (String) MapGetter.getValueByStringKey( "ref", attributes );

		if( ref != null && ! ref.isEmpty() ){

			if( isAttributeGroupTagName( tagName ) ) return new AttributeGroupDeclaration( path, ref, "ref", isGlobal );

			if( isElementGroupTagName( tagName ) ) return new ElementGroupDeclaration( path, ref, "ref", isGlobal );
		}

		if( isAttributeGroupTagName( tagName ) ) return new AttributeGroupDefinition( path, isGlobal );

		if( isElementGroupTagName( tagName ) ) return new ElementGroupDefinition( path, isGlobal );


		return UnknownClassesController.getUnknownTag( tagName, XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel() );
	}

	public static Map<String, String> getElementGroupKinds(){

		return elementGroupKinds;
	}

	public static String getFlatElementGroupKinds(){

		return MapPrinting.printCommaSeparated( getElementGroupKinds() );
	}

	public static Map<String, String> getAttributeGroupKinds(){

		return attributeGroupKinds;
	}

	public static String getFlatAttributeGroupKinds(){

		return MapPrinting.printCommaSeparated( getAttributeGroupKinds() );
	}

	public static Map<String, String> getGroupKinds(){

		return MapTransfomation.concat( getElementGroupKinds(), getAttributeGroupKinds() );
	}

	public static String getFlatGroupKinds(){

		return MapPrinting.printCommaSeparated( getGroupKinds() );
	}
}
