package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.identityConstraint;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.Annotation;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;


/**
 * 
 * @author dathanas
 * This class models the 'unique' and 'key' tags.
 */
class IdentityConstraint extends Component {


	//Variables.
	private static final long serialVersionUID = 8833067976867734094L;


	//Constructor.
	protected IdentityConstraint( List<Map<String, String>> path, boolean isGlobal ){

		super( path, isGlobal );
	}


	//Constraint checkers.
	private String checkConstraint( Object object ){

		return IdentityConstraintController.isSelectorOrFieldKind( object ) && ( (SelectorOrField) object ).isSelector() && hasSelector() ? ObjectConstraintsManipulation.setConnectionError( this, object, "It should contain only one 'selector' tag" ) : null;
	}

	private String checkConstraint(){

		if( ! hasSelector() ) return ObjectConstraintsManipulation.setContainmentError( this, getSelectors() );

		if( ! hasField() ) return ObjectConstraintsManipulation.setContainmentError( this, getFields() );


		return null;
	}


	//Setters.
	public Object add( Object object ){

		if( ObjectConstraintsManipulation.setContainmentError( this, object ) == null ){

			if(  IdentityConstraintController.isSelectorOrFieldKind( object ) && checkConstraint( object ) == null ) addComponent( (Component) object );

			else if( XMLSchemaRepresentationController.isAnnotationKind( object ) ) setAnnotation( (Annotation) object );

			else ObjectConstraintsManipulation.setConnectionError ( this, object, "Valid content:\n\n1. The selector or fields: " + IdentityConstraintController.getFlatSelectorOrFieldKinds() + "\n2. Annotation" );
		}

		return this;
	}


	//Getters.
	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation() ) return getConstraintViolation();


		checkConstraint();


		super.getFinalConstraintViolation();


		return getConstraintViolation();
	}

	protected boolean hasSelector(){

		for( int i = 0; hasComponents() && i < getComponents().size(); ++i ) if( IdentityConstraintController.isSelectorOrFieldKind( getComponents().get( i ) ) && ( (SelectorOrField) getComponents().get( i ) ).isSelector() ) return true;

		return false;
	}

	protected boolean hasField(){

		for( int i = 0; hasComponents() && i < getComponents().size(); ++i ) if( IdentityConstraintController.isSelectorOrFieldKind( getComponents().get( i ) ) && ( (SelectorOrField) getComponents().get( i ) ).isField() ) return true;

		return false;
	}

	protected List<Component> getSelectors(){

		List<Component> selectors = null;

		for( int i = 0; hasComponents() && i < getComponents().size(); ++i )

			if( IdentityConstraintController.isSelectorOrFieldKind( getComponents().get( i ) ) && ( (SelectorOrField) getComponents().get( i ) ).isSelector() ){

				if( selectors == null ) selectors = new ArrayList<Component>();

				selectors.add( getComponents().get( i ) );
			}

		return selectors;
	}

	protected List<Component> getFields(){

		List<Component> fields = null;

		for( int i = 0; hasComponents() && i < getComponents().size(); ++i )

			if( IdentityConstraintController.isSelectorOrFieldKind( getComponents().get( i ) ) && ( (SelectorOrField) getComponents().get( i ) ).isField() ){

				if( fields == null ) fields = new ArrayList<Component>();

				fields.add( getComponents().get( i ) );
			}


		return fields;
	}

	public String getSchemaRepresentationModel(){

		return XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel();
	}

	public String getSchemaKind(){

		return SchemaRepresentationController.getSchemaKind();
	}

	public String getTagName(){

		return (String) MapGetter.getValueByStringKey( SchemaRepresentationController.getTagNameKey(), getTagAttributes() );
	}

	@SuppressWarnings("serial")
	public String getTag(){

		Map<String, String> exceptedKeys = new Hashtable<String, String>() { { put( SchemaRepresentationController.getTagNameKey(), SchemaRepresentationController.getTagNameKey() ); } };


		return getTagName() + MapPrinting.printSpaceSeparated( exceptedKeys, getTagAttributes() ) + printMyOrderID();
	}
}
