package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.identityConstraint;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.ComponentTagNames;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.unknownClasses.UnknownClassesController;


/**
 * 
 * @author dathanas
 *
 */
@SuppressWarnings("serial")
public class IdentityConstraintController {


	//Variables.
	private static Map<String, String> identityConstraintsKinds = new Hashtable<String, String>() { { put( "IdentityConstraint", "IdentityConstraint" ); put( "IdentityConstraintKeyRef", "IdentityConstraintKeyRef" ); } };
	private static Map<String, String> selectorOrFieldKinds = new Hashtable<String, String>() { { put( "SelectorOrField", "SelectorOrField" ); } };


	//Verifiers.
	public static boolean isIdentityConstraintKind( Object object ){

		return object != null ? ( object.getClass().equals( IdentityConstraint.class ) || object.getClass().equals( IdentityConstraintKeyRef.class ) ) : false;
	}

	public static boolean isSelectorOrFieldKind( Object object ){

		return object != null ? object.getClass().equals( SelectorOrField.class ) : false;
	}


	public static boolean isIdentityConstraintTagName( String tagName ){

		return tagName != null && ( tagName.equals( ComponentTagNames.uniqueTagName ) || tagName.equals( ComponentTagNames.keyTagName ) );
	}

	private static boolean isIdentityConstraintKeyRefTagName( String tagName ){

		return tagName != null && tagName.equals( ComponentTagNames.keyRefTagName );
	}

	public static boolean isSelectorOrFieldTagName( String tagName ){

		return tagName != null && ( tagName.equals( ComponentTagNames.selectorTagName ) || tagName.equals( ComponentTagNames.fieldTagName ) );
	}


	//Getters.
	public static Object getTagObject( List<Map<String, String>> path, boolean isGlobal ){

		String tagName = ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path );


		if( isIdentityConstraintTagName( tagName ) ) return new IdentityConstraint( path, isGlobal );

		else if( isIdentityConstraintKeyRefTagName( tagName ) ) return new IdentityConstraintKeyRef( path, isGlobal );

		else if( isSelectorOrFieldTagName( tagName ) ) return new SelectorOrField( path );


		return UnknownClassesController.getUnknownTag( tagName, XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel() );
	}


	public static Map<String, String> getIdentityConstaintsKinds(){

		return identityConstraintsKinds;
	}

	public static String getFlatIdentityConstaintsKinds(){

		return MapPrinting.printCommaSeparated( identityConstraintsKinds );
	}

	public static Map<String, String> getSelectorOrFieldKinds(){

		return selectorOrFieldKinds;
	}

	public static String getFlatSelectorOrFieldKinds(){

		return MapPrinting.printCommaSeparated( selectorOrFieldKinds );
	}
}
