package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.structure.simpleType;

import java.util.List;
import java.util.Map;

import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.Annotation;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.ReferencedComponent;
import org.ow2.choreos.schemaRepresentation.xml.Structure;
import org.ow2.choreos.schemaRepresentation.xml.StructureTagNames;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.dataType.DataTypeController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.structure.restrictionFacet.RestrictionFacetController;


/**
 * 
 * @author dathanas
 * This class models the 'restriction' tag of a 'simpleType' that declares a 'base' attribute (which is another 'simpleType' or built-in type).
 */
class SimpleTypeRestriction extends Structure {


	//Variables.
	private static final long serialVersionUID = -2257854847228560445L;


	//Constructor.
	protected SimpleTypeRestriction( List<Map<String, String>> path ){

		super( path );


		String base = (String) MapGetter.getValueByStringKey( "base", getTagAttributes() );

		addReferencedComponent( new ReferencedComponent( base, "base" ) );
	}


	//Constraint checkers.
	private String checkConstraint1( Object object ){

		return hasSimpleType() ? ObjectConstraintsManipulation.setConnectionError( this, object, "The '" + getTag() + "' cannot contain more than one simpleTypes") : null;
	}


	//Setters.
	public Object add( Object object ){

		if( ObjectConstraintsManipulation.setContainmentError( this, object ) == null ){

			if( RestrictionFacetController.isRestrictionFacetKind( object ) ) addStructure( (Structure) object );

			else if( DataTypeController.isSimpleTypeKind( object ) && checkConstraint1( object ) == null ) addComponent( (Component) object );

			else if( XMLSchemaRepresentationController.isAnnotationKind( object ) ) setAnnotation( (Annotation) object );

			else{

				String message1 = "Valid content:\n1. The simpleType structures: " + DataTypeController.getFlatSimpleTypeKinds();

				String message2 = "\n2. The restriction facet structures: " + RestrictionFacetController.getFlatFestrictionFacetKinds();

				String message3 = "\n3. Annotation";

				ObjectConstraintsManipulation.setConnectionError ( this, object, message1 + message2 + message3 );
			}
		}

		return this;
	}


	//Getters.
	private boolean hasSimpleType(){

		for( int i = 0; hasComponents() && i < getComponents().size(); ++i ) if( DataTypeController.isSimpleTypeKind( getComponents().get( i ) ) ) return true;

		return false;
	}

	public String getSchemaRepresentationModel(){

		return XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel();
	}

	public String getSchemaKind(){

		return SchemaRepresentationController.getSchemaKind();
	}

	public String getTagName(){

		return StructureTagNames.restrictionTagName;
	}

	public String getTag(){

		return getTagName() + MapPrinting.printSpaceSeparated( SchemaRepresentationController.getTagNameKey(), getTagAttributes() ) + printMyOrderID();
	}
}
