package org.ow2.choreos.schemaRepresentation.xml.completelyStructured.structure.simpleType;

import java.util.Hashtable;
import java.util.Map;

import org.ow2.choreos.dataStructures.list.listManipulation.ListGetters;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.StructureTagNames;
import org.ow2.choreos.schemaRepresentation.xml.XMLSchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.completelyStructured.component.dataType.DataTypeController;
import org.ow2.choreos.unknownClasses.UnknownClassesController;


/**
 * 
 * @author dathanas
 *
 */
@SuppressWarnings("serial")
public class SimpleTypeStructureController {


	//Variables.
	private static Map<String, String> simpleTypeKinds = new Hashtable<String, String>() { { put( "Union", "Union" ); put( "SimpleTypeRestriction", "SimpleTypeRestriction" ); put( "List", "List" ); } };


	//Verifiers.
	public static boolean isSimpleTypeStructureKind( Object object ){

		return MapGetter.objectNameBelongsToMapStrings( object, simpleTypeKinds );
	}


	public static boolean isSimpleTypeStructureTagName( java.util.List<Map<String, String>> path, String tagName ){

		return isSimpleTypeRestrictionTagName( path, tagName ) || isListTagName( tagName ) || isUnionTagName( tagName ) ? true : false;
	}

	public static boolean isSimpleTypeStructureTagName( String tagName ){

		return isSimpleTypeRestrictionTagName( tagName ) || isListTagName( tagName ) || isUnionTagName( tagName ) ? true : false;
	}

	private static boolean isSimpleTypeRestrictionTagName( String tagName ){

		return tagName != null && tagName.equals( StructureTagNames.restrictionTagName ) ? true : false;
	}

	private static boolean isListTagName( String tagName ){

		return tagName != null && tagName.equals( List.tagName ) ? true : false;
	}

	private static boolean isUnionTagName( String tagName ){

		return tagName != null && tagName.equals( Union.tagName ) ? true : false;
	}

	private static boolean isSimpleTypeRestrictionTagName( java.util.List<Map<String, String>> path, String tagName ){

		String prevPrevTagName = ListGetters.getPrevPrevElementValue( SchemaRepresentationController.getTagNameKey(), path );


		return tagName != null && tagName.equals( StructureTagNames.restrictionTagName ) && prevPrevTagName != null && prevPrevTagName.equals( DataTypeController.getSimpleTypeTagName() ) ? true : false;
	}


	//Getters.
	public static Object getTagObject( java.util.List<Map<String, String>> path ){

		String tagName = ListGetters.getLastListElementValue( SchemaRepresentationController.getTagNameKey(), path );


		if( isSimpleTypeRestrictionTagName( tagName ) ) return new SimpleTypeRestriction( path );

		if( isListTagName( tagName ) ) return new List( path );

		if( isUnionTagName( tagName ) ) return new Union( path );


		return UnknownClassesController.getUnknownTag( tagName, XMLSchemaRepresentationController.getXMLSchemaCompletelyStructuredRepresentationModel() );
	}

	public static Map<String, String> getSimpleTypeStructureKinds(){

		return simpleTypeKinds;
	}

	public static String getFlatSimpleTypeStructureKinds(){

		return MapPrinting.printCommaSeparated( getSimpleTypeStructureKinds() );
	}
}
