package org.ow2.choreos.serviceBase;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.choreos.abstractionBase.FunctionalAbstraction;
import org.ow2.choreos.abstractionBase.Hierarchy;
import org.ow2.choreos.abstractionBase.IndependentHierarchies;
import org.ow2.choreos.abstractionBase.NonFunctionalAbstraction;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.schema.ComponentsAndStructuresMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.InterfacesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.MessageTypesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.MessagesMapping;
import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.OperationsMapping;
import org.ow2.choreos.dataStructures.interval.Interval;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.ReferencedComponent;
import org.ow2.choreos.schemaRepresentation.xml.Structure;
import org.ow2.choreos.serviceRepresentation.functional.Message;
import org.ow2.choreos.serviceRepresentation.functional.MessageType;
import org.ow2.choreos.serviceRepresentation.functional.Operation;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;

/**
 * 
 * @author dathanas
 *
 * This class contains methods to store abstraction objects (and its containing objects) to the DBMS
 */
public class AbstractionsStorage {
	
	private Statement insertStatement;	// empty SQL statement
	private ResultSet rs;
	private Statement stmt;
	private String hierarchyClass;
	
	//Constructor
	AbstractionsStorage(){
		insertStatement = null;
		rs = null;
		stmt = null;
	}
	
	//returns the (autoincrement) ID of the last inserted row in the DBMS -- needed to insert the foreign keys back to the objects
	int getLastInsertedId(Connection dbConnection){
		
		try {
			stmt = dbConnection.createStatement();
			rs = stmt.executeQuery("SELECT LAST_INSERT_ID()");
			if (rs.next()) {
				int lastInsertedID = rs.getInt(1);
				return lastInsertedID;
			} 
		}
		catch(Exception e) {
		     System.err.println("Exception: " + e.getMessage());
		}
	  	return (-1);
	}

	//stores the 'Hierarchy' object 'tmpHierachy' (parameter) and return its ID
	public int storeHierarchy(Connection dbConnection, Hierarchy tmpHierachy){
		try {
			insertStatement = dbConnection.createStatement();
  	  		String upd = new String();
  	  		
  	  		hierarchyClass = tmpHierachy.getClass().getName();
  	  		if (tmpHierachy.getClass().getName().equals("org.ow2.choreos.abstractionBase.IndependentHierarchies")){
  	  			upd = new String ("INSERT INTO hierarchies(`H_C_ID`,`H_MODE`,`H_DIST_CALC_ALGO`) VALUES(" + tmpHierachy.getCollectionID() + "," + tmpHierachy.getMode() + ",'" + ((IndependentHierarchies) tmpHierachy).getFunctionalHierarchy().getNamesDistanceCalculatorAlgorithm() + "')");
  	  		}
  	  		else{
  	  			upd = new String ("INSERT INTO hierarchies(`H_C_ID`,`H_MODE`,`H_DIST_CALC_ALGO`) VALUES(" + tmpHierachy.getCollectionID() + "," + tmpHierachy.getMode() + ",'" + "null" + "')");
  	  		}
 
  	  		insertStatement.executeUpdate(upd);
  	  		return getLastInsertedId(dbConnection);
		}
		catch(Exception e) {
	        System.err.println("Exception storeHierarchy: " + e.getMessage());
	    }
		return (-1);
	}
	
	//stores the 'Abstraction' object (parameter) and return its ID. This object can be either FunctionalAbstraction or NonFunctionalAbstraction
	public int storeAbstraction(Connection dbConnection, FunctionalAbstraction tmpFunctionalAbstraction, NonFunctionalAbstraction tmpNonFunctionalAbstraction, int hierachiesCounter){
		int functionalAbstractionID;
		int nonFunctionalAbstractionID;
		
		//this is a functional abstraction to store
		if (tmpFunctionalAbstraction != null){
			
			functionalAbstractionID = tmpFunctionalAbstraction.getID();
			try {
	  	  		insertStatement = dbConnection.createStatement();
	  	  		String upd = new String ("INSERT INTO abstractions(`A_FA_ID`,`A_NFA_ID`,`A_TYPE`,`A_H_ID`) VALUES(" + functionalAbstractionID + ", -1, 'FUNCTIONAL'," + hierachiesCounter + ")");
	  	  		insertStatement.executeUpdate(upd);
	  	  		return getLastInsertedId(dbConnection);
			}
			catch(Exception e) {
		          System.err.println("Exception storeAbstraction: " + e.getMessage());
		    }
		}
		
		else{	//this is a non-functional abstraction to store
			nonFunctionalAbstractionID = tmpNonFunctionalAbstraction.getNonFunctionalAbstractionID();
			try {
	  	  		insertStatement = dbConnection.createStatement();
	  	  		String upd = new String ("INSERT INTO abstractions(`A_FA_ID`,`A_NFA_ID`,`A_TYPE`,`A_H_ID`) VALUES(-1, " + nonFunctionalAbstractionID + ", 'NONFUNCTIONAL'," + hierachiesCounter + ")");	
	  	  		insertStatement.executeUpdate(upd);
	  	  		return getLastInsertedId(dbConnection);
			}
			catch(Exception e) {
		          System.err.println("Exception storeAbstraction: " + e.getMessage());
		    }
		}
		return (-1);
		
	}
	
	//stores the 'RepresentativeInterface' object (parameter) of a functional abstraction and return its ID
	public int storeRepresentativeInterface(Connection dbConnection, ServiceInterface representativeInterface, int abstractionCounter){
		
		String name = new String (representativeInterface.getName());
		String profile = null;
		String behavioral = null;
		String constraints = null;
		
		if (representativeInterface.getProfile() != null) {
			profile = representativeInterface.getProfile();
		}
		if (representativeInterface.getBehavioralSpecification() != null) {
			behavioral =representativeInterface.getBehavioralSpecification();
		}
		if (representativeInterface.getConstraints() != null){
			constraints = representativeInterface.getConstraints();
		}
						 		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO representativeinterfaces(`RSI_NAME`,`RSI_PROFILE`,`RSI_BEHAV`,`RSI_CONSTR`,`RSI_A_ID`) VALUES('" + name + "','" + profile + "','" + behavioral + "','" +constraints + "'," + abstractionCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		
  	  		return getLastInsertedId(dbConnection);
  	  		
		} 
		catch(Exception e) {
	          System.err.println("Exception storeRepresentativeInterface: " + e.getMessage());
	    }
		return (-1);
	}
	
	//stores the 'RepresentativeOperation' object (parameter) of a RepresentativeInterface and return its ID
	public int storeRepresentativeOperation(Connection dbConnection, Operation representativeOperation, int representativeInterfaceCounter){
		String name = new String (representativeOperation.getName());
		String profile = null; 
		String pre = null;
		String post = null;
		if (representativeOperation.getProfile() != null) {
			profile = representativeOperation.getProfile();
		}
		if (representativeOperation.getPreConditions() != null) {
			pre = representativeOperation.getPreConditions();
		}
		if (representativeOperation.getPostConditions() != null) {
			post = representativeOperation.getPostConditions();
		}
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO representativeoperations(`ROP_NAME`,`ROP_PROFILE`,`ROP_PRE`,`ROP_POST`,`ROP_RSI_ID`) VALUES('" + name + "','" + profile + "','" + pre + "','" + post + "'," + representativeInterfaceCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	
  	  		return getLastInsertedId(dbConnection);
  	  		
		} 
		catch(Exception e) {
	          System.err.println("Exception storeRepresentativeOperation: " + e.getMessage());
	    }
		
		return (-1);
	}
	
	//stores the 'RepresentativeMessage' object (parameter) of a RepresentativeOperation and return its ID
	public int storeRepresentativeMessage(Message sMessage, Connection dbConnection, int operationCounter, String type){
		String name = new String (sMessage.getName());
		boolean hasMessageTypes = sMessage.hasMessageTypes();
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO representativemessages(`RMS_NAME`,`RMS_HAS_MESS_TYPES`,`RMS_TYPE`,`RMS_ROP_ID`) VALUES('" + name + "'," + hasMessageTypes + ",'" + type + "'," + operationCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		
  	  		return getLastInsertedId(dbConnection);
  	  		
		} 
		catch(Exception e) {
	          System.err.println("Exception storeRepresentativeMessage: " + e.getMessage());
	    }
		return(-1);
	 }
	
	
	public int storeRepresentativeReferencedComponent(ReferencedComponent tmpReferencedComponent, Connection dbConnection, int messageTypeCounter) {
		String prefix = null;
		String type = null;
		String prefix_type = null;
		String typeKind = tmpReferencedComponent.getTypeKind();

		if (tmpReferencedComponent.getType() != null){
			type = tmpReferencedComponent.getType();
		}
		
		if (tmpReferencedComponent.getPrefix() != null){	//if a prefix exists, concatenate it with 'type'
			prefix = new String (tmpReferencedComponent.getPrefix());
			prefix_type = prefix + ":" + type;
		}
		else{
			prefix_type = type;
		}
						
		String allTagAttributes = new String("");
		String concatenatedSchemaPath = new String("");
		String concatenatedClassPath = new String("");
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO representativecomponentsandstructures(`RCOM_NAME`,`RCOM_TYPE`,`RCOM_TYPEKIND`,`RCOM_ISGLOBAL`,`RCOM_TAG_ATTRIBUTES`,`RCOM_SCHEMA_PATH`,`RCOM_CLASS_PATH`,`RCOM_MODEL`,`RCOM_IS_COMPONENT`,`RCOM_MT_ID`) VALUES('" + null + "','" + type + "','" + typeKind + "'," + true + ",'" + allTagAttributes + "','" + concatenatedSchemaPath + "','" + concatenatedClassPath + "','" + "'," + true + "," + messageTypeCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		
  	  		return getLastInsertedId(dbConnection);
		} 
		catch(Exception e) {
	          System.err.println("Exception storeRepresentativeReferencedComponent: " + e.getMessage());
	    }
		return (-1);
	}
	
	@SuppressWarnings("rawtypes")
	public int storeRepresentativeLeafComponent(Component tmpLeafComponent, Connection dbConnection, int reprMessageTypesCounter){
		String model = new String (tmpLeafComponent.getSchemaRepresentationModel());
		String name = new String ();
		if (tmpLeafComponent != null){
			name = tmpLeafComponent.getName();
		}
		boolean isGlobal = tmpLeafComponent.isGlobal();
				
		Map<String, String> tagAttributes = tmpLeafComponent.getTagAttributes();
		String allTagAttributes = new String("");
		boolean flag = false;
		
		if( tagAttributes != null ){
			Set tagSet = tagAttributes.entrySet();
		    Iterator tagIterator = tagSet.iterator();
		    while (tagIterator.hasNext()) {
		    	Map.Entry entry = (Map.Entry) tagIterator.next();
		    	String attribute = (String) entry.getValue();
				if (flag){
					allTagAttributes = allTagAttributes.concat("," + entry.getKey().toString() + "-" + attribute);
				}
				else{
					flag = true;
					allTagAttributes = allTagAttributes.concat(entry.getKey().toString() + "-" + attribute);
				}
		    }
		} 
		
		String[] schemaPath = tmpLeafComponent.getSchemaPath();
		String concatenatedSchemaPath = new String("");
		for (int i = 0; i < schemaPath.length; i++){
			if (i == schemaPath.length - 1){
				concatenatedSchemaPath = concatenatedSchemaPath.concat(schemaPath[i]);
			}
			else{
				concatenatedSchemaPath = concatenatedSchemaPath.concat(schemaPath[i] + "-");
			}
		}
		
		String[] classPath = tmpLeafComponent.getClassPath();
		String concatenatedClassPath = new String("");
		for (int i = 0; classPath != null && i < classPath.length; i++){
			if (i == classPath.length - 1){
				concatenatedClassPath = concatenatedClassPath.concat(classPath[i]);
			}
			else{
				concatenatedClassPath = concatenatedClassPath.concat(classPath[i] + "-");
			}
		}
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO representativecomponentsandstructures(`RCOM_NAME`,`RCOM_TYPE`,`RCOM_TYPEKIND`,`RCOM_ISGLOBAL`,`RCOM_TAG_ATTRIBUTES`,`RCOM_SCHEMA_PATH`,`RCOM_CLASS_PATH`,`RCOM_MODEL`,`RCOM_IS_COMPONENT`,`RCOM_MT_ID`) VALUES('" + name + "','" + null + "','" + null + "'," + isGlobal + ",'" + allTagAttributes + "','" + concatenatedSchemaPath + "','" + concatenatedClassPath + "','" + model + "'," + true + "," + reprMessageTypesCounter + ")"); 	  		
  	  		insertStatement.executeUpdate(upd);
  	  		
  	  		return getLastInsertedId(dbConnection);
		} 
		catch(Exception e) {
	          System.err.println("Exception storeRepresentativeLeafComponent: " + e.getMessage());
	    }
		return (-1);
	}
	
	@SuppressWarnings("rawtypes")
	public int storeRepresentativeLeafStructure(Structure tmpStructure, Connection dbConnection, int reprMessageTypesCounter){
		String model = new String (tmpStructure.getSchemaRepresentationModel());
		Map<String, String> tagAttributes = tmpStructure.getTagAttributes();
		String allTagAttributes = new String("");
		boolean flag = false;
		
		if( tagAttributes != null ){
			Set tagSet = tagAttributes.entrySet();
		    Iterator tagIterator = tagSet.iterator();
		    while (tagIterator.hasNext()) {
		    	Map.Entry entry = (Map.Entry) tagIterator.next();
		    	String attribute = (String) entry.getValue();
				if (flag){
					allTagAttributes = allTagAttributes.concat("," + entry.getKey().toString() + "-" + attribute);
				}
				else{
					flag = true;
					allTagAttributes = allTagAttributes.concat(entry.getKey().toString() + "-" + attribute);
				}
		    }
		}
		
		String[] schemaPath = tmpStructure.getSchemaPath();
		String concatenatedSchemaPath = new String("");
		for (int i = 0; i < schemaPath.length; i++){
			if (i == schemaPath.length - 1){
				concatenatedSchemaPath = concatenatedSchemaPath.concat(schemaPath[i]);
			}
			else{
				concatenatedSchemaPath = concatenatedSchemaPath.concat(schemaPath[i] + "-");
			}
		}
		
		String[] classPath = tmpStructure.getClassPath();
		String concatenatedClassPath = new String("");
		for (int i = 0; i < classPath.length; i++){
			if (i == classPath.length - 1){
				concatenatedClassPath = concatenatedClassPath.concat(classPath[i]);
			}
			else{
				concatenatedClassPath = concatenatedClassPath.concat(classPath[i] + "-");
			}
		}
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO representativecomponentsandstructures(`RCOM_NAME`,`RCOM_TYPE`,`RCOM_TYPEKIND`,`RCOM_ISGLOBAL`,`RCOM_TAG_ATTRIBUTES`,`RCOM_SCHEMA_PATH`,`RCOM_CLASS_PATH`,`RCOM_MODEL`,`RCOM_IS_COMPONENT`,`RCOM_MT_ID`) VALUES(" + tmpStructure.getID() + "," + null + "," + null + "," + false + ",'" + allTagAttributes + "','" + concatenatedSchemaPath + "','" + concatenatedClassPath + "','" + model + "'," + false + ","  + reprMessageTypesCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		
  	  		return getLastInsertedId(dbConnection);
  	  	
		} 
		catch(Exception e) {
	          System.err.println("Exception storeRepresentativeLeafStructure: " + e.getMessage());
	    }
		return (-1);
		
	}
	
	//stores the 'RepresentativeMessageType' object (parameter) of a RepresentativeMessage and return its ID
	public int storeRepresentativeMessageType(MessageType sMessageType, Connection dbConnection, int messageCounter){
		String name = new String (sMessageType.getName());
		String prefix = null;
		String type = null;
		String prefix_type = null;
		String typeKind = null;
		
		if (sMessageType.getReferencedComponent().getType() != null){
			type = new String (sMessageType.getReferencedComponent().getType());
		}
		
		if (sMessageType.getReferencedComponent().getPrefix() != null){	//if a prefix exists, concatenate it with 'type'
			prefix = new String (sMessageType.getReferencedComponent().getPrefix());
			prefix_type = prefix + ":" + type;
		}
		else{
			prefix_type = type;
		}

		try {
			insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO representativemessagetypes(`RMT_NAME`,`RMT_PREFIX_TYPE`,`RMT_TYPEKIND`,`RMT_TYPE`,`RMT_RMS_ID`) VALUES('" + name + "','" + prefix_type + "','" + typeKind + "','" + type + "'," + messageCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		return getLastInsertedId(dbConnection);
  	  	
		}
		catch(Exception e) {
	          System.err.println("Exception storeRepresentativeMessageType: " + e.getMessage());
	    }
		return (-1);
		
	 }
	
		
	//stores the 'RepresentedInterface' object 'tmpInterface' (parameter) of a functional abstraction and return its ID
	//'tmpInterface' is already stored in table 'SERVICEINTERFACES', so only its ID must be stored here
	public void storeRepresentedInterface(Connection dbConnection, ServiceInterface tmpInterface, int abstractionCounter){
		int representedId;
		
		try {
			Statement repInstanceStmt = dbConnection.createStatement();
			ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT SI_ID FROM SERVICEINTERFACES WHERE SI_NAME = '" + tmpInterface.getName() + "' AND SI_ST_ID =" + tmpInterface.getDBKey());
			if (repInstanceSet.next()) {
				representedId = repInstanceSet.getInt(1);
			}
			else {
				representedId = -1;
			}
			if (representedId != -1) {
  	  			insertStatement = dbConnection.createStatement();
  	  			String upd = new String ("INSERT INTO representedinterfaces(`RI_ID`,`RI_FA_ID`) VALUES(" + representedId + "," + abstractionCounter + ")");
  	  			insertStatement.executeUpdate(upd);
			}
			else{
				//System.out.println("**RepresentedInterface is not included in actual Interfaces**");
			}
			
		}
		catch(Exception e) {
	          System.err.println("Exception storeRepresentedInterface: " + e.getMessage());
	    }
	}
	
	//stores the 'RepresentedInstance' object 'tmpInstance' (parameter) of a functional abstraction and return its ID
	//'tmpInstance' is already stored in table 'SERVICEINSTANCES', so only its ID must be stored here
	public void storeRepresentedInstance(Connection dbConnection, ServiceInstance tmpInstance, int abstractionCounter){
		int representedId;
		
		try {
			Statement repInstanceStmt = dbConnection.createStatement();
			ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT SIS_ID FROM SERVICEINSTANCES WHERE SIS_NAME = '" + tmpInstance.getName() + "' AND SIS_SI_ID =" + tmpInstance.getDBKey());
			if (repInstanceSet.next()) {
				representedId = repInstanceSet.getInt(1);
			}
			else {
				representedId = -1;
			}
			
			if (representedId != -1){
  	  			insertStatement = dbConnection.createStatement();
  	  			String upd = new String ("INSERT INTO representedinstances(`RIS_ID`,`RIS_NFA_ID`) VALUES(" + representedId + "," + abstractionCounter + ")");
  	  			insertStatement.executeUpdate(upd);
			}
			else{
				//System.out.println("**RepresentedInstance is not included in actual Instances**");
			}
		}
		catch(Exception e) {
	          System.err.println("Exception storeRepresentedInstance: " + e.getMessage());
	    }
	}
	
	//stores the 'InterfacesMapping' object 'tmpInterfaceMapping' (parameter) of a functional abstraction and return its ID
	public int storeInterfacesMapping(Connection dbConnection, InterfacesMapping tmpInterfaceMapping, int abstractionsCounter){
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO interfacesmappings(`IM_DISTANCE`,`IM_FA_ID`) VALUES(" + tmpInterfaceMapping.getDistance() + "," + abstractionsCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		return getLastInsertedId(dbConnection);
		}
		catch(Exception e) {
	          System.err.println("Exception storeInterfacesMapping: " + e.getMessage());
	    }
		
		return (-1);
	}
	
	//stores the 'OperationsMapping' object 'tmpMapOperation' (parameter) of a functional abstraction and return its ID
	public int storeOperationsMapping(Connection dbConnection, OperationsMapping tmpMapOperation, int interfaceMappingsCounter){
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO operationsmappings(`OM_DISTANCE`,`OM_IM_ID`) VALUES(" + tmpMapOperation.getDistance() + "," + interfaceMappingsCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		return getLastInsertedId(dbConnection);
		}
		catch(Exception e) {
	          System.err.println("Exception storeOperationsMapping: " + e.getMessage());
	    }
		
		return (-1);
	
	}
	
	//stores the 'MappedInterface' object 'tmpMapInterface' (parameter) of an InterfaceMapping object and return its ID
	public void storeMappedInterface(Connection dbConnection, ServiceInterface tmpMapInterface, int interfaceMappingsCounter, int representativeInterfaceCounter){
		int siId;
		
		try {
			Statement repInstanceStmt = dbConnection.createStatement();
			if (tmpMapInterface.isConcrete()){
				ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT SI_ID FROM SERVICEINTERFACES WHERE SI_NAME = '" + tmpMapInterface.getName() + "' AND SI_ST_ID =" + tmpMapInterface.getDBKey());
				if (repInstanceSet.next()) {
					siId = repInstanceSet.getInt(1);
				}
				else {
					siId = -1;
				}
			
				if (siId != -1) {
  	  				insertStatement = dbConnection.createStatement();
  	  				String upd = new String ("INSERT INTO mappedinterfaces(`MI_SI_ID`,`MI_RI_ID`,`MI_IM_ID`, `MI_RSI_ID`) VALUES(" + siId + "," + null + "," + interfaceMappingsCounter + "," + representativeInterfaceCounter + ")");
  	  				insertStatement.executeUpdate(upd);
				}
				else{
					System.out.println("**MappedInterface is not included in actual Interfaces**");
				}
			}
			
			else {
				ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT RSI_ID FROM REPRESENTATIVEINTERFACES WHERE RSI_NAME = '" + tmpMapInterface.getName() + "' AND RSI_A_ID =" + tmpMapInterface.getDBKey());
				
				if (repInstanceSet.next()) {
					siId = repInstanceSet.getInt(1);
				}
				else {
					siId = -1;
				}
				
				if (siId != -1) {
	  	  			insertStatement = dbConnection.createStatement();
	  	  			String upd = new String ("INSERT INTO mappedinterfaces(`MI_SI_ID`,`MI_RI_ID`,`MI_IM_ID`, `MI_RSI_ID`) VALUES(" + null + "," + siId + "," + interfaceMappingsCounter + "," + representativeInterfaceCounter + ")");
	  	  			insertStatement.executeUpdate(upd);
				}
				else{
					//System.out.println("**MappedInterface is not included in Representative Interfaces**");
				}
			}
  	  		
		}
		catch(Exception e) {
	          System.err.println("Exception storeMappedInterface: " + e.getMessage());
	    }
	}
	
	//stores the 'MappedOperation' object 'tmpMappedOperation' (parameter) of an OperationMapping object and return its ID
	//'tmpMappedOperation' is already stored in table 'OPERATIONS', so only its ID must be stored here
	public void storeMappedOperation (Connection dbConnection, Operation tmpMappedOperation, int operationsMappingsCounter, int reprOperationsCounter){
		int opId;
		
		try {
			Statement repInstanceStmt = dbConnection.createStatement();
			
			if (tmpMappedOperation.isConcrete()){
			
				ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT OP_ID FROM OPERATIONS WHERE OP_NAME = '" + tmpMappedOperation.getName() + "' AND OP_SI_ID =" + tmpMappedOperation.getDBKey());
				if (repInstanceSet.next()) {
					opId = repInstanceSet.getInt(1);
				}
				else {
					opId = -1;
				}
			
				if (opId != -1) {
					insertStatement = dbConnection.createStatement();
					String upd = new String ("INSERT INTO mappedoperations(`MO_OP_ID`,`MO_ROP_ID`,`MO_OM_ID`, `MO_RO_ID`) VALUES(" + opId + "," + null + "," + operationsMappingsCounter + "," + reprOperationsCounter + ")");
					insertStatement.executeUpdate(upd);
				}
				else{
					//System.out.println("**MappedOperation is not included in actual Operations**");
				}
			}
			else{
				ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT ROP_ID FROM REPRESENTATIVEOPERATIONS WHERE ROP_NAME = '" + tmpMappedOperation.getName() + "' AND ROP_RSI_ID =" + tmpMappedOperation.getDBKey());
				if (repInstanceSet.next()) {
					opId = repInstanceSet.getInt(1);
				}
				else {
					opId = -1;
				}
			
				if (opId != -1) {
					insertStatement = dbConnection.createStatement();
					String upd = new String ("INSERT INTO mappedoperations(`MO_OP_ID`,`MO_ROP_ID`,`MO_OM_ID`, `MO_RO_ID`) VALUES(" + null + "," + opId + "," + operationsMappingsCounter + "," + reprOperationsCounter + ")");
					insertStatement.executeUpdate(upd);
				}
				else{
					System.out.println("**MappedOperation is not included in Representative Operations**");
				}
			}
  	  		
		}
		catch(Exception e) {
	          System.err.println("Exception storeMappedOperation: " + e.getMessage());
	    }
	}

	//stores the 'MessagesMapping' object 'operationMessageMapping' (parameter) of an OperationMapping object and return its ID
	public int storeMessagesMapping(Connection dbConnection, MessagesMapping operationMessageMapping,  int operationsMappingsCounter, String type){
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO messagesmappings(`MM_DISTANCE`,`MM_TYPE`,`MM_OM_ID`) VALUES(" + operationMessageMapping.getDistance() + ",'" + type + "'," + operationsMappingsCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		return getLastInsertedId(dbConnection);
		}
		catch(Exception e) {
	          System.err.println("Exception storeMessagesMapping: " + e.getMessage());
	    }
		
		return (-1);
	}
	

	//stores the 'MessageTypesMapping' object 'tmpMessageTypesMapping' (parameter) of an MessageMapping object and return its ID
	public int storeMessageTypesMapping(Connection dbConnection, MessageTypesMapping tmpMessageTypesMapping,  int operationsMappingsCounter){
			
		try {
	  		insertStatement = dbConnection.createStatement();
	  		String upd = new String ("INSERT INTO messagetypesmappings(`MTM_DISTANCE`,`MTM_MM_ID`) VALUES(" + tmpMessageTypesMapping.getDistance() + "," + operationsMappingsCounter + ")");
	  		insertStatement.executeUpdate(upd);
	  		return getLastInsertedId(dbConnection);
		}
		catch(Exception e) {
			System.err.println("Exception storeMessageTypesMapping: " + e.getMessage());
	    }
			
		return (-1);
	}
	
	
	//stores the 'ComponentsAndStructuresMapping' object 'tmpMessageTypesMapping' (parameter) of an MessageMapping object and return its ID
	public int storeComponentsAndStructuresMapping(Connection dbConnection, ComponentsAndStructuresMapping componentsAndStructuresMappings,  int messageTypesMappingsCounter, int componentsAndStructuresMappingsCounter){
			
		try {
	  		insertStatement = dbConnection.createStatement();
	  		String upd = new String();
	  		if (messageTypesMappingsCounter == -1){
	  			upd = new String ("INSERT INTO componentsandstructuresmappings(`CM_DISTANCE`,`CM_MTM_ID`, `CM_CM_ID`) VALUES(" + componentsAndStructuresMappings.getDistance() + "," + null + "," + componentsAndStructuresMappingsCounter + ")");
	  		}
	  		else{
	  			upd = new String ("INSERT INTO componentsandstructuresmappings(`CM_DISTANCE`,`CM_MTM_ID`, `CM_CM_ID`) VALUES(" + componentsAndStructuresMappings.getDistance() + "," + messageTypesMappingsCounter + "," + null + ")");
	  		}

	  		insertStatement.executeUpdate(upd);
	  		return getLastInsertedId(dbConnection);
		}
		catch(Exception e) {
			System.err.println("Exception storeComponentsAndStructuresMapping: " + e.getMessage());
	    }
			
		return (-1);
	}
	
	//stores the 'MappedMessage' object 'tmpMappedMessage' (parameter) of an messagesMapping object and return its ID
	//'tmpMappedMessage' is already stored in table 'MESSAGES', so only its ID must be stored here
	public void storeMappedMessage(Connection dbConnection, Message tmpMappedMessage, int messageMappingsCounter, int reprMessagesCounter){
		int msId;
		
		try {
			Statement repInstanceStmt = dbConnection.createStatement();
			
			if (tmpMappedMessage.isConcrete()){
			
				ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT MS_ID FROM MESSAGES WHERE MS_NAME = '" + tmpMappedMessage.getName() + "' AND MS_OP_ID =" + tmpMappedMessage.getDBKey());
				if (repInstanceSet.next()) {
					msId = repInstanceSet.getInt(1);
				}
				else {
					msId = -1;
				}
			
				if (msId != -1) {
  	  				insertStatement = dbConnection.createStatement();
  	  				String upd = new String ("INSERT INTO mappedmessages(`MM_MS_ID`,`MM_RM_ID`,`MM_MM_ID`, `MM_RMS_ID`) VALUES(" + msId + "," + null + "," + messageMappingsCounter + "," + reprMessagesCounter + ")");
  	  				insertStatement.executeUpdate(upd);
				}
				else{
					//System.out.println("**Mapped message is not included in actual messages**" + tmpMappedMessage.getName() + "--" + tmpMappedMessage.getDBKey());
				}
			}
			
			else{
				ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT RMS_ID FROM REPRESENTATIVEMESSAGES WHERE RMS_NAME = '" + tmpMappedMessage.getName() + "' AND RMS_ROP_ID =" + tmpMappedMessage.getDBKey());
				if (repInstanceSet.next()) {
					msId = repInstanceSet.getInt(1);
				}
				else {
					msId = -1;
				}
			
				if (msId != -1) {
  	  				insertStatement = dbConnection.createStatement();
  	  				String upd = new String ("INSERT INTO mappedmessages(`MM_MS_ID`,`MM_RM_ID`,`MM_MM_ID`, `MM_RMS_ID`) VALUES(" + null + "," + msId + "," + messageMappingsCounter + "," + reprMessagesCounter + ")");
  	  				insertStatement.executeUpdate(upd);
				}
				else{
					//System.out.println("**MappedMessage is not included in Representative Messages**" + tmpMappedMessage.getName() + "--" + tmpMappedMessage.getDBKey());
				}
			}
  	  		
		}
		catch(Exception e) {
	          System.err.println("Exception storeMappedMessage: " + e.getMessage());
	    }
	}
	

	//stores the 'MappedMessageType' object 'tmpMappedMessageType' (parameter) of an messageTypesMapping object and return its ID
	//'tmpMappedMessageType' is already stored in table 'MESSAGES'or 'REPRESENTATIVEMESSAGES', so only its ID must be stored here
	public void storeMappedMessageType(Connection dbConnection, MessageType tmpMappedMessageType, int messageTypeMappingsCounter, int reprMessageTypesCounter){
		int mstId;
		
		try {
			Statement repInstanceStmt = dbConnection.createStatement();
			
			if (tmpMappedMessageType.isConcrete()){
				
				ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT MT_ID FROM MESSAGETYPES WHERE MT_NAME = '" + tmpMappedMessageType.getName() + "' AND MT_MS_ID =" + tmpMappedMessageType.getDBKey());
				if (repInstanceSet.next()) {
					mstId = repInstanceSet.getInt(1);
				}
				else {
					mstId = -1;
				}
				
				if (mstId != -1) {
					insertStatement = dbConnection.createStatement();
					String upd = new String ("INSERT INTO mappedmessagetypes(`MMT_MT_ID`,`MMT_RMST_ID`,`MMT_MTM_ID`, `MMT_RMT_ID`) VALUES(" + mstId + "," + null + "," + messageTypeMappingsCounter + "," + reprMessageTypesCounter + ")");
					insertStatement.executeUpdate(upd);
				}
				else{
					//System.out.println("**Mapped MessageType is not included in actual messagetypes**" + tmpMappedMessageType.getName() + "--" + tmpMappedMessageType.getDBKey());
				}
			}
				
			else{
				ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT RMT_ID FROM REPRESENTATIVEMESSAGETYPES WHERE RMT_NAME = '" + tmpMappedMessageType.getName() + "' AND RMT_RMS_ID =" + tmpMappedMessageType.getDBKey());
				if (repInstanceSet.next()) {
					mstId = repInstanceSet.getInt(1);
				}
				else {
					mstId = -1;
				}
				
				if (mstId != -1) {
					insertStatement = dbConnection.createStatement();
					String upd = new String ("INSERT INTO mappedmessagetypes(`MMT_MT_ID`,`MMT_RMST_ID`,`MMT_MTM_ID`, `MMT_RMT_ID`) VALUES(" + null + "," + mstId + "," + messageTypeMappingsCounter + "," + reprMessageTypesCounter + ")");

					insertStatement.executeUpdate(upd);
				}
				else{
					//System.out.println("**Mapped MessageType is not included in representative messagetypes**" + tmpMappedMessageType.getName() + "--" + tmpMappedMessageType.getDBKey());
				}
			}
	  	  		
		}
		catch(Exception e) {
			System.err.println("Exception storeMappedMessageType: " + e.getMessage());
	    }
	}
	

	//TODO: stores the 'MappedComponent' object 'tmpMappedComponent' (parameter) of an ComponentsMapping object and return its ID
	//'tmpMappedComponent' is already stored in table 'COMPONENTS'or 'REPRESENTATIVECOMPONENTS', so only its ID must be stored here
	public void storeMappedComponent(Connection dbConnection, Component tmpMappedComponent, int componentMappingsCounter, int reprComponentCounter){
		int comId;
			
		try {
			Statement repInstanceStmt = dbConnection.createStatement();
			
			if (tmpMappedComponent.isConcrete()){
				
				ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT COM_ID FROM componentsandstructures WHERE COM_NAME = '" + tmpMappedComponent.getName() + "' AND COM_MT_ID =" + tmpMappedComponent.getDBKey());
				if (repInstanceSet.next()) {
					comId = repInstanceSet.getInt(1);
				}
				else {
					comId = -1;
				}
				
				if (comId != -1) {
					insertStatement = dbConnection.createStatement();
					String upd = new String ("INSERT INTO mappedcomponentsandstructures(`MC_COM_ID`,`MC_RC_ID`,`MC_CM_ID`, `MC_RCOM_ID`) VALUES(" + comId + "," + null + "," + componentMappingsCounter + "," + reprComponentCounter + ")");
					insertStatement.executeUpdate(upd);
				}
				else{
					//System.out.println("**Mapped component is not included in actual component**" + tmpMappedComponent.getName() + "--" + tmpMappedComponent.getDBKey());
				}
			}
				
			else{
				ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT RCOM_ID FROM representativecomponentsandstructures WHERE RCOM_NAME = '" + tmpMappedComponent.getName() + "' AND RCOM_MT_ID =" + tmpMappedComponent.getDBKey());
				if (repInstanceSet.next()) {
					comId = repInstanceSet.getInt(1);
				}
				else {
					comId = -1;
				}
			
				if (comId != -1) {
					insertStatement = dbConnection.createStatement();
					String upd = new String ("INSERT INTO mappedcomponentsandstructures(`MC_COM_ID`,`MC_RC_ID`,`MC_CM_ID`, `MC_RCOM_ID`) VALUES(" + null + "," + comId + "," + componentMappingsCounter + "," + reprComponentCounter + ")");
					insertStatement.executeUpdate(upd);
				}
				else{
					//System.out.println("**Mapped Component is not included in representative components**" + tmpMappedComponent.getName() + "--" + tmpMappedComponent.getDBKey());
				}
			}
		 	  		
		}
		catch(Exception e) {
			System.err.println("Exception storeMappedComponent: " + e.getMessage());
	    }
	}
		
	
	public void storeMappedStructure(Connection dbConnection, Structure tmpMappedStructure, int componentMappingsCounter, int reprComponentCounter){
		int comId;
			
		try {
			Statement repInstanceStmt = dbConnection.createStatement();
			
			if (tmpMappedStructure.isConcrete()){
				
				ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT COM_ID FROM componentsandstructures WHERE COM_NAME = '" + tmpMappedStructure.getID() + "' AND COM_MT_ID =" + tmpMappedStructure.getDBKey());
				if (repInstanceSet.next()) {
					comId = repInstanceSet.getInt(1);
				}
				else {
					comId = -1;
				}
				
				if (comId != -1) {
					insertStatement = dbConnection.createStatement();
					String upd = new String ("INSERT INTO mappedcomponentsandstructures(`MC_COM_ID`,`MC_RC_ID`,`MC_CM_ID`, `MC_RCOM_ID`) VALUES(" + comId + "," + null + "," + componentMappingsCounter + "," + reprComponentCounter + ")");
					insertStatement.executeUpdate(upd);
				}
				else{
					//System.out.println("**Mapped Structure is not included in actual Structures**" + tmpMappedStructure.getID() + "--" + tmpMappedStructure.getDBKey());
				}
			}
				
			else{
				ResultSet repInstanceSet = repInstanceStmt.executeQuery("SELECT RCOM_ID FROM representativecomponentsandstructures WHERE RCOM_NAME = '" + tmpMappedStructure.getID() + "' AND RCOM_MT_ID =" + tmpMappedStructure.getDBKey());
				if (repInstanceSet.next()) {
					comId = repInstanceSet.getInt(1);
				}
				else {
					comId = -1;
				}
			
				if (comId != -1) {
					insertStatement = dbConnection.createStatement();
					String upd = new String ("INSERT INTO mappedcomponentsandstructures(`MC_COM_ID`,`MC_RC_ID`,`MC_CM_ID`, `MC_RCOM_ID`) VALUES(" + null + "," + comId + "," + componentMappingsCounter + "," + reprComponentCounter + ")");
					insertStatement.executeUpdate(upd);
				}
				else{
					//System.out.println("**Mapped Structure is not included in representative Structures**" + tmpMappedStructure.getID() + "--" + tmpMappedStructure.getDBKey());
				}
			}
		 	  		
		}
		catch(Exception e) {
			System.err.println("Exception storeMappedStructure: " + e.getMessage());
	    }
	}
	
	
	//stores the 'NonFunctionalDescription' object (parameter) of a NonFunctionalAbstraction object and return its ID
	public void storeNonFunctionalDescription(String property, String value, Connection dbConnection, int abstractionsCounter){
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO nfanonfuncdescriptions(`NFD_PROPERTY`,`NFD_VALUE`,`NFD_NFA_ID`) VALUES('" + property + "','" + value + "'," + abstractionsCounter + ")");
  	  		insertStatement.executeUpdate(upd);
		} 
		catch(Exception e) {
	          System.err.println("Exception storeNonFunctionalDescription: " + e.getMessage());
	    }
	}
	
	//stores the 'Range' object (parameter) of a NonFunctionalAbstraction object and return its ID
	@SuppressWarnings("rawtypes")
	public void storeRange(Connection dbConnection, Interval tmpRange, String property, int abstractionsCounter){
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO nfabstractionsranges(`NFR_PROPERTY`,`NFR_VALUE_FROM`, `NFR_VALUE_TO`, `NFR_NFA_ID`) VALUES('" + property + "'," + Double.parseDouble(tmpRange.getStart().toString()) + "," + Double.parseDouble(tmpRange.getEnd().toString()) + ","+ abstractionsCounter + ")");
  	  		insertStatement.executeUpdate(upd);
		}
		catch(Exception e) {
	          System.err.println("Exception storeRange: " + e.getMessage());
	    }
		
	}
	
	//stores the descendants of a FunctionalAbstraction or a NonFunctionalAbstraction object
	public void storeDescentants(Connection dbConnection, FunctionalAbstraction tmpFunctionalAbstraction, NonFunctionalAbstraction tmpNonFunctionalAbstraction){
		int AncestorId = -1;
		
		if (tmpNonFunctionalAbstraction == null){
			if (tmpFunctionalAbstraction.getDescendants() != null) {
				try {
					Statement stmt = dbConnection.createStatement();
					
					ResultSet repAbstractionSet = stmt.executeQuery("SELECT A_ID FROM ABSTRACTIONS WHERE A_FA_ID = " + tmpFunctionalAbstraction.getID() + " AND A_ID = " + tmpFunctionalAbstraction.getDBKey() + ";");
					if (repAbstractionSet.next()) {
						AncestorId = repAbstractionSet.getInt(1);
					}
				}
				catch(Exception e) {
					System.err.println("Exception storeDescentants: " + e.getMessage());
				}
						
				List<FunctionalAbstraction> descendants = tmpFunctionalAbstraction.getDescendants();

				for (int i = 0; i < descendants.size() ; i++){
					try {
						int descendantId = -1;
						Statement stmt = dbConnection.createStatement();
						
						ResultSet repAbstractionSet = stmt.executeQuery("SELECT A_ID FROM ABSTRACTIONS WHERE A_FA_ID = " + descendants.get(i).getID() + " AND A_ID = " + descendants.get(i).getDBKey() + ";");
						if (repAbstractionSet.next()) {
							descendantId = repAbstractionSet.getInt(1);
						}
					
						insertStatement = dbConnection.createStatement();
						String upd = new String ("INSERT INTO hier_edges(`AD_ANC_ID`, `AD_DES_ID`) VALUES(" + AncestorId + "," + descendantId + ")");
						insertStatement.executeUpdate(upd);
						
					}
					catch(Exception e) {
						System.err.println("Exception storeDescentants: " + e.getMessage());
					}
				}
			}
		}
		
		else if (tmpFunctionalAbstraction == null){
			if (tmpNonFunctionalAbstraction.getDescendants() != null) {
				try {
					Statement stmt = dbConnection.createStatement();
					
					ResultSet repAbstractionSet = stmt.executeQuery("SELECT A_ID FROM ABSTRACTIONS WHERE A_NFA_ID = " + tmpNonFunctionalAbstraction.getNonFunctionalAbstractionID() + " AND A_ID = " + tmpNonFunctionalAbstraction.getDBKey() + ";");
					if (repAbstractionSet.next()) {
						AncestorId = repAbstractionSet.getInt(1);
					}
				}
				catch(Exception e) {
					System.err.println("Exception storeDescentants: " + e.getMessage());
				}
						
				ArrayList<NonFunctionalAbstraction> descendants = tmpNonFunctionalAbstraction.getDescendants();
				for (int i = 0; i < descendants.size() ; i++){
					try {
						int DescendantId = -1;
						Statement stmt = dbConnection.createStatement();
						
						String q = new String ("SELECT A_ID FROM ABSTRACTIONS WHERE A_NFA_ID = " + descendants.get(i).getNonFunctionalAbstractionID() + " AND A_ID = " + descendants.get(i).getDBKey() + ";");
						ResultSet repAbstractionSet = stmt.executeQuery(q);
						
						if (repAbstractionSet.next()) {
							DescendantId = repAbstractionSet.getInt(1);
							
						}
					
						insertStatement = dbConnection.createStatement();
						String upd = new String ("INSERT INTO hier_edges(`AD_ANC_ID`, `AD_DES_ID`) VALUES(" + AncestorId + "," + DescendantId + ")");
						insertStatement.executeUpdate(upd);
					}
					catch(Exception e) {
						System.err.println("Exception storeDescentants: " + e.getMessage());
					}
				}
			}
				
		}//end of if - else
	}//end of storeDescendants

}