package org.ow2.choreos.serviceBase;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.ReferencedComponent;
import org.ow2.choreos.schemaRepresentation.xml.Structure;
import org.ow2.choreos.serviceRegistration.ServiceCollection;
import org.ow2.choreos.serviceRegistration.ServiceSource;
import org.ow2.choreos.serviceRepresentation.functional.DocOfFuncProperties;
import org.ow2.choreos.serviceRepresentation.functional.Message;
import org.ow2.choreos.serviceRepresentation.functional.MessageType;
import org.ow2.choreos.serviceRepresentation.functional.Operation;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInstance;
import org.ow2.choreos.serviceRepresentation.functional.ServiceInterface;
import org.ow2.choreos.serviceRepresentation.functional.ServiceType;
import org.ow2.choreos.serviceRepresentation.nonFunctional.NonFuncDescription;
import org.ow2.choreos.serviceRepresentation.nonFunctional.QualityProperties;

/**
 * 
 * @author dathanas
 *
 * This class contains methods to store the objects of the Service model to the DBMS
 */
public class ServiceRepresentationStorage {
	
	Statement insertStatement;	// empty SQL statement
	ResultSet rs;
	Statement stmt;
	
	//Constructor
	public ServiceRepresentationStorage(){
		insertStatement = null;
		rs = null;
		stmt = null;
	}
	
	//returns the (autoincrement) ID of the last inserted row in the DBMS -- needed to insert the foreign keys back to the objects
	int getLastInsertedId(Connection dbConnection){
		
		try {
			stmt = dbConnection.createStatement();
			rs = stmt.executeQuery("SELECT LAST_INSERT_ID()");
			if (rs.next()) {
				int lastInsertedID = rs.getInt(1);
				return lastInsertedID;
			} 
		}
		catch(Exception e) {
		     System.err.println("Exception getLastInsertedId: " + e.getMessage());
		}
	  	return (-1);
	}
	
	//stores the 'Collection' object sCollection (parameter) and return its ID
	public int storeCurrCollection(ServiceCollection sCollection, Connection dbConnection)
	{
		int collectionId = sCollection.getCollectionID();
		
		try {
			
  	  		insertStatement = dbConnection.createStatement();
  	  		
  	  		String upd = new String ("INSERT INTO servicecollections VALUES("+ collectionId +")");
  	  		insertStatement.executeUpdate(upd);
  	  		return getLastInsertedId(dbConnection);
  	  		  	  			
		} catch(Exception e) {
	          System.err.println("Exception storeCurrCollection: " + e.getMessage());
	    }
		return (-1);
		
	 }

	//stores the 'ServiceSource' object sSource (parameter) and return its ID
	public int storeServiceSource(ServiceSource sSource, Connection dbConnection, int scId)	{
		String ref2Source = new String (sSource.getRef2Source());
		int numOfParsedFuncProperties = sSource.getNumOfParsedFuncDocs();
		int numOfParsableFuncProperties = sSource.getNumOfParsableFuncDocs();
		
		try {
			insertStatement = dbConnection.createStatement();
			
			String temp1 = "\\\\";
  	  		String temp2 = "\\\\\\\\";
  	  		ref2Source = ref2Source.replaceAll(temp1 , temp2);
  	  		
  	  		String upd = new String ("INSERT INTO servicesources(`SS_REF2SOURCE_ID`,`SS_PARSED_FD`,`SS_PARSEABLE_FD`,`SS_SC_ID`) VALUES('" + ref2Source + "'," + numOfParsedFuncProperties + "," + numOfParsableFuncProperties + "," + scId + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		return getLastInsertedId(dbConnection);
	  			  	 	  	
		}
		catch(Exception e) {
	          System.err.println("Exception storeServiceSource: " + e.getMessage());
	    }
		return (-1);
		
	 }
	
	//stores the 'DocOfFuncProperties' object dDocOfFuncProps (parameter) and return its ID
	public int storeDocOfFuncProperties(Connection dbConnection, DocOfFuncProperties dDocOfFuncProps, int sourceCounter)
	{
		String ref2DocOfFuncProperties = new String (dDocOfFuncProps.getRef2DocOfFuncProperties());
		
		boolean hasXmlSchemas = dDocOfFuncProps.hasSchemas();
				
		try {
			insertStatement = dbConnection.createStatement();
			
			//ref2DocOfFuncProperties contains zero or more backslash characters which are special characters in Java
			String temp1 = "\\\\";
			String temp2 = "\\\\\\\\";
			ref2DocOfFuncProperties = ref2DocOfFuncProperties.replaceAll(temp1 , temp2);
									
  	  		String upd = new String ("INSERT INTO docOffuncproperties(`DOCFUNCPROPS_REF2DOCOFFUNCPROPS_ID`,`DOCFUNCPROPS_HAS_XML_SCHEMAS`,`DOCFUNCPROPS_SS_ID`) VALUES('" + ref2DocOfFuncProperties + "'," + hasXmlSchemas + "," + sourceCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		return getLastInsertedId(dbConnection);
  	  		
		}
		catch(Exception e) {
	          System.err.println("Exception storeDocOfFuncProperties: " + e.getMessage());
	    }
		return (-1);
	}
	
	//stores the 'ServiceType' object sType (parameter) and return its ID
	public int storeServiceType(ServiceType sType, Connection dbConnection, int ssId){
		String name = new String (sType.getName());
		String profile = null;
		if (sType.getProfile() != null) {
			profile = sType.getProfile();
		}
		
		String style = new String (sType.getStyle());
		String specification = new String (sType.getSpecification());
			
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO servicetypes(`ST_NAME`,`ST_PROFILE`,`ST_STYLE`,`ST_SPEC`,`ST_DOCFUNCPROPS_ID`) VALUES('" + name + "','" + profile + "','" + style + "','" + specification + "'," + ssId + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		return getLastInsertedId(dbConnection); 
  	  		
		} 
		catch(Exception e) {
	          System.err.println("Exception storeServiceType: " + e.getMessage());
	    }
		return (-1);
	 }
	
	//stores the 'ServiceInterface' object sInterface (parameter) and return its ID
	public int storeServiceInterface(ServiceInterface sInterface, Connection dbConnection, int stId){
		String name = new String (sInterface.getName());
		String profile = null;
		String behavioral = null;
		String constraints = null;
		
		//check if the values in the object are not null
		if (sInterface.getProfile() != null) {
			profile = sInterface.getProfile();
		}
		if (sInterface.getBehavioralSpecification() != null) {
			behavioral = sInterface.getBehavioralSpecification();
		}
		if (sInterface.getConstraints() != null){
			constraints = sInterface.getConstraints();
		}
				 		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO serviceinterfaces(`SI_NAME`,`SI_PROFILE`,`SI_BEHAV`,`SI_CONSTR`,`SI_ST_ID`) VALUES('" + name + "','" + profile + "','" + behavioral + "','" +constraints + "'," + stId + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		
  	  		return getLastInsertedId(dbConnection);
  	  		
		} 
		catch(Exception e) {
	          System.err.println("Exception storeServiceInterface: " + e.getMessage());
	    }
		return (-1);
		
	 }
	
	//stores the 'ServiceInstance' object sInstance (parameter) and return its ID
	public int storeServiceInstance(ServiceInstance sInstance, Connection dbConnection, int siId){
		String name = new String (sInstance.getName());
		String protocol = new String (sInstance.getProtocol());
		String uri = new String (sInstance.getURI());
		String implementDetails = null;
		String requirements = null;
		if (sInstance.getImplementationDetails() != null) {
			implementDetails = sInstance.getImplementationDetails();
		}
		if (sInstance.getRequirements() != null) {
			requirements = sInstance.getRequirements();
		}
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO serviceinstances(`SIS_NAME`,`SIS_PROTO`,`SIS_URI`,`SIS_IMP_DET`,`SIS_REQS`,`SIS_SI_ID`) VALUES('" + name + "','" + protocol + "','" + uri + "','" + implementDetails + "','" + requirements + "'," + siId + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		
  	  		return getLastInsertedId(dbConnection);
  	  		
		} 
		catch(Exception e) {
	          System.err.println("Exception storeServiceInstance: " + e.getMessage());
	    }
		return (-1);
		
	 }
	
	//stores the 'Operation' object sOperation (parameter) and return its ID
	public int storeOperation(Operation sOperation, Connection dbConnection, int siId){
		String name = new String (sOperation.getName());
		
		String profile = null; 
		String pre = null;
		String post = null;
		if (sOperation.getProfile() != null) {
			profile = sOperation.getProfile();
		}
		if (sOperation.getPreConditions() != null) {
			pre = sOperation.getPreConditions();
		}
		if (sOperation.getPostConditions() != null) {
			post = sOperation.getPostConditions();
		}
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO operations(`OP_NAME`,`OP_PROFILE`,`OP_PRE`,`OP_POST`,`OP_SI_ID`) VALUES('" + name + "','" + profile + "','" + pre + "','" + post + "'," + siId + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		
  	  		return getLastInsertedId(dbConnection);
  	  		
		} 
		catch(Exception e) {
	          System.err.println("Exception storeOperation: " + e.getMessage());
	    }
		return (-1);
	 }
	
	//stores the 'Message' object sMessage (parameter) and return its ID
	public int storeMessage(Message sMessage, Connection dbConnection, int operationCounter, String type){
		String name = new String (sMessage.getName());
		boolean hasMessageTypes = sMessage.hasMessageTypes();
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO messages(`MS_NAME`,`MS_HAS_MESS_TYPES`,`MS_TYPE`,`MS_OP_ID`) VALUES('" + name + "'," + hasMessageTypes + ",'" + type + "'," + operationCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		
  	  		return getLastInsertedId(dbConnection);
  	  		
		} 
		catch(Exception e) {
	          System.err.println("Exception storeMessage: " + e.getMessage());
	    }
		return(-1);
	 }
	
	//stores the 'NonFuncDescription' object sNonFuncDescription (parameter) and return its ID
	public int storeNonFuncDescription(NonFuncDescription sNonFuncDescription, Connection dbConnection, int nfSisId){
		String ref2Source = new String (sNonFuncDescription.getRef2Source());
		String ref2DocOfFuncProperties = new String (sNonFuncDescription.getRef2DocOfFuncProperties());
		String serviceTypeName = new String (sNonFuncDescription.getServiceTypeName());
		String serviceInstanceName = new String (sNonFuncDescription.getServiceInstanceName());
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String temp1 = "\\\\";
	  		String temp2 = "\\\\\\\\";
	  		ref2Source = ref2Source.replaceAll(temp1 , temp2);
	  		ref2DocOfFuncProperties = ref2DocOfFuncProperties.replaceAll(temp1 , temp2);
  	  		
  	  		String upd = new String ("INSERT INTO nonfuncdescriptions(`NF_REF2SOURCE_ID`,`NF_REF2DOCOFFUNCPROPS`,`NF_STYPE_NAME`,`NF_SINSTANCE_NAME`,`NF_SIS_ID`) VALUES('" + ref2Source + "','" + ref2DocOfFuncProperties + "','" + serviceTypeName + "','" + serviceInstanceName + "'," + nfSisId + ")");
  	  		insertStatement.executeUpdate(upd);
  	 
  	  		return getLastInsertedId(dbConnection);
  	  		
		} 
		catch(Exception e) {
	          System.err.println("Exception storeNonFuncDescription: " + e.getMessage());
	    }
		return (-1);
	 }
	
	
	public int storePropertyValue(String property, Double value, Connection dbConnection, int nfDescriptionCounter){						
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO propertiesvalues(`PV_PROPERTY_NAME`,`PV_VALUE`,`PV_NF_ID`) VALUES('" + property + "'," + value + "," + nfDescriptionCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		return getLastInsertedId(dbConnection);
  	  		
		} 
		catch(Exception e) {
	          System.err.println("Exception storePropertyValue: " + e.getMessage());
	    }
		return (-1);
	 }
	
	
	public void storeQualityProperties(Connection dbConnection, String property, QualityProperties value, int instanceCounter){
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		
  	  		String temp1 = "\\\\";
  	  		String temp2 = "\\\\\\\\";
  	  		property = property.replaceAll(temp1 , temp2);
		
  	  		String upd = new String ("INSERT INTO qualityproperties(`QP_SOURCE_NAME`,`QP_SIS_ID`) VALUES('" + property + "'," + instanceCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		int qpId = getLastInsertedId(dbConnection);
  	  		
  	  		ArrayList<String> properties = value.getProperties();
  	  		for (int i = 0; i < properties.size(); i++){
  	  			upd = "INSERT INTO QPPropertiesNames(`QPP_PROPERTY_NAME`,`QPP_QP_ID`) VALUES('" + properties.get(i) + "'," + qpId + ")";
  	  			insertStatement.executeUpdate(upd);
  	  			int qpPropertyId = getLastInsertedId(dbConnection);
  	  			ArrayList<String> domains = value.getDomain(i);
	  			for (int j = 0; j < domains.size(); j++){
	  				upd = "INSERT INTO QPRankings(`QPD_RANKING`,`QPD_QPP_ID`) VALUES('" + domains.get(j) + "'," + qpPropertyId/*i*/ + ")";
	  				insertStatement.executeUpdate(upd);
	  			}
  	  		}
  	  	
		} 
		catch(Exception e) {
	          System.err.println("Exception storeQualityProperties: " + e.getMessage());
	    }
	}
	
	public int storeReferencedComponent(ReferencedComponent tmpReferencedComponent, Connection dbConnection, int messageTypeCounter) {
		String prefix = null;
		String type = null;
		String prefix_type = null;
		String typeKind = tmpReferencedComponent.getTypeKind();

		if (tmpReferencedComponent.getType() != null){
			type = tmpReferencedComponent.getType();
		}
		
		if (tmpReferencedComponent.getPrefix() != null){	//if a prefix exists, concatenate it with 'type'
			prefix = new String (tmpReferencedComponent.getPrefix());
			prefix_type = prefix + ":" + type;
		}
		else{
			prefix_type = type;
		}
			
		String allTagAttributes = new String("");
		String concatenatedSchemaPath = new String("");
		String concatenatedClassPath = new String("");
				
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO componentsandstructures(`COM_NAME`,`COM_TYPE`,`COM_TYPEKIND`,`COM_ISGLOBAL`,`COM_TAG_ATTRIBUTES`,`COM_SCHEMA_PATH`,`COM_CLASS_PATH`,`COM_MODEL`,`COM_IS_COMPONENT`,`COM_MT_ID`) VALUES('" + null + "','" + type + "','" + typeKind + "'," + true + ",'" + allTagAttributes + "','" + concatenatedSchemaPath + "','" + concatenatedClassPath + "','" + "'," + true + "," + messageTypeCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		
  	  		return getLastInsertedId(dbConnection);
		} 
		catch(Exception e) {
	          System.err.println("Exception storeReferencedComponent: " + e.getMessage());
	    }
		return (-1);
	}
	
	
	@SuppressWarnings("rawtypes")
	public int storeLeafComponent(Component tmpComponent, Connection dbConnection, int messageTypeCounter) {
		String name = new String (tmpComponent.getName());
		String model = new String (tmpComponent.getSchemaRepresentationModel());
		boolean isGlobal = tmpComponent.isGlobal();
		
		Map<String, String> tagAttributes = tmpComponent.getTagAttributes();
		String allTagAttributes = new String("");
		boolean flag = false;
				
		if( tagAttributes != null ){
			//parse and store Properties and Values
			Set tagSet = tagAttributes.entrySet();
		    Iterator tagIterator = tagSet.iterator();
		    while (tagIterator.hasNext()) {
		    	Map.Entry entry = (Map.Entry) tagIterator.next();
		    	String attribute = (String) entry.getValue();
				if (flag){
					allTagAttributes = allTagAttributes.concat("," + entry.getKey().toString() + "-" + attribute);
				}
				else{
					flag = true;
					allTagAttributes = allTagAttributes.concat(entry.getKey().toString() + "-" + attribute);
				}
		    }
		} 
		

		String[] schemaPath = tmpComponent.getSchemaPath();
		String concatenatedSchemaPath = new String("");
		for (int i = 0; i < schemaPath.length; i++){
			if (i == schemaPath.length - 1){
				concatenatedSchemaPath = concatenatedSchemaPath.concat(schemaPath[i]);
			}
			else{
				concatenatedSchemaPath = concatenatedSchemaPath.concat(schemaPath[i] + "-");
			}
		}
		

		String[] classPath = tmpComponent.getClassPath();
		String concatenatedClassPath = new String("");
		for (int i = 0; classPath != null && i < classPath.length; i++){
			if (i == classPath.length - 1){
				concatenatedClassPath = concatenatedClassPath.concat(classPath[i]);
			}
			else{
				concatenatedClassPath = concatenatedClassPath.concat(classPath[i] + "-");
			}
		}
		
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO componentsandstructures(`COM_NAME`,`COM_TYPE`,`COM_TYPEKIND`,`COM_ISGLOBAL`,`COM_TAG_ATTRIBUTES`,`COM_SCHEMA_PATH`,`COM_CLASS_PATH`,`COM_MODEL`,`COM_IS_COMPONENT`,`COM_MT_ID`) VALUES('" + name + "'," + null + "," + null + "," + isGlobal + ",'" + allTagAttributes + "','" + concatenatedSchemaPath + "','" + concatenatedClassPath + "','" + model + "'," + true + "," + messageTypeCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		
  	  		return getLastInsertedId(dbConnection);
		} 
		catch(Exception e) {
	          System.err.println("Exception storeLeafComponent: " + e.getMessage());
	    }
		return (-1);
	}
	
	
	@SuppressWarnings("rawtypes")
	public int storeLeafStructure(Structure tmpStructure, Connection dbConnection, int messageTypeCounter) {
		
		String model = new String (tmpStructure.getSchemaRepresentationModel());
		Map<String, String> tagAttributes = tmpStructure.getTagAttributes();
		String allTagAttributes = new String("");
		boolean flag = false;
		
		if( tagAttributes != null ){
			//parse and store Properties and Values
			Set tagSet = tagAttributes.entrySet();
		    Iterator tagIterator = tagSet.iterator();
		    while (tagIterator.hasNext()) {
		    	Map.Entry entry = (Map.Entry) tagIterator.next();
		    	String attribute = (String) entry.getValue();
				if (flag){
					allTagAttributes = allTagAttributes.concat("," + entry.getKey().toString() + "-" + attribute);
				}
				else{
					flag = true;
					allTagAttributes = allTagAttributes.concat(entry.getKey().toString() + "-" + attribute);
				}
		    }
		} 
		
		
		String[] schemaPath = tmpStructure.getSchemaPath();
		String concatenatedSchemaPath = new String("");
		for (int i = 0; i < schemaPath.length; i++){
			if (i == schemaPath.length - 1){
				concatenatedSchemaPath = concatenatedSchemaPath.concat(schemaPath[i]);
			}
			else{
				concatenatedSchemaPath = concatenatedSchemaPath.concat(schemaPath[i] + "-");
			}
		}
		

		String[] classPath = tmpStructure.getClassPath();
		String concatenatedClassPath = new String("");
		for (int i = 0; i < classPath.length; i++){
			if (i == classPath.length - 1){
				concatenatedClassPath = concatenatedClassPath.concat(classPath[i]);
			}
			else{
				concatenatedClassPath = concatenatedClassPath.concat(classPath[i] + "-");
			}
		}
		
		
		try {
  	  		insertStatement = dbConnection.createStatement();
  	  		String upd = new String ("INSERT INTO componentsandstructures(`COM_NAME`,`COM_TYPE`,`COM_TYPEKIND`,`COM_ISGLOBAL`,`COM_TAG_ATTRIBUTES`,`COM_SCHEMA_PATH`,`COM_CLASS_PATH`,`COM_MODEL`,`COM_IS_COMPONENT`,`COM_MT_ID`) VALUES(" + tmpStructure.getID() + "," + null + "," + null + "," + false + ",'" + allTagAttributes + "','" + concatenatedSchemaPath + "','" + concatenatedClassPath + "','" + model + "'," + false + ","  + messageTypeCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		
  	  		return getLastInsertedId(dbConnection);
  	  	
		} 
		catch(Exception e) {
	          System.err.println("Exception storeLeafStructure: " + e.getMessage());
	    }
		return (-1);
	}
	
	//stores the 'MessageType' object sMessageType (parameter) and return its ID
	public int storeMessageType(MessageType sMessageType, Connection dbConnection, int messageCounter){
		String name = new String (sMessageType.getName());
		String prefix = null;
		String type = null;
		String prefix_type = null;
		String typeKind = sMessageType.getReferencedComponent().getTypeKind();
		
		if (sMessageType.getReferencedComponent().getType() != null){
			type = new String (sMessageType.getReferencedComponent().getType());
		}
		
		if (sMessageType.getReferencedComponent().getPrefix() != null){	//if a prefix exists, concatenate it with 'type'
			prefix = new String (sMessageType.getReferencedComponent().getPrefix());
			prefix_type = prefix + ":" + type;
		}
		else{
			prefix_type = type;
		}

		try {
			insertStatement = dbConnection.createStatement();
			String upd = new String ("INSERT INTO messagetypes(`MT_NAME`,`MT_PREFIX_TYPE`,`MT_TYPEKIND`,`MT_TYPE`,`MT_MS_ID`) VALUES('" + name + "','" + prefix_type + "','" + typeKind + "','" + type + "'," + messageCounter + ")");
  	  		insertStatement.executeUpdate(upd);
  	  		
  	  		return getLastInsertedId(dbConnection);
  	  	
		}
		catch(Exception e) {
	          System.err.println("Exception storeMessageType: " + e.getMessage());
	    }
		return (-1);
		
	 }
	
}

