package org.ow2.choreos.serviceRepresentation.functional;

import java.util.Hashtable;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapPrinting;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapTransfomation;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.schemaRepresentation.SchemaRepresentationController;
import org.ow2.choreos.schemaRepresentation.xml.Component;
import org.ow2.choreos.schemaRepresentation.xml.ReferencedComponent;


/**
 * 
 * @author dathanas
 *
 */
public class MessageType implements java.io.Serializable {


	//Variables.
	private static final long serialVersionUID = 3709115780808063454L;

	private Map<String, String> tagAttributes;

	private String name;

	private ReferencedComponent referencedComponent;

	private boolean hasUnmaterializedReferencedComponent;

	private Map<String, Boolean> hasTransformedReferencedComponent;//The String is the 'schemaRepresentationModel'.

	private int dbKey;
	
	private boolean concrete; //actual (true) or representative (false)

	private String constraintViolation;
	private boolean hasConstraintViolation;


	//Constructor 1.
	public MessageType( Map<String, String> cTagAttributes ){

		String name = (String) MapGetter.getValueByStringKey( "name", cTagAttributes );

		String type = MapGetter.getStringValueByCandidateKeysLowerCase( ReferencedComponent.typeKinds, cTagAttributes );

		String typeKind = MapGetter.getStringKeyByCandidateKeysLowerCase( ReferencedComponent.typeKinds, cTagAttributes );


		referencedComponent = new ReferencedComponent( type, typeKind );

		if( referencedComponent.hasConstraintViolation() ) ObjectConstraintsManipulation.setErrorAttribute( this, typeKind, referencedComponent.getConstraintViolation() );

		else hasUnmaterializedReferencedComponent = referencedComponent.hasUnmaterializedComponent();


		setTagAttributes( cTagAttributes );


		initVariables( name );
	}

	//Construcor 2: It is used for extraction purposes.
	public MessageType( String cName, String cType, String typeKind, Component materializedComponent ){

		referencedComponent = new ReferencedComponent( cType, typeKind, materializedComponent );

		if( referencedComponent.hasConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, referencedComponent );

		else hasUnmaterializedReferencedComponent = referencedComponent.hasUnmaterializedComponent();


		setTagAttributes( cName, cType, typeKind );


		initVariables( cName );
	}

	//Constructor 3: It is used for retrieval purposes.
	public MessageType( String cName, String cType, String typeKind ){

		referencedComponent = new ReferencedComponent( cType, typeKind );

		if( referencedComponent.hasConstraintViolation() ) ObjectConstraintsManipulation.setErrorAttribute( this, typeKind, referencedComponent.getConstraintViolation() );

		else hasUnmaterializedReferencedComponent = referencedComponent.hasUnmaterializedComponent();


		setTagAttributes( cName, cType, typeKind );


		initVariables( cName );
	}
	
	//Empty constructor used for xml transformation
	public MessageType(){
		
	}

	//Methods used by the constructors.
	private void initVariables( String cName ){

		if( checkConstraint( cName ) == null ) name = cName;

		concrete = true;


		PrintMessages.print( PrintMessages.wsdlPrint, hasConstraintViolation(), this );
	}

	private void setTagAttributes( Map<String, String> cTagAttributes ){

		String name = (String) MapGetter.getValueByStringKey( "name", cTagAttributes );

		String type = MapGetter.getStringValueByCandidateKeysLowerCase( ReferencedComponent.typeKinds, cTagAttributes );

		String typeKind = MapGetter.getStringKeyByCandidateKeysLowerCase( ReferencedComponent.typeKinds, cTagAttributes );


		setTagAttributes( name, type, typeKind );


		Map<String, String> qualifierNamespaces = MapGetter.getFilteredMapValues( "xmlns", ":", cTagAttributes );

		tagAttributes = MapTransfomation.concat( tagAttributes, qualifierNamespaces );
	}

	private void setTagAttributes( String cName, String cType, String typeKind ){

		tagAttributes = new Hashtable<String, String>();


		tagAttributes.put( SchemaRepresentationController.getTagNameKey(), "messageType" );

		if( cName != null && ! cName.isEmpty() ) tagAttributes.put( "name", cName );

		if( cType != null && ! cType.isEmpty() && typeKind != null && ! typeKind.isEmpty() ) tagAttributes.put( typeKind, cType );
	}


	//Constraint checkers.
	private String checkConstraint( String value ){

		return value == null ? setConstraintViolation( "Missing value" ) : null;
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	void connect( Map<String, String> namespaceQualifiers, String docTargetNamespace, Map<String, String> schemaQualifierNamespaces, Map<String, Object> indexGlobalComponentsWithoutTargetNamespaces, Map<String, Object> indexGlobalComponentsWithTargetNamespaces, String schemaRepresentationModel, String schemaDefinitionNamespace ){

		PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "START - connect: " + getTag() + " - hasUnmaterializedReferencedType= " + hasUnmaterializedReferencedComponent + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel + " - schemaDefinitionNamespace= " + schemaDefinitionNamespace + " - docTargetNamespace= " + docTargetNamespace );

		if( hasUnmaterializedReferencedComponent && ! hasConstraintViolation ){


			referencedComponent.replacePrefix( namespaceQualifiers, docTargetNamespace );

			replacePrefix();


			Map<String, String> tagQualifierNamespaces = MapGetter.getFilteredMapValues( "xmlns", ":", tagAttributes );

			Map<String, Object> indexLocalComponentsWithoutTargetNamespaces = null;

			referencedComponent.connect( this, schemaQualifierNamespaces, tagQualifierNamespaces, indexLocalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces, schemaRepresentationModel, schemaDefinitionNamespace );


			if( referencedComponent.hasUnmaterializedComponent() ) return;


			hasUnmaterializedReferencedComponent = false;
		}

		PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "END - connect: " + getTag() + " - hasUnmaterializedReferencedType= " + hasUnmaterializedReferencedComponent + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel + " - schemaDefinitionNamespace= " + schemaDefinitionNamespace + " - docTargetNamespace= " + docTargetNamespace );
	}

	private void replacePrefix(){

		String typeKind = referencedComponent != null ? referencedComponent.getTypeKind() : null;

		String prefix = referencedComponent != null ? referencedComponent.getPrefix() : null;

		String type = referencedComponent != null ? referencedComponent.getType() : null;


		if( typeKind != null && type != null ){

			if( prefix != null ) tagAttributes.put( typeKind, prefix + ":" + type );

			else tagAttributes.put( typeKind, type );
		}
	}


	void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedReferencedType= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedReferencedComponent ) + " hasConstraintViolation= " + hasConstraintViolation );

		if( ! MapGetter.getBooleanValueByKey( targetModel, hasTransformedReferencedComponent ) && ! hasConstraintViolation ){


			referencedComponent.transform( targetModel );


			if( ! referencedComponent.hasTransformedReferencedComponent( targetModel ) ) return;


			if( hasTransformedReferencedComponent == null ) hasTransformedReferencedComponent = new Hashtable<String, Boolean>();

			hasTransformedReferencedComponent.put( targetModel, true );
		}

		PrintMessages.print( PrintMessages.transformationPrint, "END - transform: " + getTag() + " - hasTransformedReferencedType= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedReferencedComponent ) + " hasConstraintViolation= " + hasConstraintViolation );
	}

	public void setDBKey( int cDBKey ){

		dbKey = cDBKey;
	}
	
	public void setConcrete( boolean cIsConcrete ){

		concrete = cIsConcrete;
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any checkConstraint method.


		if( referencedComponent.hasFinalConstraintViolation( this ) ) ObjectConstraintsManipulation.setContainmentError ( this, referencedComponent );


		return getConstraintViolation();
	}

	boolean hasUnmaterializedReferencedComponent(){

		return hasUnmaterializedReferencedComponent;
	}

	boolean hasTransformedReferencedComponent( String targetModel ){

		return MapGetter.getBooleanValueByKey( targetModel, hasTransformedReferencedComponent );
	}

	public String getName(){

		return name;
	}

	public ReferencedComponent getReferencedComponent(){

		return referencedComponent;
	}

	public int getDBKey(){

		return dbKey;
	}

	public Map<String, String> getTagAttributes(){

		return tagAttributes;
	}

	public boolean isConcrete(){

		return concrete;
	}

	public String getTagName(){

		return "messageType";
	}

	public String getTag(){

		String tag = null;


		if( name != null && referencedComponent != null ) tag = getTagName() + " name= '" + name + "'" + referencedComponent.getTag();

		else if( name == null && referencedComponent != null ) tag = getTagName() + referencedComponent.getTag();

		else if( name != null && referencedComponent == null ) tag = getTagName() + " name= '" + name + "'";

		else tag = getTagName() + " ";


		return tag;
	}


	//Visualization.
	@SuppressWarnings("serial")
	public DefaultMutableTreeNode visualize(){

		Map<String, String> exceptedKeys = new Hashtable<String, String>() { { put( SchemaRepresentationController.getTagNameKey(), SchemaRepresentationController.getTagNameKey() ); } };


		DefaultMutableTreeNode node = new DefaultMutableTreeNode( getTagName() + MapPrinting.printSpaceSeparated( exceptedKeys, getTagAttributes() ) );


		if( ! referencedComponent.isBuiltInType() ) node.add( referencedComponent.visualize() );


		return node;
	}
	
	public DefaultMutableTreeNode visualizeForQueryResults2(){

		DefaultMutableTreeNode node = new DefaultMutableTreeNode( getTag() );

		if( ! referencedComponent.isBuiltInType() ) node.add( referencedComponent.visualize() );

		return node;
	}
	
	public DefaultMutableTreeNode visualizeForQueryResults(){

		if( hasConstraintViolation ) return null;

		DefaultMutableTreeNode node = new DefaultMutableTreeNode( getTag() );

		//if( ! referencedComponent.isBuiltInType() ) node.add( referencedComponent.visualize() );
		
		return node;
	}
	

	//Setters used for xml transformation
	
	public void setxName( String cName ){
		name = cName;
	}
		
	public void setxTagAttributes( Map<String, String> cTagAttributes ){
		tagAttributes = cTagAttributes;
	}
		
	public void setxReferencedComponent( ReferencedComponent cReferencedComponent ){
		referencedComponent = cReferencedComponent;
	}
		
	public void setxHasUnmaterializedReferencedComponent( boolean cHasUnmaterializedReferencedComponent ){
		hasUnmaterializedReferencedComponent = cHasUnmaterializedReferencedComponent;
	}
		
	public void setxHasTransformedReferencedComponent( Map<String, Boolean> cHasTransformedReferencedComponent ){
		hasTransformedReferencedComponent = cHasTransformedReferencedComponent;
	}
		
	public void setxConstraintViolation( String cConstraintViolation ){
		constraintViolation = cConstraintViolation;
	}
		
	public void setxHasConstraintViolation( boolean cHasConstraintViolation ){
		hasConstraintViolation = cHasConstraintViolation;
	}
		
	public void setxDBKey( int cDBKey ){
		dbKey = cDBKey;
	}
		
	public void setxConcrete( boolean cConcrete ){
		concrete = cConcrete;
	}
	
	//Getters used for xml transformation
	
	public String getxName(){
		return name;
	}
		
	public Map<String, String> getxTagAttributes(){
		return tagAttributes;
	}
		
	public ReferencedComponent getxReferencedComponent(){
		return referencedComponent;
	}
		
	public boolean getxHasUnmaterializedReferencedComponent(){
		return hasUnmaterializedReferencedComponent;
	}
		
	public Map<String, Boolean> getxHasTransformedReferencedComponent(){
		return hasTransformedReferencedComponent;
	}
		
	public String getxConstraintViolation(){
		return constraintViolation;
	}
		
	public boolean getxHasConstraintViolation(){
		return hasConstraintViolation;
	}
		
	public int getxDBKey(){
		return dbKey;
	}
		
	public boolean getxConcrete(){
		return concrete;
	}
}
