package org.ow2.choreos.serviceRepresentation.functional;

import java.util.Hashtable;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;


/**
 * 
 * @author dathanas
 *
 */
public class Operation implements java.io.Serializable {


	//Variables.
	private static final long serialVersionUID = -4479327557836278128L;

	private String name;
	private String p;						//Optional: interface profile.

	private Message inMessage;				//It may be null!
	private boolean hasInMessage;

	private Message outMessage;				//It may be null!
	private boolean hasOutMessage;

	private boolean hasUnconnectedMessages;

	private Map<String, Boolean> hasTransformedMessages;//The String is the representation model.

	private String pre;						//Optional: pre-conditions.
	private String post;					//Optional: post-conditions.

	private String constraintViolation;
	private boolean hasConstraintViolation;

	private int dbKey;

	private int dbAbstractionKey;
	
	private boolean concrete; //concrete (true) or representative (false)


	//Constructor.
	public Operation( String cName, Message inMessage, Message outMessage ){

		name = cName;

		setInMessage( inMessage );

		setOutMessage( outMessage );

		hasUnconnectedMessages = true;

		hasConstraintViolation = false;

		concrete = true;


		PrintMessages.print( PrintMessages.wsdlPrint, hasConstraintViolation(), this );
	}
	
	public Operation(){
		
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	public void setInMessage( Message cInMessage ){
if(cInMessage == null){ hasInMessage = false;
		return;}
		if( ObjectConstraintsManipulation.setContainmentError( this, cInMessage ) == null ){

			inMessage = cInMessage;

			hasInMessage = true;
		}

		else hasInMessage = false;
	}

	public void setOutMessage( Message cOutMessage ){
if(cOutMessage == null){ hasOutMessage = false;
return;}
		if( ObjectConstraintsManipulation.setContainmentError( this, cOutMessage ) == null ){

			outMessage = cOutMessage;

			hasOutMessage = true;
		}

		else hasOutMessage = false;
	}

	public void setDBKey( int cDBKey ){

		dbKey = cDBKey;
	}

	public void setDBAbstractionKey( int cDBKey ){

		dbAbstractionKey = cDBKey;
	}
	
	public void setConcrete( boolean cIsConcrete ){

		concrete = cIsConcrete;
	}

	void connect( Map<String, String> namespaceQualifiers, String docTargetNamespace, Map<String, String> qualifierNamespaces, Map<String, Object> indexGlobalComponentsWithoutTargetNamespaces, Map<String, Object> indexGlobalComponentsWithTargetNamespaces, String schemaRepresentationModel, String schemaDefinitionNamespace ){

		PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "START - connect: " + getTag() + " - hasUnconnectedMessages= " + hasUnconnectedMessages + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel + " - schemaDefinitionNamespace= " + schemaDefinitionNamespace );

		if( hasUnconnectedMessages && ! hasConstraintViolation ){

			if( hasInMessage ){


				inMessage.connect( namespaceQualifiers, docTargetNamespace, qualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces, schemaRepresentationModel, schemaDefinitionNamespace );

				if( inMessage.hasUnconnectedMessageTypes() ) return;
			}


			if( hasOutMessage ){


				outMessage.connect( namespaceQualifiers, docTargetNamespace, qualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces, schemaRepresentationModel, schemaDefinitionNamespace );

				if( outMessage.hasUnconnectedMessageTypes() ) return;
			}

			hasUnconnectedMessages = false;
		}

		PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "END - connect: " + getTag() + " - hasUnconnectedMessages= " + hasUnconnectedMessages + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel + " - schemaDefinitionNamespace= " + schemaDefinitionNamespace );
	}

	void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedMessages= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedMessages ) + " hasConstraintViolation= " + hasConstraintViolation );

		if( ! MapGetter.getBooleanValueByKey( targetModel, hasTransformedMessages ) && ! hasConstraintViolation ){

			if( hasInMessage ){


				inMessage.transform( targetModel );


				if( ! inMessage.hasTransformedMessageTypes( targetModel ) ) return;
			}

			if( hasOutMessage ){


				outMessage.transform( targetModel );


				if( ! outMessage.hasTransformedMessageTypes( targetModel ) ) return;
			}

			if( hasTransformedMessages == null ) hasTransformedMessages = new Hashtable<String, Boolean>();
if(targetModel != null)
			hasTransformedMessages.put( targetModel, true );
		}

		PrintMessages.print( PrintMessages.transformationPrint, "END - transform: " + getTag() + " - hasTransformedMessages= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedMessages ) + " hasConstraintViolation= " + hasConstraintViolation );
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any 'checkConstraint()' method.


		if( hasInMessage && inMessage.hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, inMessage );

		if( hasOutMessage && outMessage.hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, outMessage );


		return constraintViolation;
	}

	boolean hasUnconnectedMessages(){

		return hasUnconnectedMessages;
	}

	boolean hasTransformedMessages( String targetModel ){

		return MapGetter.getBooleanValueByKey( targetModel, hasTransformedMessages );
	}

	public int getDBKey(){

		return dbKey;
	}
	
	public boolean isConcrete(){

		return concrete;
	}
	
	public int getDBAbstractionKey(){

		return dbAbstractionKey;
	}

	public String getName(){

		return name;
	}

	public Message getInMessage(){

		return inMessage;
	}

	public Message getOutMessage(){

		return outMessage;
	}

	public String getProfile(){

		return p;
	}

	public String getPreConditions(){
		return pre;
	}

	public String getPostConditions(){
		return post;
	}

	public String getTag(){

		return "operation name= '" + name + "'";
	}


	//Visualization.
	DefaultMutableTreeNode visualize( Integer label, String endOfPath ){//This function is used by the 'Visualizer' class.

		DefaultMutableTreeNode operation = new DefaultMutableTreeNode("Operation (" + label + "): " + name);

		if( endOfPath != null && ! endOfPath.equals("Service Operations") && endOfPath.equals("Service Messages") ){

			DefaultMutableTreeNode in;

			if( hasInMessage ) in = inMessage.visualize("Input message ");

			else in = new DefaultMutableTreeNode("Undefined input message");

			operation.add( in );
	

			DefaultMutableTreeNode out;

			if( hasOutMessage ) out = outMessage.visualize("Output message ");

			else out = new DefaultMutableTreeNode("Undefined output message");
	
			operation.add( out );
		}

    	return operation;
	}

	DefaultMutableTreeNode visualize( Integer label ){//This function is used by the 'Visualizer' class.

		DefaultMutableTreeNode operation = new DefaultMutableTreeNode( "Operation (" + label + "): " + name );


		DefaultMutableTreeNode in;

		if( hasInMessage ) in = inMessage.visualize( "Input message " );

		else in = new DefaultMutableTreeNode( "Undefined input message" );

		operation.add( in );
	

		DefaultMutableTreeNode out;

		if( hasOutMessage ) out = outMessage.visualize( "Output message " );

		else out = new DefaultMutableTreeNode( "Undefined output message" );
	
		operation.add(out);


    	return operation;
	}
	
	
	public DefaultMutableTreeNode visualizeForQueryResults( Integer label ){//This function is used by the 'Visualizer' class.

		if( hasConstraintViolation ) return null;


		DefaultMutableTreeNode operation = new DefaultMutableTreeNode("Operation (" + label + "): " + name);


		DefaultMutableTreeNode in;

		if( hasInMessage ) in = inMessage.visualizeForQueryResults( "Input message " );

		else in = new DefaultMutableTreeNode( "Undefined input message" );

		operation.add( in );
	

		DefaultMutableTreeNode out;

		if( hasOutMessage ) out = outMessage.visualizeForQueryResults( "Output message " );

		else out = new DefaultMutableTreeNode( "Undefined output message" );
	
		operation.add(out);


    	return operation;
	}
	
	public DefaultMutableTreeNode visualizeForQueryResults2( Integer label ){//This function is used by the 'Visualizer' class.

		DefaultMutableTreeNode operation = new DefaultMutableTreeNode( "Operation (" + label + "): " + name );


		DefaultMutableTreeNode in;

		if( hasInMessage ) in = inMessage.visualizeForQueryResults2( "Input message " );

		else in = new DefaultMutableTreeNode( "Undefined input message" );

		operation.add( in );
	

		DefaultMutableTreeNode out;

		if( hasOutMessage ) out = outMessage.visualizeForQueryResults2( "Output message " );

		else out = new DefaultMutableTreeNode( "Undefined output message" );
	
		operation.add(out);


    	return operation;
	}
	

	//Setters used for xml transformation
		
	public void setxName( String cName ){
		name = cName;
	}
		
	public void setxInMessage( Message cInMessage ){
		inMessage = cInMessage;
	}
		
	public void setxHasInMessage( boolean cHasInMessage ){
		hasInMessage = cHasInMessage;
	}
		
	public void setxOutMessage( Message cOutMessage ){
		outMessage = cOutMessage;
	}
		
	public void setxHasOutMessage( boolean cHasOutMessage ){
		hasOutMessage = cHasOutMessage;
	}
		
	public void setxHasTransformedMessages( Map<String, Boolean> cHasTransformedMessages ){
		hasTransformedMessages = cHasTransformedMessages;
	}
	
	public void setxPre( String cPre ){
		pre = cPre;
	}
	
	public void setxPost( String cPost ){
		post = cPost;
	}
		
	public void setxP( String cP ){
		p = cP;
	}
		
	public void setxConstraintViolation( String cConstraintViolation ){
		constraintViolation = cConstraintViolation;
	}
		
	public void setxHasConstraintViolation( boolean cHasConstraintViolation ){
		hasConstraintViolation = cHasConstraintViolation;
	}
		
	public void setxDBKey( int cDBKey ){
		dbKey = cDBKey;
	}
		
	public void setxDBAbstractionKey( int cDBAbstractionKey ){
		dbAbstractionKey = cDBAbstractionKey;
	}
		
	public void setxConcrete( boolean cConcrete ){
		concrete = cConcrete;
	}
	
	//Getters used for xml transformation
	
	public String getxName(){
		return name;
	}
		
	public Message getxInMessage(){
		return inMessage;
	}
		
	public boolean getxHasInMessage(){
		return hasInMessage;
	}
		
	public Message getxOutMessage(){
		return outMessage;
	}
		
	public boolean getxHasOutMessage(){
		return hasOutMessage;
	}
		
	public Map<String, Boolean> getxHasTransformedMessages(){
		return hasTransformedMessages;
	}
	
	public String getxPre(){
		return pre;
	}
	
	public String getxPost(){
		return post;
	}
		
	public String getxP(){
		return p;
	}
		
	public String getxConstraintViolation(){
		return constraintViolation;
	}
		
	public boolean getxHasConstraintViolation(){
		return hasConstraintViolation;
	}
		
	public int getxDBKey(){
		return dbKey;
	}
		
	public int getxDBAbstractionKey(){
		return dbAbstractionKey;
	}
		
	public boolean getxConcrete(){
		return concrete;
	}
		
}
