package org.ow2.choreos.serviceRepresentation.functional;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.tree.DefaultMutableTreeNode;

import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.serviceRepresentation.nonFunctional.NonFuncDescription;
import org.ow2.choreos.serviceRepresentation.nonFunctional.QualityProperties;


/**
 * 
 * @author dathanas
 *
 */
public class ServiceInstance implements java.io.Serializable{


	//Variables.
	private static final long serialVersionUID = -1490569813557283252L;

	private String name;
	private String protocol;				//Protocol: soap, http, soap12.
	private String uri;						//Endpoint.

	private Map<String, QualityProperties> qualityProperties;//The quality properties are indexed by the sources!
	private boolean hasQualityProperties;

	private Map<String, NonFuncDescription> nonFuncDescriptions;//The non-functional descriptions are indexed by the sources!

	private String d;						//Optional: description of implementation details.
	private String R;						//Optional: set of requirements.

	private String constraintViolation;
	private boolean hasConstraintViolation;

	private int dbKey;


	//Constructor.
	public ServiceInstance( String cName, String cPprotocol, String URI ){

		name = cName;           
		protocol = cPprotocol;
		uri = URI;

		hasConstraintViolation = false;

		PrintMessages.print( PrintMessages.wsdlPrint, hasConstraintViolation(), this );
	}
	
	
	//Empty constructor used for xml transformation
	public ServiceInstance(){
		
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return ( getFinalConstraintViolation() != null ) ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//It did not find any checkConstraint method.


		//TO check its components ( non-functional ).

		return constraintViolation;
	}

	public int getDBKey(){

		return dbKey;
	}

	public String getName(){

		return name;
	}

	public String getURI(){

		return uri;
	}

	public String getProtocol(){

		return protocol;
	}

	public String getImplementationDetails(){

		return d;
	}

	public String getRequirements(){

		return R;
	}

	public boolean hasNonFunctionalDescription( String ref2Source ){

		if( nonFuncDescriptions == null || ( nonFuncDescriptions != null && nonFuncDescriptions.get( ref2Source ) == null ) ) return false;

		return true;
	}

	public QualityProperties getQualityProperties( String ref2Source ){

		if( hasQualityProperties )

			return qualityProperties.get( ref2Source );

		return null;
	}

	public Map<String, QualityProperties> getQualityProperties(){

		return qualityProperties;
	}
	
	Boolean existNonFuncDescriptions(){

		if( nonFuncDescriptions != null ) return true;

		return false;
	}

	public String getTag(){

		return "service instance name= '" + name + "'";
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	public void setDBKey( int cDBKey ){

		dbKey = cDBKey;
	}

	public void set( NonFuncDescription nonFuncDescription, QualityProperties qualityProperties ){

		if(nonFuncDescriptions == null) nonFuncDescriptions = new Hashtable<String, NonFuncDescription>();

		nonFuncDescriptions.put( nonFuncDescription.getRef2Source(), nonFuncDescription );

		setQualityProperties( nonFuncDescription.getRef2Source(), qualityProperties );
	}

	public void setQualityProperties( String ref2Source, QualityProperties qualityProperties ){

		if( ! hasQualityProperties ){

			this.qualityProperties = new Hashtable<String, QualityProperties>();
			hasQualityProperties = true;
		}

		this.qualityProperties.put( ref2Source, qualityProperties );
	}

	public Map<String, NonFuncDescription> getNonFuncDescriptions(){
		return nonFuncDescriptions;
	}

	public Double getValue( String ref2Source, String property ){
		return nonFuncDescriptions.get( ref2Source ).getValue( property );
	}

	//This function is used by the 'Visualizer' class.
	DefaultMutableTreeNode visualize( int label, String ref2Source ){

		DefaultMutableTreeNode node = new DefaultMutableTreeNode( "Service instance(" + label + "): " + name + " - (protocol= " + protocol + ", uri= " + uri + ")");

		if( nonFuncDescriptions == null || ( nonFuncDescriptions != null && nonFuncDescriptions.get( ref2Source ) == null ) ){

			return node;
		}

		node.add( nonFuncDescriptions.get(ref2Source).visualize() );

		return node;
	}

	DefaultMutableTreeNode visualize( Integer label, String ref2Source, DefaultMutableTreeNode node ){

		DefaultMutableTreeNode child = new DefaultMutableTreeNode( "Service instance(" + label + "): " + name + " - " + protocol + ": " + uri );
		node.add( child );

		if(nonFuncDescriptions == null || ( nonFuncDescriptions != null && nonFuncDescriptions.get( ref2Source ) == null)) return child;

		child.add( nonFuncDescriptions.get(ref2Source).visualize() );

		return node;
	}

	DefaultMutableTreeNode visualize( String ref2Source, DefaultMutableTreeNode node ){

		if(nonFuncDescriptions == null || ( nonFuncDescriptions != null && nonFuncDescriptions.get( ref2Source ) == null)) return node;

		node.add( nonFuncDescriptions.get(ref2Source).visualize() );

		return node;
	}

	public DefaultMutableTreeNode visualize( Integer label, DefaultMutableTreeNode node ){

		DefaultMutableTreeNode child = new DefaultMutableTreeNode( "Service instance (" + label + "): Name = " + name);
		node.add( child );

		return node;
	}
	
	
	//------------------------------------------------------------------
	@SuppressWarnings("unchecked")
	public DefaultMutableTreeNode visualizeForQueryResults( int label, String ref2Source ){

		DefaultMutableTreeNode node = new DefaultMutableTreeNode( "Service instance(" + label + "): " + name + " - (protocol= " + protocol + ", uri= " + uri + ")");

		//if( nonFuncDescriptions == null || ( nonFuncDescriptions != null && nonFuncDescriptions.get( ref2Source ) == null ) ){

		//	return node;
		//}		
				
		if (nonFuncDescriptions != null ){
			Set nonFuncDescriptionsSet = nonFuncDescriptions.entrySet();
			Iterator nonFuncDescriptionsIterator = nonFuncDescriptionsSet.iterator();
			while (nonFuncDescriptionsIterator.hasNext()) {
				//System.out.println("IXAAAAAAAAA");
				Map.Entry entry = (Map.Entry) nonFuncDescriptionsIterator.next();
				node.add( ((NonFuncDescription) entry.getValue()).visualize() );
			}
		}
				
		//node.add( nonFuncDescriptions.get(ref2Source).visualize() );
			
		return node;
	}
	//------------------------------------------------------------------
	
	
	//Setters used for xml transformation
	
	public void setxName( String cName ){
		name = cName;
	}
		
	public void setxProtocol( String cProtocol ){
		protocol = cProtocol;
	}
		
	public void setxUri( String cUri ){
		uri = cUri;
	}
		
	public void setxQualityProperties( Map<String, QualityProperties> cQualityProperties ){
		qualityProperties = cQualityProperties;
	}
		
	public void setxHasQualityProperties( boolean cHasQualityProperties ){
		hasQualityProperties = cHasQualityProperties;
	}
	
	public void setxNonFuncDescriptions( Map<String, NonFuncDescription> cNonFuncDescriptions ){
		nonFuncDescriptions = cNonFuncDescriptions;
	}
		
	public void setxD( String cD ){
		d = cD;
	}
		
	public void setxR( String cR ){
		R = cR;
	}
		
	public void setxConstraintViolation( String cConstraintViolation ){
		constraintViolation = cConstraintViolation;
	}
		
	public void setxHasConstraintViolation( boolean cHasConstraintViolation ){
		hasConstraintViolation = cHasConstraintViolation;
	}
		
	public void setxDBKey( int cDBKey ){
		dbKey = cDBKey;
	}
	
	//Getters used for xml transformation
	
	public String getxName(){
		return name;
	}
		
	public String getxProtocol(){
		return protocol;
	}
		
	public String getxUri(){
		return uri;
	}
		
	public Map<String, QualityProperties> getxQualityProperties(){
		return qualityProperties;
	}
		
	public boolean getxHasQualityProperties(){
		return hasQualityProperties;
	}
	
	public Map<String, NonFuncDescription> getxNonFuncDescriptions(){
		return nonFuncDescriptions;
	}
		
	public String getxD(){
		return d;
	}
		
	public String getxR(){
		return R;
	}
		
	public String getxConstraintViolation(){
		return constraintViolation;
	}
		
	public boolean getxHasConstraintViolation(){
		return hasConstraintViolation;
	}
		
	public int getxDBKey(){
		return dbKey;
	}
		
}
