package org.ow2.choreos.serviceRepresentation.functional;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import javax.swing.tree.DefaultMutableTreeNode;

import org.omg.CORBA.IntHolder;
import org.ow2.choreos.dataStructures.map.mapManipulation.MapGetter;
import org.ow2.choreos.gui.PrintMessages;
import org.ow2.choreos.parser.functionalDocuments.objectConstraintsManipulation.ObjectConstraintsManipulation;
import org.ow2.choreos.serviceRepresentation.nonFunctional.NonFuncDescription;
import org.ow2.choreos.serviceRepresentation.nonFunctional.QualityProperties;


/**
 * 
 * @author dathanas
 *
 */
public class ServiceInterface implements java.io.Serializable{


	//Variables.
	private static final long serialVersionUID = 2447804158911166383L;

	private String name;

	private String serviceTypeName;

	private List<Operation> operations;
	private boolean hasOperations;

	private boolean hasUnconnectedOperations;

	private Map<String, Boolean> hasTransformedOperations;//The String is the representation model.

	private List<ServiceInstance> serviceInstances;

	private String p;//Optional: operation profile.

	private String b;//Optional: behavioral specification.
	private boolean hasBehavioralSpecification;

	private String C;//Optional: a set of constraints.

	private String constraintViolation;
	private boolean hasConstraintViolation;

	private int dbKey;

	private int dbAbstractionKey;

	private boolean concrete; //actual (true) or representative (false)


	//Constructor.
	public ServiceInterface( String cName ){

		name = cName;

		hasOperations = false;

		hasUnconnectedOperations = false;

		hasBehavioralSpecification = false;

		hasConstraintViolation = false;

		concrete = true;


		PrintMessages.print( PrintMessages.wsdlPrint, hasConstraintViolation(), this );
	}
	
	
	//Empty constructor used for xml transformation
	public ServiceInterface(){
		
	}
	
	public ServiceInterface(ServiceInterface serviceInterface){
		this.name = serviceInterface.getName();
		this.serviceTypeName = serviceInterface.getServiceTypeName();
		this.operations = serviceInterface.getOperations();
		this.hasOperations = serviceInterface.getxHasOperations();
		this.hasUnconnectedOperations = serviceInterface.hasUnconnectedOperations();
		this.hasTransformedOperations = serviceInterface.getxHasTransformedOperations();
		this.serviceInstances = serviceInterface.getServiceInstances();
		this.p = serviceInterface.getProfile();
		this.b = serviceInterface.getBehavioralSpecification();
		this.hasBehavioralSpecification = serviceInterface.getxHasBehavioralSpecification();
		this.C = serviceInterface.getConstraints();
		this.constraintViolation = serviceInterface.getConstraintViolation();
		this.hasConstraintViolation = serviceInterface.hasConstraintViolation();
		this.dbKey = serviceInterface.getDBKey();
		this.dbAbstractionKey = serviceInterface.getDBAbstractionKey();
		this.concrete = serviceInterface.isConcrete();
	}


	//Setters.
	public String setConstraintViolation( String cConstraintViolation ){

		if( ! hasConstraintViolation && cConstraintViolation != null ){

			constraintViolation = cConstraintViolation;

			hasConstraintViolation = true;
		}

		return constraintViolation;
	}

	public void setDBKey( int cDBKey ){

		dbKey = cDBKey;
	}
	
	public void setConcrete( boolean cIsConcrete ){

		concrete = cIsConcrete;
	}

	public void setDBAbstractionKey( int cDBKey ){

		dbAbstractionKey = cDBKey;
	}

	public void setServiceTypeName( String cServiceTypeName ){

		serviceTypeName = cServiceTypeName;
	}

	public void add( Operation operation ){

		if( ObjectConstraintsManipulation.setContainmentError( this, operation ) == null ){

			if( operations == null ){

				operations = new ArrayList<Operation>();

				hasOperations = true;
			}

			operations.add( operation );

			hasUnconnectedOperations = true;
		}
	}

	void connect( Map<String, String> namespaceQualifiers, String docTargetNamespace, Map<String, String> qualifierNamespaces, Map<String, Object> indexGlobalComponentsWithoutTargetNamespaces, Map<String, Object> indexGlobalComponentsWithTargetNamespaces, String schemaRepresentationModel, String schemaDefinitionNamespace ){

		PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "START - connect: " + getTag() + " - hasUnconnectedOperations= " + hasUnconnectedOperations + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel + " - schemaDefinitionNamespace= " + schemaDefinitionNamespace );

		if( hasUnconnectedOperations && ! hasConstraintViolation ){

			for( int i = 0; hasOperations && i < operations.size(); ++i ){


				operations.get( i ).connect( namespaceQualifiers, docTargetNamespace, qualifierNamespaces, indexGlobalComponentsWithoutTargetNamespaces, indexGlobalComponentsWithTargetNamespaces, schemaRepresentationModel, schemaDefinitionNamespace );

				if( operations.get( i ).hasUnconnectedMessages() ) return;
			}

			hasUnconnectedOperations = false;
		}

		PrintMessages.print( PrintMessages.connectionWSDLWithXMLPrint, "END - connect: " + getTag() + " - hasUnconnectedOperations= " + hasUnconnectedOperations + " hasConstraintViolation= " + hasConstraintViolation + " - schemaRepresentationModel= " + schemaRepresentationModel + " - schemaDefinitionNamespace= " + schemaDefinitionNamespace );
	}

	public void transform( String targetModel ){

		PrintMessages.print( PrintMessages.transformationPrint, "START - transform: " + getTag() + " - hasTransformedOperations= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedOperations ) + " hasConstraintViolation= " + hasConstraintViolation );

		if( ! MapGetter.getBooleanValueByKey( targetModel, hasTransformedOperations ) && ! hasConstraintViolation ){

			for( int i = 0; hasOperations && i < operations.size(); ++i ){


				operations.get( i ).transform( targetModel );


				if( ! operations.get( i ).hasTransformedMessages( targetModel ) ) return;
			}

			if( hasTransformedOperations == null ) hasTransformedOperations = new Hashtable<String, Boolean>();

			if(targetModel != null) hasTransformedOperations.put( targetModel, true );
		}

		PrintMessages.print( PrintMessages.transformationPrint, "END -transform: " + getTag() + " - hasTransformedOperations= " + MapGetter.getBooleanValueByKey( targetModel, hasTransformedOperations ) + " hasConstraintViolation= " + hasConstraintViolation );
	}

	public void addServiceInstance(ServiceInstance interfaceInstance){

		if( serviceInstances == null ) serviceInstances = new ArrayList<ServiceInstance>();

		serviceInstances.add(interfaceInstance);
	}

	void set( NonFuncDescription nonFuncDescription, QualityProperties qualityProperties ){

		if( nonFuncDescription.getServiceInstanceName() != null ){

			ServiceInstance serviceInstance = lookUp( nonFuncDescription.getServiceInstanceName() );

			if( serviceInstance != null ) serviceInstance.set( nonFuncDescription, qualityProperties );
		}

		else addInAllServiceInstances( nonFuncDescription, qualityProperties );
	}

	private void addInAllServiceInstances( NonFuncDescription nonFuncDescription, QualityProperties qualityProperties){

		for( int i = 0; serviceInstances != null && i < serviceInstances.size(); ++i ) serviceInstances.get(i).set( nonFuncDescription, qualityProperties ); 
	}

	public void seteBehavioralSpecification( String behavioralSpecification ){

		if( behavioralSpecification != null ){

			b = behavioralSpecification;

			hasBehavioralSpecification = true;
		}
	}


	//Getters.
	public boolean hasConstraintViolation(){

		return hasConstraintViolation;
	}

	public String getConstraintViolation(){

		return constraintViolation;
	}

	public boolean hasFinalConstraintViolation(){

		return getFinalConstraintViolation() != null ? true : false;
	}

	public String getFinalConstraintViolation(){//It accumulates the constraints that are not called by the constructor.

		if( hasConstraintViolation ) return constraintViolation;


		//I did not find any 'checkConstraint()' method.


		for( int i = 0; operations != null && i < operations.size(); ++i ) if( operations.get( i ).hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, operations.get( i ) );

		for( int i = 0; serviceInstances != null && i < serviceInstances.size(); ++i ) if( serviceInstances.get( i ).hasFinalConstraintViolation() ) ObjectConstraintsManipulation.setContainmentError( this, serviceInstances.get( i ) );


		return constraintViolation;
	}

	boolean hasUnconnectedOperations(){

		return hasUnconnectedOperations;
	}

	boolean hasTransformedOperations( String targetModel ){

		return MapGetter.getBooleanValueByKey( targetModel, hasTransformedOperations );
	}

	public int getDBKey(){

		return dbKey;
	}
	
	public boolean isConcrete(){

		return concrete;
	}

	public int getDBAbstractionKey(){

		return dbAbstractionKey;
	}
	
	public String getName(){

		return name;
	}

	public String getServiceTypeName(){

		return serviceTypeName;
	}

	public String getProfile(){

		return p;
	}

	public String getBehavioralSpecification(){

		return b;
	}

	public String getConstraints(){

		return C;
	}

	int getNumOfInstances(){

		if( serviceInstances == null ) return 0;

		return serviceInstances.size();
	}

	int getNumOfInstancesWithNonFunctionalDescription( String ref2Source ){

		int num = 0;

		for( int i = 0; serviceInstances != null && i < serviceInstances.size(); ++i ) if( serviceInstances.get( i ).hasNonFunctionalDescription( ref2Source ) ) ++num;

		return num;
	}

	public boolean hasServiceInstances(){

		if( serviceInstances != null ) return true;

		return false;
	}

	public String getTag(){

		return "service interface name= '" + name + "'";
	}

	String getRef2SourceOfServiceInstance( String ref2Source, ServiceInstance serviceInstance ){

		for( int i = 0; serviceInstances != null && i < serviceInstances.size(); ++i )

			if( serviceInstances.get(i).equals( serviceInstance ) ) return ref2Source;

		return null;
	}

	ServiceInterface getInterface( ServiceInstance serviceInstance ){

		for( int i = 0; serviceInstances != null && i < serviceInstances.size(); ++i )

			if( serviceInstances.get(i).equals( serviceInstance ) ) return this;

		return null;
	}

	public List<Operation> getOperations(){

		return operations;
	}

	public List<ServiceInstance> getServiceInstances(){

		return serviceInstances;
	}

	private ServiceInstance lookUp( String serviceInstanceName ){

		for( int i = 0; serviceInstances != null && i < serviceInstances.size(); ++i ) if( serviceInstanceName != null && serviceInstances.get(i).getName().equals( serviceInstanceName ) ) return serviceInstances.get(i);


		return null;
	}


	//Non-functional descriptions.
	QualityProperties getQualityProperties( String ref2Source ){

		for( int i = 0; serviceInstances != null && i < serviceInstances.size(); )

			return serviceInstances.get( i).getQualityProperties( ref2Source );

		return null;
	}

	Boolean existNonFuncDescriptions(){

		for( int i = 0; serviceInstances != null && i < serviceInstances.size(); ++i )

			if( serviceInstances.get(i).existNonFuncDescriptions() ) return true;

		return false;
	}

	//I constructed the following two functions in order to visualize the service instance in a non-functional abstraction! 
	DefaultMutableTreeNode visualize( Integer label, ServiceInstance serviceInstance, String ref2Source, DefaultMutableTreeNode node ){//This function is dedicated for visualizing non-functional hierarchy!

		for( int k = 0; k < serviceInstances.size(); ++k )

			if( serviceInstances.get(k).equals( serviceInstance) ){

				DefaultMutableTreeNode child1 = new DefaultMutableTreeNode( "Service instance (" + label + "): " + serviceInstances.get(k).getName());
				node.add( child1 );

				child1 = serviceInstances.get( k ).visualize( ref2Source, child1 );//Visualizing the non-functional description!

				DefaultMutableTreeNode i = new DefaultMutableTreeNode("Service interface: " + name);
				child1.add( i );

			   	for( int m = 0; m < operations.size(); m++){

		    		label = m + 1;

		   			DefaultMutableTreeNode operation = operations.get( m ).visualize( label );

		   			i.add(operation);
		   		}
			}

    	return node;
	}

	DefaultMutableTreeNode visualizeSI( String schemaRepresentatiobModel, DefaultMutableTreeNode node ){//TODO: to be completed.

		DefaultMutableTreeNode i = new DefaultMutableTreeNode("Service interface: " + name);

	   	if( node == null) node = i;
	   	else node.add( i );

	   	/*for( int k = 0; k < operations.size(); k++){

    		Integer label = k + 1;

   			DefaultMutableTreeNode operation = operations.get(k).visualize( label, structured );

   			i.add(operation);
   		}*/

    	return node;
	}


	protected DefaultMutableTreeNode visualize( Integer label, String ref2Source, String endOfPath ){//This function is used by the 'Visualizer' class.

		DefaultMutableTreeNode interfaceNode = null; 

		if( endOfPath != null && ( endOfPath.equals("Service Interfaces") || endOfPath.equals("Service Instances") || endOfPath.equals("Service Operations") || endOfPath.equals("Service Messages") ) ){

			interfaceNode = new DefaultMutableTreeNode("Service interface(" + label + "): " + name);

			if( hasBehavioralSpecification ){

				DefaultMutableTreeNode behavioralNode = new DefaultMutableTreeNode( "Behavioral specification:" );

				behavioralNode.add( new DefaultMutableTreeNode( b ) );

				interfaceNode.add( behavioralNode );
			}
		}

		if( endOfPath != null && endOfPath.equals("Service Instances") ){

			for( int m = 0; m < serviceInstances.size(); ++m ){

				int instanceLabel = m + 1;

				DefaultMutableTreeNode instanceNode = serviceInstances.get( m ).visualize( instanceLabel, ref2Source);
				interfaceNode.add( instanceNode );
			}
		}

		if( endOfPath != null && ( endOfPath.equals("Service Operations") || endOfPath.equals("Service Messages") ) ){

			for( int m = 0; m < operations.size(); m++ ){

				int operationLabel = m + 1;

				DefaultMutableTreeNode operation = operations.get( m ).visualize( operationLabel, endOfPath );
				interfaceNode.add(operation);
			}
		}

    	return interfaceNode;
	}

	protected DefaultMutableTreeNode visualize( Integer label, String ref2Source ){//This function is used by the 'Visualizer' class.

		if( hasConstraintViolation() ) return null;


		DefaultMutableTreeNode interfaceNode = new DefaultMutableTreeNode("Service interface(" + label + "): " + name);


		for( int m = 0; m < serviceInstances.size(); ++m ){

			int instanceLabel = m + 1;

			DefaultMutableTreeNode instanceNode = serviceInstances.get( m ).visualize( instanceLabel, ref2Source);

			interfaceNode.add( instanceNode );
		}

		for( int m = 0; m < operations.size(); m++ ){

			int operationLabel = m + 1;

			DefaultMutableTreeNode operation = operations.get( m ).visualize( operationLabel );

			interfaceNode.add(operation);
		}


    	return interfaceNode;
	}

	DefaultMutableTreeNode visualize( IntHolder labelOfInstances, String ref2Source, DefaultMutableTreeNode node ){

		DefaultMutableTreeNode serviceInstance = null;
		DefaultMutableTreeNode i = null;

		for( int k = 0; k < serviceInstances.size(); ++k ){

			if( labelOfInstances != null)
				serviceInstance = new DefaultMutableTreeNode( "Service instance (" + ++labelOfInstances.value + "): " + serviceInstances.get(k).getName() );

			else
				serviceInstance = new DefaultMutableTreeNode( "Service instance: " + serviceInstances.get(k).getName() );

			node.add( serviceInstance );

			i = new DefaultMutableTreeNode("Service Interface: " + name);
	   		serviceInstance.add(i);
		}

    	return node;
	}

	public DefaultMutableTreeNode visualize( Integer label, DefaultMutableTreeNode node ){//This is the case of visualizing the interface of a represented service in a functional hierarchy.

		DefaultMutableTreeNode i = new DefaultMutableTreeNode("Service interface (" + label + "): " + name);
	   	node.add( i );

    	return node;
	}

	public DefaultMutableTreeNode visualize( Integer label, String schemaRepresentationModel, DefaultMutableTreeNode node ){//TODO: to be implemented: This is the case of visualizing the interface of a represented service in a functional hierarchy but with the whole structure.

		DefaultMutableTreeNode i = new DefaultMutableTreeNode("Service interface (" + label + "): " + name);

	   	if( node == null) node = i;

	   	else node.add( i );

	   	/*for( int k = 0; k < operations.size(); k++){

    		label = k + 1;

   			DefaultMutableTreeNode operation = operations.get(k).visualize( label, schemaRepresentationModel );

   			i.add(operation);
   		}*/

    	return node;
	}

	public DefaultMutableTreeNode visualize( DefaultMutableTreeNode node ){

		DefaultMutableTreeNode i = new DefaultMutableTreeNode( "Abstraction interface: " + name );

		node.add( i );


		for( int k = 0; operations != null && k < operations.size(); k++){

    		int label = k + 1;

   			DefaultMutableTreeNode operation = operations.get(k).visualize( label );

   			i.add(operation);
   		}

    	return node;
	}
	
	
	//------------------------------------------------------------------------
	public DefaultMutableTreeNode visualizeForQueryResults( Integer label, String ref2Source ){//This function is used by the 'Visualizer' class.

		if( hasConstraintViolation() ) return null;

		DefaultMutableTreeNode interfaceNode = new DefaultMutableTreeNode("Service interface(" + label + "): " + name);

		if( hasBehavioralSpecification ){

			DefaultMutableTreeNode behavioralNode = new DefaultMutableTreeNode( "Behavioral specification:" );

			behavioralNode.add( new DefaultMutableTreeNode( b ) );

			interfaceNode.add( behavioralNode );
		}
		
		if (serviceInstances != null) {
			for( int m = 0; m < serviceInstances.size(); ++m ){

				int instanceLabel = m + 1;

				DefaultMutableTreeNode instanceNode = serviceInstances.get( m ).visualizeForQueryResults( instanceLabel, ref2Source);

				interfaceNode.add( instanceNode );
			}
		}
				
		for( int m = 0; m < operations.size(); m++ ){

			int operationLabel = m + 1;

			DefaultMutableTreeNode operation = operations.get( m ).visualizeForQueryResults( operationLabel );

			interfaceNode.add(operation);
		}

    	return interfaceNode;
	}
	
	
	public DefaultMutableTreeNode visualizeForQueryResults2( DefaultMutableTreeNode node ){

		DefaultMutableTreeNode i = new DefaultMutableTreeNode( "Abstraction interface: " + name );

		node.add( i );

		if( hasBehavioralSpecification ){

			DefaultMutableTreeNode behavioralNode = new DefaultMutableTreeNode( "Behavioral specification:" );

			behavioralNode.add( new DefaultMutableTreeNode( b ) );

			i.add( behavioralNode );
		}

		for( int k = 0; k < operations.size(); k++){

    		int label = k + 1;

   			DefaultMutableTreeNode operation = operations.get(k).visualizeForQueryResults2( label );

   			i.add(operation);
   		}

    	return node;
	}
	//------------------------------------------------------------------------
	
	
	//Setters used for xml transformation
	
	public void setxName( String cName ){
		name = cName;
	}
		
	public void setxServiceTypeName( String cServiceTypeName ){
		serviceTypeName = cServiceTypeName;
	}
		
	public void setxOperations( List<Operation> cOperations ){
		operations = cOperations;
	}
		
	public void setxHasOperations( boolean cHasOperations ){
		hasOperations = cHasOperations;
	}
		
	public void setxHasUnconnectedOperations( boolean cHasUnconnectedOperations ){
		hasUnconnectedOperations = cHasUnconnectedOperations;
	}
		
	public void setxHasTransformedOperations( Map<String, Boolean> cHasTransformedOperations ){
		hasTransformedOperations = cHasTransformedOperations;
	}
		
	public void setxServiceInstances( List<ServiceInstance> cServiceInstances ){
		serviceInstances = cServiceInstances;
	}
		
	public void setxP( String cP ){
		p = cP;
	}
		
	public void setxB( String cB ){
		b = cB;
	}
		
	public void setxHasBehavioralSpecification( boolean cHasBehavioralSpecification ){
		hasBehavioralSpecification = cHasBehavioralSpecification;
	}
		
	public void setxC( String cC ){
		C = cC;
	}
		
	public void setxConstraintViolation( String cConstraintViolation ){
		constraintViolation = cConstraintViolation;
	}
		
	public void setxHasConstraintViolation( boolean cHasConstraintViolation ){
		hasConstraintViolation = cHasConstraintViolation;
	}
		
	public void setxDBKey( int cDBKey ){
		dbKey = cDBKey;
	}
		
	public void setxDBAbstractionKey( int cDBAbstractionKey ){
		dbAbstractionKey = cDBAbstractionKey;
	}
		
	public void setxConcrete( boolean cConcrete ){
		concrete = cConcrete;
	}
		
	//Getters used for xml transformation
		
	public String getxName(){
		return name;
	}
		
	public String getxServiceTypeName(){
		return serviceTypeName;
	}
		
	public List<Operation> getxOperations(){
		return operations;
	}
		
	public boolean getxHasOperations(){
		return hasOperations;
	}
		
	public boolean getxHasUnconnectedOperations(){
		return hasUnconnectedOperations;
	}
		
	public Map<String, Boolean> getxHasTransformedOperations(){
		return hasTransformedOperations;
	}
		
	public List<ServiceInstance> getxServiceInstances(){
		return serviceInstances;
	}
		
	public String getxP(){
		return p;
	}
		
	public String getxB(){
		return b;
	}
		
	public boolean getxHasBehavioralSpecification(){
		return hasBehavioralSpecification;
	}
		
	public String getxC(){
		return C;
	}
		
	public String getxConstraintViolation(){
		return constraintViolation;
	}
		
	public boolean getxHasConstraintViolation(){
		return hasConstraintViolation;
	}
		
	public int getxDBKey(){
		return dbKey;
	}
		
	public int getxDBAbstractionKey( int cDBAbstractionKey ){
		return dbAbstractionKey = cDBAbstractionKey;
	}
		
	public boolean getxConcrete( boolean cConcrete ){
		return concrete = cConcrete;
	}
}


