package org.ow2.choreos.serviceSourcesPreProcessing;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.swing.InputVerifier;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.JSlider;
import javax.swing.SwingConstants;
import javax.swing.JProgressBar;

import org.ow2.choreos.parser.freeTextDocuments.Tokenizer;
import org.ow2.choreos.abstractionDrivenServiceOrganization.distanceCalculator.names.NamesDistanceCalculator;
import java.awt.FlowLayout;

public class ServiceSourcesPreProcessor extends JFrame {

	private JPanel contentPane;
	private JPanel preferencesPanel;
	private JButton browseButton;
	private JButton startButton;
	private JButton selectODirButton;
	private JTextField bucketSizeField;
	private JSlider similaritySlider;
	private JLabel directoryLabel;
	private JLabel sliderValueLabel;
	private JLabel bucketSizeLabel;
	private JLabel similarityThresholdLabel;
	private JLabel outputDirectoryLabel;
	private JProgressBar progressBar;

	private File sourceDirectory;
	private String outputDirPath;
	private String outputParentDirPath = "src/main/resources/WSDLv1.1/ServiceSourcesPreprocessing";
	private ArrayList<Bucket> buckets;
	private ArrayList<Bucket> closedBuckets;
	private int maxBucketSize;
	private int bucketCount;
	private double similarityThreshold;
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					ServiceSourcesPreProcessor frame = new ServiceSourcesPreProcessor();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the frame.
	 */
	public ServiceSourcesPreProcessor() {
		setTitle("AoSBM - Service Sources Pre Processing");
		
		maxBucketSize = 5;
		similarityThreshold = 0.5;
		
		initComponets();
	}

	private void initComponets() {
		setMinimumSize(new Dimension(600, 300));
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 300, 480);
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		contentPane.setLayout(new BorderLayout(0, 0));
		setContentPane(contentPane);
		
		JPanel dirSelectionPanel = new JPanel();
		dirSelectionPanel.setBorder(new EtchedBorder());
		GridBagLayout gbl_dirSelectionPanel = new GridBagLayout();
		gbl_dirSelectionPanel.columnWidths = new int[]{134, 75, 0};
		gbl_dirSelectionPanel.rowHeights = new int[]{23, 0};
		gbl_dirSelectionPanel.columnWeights = new double[]{1.0, 0.0, Double.MIN_VALUE};
		gbl_dirSelectionPanel.rowWeights = new double[]{0.0, Double.MIN_VALUE};
		dirSelectionPanel.setLayout(gbl_dirSelectionPanel);
		
		
		directoryLabel = new JLabel("");
		directoryLabel.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				browseActionPerfomed();
			}
		});
		directoryLabel.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		directoryLabel.setForeground(Color.GRAY);
		directoryLabel.setFont(new Font("Segoe UI Symbol", Font.PLAIN, 12));
		directoryLabel.setBorder(new LineBorder(Color.LIGHT_GRAY));
		GridBagConstraints gbc_selectedDirLabel = new GridBagConstraints();
		gbc_selectedDirLabel.fill = GridBagConstraints.BOTH;
		gbc_selectedDirLabel.insets = new Insets(0, 0, 0, 5);
		gbc_selectedDirLabel.gridx = 0;
		gbc_selectedDirLabel.gridy = 0;
		dirSelectionPanel.add(directoryLabel, gbc_selectedDirLabel);
		
		contentPane.add(dirSelectionPanel, BorderLayout.NORTH);
		
		browseButton = new JButton("Browse..");
		directoryLabel.setLabelFor(browseButton);
		browseButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				browseActionPerfomed();
			}
		});
		GridBagConstraints gbc_browseButton = new GridBagConstraints();
		gbc_browseButton.fill = GridBagConstraints.VERTICAL;
		gbc_browseButton.anchor = GridBagConstraints.EAST;
		gbc_browseButton.gridx = 1;
		gbc_browseButton.gridy = 0;
		dirSelectionPanel.add(browseButton, gbc_browseButton);
		
		preferencesPanel = new JPanel();
		preferencesPanel.setBorder(new EtchedBorder(EtchedBorder.LOWERED, null, null));
		contentPane.add(preferencesPanel, BorderLayout.CENTER);
		
		bucketSizeLabel = new JLabel("Max bucket Size(# of services):");
		
		bucketSizeField = new JTextField(maxBucketSize + "");
		bucketSizeField.setInputVerifier(new InputVerifier() {
			
			@Override
			public boolean verify(JComponent input) {
				try{
					maxBucketSize = Integer.parseInt(((JTextField)input).getText());
					return true;
				}
				catch(NumberFormatException e){
					JOptionPane.showMessageDialog(getContentPane(), "Maximum bucket size must be an integer number!");
					((JTextField)input).setText(maxBucketSize + "");
					return false;
				}
			}
		});
		bucketSizeField.setColumns(10);
		
		similarityThresholdLabel = new JLabel("Similarity Threshold:");
		
		similaritySlider = new JSlider();
		Hashtable<Integer,JLabel> labelTable = new Hashtable<Integer,JLabel>();
		labelTable.put(0,new JLabel("0"));
		labelTable.put(100,new JLabel("1"));
		similaritySlider.setPaintLabels(true);
		similaritySlider.setMajorTickSpacing(10);
		similaritySlider.setLabelTable(labelTable);
		similaritySlider.setValue((int)(similarityThreshold*100));
		similaritySlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				similaritySliderStateChanged(e);
			}
		});
		
		sliderValueLabel = new JLabel(similarityThreshold + "");
		sliderValueLabel.setHorizontalAlignment(SwingConstants.CENTER);
		sliderValueLabel.setBorder(new LineBorder(Color.LIGHT_GRAY));
		
		startButton = new JButton("Start");
		startButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				startButtonActionPerfomed();
			}
		});
		
		outputDirectoryLabel = new JLabel("Output directory:");
		outputDirectoryLabel.setFont(new Font("Dialog", Font.PLAIN, 12));
		
		selectODirButton = new JButton("Select output directory");
		selectODirButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser fileChooser = new JFileChooser( "src/main/resources/" );
				fileChooser.setDialogTitle("Choose an output directory");
				fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				if(fileChooser.showDialog(getContentPane(), "Open") == JFileChooser.APPROVE_OPTION){
					outputParentDirPath = fileChooser.getSelectedFile().getAbsolutePath();
				}
			}
		});
		selectODirButton.setFont(new Font("Dialog", Font.BOLD, 10));
		
		GroupLayout gl_preferencesPanel = new GroupLayout(preferencesPanel);
		gl_preferencesPanel.setHorizontalGroup(
			gl_preferencesPanel.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_preferencesPanel.createSequentialGroup()
					.addContainerGap()
					.addGroup(gl_preferencesPanel.createParallelGroup(Alignment.LEADING)
						.addGroup(gl_preferencesPanel.createParallelGroup(Alignment.LEADING)
							.addGroup(gl_preferencesPanel.createSequentialGroup()
								.addGroup(gl_preferencesPanel.createParallelGroup(Alignment.LEADING)
									.addGroup(gl_preferencesPanel.createSequentialGroup()
										.addGroup(gl_preferencesPanel.createParallelGroup(Alignment.TRAILING)
											.addComponent(bucketSizeLabel)
											.addComponent(similarityThresholdLabel))
										.addPreferredGap(ComponentPlacement.RELATED)
										.addGroup(gl_preferencesPanel.createParallelGroup(Alignment.TRAILING)
											.addComponent(bucketSizeField, GroupLayout.DEFAULT_SIZE, 365, Short.MAX_VALUE)
											.addGroup(gl_preferencesPanel.createSequentialGroup()
												.addComponent(similaritySlider, GroupLayout.DEFAULT_SIZE, 303, Short.MAX_VALUE)
												.addPreferredGap(ComponentPlacement.RELATED)
												.addComponent(sliderValueLabel, GroupLayout.PREFERRED_SIZE, 50, GroupLayout.PREFERRED_SIZE))))
									.addComponent(startButton, Alignment.TRAILING))
								.addContainerGap())
							.addGroup(gl_preferencesPanel.createSequentialGroup()
								.addComponent(outputDirectoryLabel, GroupLayout.DEFAULT_SIZE, 554, Short.MAX_VALUE)
								.addContainerGap()))
						.addGroup(Alignment.TRAILING, gl_preferencesPanel.createSequentialGroup()
							.addComponent(selectODirButton)
							.addContainerGap())))
		);
		gl_preferencesPanel.setVerticalGroup(
			gl_preferencesPanel.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_preferencesPanel.createSequentialGroup()
					.addContainerGap()
					.addGroup(gl_preferencesPanel.createParallelGroup(Alignment.BASELINE)
						.addComponent(bucketSizeLabel)
						.addComponent(bucketSizeField, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addGroup(gl_preferencesPanel.createParallelGroup(Alignment.LEADING, false)
						.addComponent(sliderValueLabel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
						.addComponent(similarityThresholdLabel)
						.addComponent(similaritySlider, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addComponent(startButton)
					.addPreferredGap(ComponentPlacement.RELATED, 218, Short.MAX_VALUE)
					.addComponent(selectODirButton)
					.addPreferredGap(ComponentPlacement.RELATED)
					.addComponent(outputDirectoryLabel)
					.addContainerGap())
		);
		preferencesPanel.setLayout(gl_preferencesPanel);
		
		JPanel bottomPanel = new JPanel();
		contentPane.add(bottomPanel, BorderLayout.SOUTH);
		bottomPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 5));
		
		progressBar = new JProgressBar();
		bottomPanel.add(progressBar);
		
		enablePreferencesComponents(false);
	}

	protected void startButtonActionPerfomed() {		
		buckets = new ArrayList<Bucket>();
		closedBuckets = new ArrayList<Bucket>();
		bucketCount = 0;

		outputDirectoryLabel.setText("Output directory: ");
		enablePreferencesComponents(false);
		progressBar.setIndeterminate(true);
		
		new Thread(new Runnable(){
			public void run(){
				outputDirPath = outputParentDirPath + "\\" + sourceDirectory.getName() + ".output";
				if (Files.exists(Paths.get(outputDirPath))) {
					String tempPath = outputDirPath;
					int count = 0;
					do {
						tempPath = outputDirPath + ++count;System.out.println(tempPath);
					} while (Files.exists(Paths.get(tempPath)));
					outputDirPath = tempPath;
				}
				try {
					Files.createDirectories(Paths.get(outputDirPath));
					
					traverse(sourceDirectory);
					
					for(Bucket bucket:closedBuckets){
						bucket.applyRepresentative();
					}
					for(Bucket bucket:buckets){
						bucket.applyRepresentative();
					}
				} catch (IOException e) {
					e.printStackTrace();
				} finally {
					progressBar.setIndeterminate(false);
					enablePreferencesComponents(true);
					outputDirectoryLabel.setText("Output directory: " + outputDirPath);
				}
			}
		}).start();
	}
	
	private void traverse(File directory) throws IOException{
		if(directory.listFiles() != null){
			int length = directory.listFiles().length;
			for(int i = 0;i<length;i++){
				File file = directory.listFiles()[i];
				if(file.isFile()){
					String extension = Tokenizer.findExtension(file.getName());
					if(extension.equals("wsdl")){
						Bucket bucket = getBucketFor(file);
						moveFile(file.getAbsolutePath(), bucket.getPath()+"/"+file.getName());
					}
				}
				else if( file.isDirectory()){
					traverse(file);
				}
			}
		}
	}
	
	private void moveFile(String sourcePath, String targetPath) throws IOException{
		Path source = Paths.get(sourcePath);
		Path target = Paths.get(targetPath);
		if(Files.exists(target)){
			String name = Tokenizer.getPrefix(targetPath, ".");
			String extension = Tokenizer.findExtension(targetPath);
			target = Paths.get(name + "." + System.currentTimeMillis() + "." + extension);
		}
		//TODO: Move file and not copy
		Files.copy(source, target);
		
	}
	
	private Bucket getBucketFor(File file) throws IOException{
		double minDistance = Double.MAX_VALUE;
		Bucket minBucket = null;
		String fileName = Tokenizer.getPrefix(file.getName(), ".");
		/* 'buckets' list is surely not null(since it was instantiated beforehand)!
		 * Thus the use of 'for each' loop*/
		for(Bucket bucket:buckets){
			double distance = NamesDistanceCalculator.calculate("EditDistanceCalculator", bucket.getRepresentative(), fileName);
			if(distance < minDistance){
				minDistance = distance;
				minBucket = bucket;
			}
		}
		if(minBucket == null || minDistance > similarityThreshold){
			return createBucket(file);
		}
		else {
			minBucket.setRepresentative(NamesDistanceCalculator.extractName("EditDistanceCalculator", new String[]{minBucket.getRepresentative(),fileName}));
			if(minBucket.getNumOfFiles() >= maxBucketSize){
				buckets.remove(minBucket);
				closedBuckets.add(minBucket);
			}
			return minBucket;
		}
	}
	
	
	private Bucket createBucket(File file){
		Bucket bucket = new Bucket(outputDirPath, Tokenizer.getPrefix(file.getName(),"."),bucketCount++);
		buckets.add(bucket);
		return bucket;
	}
	
	protected void similaritySliderStateChanged(ChangeEvent e) {
		if(similaritySlider.getValueIsAdjusting()){
			similarityThreshold= (double)similaritySlider.getValue()/100;
			sliderValueLabel.setText( similarityThreshold + "");
		}
	}

	
	protected void browseActionPerfomed() {
		JFileChooser fileChooser = new JFileChooser( "src/main/resources" );
		fileChooser.setDialogTitle("Choose a source directory");
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		if(fileChooser.showDialog(this, "Open") == JFileChooser.APPROVE_OPTION){
			sourceDirectory = fileChooser.getSelectedFile();
			
			directoryLabel.setText(sourceDirectory.getAbsolutePath());
			
			enablePreferencesComponents(true);
		}
	}
	
	private void enablePreferencesComponents(boolean enable){
		bucketSizeField.setEnabled(enable);
		similaritySlider.setEnabled(enable);
		sliderValueLabel.setEnabled(enable);
		similarityThresholdLabel.setEnabled(enable);
		bucketSizeLabel.setEnabled(enable);
		startButton.setEnabled(enable);
		outputDirectoryLabel.setEnabled(enable);
		selectODirButton.setEnabled(enable);
	}
}
