import org.ow2.choreos.executable_service_discovery.DummyActiveRegistry.Registry;
import org.ow2.choreos.executable_service_discovery.plugin_manager.PluginManager;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.slf4j.*;

/**
 * @author Kostas Lekkas -- lekkas@vtrip.net
 */

/**
 * Use case test for the Plugin Manager, using
 * dummy implementations for AoSBM, Service Registry
 * and Service Discovery Plugin.
 *  
 */

public class Main {
    private static Logger logger;
    private static String RegistryServicesDirPath;
    
    public static void main(String args[]) {
    	ExecutorService exec = Executors.newSingleThreadExecutor();
    	Iterator<String> it;
        LinkedList<String> list;
        PluginManager pluginmanager;
        logger = LoggerFactory.getLogger(Main.class);
        
        /*
    	if(args.length != 2) {
    		System.out.println("Usage: pluginmanagertest <PluginConfigurationFile> <RegistryServicesDirPath");
    		return;
    	}
    	*/
    	// RegistryServicesDirPath = args[1];
    	RegistryServicesDirPath = "dummy_registry_services/";

    	/* First of all, start the Services Registry */
    	exec.execute(new Runnable() {
			@Override
			public void run() {
				Registry.getInstance().start(2000, RegistryServicesDirPath, 31337);
		    	try {
		            Registry.getInstance().getConnectionAcceptThread().join();
		        } catch(InterruptedException e) {
		            logger.error("AcceptThread interrupted");
		        }
			}
		});
    	
        //PluginManager.setConfDir(args[0]);
    	PluginManager.setConfDir("plugin_conf/plugin.properties");
        pluginmanager = PluginManager.getInstance();

        MockAoSBM aosbm = new MockAoSBM();	
        pluginmanager.setRetrieveServiceCallback(aosbm);
            
        /* Test 1: load plugins from file */
        logger.info("\n--Test 1: load plugins from file--");
        pluginmanager.addPluginsFromConf();
            
        /* Test 2: explicitly load plugin */
        logger.info("\n--Test 2: explicitly load plugin--");
        pluginmanager.addPluginType("org.ow2.choreos.executable_service_discovery.DPWSPlugin");
        
        /* Test 3: view available plugins */
        logger.info("\n--Test 3: see available plugins--");
        list = pluginmanager.getAvailablePluginTypes();
        it = list.iterator();
        while(it.hasNext())
            logger.info("Available plugin: "+(String)it.next());

        /* Test 4: view loaded plugins */
        logger.info("\n--Test 4: view loaded plugins--");
        list = pluginmanager.getLoadedPluginTypes();
        it = list.iterator();
        while(it.hasNext())
            logger.info("Loaded plugin: "+(String)it.next());

        /* Test 5: remove all plugin types; */
        /* pluginmanager.removeAllPluginTypes(); */

        /* Test 6: remove specific plugin type */
        /* pluginmanager.removePluginType("test"); */

        /* Test 7: create plugin instance */
        logger.info("\n--Test 7: create plugin instance--");
        pluginmanager.createPluginInstance("org.ow2.choreos.executable_service_discovery.DummyPlugin.DummyPlugin", "localhost:31337");
        
        /* Test 8: get list of plugin instances for plugintype */
        logger.info("\n--Test 8: get list of plugin instances for plugintype--");
        list = pluginmanager.getPluginInstancesOfType("org.ow2.choreos.executable_service_discovery.DummyPlugin.DummyPlugin");
        it = list.iterator();
        while(it.hasNext())
            logger.info("Instantiated plugin: "+it.next());
            
        try {
            Thread.sleep(10000);	/* Run for 10 seconds, then destroy dummy plugin instance */
        } catch (InterruptedException ex) {
            
        }
        /* Test 9: destroy plugin instance */
        logger.info("\n--Test 9: destroy plugin instance--");
        pluginmanager.destroyPluginInstance("org.ow2.choreos.executable_service_discovery.DummyPlugin.DummyPlugin", "localhost:31337");
        
        Registry.getInstance().stop();
        exec.shutdownNow();
        System.exit(0);
    }
}
