package org.ow2.choreos.executable_service_discovery.DummyPlugin;

import org.ow2.choreos.executable_service_discovery.plugin_manager.AbstractPlugin;
import org.ow2.choreos.executable_service_discovery.plugin_manager.ServiceFilter;
import java.io.IOException;
import java.net.Socket;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.ThreadPoolExecutor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DummyPlugin extends AbstractPlugin {

	private final String UID = DummyPlugin.class.getCanonicalName();
        
	private final char 
        SUPPORTED_DISCOVERY_MODE    = AbstractPlugin.PASSIVE_DISCOVERY_MODE;
        
    private final String REGISTRY_URI;
    private final String SHORT_NAME     = "Dummy";
    private final String LONG_NAME      = "Dummy plugin for dummy registry";
    private final String AUTHOR         = "VTRIP";
    private final String DESCRIPTION    = "";
    private char CURRENT_STATUS;
    
    private Logger logger;
    private RegistryListener listener_thread;
    private ThreadPoolExecutor exec;
    private static final int POOL_SIZE  = 1;

    private boolean started = false;
    private String addr;
    private int port;
	private Object services;
    
    public DummyPlugin(String registryURI) {
        REGISTRY_URI = registryURI;
        logger = LoggerFactory.getLogger(DummyPlugin.class);
        exec = new ScheduledThreadPoolExecutor(POOL_SIZE);
        	logger.info("plugin "+registryURI+" initialized;");

    }

    @Override
    public String getUID() { return UID; }

    @Override
    public String getShortName() { return SHORT_NAME; }

    @Override
    public String getLongName() { return LONG_NAME; }

    @Override
    public String getDescription() { return DESCRIPTION;}

    @Override
    public String getAuthor() { return AUTHOR; }

    @Override
    public char getSupportedDiscoveryMode() { return SUPPORTED_DISCOVERY_MODE; }

    @Override
    public String getRegistryURI() { return REGISTRY_URI; }
    
    @Override
    public char getStatus() { return CURRENT_STATUS; }
    

	@Override
	public void start() {
        if(started)
            return;
        started = true;

        logger.info("start called");
        String host[] = REGISTRY_URI.split(":");
        if(host.length != 2)
            logger.error("wrong addr: length = "+host.length);

        addr = host[0];
        port = Integer.parseInt(host[1]);

        listener_thread = new RegistryListener(this);
        exec.execute(listener_thread);
        CURRENT_STATUS = AbstractPlugin.STATUS_STARTED;
	}

	@Override
	public void stop() {
        if(!started)
            return;

        logger.info("stop called");
        Socket s;
        s = listener_thread.getSocket();

        try {
            if(s!= null)
                s.close();
        } catch (IOException ex) {
            logger.info((ex.getMessage()));
        }

        exec.shutdownNow();
        exec = null;
        listener_thread = null;
        started = false;
        CURRENT_STATUS = AbstractPlugin.STATUS_STOPPED;
	}

	@Override
	public void restart() {
		logger.info("restart called");
		stop();
		start();
	}
        
	@Override
    public Object retrieveServices(ServiceFilter service_filter) {
        //throw new UnsupportedOperationException("Not supported.");
		return services;
    }
	
    public String getAddr() {
        return addr;
    }
    
    public int getPort() {
        return port;
    }
    
    public ThreadPoolExecutor getThreadPoolExecutor() {
        return exec;
    }
    
    public void setListenerThread(RegistryListener listener_thread) {
        this.listener_thread = listener_thread;
    }

}
