/**
 * @author Kostas Lekkas -- lekkas@vtrip.net
 */
package org.ow2.choreos.executable_service_discovery.plugin_manager;

import org.ow2.choreos.executable_service_discovery.AoSBM.PluginManagerCallback;

public interface ManageServices {
    
    /**
     * This method call results in the (indirect and asynchronous) retrieval of 
     * all service descriptions from plugins that support active discovery mode.
     * The services must match a specific filter. 
     *
     * The filter could be based on many different variables
     * (service semantics, underlying transport protocol, upper limit to
     * the number of services returned etc)
     * 
     * In addition, the targetEnvironment parameter specifies
     * the plugin instances to which the service query
     * will be forwarded.
     * 
     * @param service_filter A service filter for the requested services
     * @param env The plugins to which the query will be forwarded
     */
    public void retrieveService(ServiceFilter service_filter, TargetEnvironment env);
    
    
    /**
     * Sets the callback for the asynchronous delivery of service descriptions 
     * to the AoSBM.
     */
    public void setRetrieveServiceCallback(PluginManagerCallback callback);
    
}
