package org.ow2.choreos.registry;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.ow2.chereos.iots.common.BorderPoints;
import org.ow2.chereos.iots.common.Location;
import org.ow2.chereos.iots.common.Point;
import org.ow2.chereos.iots.common.ServiceDescription;
import org.ow2.choreos.statistics.StatUtil;

public class RegistryMap {
	public static Set<ServiceDescription> regSet = new HashSet();
	public static HashMap<String, Long> regMap = new HashMap<String, Long>();
	// static int i = 1;
	public static long timeOfReg;

	// just send the info to the storage in every method
	// add the service desciption to the reg map
	/**
	 * The method adds a service to a registry Set by creating a
	 * ServiceDescription object. For now storage is in memory and not
	 * consistent
	 * 
	 * @param serviceAddress
	 * @param deviceId
	 * @param serviceName
	 * @param serviceType
	 * @param accuracy
	 * @param sensorType
	 * @param range
	 * @param globalRegion
	 * @param concept
	 * @param dataType
	 * @param locationX
	 * @param locationY
	 * @param registrationExpiry
	 *            : the period after the registration time to keep the service
	 *            registered
	 * @param unit
	 * @param estimatedPath
	 * @return
	 * @throws SQLException
	 */
	public static boolean add(String serviceAddress, String deviceId,
			String serviceName, String serviceType, double accuracy,
			String sensorType, double range, String globalRegion,
			String concept, String dataType, double locationX,
			double locationY, long registrationExpiry, String unit,
			String estimatedPath) throws SQLException {
		timeOfReg = new Date().getTime();

		boolean result = false;
		ServiceDescription myServiceDescription = new ServiceDescription(
				deviceId, serviceAddress, serviceName, serviceType, accuracy,
				globalRegion, range, sensorType, concept, dataType, locationX,
				locationY, unit, estimatedPath);
		try {
			for (ServiceDescription s : regSet)
				if (s.getAddress().toString().equals(serviceAddress)) {
					result = false;
					return result;
				}
			regMap.put(serviceAddress, registrationExpiry);
			result = regSet.add(myServiceDescription);
			return result;

		} catch (Exception e) {
			return false;
		}
		/*
		 * regMap.put(serviceAddress, registrationExpiry); return
		 * SQLStorage.save( serviceAddress, deviceId, serviceName, serviceType,
		 * accuracy, sensorType, range, globalRegion, concept, dataType,
		 * locationX, locationY);
		 */

	}

	/**
	 * This method is used to remove a service from the registry based on its
	 * address.
	 * 
	 * @param serviceAddress
	 * @return
	 */
	public static boolean remove(String serviceAddress) {
		int index = -1;

		for (ServiceDescription s : regSet) {
			if (s.getAddress().toString().equals(serviceAddress)) {

				try {
					regSet.remove(s);
					return true;
				} catch (Exception e) {
					return false;
				}

			}
		}
		return false;
		// return SQLStorage.removeServiceReq( serviceAddress);
	}

	/**
	 * The method returns the number of registered services per service Type,
	 * which is determined based on the concept they measure and within a
	 * specific geographical region. We provide the id of the device requesting
	 * this information so that it doesn t get counted if it had already
	 * registered
	 * 
	 * @param deviceId
	 * @param globalRegion
	 * @param concepts
	 * @return
	 * @throws SQLException
	 */
	public static HashMap<String, Double> getNumberOfNodes(String deviceId,
			String globalRegion, String[] concepts) throws SQLException {
		HashMap<String, Double> NodesPerType = new HashMap<String, Double>();
		int index = -1;
		boolean checked = false;
		for (int i = 0; i < concepts.length; i++) {
			String concept = concepts[i];
			for (int j = 0; j <= i; j++) {
				if (concept.equals(concepts[j]) && i != j) {
					checked = true;
					break;
				} else
					checked = false;
			}
			if (checked == false) {
				Iterator<ServiceDescription> iter = regSet.iterator();

				while (iter.hasNext()) {
					ServiceDescription s = (ServiceDescription) iter.next();
					if (s.getConcept().toString().equals(concept)
							&& (s.getDeviceID().equals(deviceId) == false)
							&& s.getGlobalRegion().toString()
									.equals(globalRegion)) {
						if (NodesPerType.get(concept) == null)
							NodesPerType.put(concept, 1.0);
						else
							NodesPerType.put(concept,
									NodesPerType.get(concept) + 1);

					}

				}
				if (NodesPerType.get(concept) == null) {
					NodesPerType.put(concept, 0.0);
				}
			}
		}

		return NodesPerType;
		/*
		 * return SQLStorage.findNumberOfServicesReq( deviceId, globalRegion,
		 * concepts);
		 */
	}

	/**
	 * The method finds the registered services based on their geographical
	 * location and the concept they measure which is specified by the requestor
	 * 
	 * @param globalRegion
	 * @param concepts
	 * @return
	 */
	public static Set<ServiceDescription> findDevice(String globalRegion,
			String[] concepts) {
		// cleanUp();
		Set<ServiceDescription> thisSet = new HashSet();

		for (int i = 0; i < concepts.length; i++) {
			String concept = concepts[i];
			Iterator<ServiceDescription> iter = regSet.iterator();
			while (iter.hasNext()) {

				ServiceDescription s = (ServiceDescription) iter.next();
				if (s.getConcept().toString().equals(concept)
						&& s.getGlobalRegion().toString().equals(globalRegion)) {

					thisSet.add(s);

				}
			}

		}
		return thisSet;

		/*
		 * return SQLStorage.executeLookupQueryReq( globalRegion, concepts);
		 */

	}

	/**
	 * When services register, they register for a certain period, after this
	 * period they should update the registration expiry date or else they will
	 * be removed upon receiving a lookup request
	 * 
	 * @param serviceAddress
	 * @param regExpDate
	 *            : the expiry date is a time duration
	 * @return
	 */
	public static boolean updateRegisteredService(String serviceAddress,
			long regExpDate) {
		try {
			regMap.remove(serviceAddress);
			regMap.put(serviceAddress, regExpDate);
			System.out.println(" updated service " + regMap);

		} catch (Exception e) {
			return false;
		}
		return true;

	}

	/**
	 * The method removes all registered services whose registration date
	 * expired. it is called upon receiving a lookup request
	 * 
	 */
	public static void cleanUp() {
		HashMap<String, Long> tempMap = new HashMap<String, Long>();
		tempMap.putAll(regMap);
		for (Entry<String, Long> entry : regMap.entrySet()) {
			long expDate = entry.getValue();
			if (expDate <= new Date().getTime()) {
				tempMap.remove(entry.getKey());
				// System.out.println("removed service" + regMap);
			}
		}
		regMap = tempMap;

	}

	public static HashMap<String, String> getDistribution(String globalRegion)
	// throws MatlabInvocationException, MatlabConnectionException
	{
		HashMap<String, String> distResults = new HashMap<String, String>();
		Iterator<ServiceDescription> iter = regSet.iterator();
		ArrayList<Double> xCoords = new ArrayList<Double>();
		ArrayList<Double> yCoords = new ArrayList<Double>();
		while (iter.hasNext()) {
			ServiceDescription s = (ServiceDescription) iter.next();
			if (s.getGlobalRegion().toString().equals(globalRegion)) {
				xCoords.add(s.getLocationX());
				yCoords.add(s.getLocationY());
			}
		}
		String xString = "[" + xCoords.get(0);
		String yString = "[" + yCoords.get(0);
		for (int i = 1; i < xCoords.size(); i++) {
			xString = xString + ", " + xCoords;
		}
		xString = xString + "]";

		for (int i = 1; i < yCoords.size(); i++) {
			yString = yString + ", " + yCoords;
		}
		yString = yString + "]";

		// String distributionForXCoords = getDistFromMatlab(xString);
		// String distributionForYCoords = getDistFromMatlab(yString) ;
		// distResults.put("Distribution X", distributionForXCoords);
		// distResults.put("Distribution Y", distributionForYCoords);

		return distResults;

	}

	public static String getDistFromMatlab(String data)
	// throws MatlabInvocationException, MatlabConnectionException
	{
		// MatlabProxyFactory factory = new MatlabProxyFactory();
		// MatlabProxy proxy = factory.getProxy();
		// String result = " ";
		// MatlabTypeConverter processor = new MatlabTypeConverter(proxy);

		// proxy.eval("data = " + data);
		// proxy.eval("[D PD] = allfitdist(data);");
		// proxy.eval("ans = D(1)");
		// proxy.eval("answer = D(1).Params");
		// MatlabNumericArray array = processor.getNumericArray("answer");
		// result = ((Object[]) ((Object[]) ((Object[])
		// proxy.getVariable("ans"))[1])[0])[0]
		// .toString();
		// for (int i = 0; i < array.getLength(); i++) {
		// result = result + " " + array.getRealValue(i);

		// }

		// proxy.disconnect();
		String result = "";
		return result;

	}

	/**
	 * The method removes all registered services
	 * 
	 * @return
	 */
	public static boolean empty() {
		regSet = new HashSet<ServiceDescription>();
		regMap = new HashMap<String, Long>();
		regSet.clear();
		regMap.clear();
		return true;
	}

	/**
	 * This method returns the subset of devices based on a required size and a
	 * known distribution. It will call the StatUtils method which will
	 * determine randomly which devices to return as discovered. for now we
	 * assume probabilistic lookup works for one concept (it is used for events
	 * that spread and events that need 6 sensors)
	 * 
	 * 
	 * @param globalRegion
	 * @param concepts
	 * @param distribution
	 *            : the distribution that nodes to select to follow
	 * @param dmaxVal
	 *            : the maximum distance beyond which nodes are not considered
	 * @param subsetSizeVal
	 *            : the total number to nodes to select
	 * @param locOfInterest
	 *            : the location at which the event of interest is taking place
	 * @return
	 */
	public static Set<ServiceDescription> findSubSetOfDevice(
			String globalRegion, String[] concepts, String distribution,
			double dmaxVal, int subsetSizeVal, Point locOfInterest, long t) {
		double ratio = 10;
		Set<ServiceDescription> fullSet = new HashSet<ServiceDescription>();
		Set<ServiceDescription> subSet = new HashSet<ServiceDescription>();
		Map<String, List<Location>> fullMap = new HashMap<String, List<Location>>();
		List<Location> estimatedPathPoints = new ArrayList<Location>();

		fullSet = findDevice(globalRegion, concepts);
		double radius = 0;
		for (ServiceDescription s : fullSet) {
			estimatedPathPoints = new ArrayList<Location>();
			String estimatedPathString = s.getEstimatedPath();
			String[] estimatedPathArr = estimatedPathString.split(";");
			radius = s.getRange();
			for (int i = 0; i < estimatedPathArr.length; i++) {
				String[] locCoords = estimatedPathArr[i].split(",");
				double xCoord = Double.parseDouble(locCoords[0]);
				double yCoord = Double.parseDouble(locCoords[1]);
				Long time = Long.parseLong(locCoords[2]);
				// System.out.println(xCoord);
				estimatedPathPoints.add(new Location(xCoord, yCoord, time));

			}
			fullMap.put(s.getAddress(), estimatedPathPoints);
		}
		List<String> selectedSubSet = new ArrayList<String>();
		if (distribution.equals("normal"))
			selectedSubSet = StatUtil.getFinalSubSet(fullMap, dmaxVal,
					locOfInterest, subsetSizeVal, ratio, t);
		if (dmaxVal == 0)
			selectedSubSet = StatUtil.getUniformFinalSubSet(fullMap,
					new BorderPoints(new Point(locOfInterest.getX() - radius,
							locOfInterest.getY() - radius), new Point(
							locOfInterest.getX() + radius, locOfInterest.getY()
									- radius), new Point(locOfInterest.getX()
							- radius, locOfInterest.getY() + radius),
							new Point(locOfInterest.getX() + radius,
									locOfInterest.getY() + radius)), 6, 100, t);
		for (String service : selectedSubSet) {
			for (ServiceDescription sd : fullSet) {
				if (sd.getAddress().equals(service))
					subSet.add(sd);
			}
		}

		return subSet;
	}

	public static Set<ServiceDescription> findUniformSubSetOfDevice(
			String globalRegion, String[] concepts, String distribution,
			int subsetSizeVal, BorderPoints AOI, long t) {
		double ratio = 20;
		Set<ServiceDescription> fullSet = new HashSet<ServiceDescription>();
		Set<ServiceDescription> subSet = new HashSet<ServiceDescription>();
		Map<String, List<Location>> fullMap = new HashMap<String, List<Location>>();
		List<Location> estimatedPathPoints = new ArrayList<Location>();

		fullSet = findDevice(globalRegion, concepts);
		double radius = 0;
		for (ServiceDescription s : fullSet) {
			estimatedPathPoints = new ArrayList<Location>();
			String estimatedPathString = s.getEstimatedPath();
			String[] estimatedPathArr = estimatedPathString.split(";");
			radius = s.getRange();
			for (int i = 0; i < estimatedPathArr.length; i++) {
				String[] locCoords = estimatedPathArr[i].split(",");
				double xCoord = Double.parseDouble(locCoords[0]);
				double yCoord = Double.parseDouble(locCoords[1]);
				Long time = Long.parseLong(locCoords[2]);
				// System.out.println(xCoord);
				estimatedPathPoints.add(new Location(xCoord, yCoord, time));

			}
			fullMap.put(s.getAddress(), estimatedPathPoints);
		}
		List<String> selectedSubSet = new ArrayList<String>();

		if (distribution.equals("uniform"))
			selectedSubSet = StatUtil.getUniformFinalSubSet(fullMap, AOI,
					subsetSizeVal, ratio, t);
		for (String service : selectedSubSet) {
			for (ServiceDescription sd : fullSet) {
				if (sd.getAddress().equals(service))
					subSet.add(sd);
			}
		}

		return subSet;
	}

	public static boolean decide(String estimatedPath, String concept,
			double threshold, double range) {
		List<Location> mainPathPoints = new ArrayList<Location>();
		String[] mainPathArr = estimatedPath.split(";");
		for (int i = 0; i < mainPathArr.length; i++) {
			String[] mainLocCoords = mainPathArr[i].split(",");
			double mainXCoord = Double.parseDouble(mainLocCoords[0]);
			double mainYCoord = Double.parseDouble(mainLocCoords[1]);
			Long mainTime = Long.parseLong(mainLocCoords[2]);
			mainPathPoints.add(new Location(mainXCoord, mainYCoord, mainTime));
		}
		ArrayList<Location> coveredLocations = new ArrayList<Location>();
		for (ServiceDescription s : regSet) {
			if (s.getConcept().equals(concept)) {
				ArrayList<Location> estimatedPathPoints = new ArrayList<Location>();
				String path = s.getEstimatedPath();
				String[] estimatedPathArr = path.split(";");

				for (int i = 0; i < estimatedPathArr.length; i++) {
					String[] locCoords = estimatedPathArr[i].split(",");
					double xCoord = Double.parseDouble(locCoords[0]);
					double yCoord = Double.parseDouble(locCoords[1]);
					Long time = Long.parseLong(locCoords[2]);
					estimatedPathPoints.add(new Location(xCoord, yCoord, time));

				}
				ArrayList<Location> locs = StatUtil.intersects(mainPathPoints,
						estimatedPathPoints, s.getRange());
				/*
				 * for (int i = 0; i < locs.size(); i++) for (Location l : locs)
				 * if (locs.get(i).getXCoordinate() != l.getXCoordinate() &&
				 * locs.get(i).getYCoordinate() != l .getYCoordinate())
				 * coveredLocations.add(l);
				 */
				coveredLocations.addAll(locs);
			}

		}
		boolean registrationDecision = false;
		double distance = 0;
		double totalDistance = 0;
		double oldX = 0;
		double oldY = 0;
		int locationCounter = 0;
		double oldXTotal = 0;
		double oldYTotal = 0;
		for (Location ml : mainPathPoints) {

			// because there might be overlaps between sensing coverage of
			// locations, we can t go beyond
			// the distance between two locations if they are less than
			// twice the sensing range (i add full range backwads and
			// intersection of ranges covered forward)
			if (locationCounter > 0)
				totalDistance = totalDistance
						+ Math.sqrt(Math.pow((oldXTotal - ml.getXCoordinate()),
								2)
								+ Math.pow((oldYTotal - ml.getYCoordinate()), 2));

			if (coveredLocations.contains(ml) == true) {
				if (locationCounter == 0) {

					distance = distance + range;

				} else if (locationCounter < mainPathPoints.size() - 1) {
					// totalDistance = totalDistance
					// + Math.sqrt(Math.pow((oldXTotal - ml.getXCoordinate()),
					// 2)
					// + Math.pow((oldYTotal - ml.getYCoordinate()), 2));
					if (Math.sqrt(Math.pow((ml.getXCoordinate() - oldX), 2)
							+ Math.pow((ml.getYCoordinate() - oldY), 2)) >= 2 * range)
						// totalDistance = totalDistance
						// + Math.sqrt(Math.pow((oldXTotal -
						// ml.getXCoordinate()),
						// 2)
						// + Math.pow((oldYTotal - ml.getYCoordinate()), 2));
						distance = distance + 2 * range;
					else
						distance = distance
								+ Math.sqrt(Math.pow(
										(ml.getXCoordinate() - oldX), 2)
										+ Math.pow(
												(ml.getYCoordinate() - oldY), 2));
				} else if (locationCounter == mainPathPoints.size() - 1)
					if (Math.sqrt(Math.pow((ml.getXCoordinate() - oldX), 2)
							+ Math.pow((ml.getYCoordinate() - oldY), 2)) >= 2 * range)
						distance = distance + range;
					else
						distance = distance
								- range
								+ Math.sqrt(Math.pow(
										(ml.getXCoordinate() - oldX), 2)
										+ Math.pow(
												(ml.getYCoordinate() - oldY), 2));

				// double oldX = 0, oldY = 0;
				// distance = distance
				// + Math.sqrt(Math.pow((oldX - ml.getXCoordinate()), 2)
				// + Math.pow((oldY - ml.getYCoordinate()), 2));
				oldX = ml.getXCoordinate();
				oldY = ml.getYCoordinate();
			}
			oldXTotal = ml.getXCoordinate();
			oldYTotal = ml.getYCoordinate();

			locationCounter++;
		}
		System.out.println("distance " + distance);
		System.out.println("totalDistance " + totalDistance);
		System.out.println(distance/totalDistance);
		if (distance / totalDistance >= (threshold- 0.00001))
			registrationDecision = false;
		else {
			registrationDecision = true;

		}

		return registrationDecision;
	}
}